// file: 001-1-phonegap-1.1.0.js
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (typeof PhoneGap === "undefined") {

/**
 * The order of events during page load and PhoneGap startup is as follows:
 *
 * onDOMContentLoaded         Internal event that is received when the web page is loaded and parsed.
 * window.onload              Body onload event.
 * onNativeReady              Internal event that indicates the PhoneGap native side is ready.
 * onPhoneGapInit             Internal event that kicks off creation of all PhoneGap JavaScript objects (runs constructors).
 * onPhoneGapReady            Internal event fired when all PhoneGap JavaScript objects have been created
 * onPhoneGapInfoReady        Internal event fired when device properties are available
 * onDeviceReady              User event fired to indicate that PhoneGap is ready
 * onResume                   User event fired to indicate a start/resume lifecycle event
 * onPause                    User event fired to indicate a pause lifecycle event
 * onDestroy                  Internal event fired when app is being destroyed (User should use window.onunload event, not this one).
 *
 * The only PhoneGap events that user code should register for are:
 *      onDeviceReady
 *      onResume
 *
 * Listeners can be registered as:
 *      document.addEventListener("deviceready", myDeviceReadyListener, false);
 *      document.addEventListener("resume", myResumeListener, false);
 *      document.addEventListener("pause", myPauseListener, false);
 */

if (typeof(DeviceInfo) !== 'object') {
    var DeviceInfo = {};
}

/**
 * This represents the PhoneGap API itself, and provides a global namespace for accessing
 * information about the state of PhoneGap.
 * @class
 */
var PhoneGap = {
    queue: {
        ready: true,
        commands: [],
        timer: null
    },
    documentEventHandler: {},   // Collection of custom document event handlers
    windowEventHandler: {}      // Collection of custom window event handlers
};

/**
 * List of resource files loaded by PhoneGap.
 * This is used to ensure JS and other files are loaded only once.
 */
PhoneGap.resources = {base: true};

/**
 * Determine if resource has been loaded by PhoneGap
 *
 * @param name
 * @return
 */
PhoneGap.hasResource = function(name) {
    return PhoneGap.resources[name];
};

/**
 * Add a resource to list of loaded resources by PhoneGap
 *
 * @param name
 */
PhoneGap.addResource = function(name) {
    PhoneGap.resources[name] = true;
};

/**
 * Custom pub-sub channel that can have functions subscribed to it
 * @constructor
 */
PhoneGap.Channel = function (type)
{
    this.type = type;
    this.handlers = {};
    this.guid = 0;
    this.fired = false;
    this.enabled = true;
};

/**
 * Subscribes the given function to the channel. Any time that
 * Channel.fire is called so too will the function.
 * Optionally specify an execution context for the function
 * and a guid that can be used to stop subscribing to the channel.
 * Returns the guid.
 */
PhoneGap.Channel.prototype.subscribe = function(f, c, g) {
    // need a function to call
    if (f === null) { return; }

    var func = f;
    if (typeof c === "object" && typeof f === "function") { func = PhoneGap.close(c, f); }

    g = g || func.observer_guid || f.observer_guid || this.guid++;
    func.observer_guid = g;
    f.observer_guid = g;
    this.handlers[g] = func;
    return g;
};

/**
 * Like subscribe but the function is only called once and then it
 * auto-unsubscribes itself.
 */
PhoneGap.Channel.prototype.subscribeOnce = function(f, c) {
    var g = null;
    var _this = this;
    var m = function() {
        f.apply(c || null, arguments);
        _this.unsubscribe(g);
    };
    if (this.fired) {
        if (typeof c === "object" && typeof f === "function") { f = PhoneGap.close(c, f); }
        f.apply(this, this.fireArgs);
    } else {
        g = this.subscribe(m);
    }
    return g;
};

/**
 * Unsubscribes the function with the given guid from the channel.
 */
PhoneGap.Channel.prototype.unsubscribe = function(g) {
    if (typeof g === "function") { g = g.observer_guid; }
    this.handlers[g] = null;
    delete this.handlers[g];
};

/**
 * Calls all functions subscribed to this channel.
 */
PhoneGap.Channel.prototype.fire = function(e) {
    if (this.enabled) {
        var fail = false;
        var item, handler, rv;
        for (item in this.handlers) {
            if (this.handlers.hasOwnProperty(item)) {
                handler = this.handlers[item];
                if (typeof handler === "function") {
                    rv = (handler.apply(this, arguments) === false);
                    fail = fail || rv;
                }
            }
        }
        this.fired = true;
        this.fireArgs = arguments;
        return !fail;
    }
    return true;
};

/**
 * Calls the provided function only after all of the channels specified
 * have been fired.
 */
PhoneGap.Channel.join = function(h, c) {
    var i = c.length;
    var f = function() {
        if (!(--i)) {
            h();
        }
    };
    var len = i;
    var j;
    for (j=0; j<len; j++) {
        if (!c[j].fired) {
            c[j].subscribeOnce(f);
        }
        else {
            i--;
        }
    }
    if (!i) {
        h();
    }
};

/**
 * Boolean flag indicating if the PhoneGap API is available and initialized.
 */ // TODO: Remove this, it is unused here ... -jm
PhoneGap.available = DeviceInfo.uuid !== undefined;

/**
 * Add an initialization function to a queue that ensures it will run and initialize
 * application constructors only once PhoneGap has been initialized.
 * @param {Function} func The function callback you want run once PhoneGap is initialized
 */
PhoneGap.addConstructor = function(func) {
    PhoneGap.onPhoneGapInit.subscribeOnce(function() {
        try {
            func();
        } catch(e) {
            console.log("Failed to run constructor: " + e);
        }
    });
};

/**
 * Plugins object
 */
if (!window.plugins) {
    window.plugins = {};
}

/**
 * Adds a plugin object to window.plugins.
 * The plugin is accessed using window.plugins.<name>
 *
 * @param name          The plugin name
 * @param obj           The plugin object
 */
PhoneGap.addPlugin = function(name, obj) {
    if (!window.plugins[name]) {
        window.plugins[name] = obj;
    }
    else {
        console.log("Error: Plugin "+name+" already exists.");
    }
};

/**
 * onDOMContentLoaded channel is fired when the DOM content
 * of the page has been parsed.
 */
PhoneGap.onDOMContentLoaded = new PhoneGap.Channel('onDOMContentLoaded');

/**
 * onNativeReady channel is fired when the PhoneGap native code
 * has been initialized.
 */
PhoneGap.onNativeReady = new PhoneGap.Channel('onNativeReady');

/**
 * onPhoneGapInit channel is fired when the web page is fully loaded and
 * PhoneGap native code has been initialized.
 */
PhoneGap.onPhoneGapInit = new PhoneGap.Channel('onPhoneGapInit');

/**
 * onPhoneGapReady channel is fired when the JS PhoneGap objects have been created.
 */
PhoneGap.onPhoneGapReady = new PhoneGap.Channel('onPhoneGapReady');

/**
 * onPhoneGapInfoReady channel is fired when the PhoneGap device properties
 * has been set.
 */
PhoneGap.onPhoneGapInfoReady = new PhoneGap.Channel('onPhoneGapInfoReady');

/**
 * onPhoneGapConnectionReady channel is fired when the PhoneGap connection properties
 * has been set.
 */
PhoneGap.onPhoneGapConnectionReady = new PhoneGap.Channel('onPhoneGapConnectionReady');

/**
 * onResume channel is fired when the PhoneGap native code
 * resumes.
 */
PhoneGap.onResume = new PhoneGap.Channel('onResume');

/**
 * onPause channel is fired when the PhoneGap native code
 * pauses.
 */
PhoneGap.onPause = new PhoneGap.Channel('onPause');

/**
 * onDestroy channel is fired when the PhoneGap native code
 * is destroyed.  It is used internally.
 * Window.onunload should be used by the user.
 */
PhoneGap.onDestroy = new PhoneGap.Channel('onDestroy');
PhoneGap.onDestroy.subscribeOnce(function() {
    PhoneGap.shuttingDown = true;
});
PhoneGap.shuttingDown = false;

// _nativeReady is global variable that the native side can set
// to signify that the native code is ready. It is a global since
// it may be called before any PhoneGap JS is ready.
if (typeof _nativeReady !== 'undefined') { PhoneGap.onNativeReady.fire(); }

/**
 * onDeviceReady is fired only after all PhoneGap objects are created and
 * the device properties are set.
 */
PhoneGap.onDeviceReady = new PhoneGap.Channel('onDeviceReady');


// Array of channels that must fire before "deviceready" is fired
PhoneGap.deviceReadyChannelsArray = [ PhoneGap.onPhoneGapReady, PhoneGap.onPhoneGapInfoReady, PhoneGap.onPhoneGapConnectionReady];

// Hashtable of user defined channels that must also fire before "deviceready" is fired
PhoneGap.deviceReadyChannelsMap = {};

/**
 * Indicate that a feature needs to be initialized before it is ready to be used.
 * This holds up PhoneGap's "deviceready" event until the feature has been initialized
 * and PhoneGap.initComplete(feature) is called.
 *
 * @param feature {String}     The unique feature name
 */
PhoneGap.waitForInitialization = function(feature) {
    if (feature) {
        var channel = new PhoneGap.Channel(feature);
        PhoneGap.deviceReadyChannelsMap[feature] = channel;
        PhoneGap.deviceReadyChannelsArray.push(channel);
    }
};

/**
 * Indicate that initialization code has completed and the feature is ready to be used.
 *
 * @param feature {String}     The unique feature name
 */
PhoneGap.initializationComplete = function(feature) {
    var channel = PhoneGap.deviceReadyChannelsMap[feature];
    if (channel) {
        channel.fire();
    }
};

/**
 * Create all PhoneGap objects once page has fully loaded and native side is ready.
 */
PhoneGap.Channel.join(function() {

    // Start listening for XHR callbacks
    setTimeout(function() {
            if (PhoneGap.UsePolling) {
                PhoneGap.JSCallbackPolling();
            }
            else {
                var polling = prompt("usePolling", "gap_callbackServer:");
                PhoneGap.UsePolling = polling;
                if (polling == "true") {
                    PhoneGap.UsePolling = true;
                    PhoneGap.JSCallbackPolling();
                }
                else {
                    PhoneGap.UsePolling = false;
                    PhoneGap.JSCallback();
                }
            }
        }, 1);

    // Run PhoneGap constructors
    PhoneGap.onPhoneGapInit.fire();

    // Fire event to notify that all objects are created
    PhoneGap.onPhoneGapReady.fire();

    // Fire onDeviceReady event once all constructors have run and PhoneGap info has been
    // received from native side, and any user defined initialization channels.
    PhoneGap.Channel.join(function() {
        // Let native code know we are inited on JS side
        prompt("", "gap_init:");

        PhoneGap.onDeviceReady.fire();

        // Fire the onresume event, since first one happens before JavaScript is loaded
        PhoneGap.onResume.fire();
    }, PhoneGap.deviceReadyChannelsArray);

}, [ PhoneGap.onDOMContentLoaded, PhoneGap.onNativeReady ]);

// Listen for DOMContentLoaded and notify our channel subscribers
document.addEventListener('DOMContentLoaded', function() {
    PhoneGap.onDOMContentLoaded.fire();
}, false);

// Intercept calls to document.addEventListener and watch for deviceready
PhoneGap.m_document_addEventListener = document.addEventListener;

// Intercept calls to window.addEventListener
PhoneGap.m_window_addEventListener = window.addEventListener;

/**
 * Add a custom window event handler.
 *
 * @param {String} event            The event name that callback handles
 * @param {Function} callback       The event handler
 */
PhoneGap.addWindowEventHandler = function(event, callback) {
    PhoneGap.windowEventHandler[event] = callback;
}

/**
 * Add a custom document event handler.
 *
 * @param {String} event            The event name that callback handles
 * @param {Function} callback       The event handler
 */
PhoneGap.addDocumentEventHandler = function(event, callback) {
    PhoneGap.documentEventHandler[event] = callback;
}

/**
 * Intercept adding document event listeners and handle our own
 *
 * @param {Object} evt
 * @param {Function} handler
 * @param capture
 */
document.addEventListener = function(evt, handler, capture) {
    var e = evt.toLowerCase();
    if (e === 'deviceready') {
        PhoneGap.onDeviceReady.subscribeOnce(handler);
    } else if (e === 'resume') {
        PhoneGap.onResume.subscribe(handler);
        if (PhoneGap.onDeviceReady.fired) {
            PhoneGap.onResume.fire();
        }
    } else if (e === 'pause') {
        PhoneGap.onPause.subscribe(handler);
    }
    else {
        // If subscribing to Android backbutton
        if (e === 'backbutton') {
            PhoneGap.exec(null, null, "App", "overrideBackbutton", [true]);
        }
        
        // If subscribing to an event that is handled by a plugin
        else if (typeof PhoneGap.documentEventHandler[e] !== "undefined") {
            if (PhoneGap.documentEventHandler[e](e, handler, true)) {
                return; // Stop default behavior
            }
        }
        
        PhoneGap.m_document_addEventListener.call(document, evt, handler, capture);
    }
};

/**
 * Intercept adding window event listeners and handle our own
 *
 * @param {Object} evt
 * @param {Function} handler
 * @param capture
 */
window.addEventListener = function(evt, handler, capture) {
    var e = evt.toLowerCase();
        
    // If subscribing to an event that is handled by a plugin
    if (typeof PhoneGap.windowEventHandler[e] !== "undefined") {
        if (PhoneGap.windowEventHandler[e](e, handler, true)) {
            return; // Stop default behavior
        }
    }
        
    PhoneGap.m_window_addEventListener.call(window, evt, handler, capture);
};

// Intercept calls to document.removeEventListener and watch for events that
// are generated by PhoneGap native code
PhoneGap.m_document_removeEventListener = document.removeEventListener;

// Intercept calls to window.removeEventListener
PhoneGap.m_window_removeEventListener = window.removeEventListener;

/**
 * Intercept removing document event listeners and handle our own
 *
 * @param {Object} evt
 * @param {Function} handler
 * @param capture
 */
document.removeEventListener = function(evt, handler, capture) {
    var e = evt.toLowerCase();

    // If unsubscribing to Android backbutton
    if (e === 'backbutton') {
        PhoneGap.exec(null, null, "App", "overrideBackbutton", [false]);
    }

    // If unsubcribing from an event that is handled by a plugin
    if (typeof PhoneGap.documentEventHandler[e] !== "undefined") {
        if (PhoneGap.documentEventHandler[e](e, handler, false)) {
            return; // Stop default behavior
        }
    }

    PhoneGap.m_document_removeEventListener.call(document, evt, handler, capture);
};

/**
 * Intercept removing window event listeners and handle our own
 *
 * @param {Object} evt
 * @param {Function} handler
 * @param capture
 */
window.removeEventListener = function(evt, handler, capture) {
    var e = evt.toLowerCase();

    // If unsubcribing from an event that is handled by a plugin
    if (typeof PhoneGap.windowEventHandler[e] !== "undefined") {
        if (PhoneGap.windowEventHandler[e](e, handler, false)) {
            return; // Stop default behavior
        }
    }

    PhoneGap.m_window_removeEventListener.call(window, evt, handler, capture);
};

/**
 * Method to fire document event
 *
 * @param {String} type             The event type to fire
 * @param {Object} data             Data to send with event
 */
PhoneGap.fireDocumentEvent = function(type, data) {
    var e = document.createEvent('Events');
    e.initEvent(type);
    if (data) {
        for (var i in data) {
            e[i] = data[i];
        }
    }
    document.dispatchEvent(e);
};

/**
 * Method to fire window event
 *
 * @param {String} type             The event type to fire
 * @param {Object} data             Data to send with event
 */
PhoneGap.fireWindowEvent = function(type, data) {
    var e = document.createEvent('Events');
    e.initEvent(type);
    if (data) {
        for (var i in data) {
            e[i] = data[i];
        }
    }
    window.dispatchEvent(e);
};

/**
 * If JSON not included, use our own stringify. (Android 1.6)
 * The restriction on ours is that it must be an array of simple types.
 *
 * @param args
 * @return {String}
 */
PhoneGap.stringify = function(args) {
    if (typeof JSON === "undefined") {
        var s = "[";
        var i, type, start, name, nameType, a;
        for (i = 0; i < args.length; i++) {
            if (args[i] !== null) {
                if (i > 0) {
                    s = s + ",";
                }
                type = typeof args[i];
                if ((type === "number") || (type === "boolean")) {
                    s = s + args[i];
                } else if (args[i] instanceof Array) {
                    s = s + "[" + args[i] + "]";
                } else if (args[i] instanceof Object) {
                    start = true;
                    s = s + '{';
                    for (name in args[i]) {
                        if (args[i][name] !== null) {
                            if (!start) {
                                s = s + ',';
                            }
                            s = s + '"' + name + '":';
                            nameType = typeof args[i][name];
                            if ((nameType === "number") || (nameType === "boolean")) {
                                s = s + args[i][name];
                            } else if ((typeof args[i][name]) === 'function') {
                                // don't copy the functions
                                s = s + '""';
                            } else if (args[i][name] instanceof Object) {
                                s = s + PhoneGap.stringify(args[i][name]);
                            } else {
                                s = s + '"' + args[i][name] + '"';
                            }
                            start = false;
                        }
                    }
                    s = s + '}';
                } else {
                    a = args[i].replace(/\\/g, '\\\\');
                    a = a.replace(/"/g, '\\"');
                    s = s + '"' + a + '"';
                }
            }
        }
        s = s + "]";
        return s;
    } else {
        return JSON.stringify(args);
    }
};

/**
 * Does a deep clone of the object.
 *
 * @param obj
 * @return {Object}
 */
PhoneGap.clone = function(obj) {
    var i, retVal;
    if(!obj) { 
        return obj;
    }
    
    if(obj instanceof Array){
        retVal = [];
        for(i = 0; i < obj.length; ++i){
            retVal.push(PhoneGap.clone(obj[i]));
        }
        return retVal;
    }
    
    if (typeof obj === "function") {
        return obj;
    }
    
    if(!(obj instanceof Object)){
        return obj;
    }
    
    if (obj instanceof Date) {
        return obj;
    }
    
    retVal = {};
    for(i in obj){
        if(!(i in retVal) || retVal[i] !== obj[i]) {
            retVal[i] = PhoneGap.clone(obj[i]);
        }
    }
    return retVal;
};

PhoneGap.callbackId = 0;
PhoneGap.callbacks = {};
PhoneGap.callbackStatus = {
    NO_RESULT: 0,
    OK: 1,
    CLASS_NOT_FOUND_EXCEPTION: 2,
    ILLEGAL_ACCESS_EXCEPTION: 3,
    INSTANTIATION_EXCEPTION: 4,
    MALFORMED_URL_EXCEPTION: 5,
    IO_EXCEPTION: 6,
    INVALID_ACTION: 7,
    JSON_EXCEPTION: 8,
    ERROR: 9
    };


/**
 * Execute a PhoneGap command.  It is up to the native side whether this action is synch or async.
 * The native side can return:
 *      Synchronous: PluginResult object as a JSON string
 *      Asynchrounous: Empty string ""
 * If async, the native side will PhoneGap.callbackSuccess or PhoneGap.callbackError,
 * depending upon the result of the action.
 *
 * @param {Function} success    The success callback
 * @param {Function} fail       The fail callback
 * @param {String} service      The name of the service to use
 * @param {String} action       Action to be run in PhoneGap
 * @param {Array.<String>} [args]     Zero or more arguments to pass to the method
 */
PhoneGap.exec = function(success, fail, service, action, args) {
    try {
        var callbackId = service + PhoneGap.callbackId++;
        if (success || fail) {
            PhoneGap.callbacks[callbackId] = {success:success, fail:fail};
        }

        var r = prompt(PhoneGap.stringify(args), "gap:"+PhoneGap.stringify([service, action, callbackId, true]));

        // If a result was returned
        if (r.length > 0) {
            eval("var v="+r+";");

            // If status is OK, then return value back to caller
            if (v.status === PhoneGap.callbackStatus.OK) {

                // If there is a success callback, then call it now with
                // returned value
                if (success) {
                    try {
                        success(v.message);
                    } catch (e) {
                        console.log("Error in success callback: " + callbackId  + " = " + e);
                    }

                    // Clear callback if not expecting any more results
                    if (!v.keepCallback) {
                        delete PhoneGap.callbacks[callbackId];
                    }
                }
                return v.message;
            }

            // If no result
            else if (v.status === PhoneGap.callbackStatus.NO_RESULT) {

                // Clear callback if not expecting any more results
                if (!v.keepCallback) {
                    delete PhoneGap.callbacks[callbackId];
                }
            }

            // If error, then display error
            else {
                console.log("Error: Status="+v.status+" Message="+v.message);

                // If there is a fail callback, then call it now with returned value
                if (fail) {
                    try {
                        fail(v.message);
                    }
                    catch (e1) {
                        console.log("Error in error callback: "+callbackId+" = "+e1);
                    }

                    // Clear callback if not expecting any more results
                    if (!v.keepCallback) {
                        delete PhoneGap.callbacks[callbackId];
                    }
                }
                return null;
            }
        }
    } catch (e2) {
        console.log("Error: "+e2);
    }
};

/**
 * Called by native code when returning successful result from an action.
 *
 * @param callbackId
 * @param args
 */
PhoneGap.callbackSuccess = function(callbackId, args) {
    if (PhoneGap.callbacks[callbackId]) {

        // If result is to be sent to callback
        if (args.status === PhoneGap.callbackStatus.OK) {
            try {
                if (PhoneGap.callbacks[callbackId].success) {
                    PhoneGap.callbacks[callbackId].success(args.message);
                }
            }
            catch (e) {
                console.log("Error in success callback: "+callbackId+" = "+e);
            }
        }

        // Clear callback if not expecting any more results
        if (!args.keepCallback) {
            delete PhoneGap.callbacks[callbackId];
        }
    }
};

/**
 * Called by native code when returning error result from an action.
 *
 * @param callbackId
 * @param args
 */
PhoneGap.callbackError = function(callbackId, args) {
    if (PhoneGap.callbacks[callbackId]) {
        try {
            if (PhoneGap.callbacks[callbackId].fail) {
                PhoneGap.callbacks[callbackId].fail(args.message);
            }
        }
        catch (e) {
            console.log("Error in error callback: "+callbackId+" = "+e);
        }

        // Clear callback if not expecting any more results
        if (!args.keepCallback) {
            delete PhoneGap.callbacks[callbackId];
        }
    }
};


/**
 * Internal function used to dispatch the request to PhoneGap.  It processes the
 * command queue and executes the next command on the list.  If one of the
 * arguments is a JavaScript object, it will be passed on the QueryString of the
 * url, which will be turned into a dictionary on the other end.
 * @private
 */
// TODO: Is this used?
PhoneGap.run_command = function() {
    if (!PhoneGap.available || !PhoneGap.queue.ready) {
        return;
    }
    PhoneGap.queue.ready = false;

    var args = PhoneGap.queue.commands.shift();
    if (PhoneGap.queue.commands.length === 0) {
        clearInterval(PhoneGap.queue.timer);
        PhoneGap.queue.timer = null;
    }

    var uri = [];
    var dict = null;
    var i;
    for (i = 1; i < args.length; i++) {
        var arg = args[i];
        if (arg === undefined || arg === null) {
            arg = '';
        }
        if (typeof(arg) === 'object') {
            dict = arg;
        } else {
            uri.push(encodeURIComponent(arg));
        }
    }
    var url = "gap://" + args[0] + "/" + uri.join("/");
    if (dict !== null) {
        var name;
        var query_args = [];
        for (name in dict) {
            if (dict.hasOwnProperty(name) && (typeof (name) === 'string')) {
                query_args.push(encodeURIComponent(name) + "=" + encodeURIComponent(dict[name]));
            }
        }
        if (query_args.length > 0) {
            url += "?" + query_args.join("&");
        }
    }
    document.location = url;

};

PhoneGap.JSCallbackPort = null;
PhoneGap.JSCallbackToken = null;

/**
 * This is only for Android.
 *
 * Internal function that uses XHR to call into PhoneGap Java code and retrieve
 * any JavaScript code that needs to be run.  This is used for callbacks from
 * Java to JavaScript.
 */
PhoneGap.JSCallback = function() {

    // Exit if shutting down app
    if (PhoneGap.shuttingDown) {
        return;
    }

    // If polling flag was changed, start using polling from now on
    if (PhoneGap.UsePolling) {
        PhoneGap.JSCallbackPolling();
        return;
    }

    var xmlhttp = new XMLHttpRequest();

    // Callback function when XMLHttpRequest is ready
    xmlhttp.onreadystatechange=function(){
        if(xmlhttp.readyState === 4){

            // Exit if shutting down app
            if (PhoneGap.shuttingDown) {
                return;
            }

            // If callback has JavaScript statement to execute
            if (xmlhttp.status === 200) {

                // Need to url decode the response
                var msg = decodeURIComponent(xmlhttp.responseText);
                setTimeout(function() {
                    try {
                        var t = eval(msg);
                    }
                    catch (e) {
                        // If we're getting an error here, seeing the message will help in debugging
                        console.log("JSCallback: Message from Server: " + msg);
                        console.log("JSCallback Error: "+e);
                    }
                }, 1);
                setTimeout(PhoneGap.JSCallback, 1);
            }

            // If callback ping (used to keep XHR request from timing out)
            else if (xmlhttp.status === 404) {
                setTimeout(PhoneGap.JSCallback, 10);
            }

            // If security error
            else if (xmlhttp.status === 403) {
                console.log("JSCallback Error: Invalid token.  Stopping callbacks.");
            }

            // If server is stopping
            else if (xmlhttp.status === 503) {
                console.log("JSCallback Error: Service unavailable.  Stopping callbacks.");
            }

            // If request wasn't GET
            else if (xmlhttp.status === 400) {
                console.log("JSCallback Error: Bad request.  Stopping callbacks.");
            }

            // If error, revert to polling
            else {
                console.log("JSCallback Error: Request failed.");
                PhoneGap.UsePolling = true;
                PhoneGap.JSCallbackPolling();
            }
        }
    };

    if (PhoneGap.JSCallbackPort === null) {
        PhoneGap.JSCallbackPort = prompt("getPort", "gap_callbackServer:");
    }
    if (PhoneGap.JSCallbackToken === null) {
        PhoneGap.JSCallbackToken = prompt("getToken", "gap_callbackServer:");
    }
    xmlhttp.open("GET", "http://127.0.0.1:"+PhoneGap.JSCallbackPort+"/"+PhoneGap.JSCallbackToken , true);
    xmlhttp.send();
};

/**
 * The polling period to use with JSCallbackPolling.
 * This can be changed by the application.  The default is 50ms.
 */
PhoneGap.JSCallbackPollingPeriod = 50;

/**
 * Flag that can be set by the user to force polling to be used or force XHR to be used.
 */
PhoneGap.UsePolling = false;    // T=use polling, F=use XHR

/**
 * This is only for Android.
 *
 * Internal function that uses polling to call into PhoneGap Java code and retrieve
 * any JavaScript code that needs to be run.  This is used for callbacks from
 * Java to JavaScript.
 */
PhoneGap.JSCallbackPolling = function() {

    // Exit if shutting down app
    if (PhoneGap.shuttingDown) {
        return;
    }

    // If polling flag was changed, stop using polling from now on
    if (!PhoneGap.UsePolling) {
        PhoneGap.JSCallback();
        return;
    }

    var msg = prompt("", "gap_poll:");
    if (msg) {
        setTimeout(function() {
            try {
                var t = eval(""+msg);
            }
            catch (e) {
                console.log("JSCallbackPolling: Message from Server: " + msg);
                console.log("JSCallbackPolling Error: "+e);
            }
        }, 1);
        setTimeout(PhoneGap.JSCallbackPolling, 1);
    }
    else {
        setTimeout(PhoneGap.JSCallbackPolling, PhoneGap.JSCallbackPollingPeriod);
    }
};

/**
 * Create a UUID
 *
 * @return {String}
 */
PhoneGap.createUUID = function() {
    return PhoneGap.UUIDcreatePart(4) + '-' +
        PhoneGap.UUIDcreatePart(2) + '-' +
        PhoneGap.UUIDcreatePart(2) + '-' +
        PhoneGap.UUIDcreatePart(2) + '-' +
        PhoneGap.UUIDcreatePart(6);
};

PhoneGap.UUIDcreatePart = function(length) {
    var uuidpart = "";
    var i, uuidchar;
    for (i=0; i<length; i++) {
        uuidchar = parseInt((Math.random() * 256),0).toString(16);
        if (uuidchar.length === 1) {
            uuidchar = "0" + uuidchar;
        }
        uuidpart += uuidchar;
    }
    return uuidpart;
};

PhoneGap.close = function(context, func, params) {
    if (typeof params === 'undefined') {
        return function() {
            return func.apply(context, arguments);
        };
    } else {
        return function() {
            return func.apply(context, params);
        };
    }
};

/**
 * Load a JavaScript file after page has loaded.
 *
 * @param {String} jsfile               The url of the JavaScript file to load.
 * @param {Function} successCallback    The callback to call when the file has been loaded.
 */
PhoneGap.includeJavascript = function(jsfile, successCallback) {
    var id = document.getElementsByTagName("head")[0];
    var el = document.createElement('script');
    el.type = 'text/javascript';
    if (typeof successCallback === 'function') {
        el.onload = successCallback;
    }
    el.src = jsfile;
    id.appendChild(el);
};

}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("accelerometer")) {
PhoneGap.addResource("accelerometer");

/** @constructor */
var Acceleration = function(x, y, z) {
  this.x = x;
  this.y = y;
  this.z = z;
  this.timestamp = new Date().getTime();
};

/**
 * This class provides access to device accelerometer data.
 * @constructor
 */
var Accelerometer = function() {

    /**
     * The last known acceleration.  type=Acceleration()
     */
    this.lastAcceleration = null;

    /**
     * List of accelerometer watch timers
     */
    this.timers = {};
};

Accelerometer.ERROR_MSG = ["Not running", "Starting", "", "Failed to start"];

/**
 * Asynchronously aquires the current acceleration.
 *
 * @param {Function} successCallback    The function to call when the acceleration data is available
 * @param {Function} errorCallback      The function to call when there is an error getting the acceleration data. (OPTIONAL)
 * @param {AccelerationOptions} options The options for getting the accelerometer data such as timeout. (OPTIONAL)
 */
Accelerometer.prototype.getCurrentAcceleration = function(successCallback, errorCallback, options) {

    // successCallback required
    if (typeof successCallback !== "function") {
        console.log("Accelerometer Error: successCallback is not a function");
        return;
    }

    // errorCallback optional
    if (errorCallback && (typeof errorCallback !== "function")) {
        console.log("Accelerometer Error: errorCallback is not a function");
        return;
    }

    // Get acceleration
    PhoneGap.exec(successCallback, errorCallback, "Accelerometer", "getAcceleration", []);
};

/**
 * Asynchronously aquires the acceleration repeatedly at a given interval.
 *
 * @param {Function} successCallback    The function to call each time the acceleration data is available
 * @param {Function} errorCallback      The function to call when there is an error getting the acceleration data. (OPTIONAL)
 * @param {AccelerationOptions} options The options for getting the accelerometer data such as timeout. (OPTIONAL)
 * @return String                       The watch id that must be passed to #clearWatch to stop watching.
 */
Accelerometer.prototype.watchAcceleration = function(successCallback, errorCallback, options) {

    // Default interval (10 sec)
    var frequency = (options !== undefined)? options.frequency : 10000;

    // successCallback required
    if (typeof successCallback !== "function") {
        console.log("Accelerometer Error: successCallback is not a function");
        return;
    }

    // errorCallback optional
    if (errorCallback && (typeof errorCallback !== "function")) {
        console.log("Accelerometer Error: errorCallback is not a function");
        return;
    }

    // Make sure accelerometer timeout > frequency + 10 sec
    PhoneGap.exec(
        function(timeout) {
            if (timeout < (frequency + 10000)) {
                PhoneGap.exec(null, null, "Accelerometer", "setTimeout", [frequency + 10000]);
            }
        },
        function(e) { }, "Accelerometer", "getTimeout", []);

    // Start watch timer
    var id = PhoneGap.createUUID();
    navigator.accelerometer.timers[id] = setInterval(function() {
        PhoneGap.exec(successCallback, errorCallback, "Accelerometer", "getAcceleration", []);
    }, (frequency ? frequency : 1));

    return id;
};

/**
 * Clears the specified accelerometer watch.
 *
 * @param {String} id       The id of the watch returned from #watchAcceleration.
 */
Accelerometer.prototype.clearWatch = function(id) {

    // Stop javascript timer & remove from timer list
    if (id && navigator.accelerometer.timers[id] !== undefined) {
        clearInterval(navigator.accelerometer.timers[id]);
        delete navigator.accelerometer.timers[id];
    }
};

PhoneGap.addConstructor(function() {
    if (typeof navigator.accelerometer === "undefined") {
        navigator.accelerometer = new Accelerometer();
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("app")) {
PhoneGap.addResource("app");
(function() {

/**
 * Constructor
 * @constructor
 */
var App = function() {};

/**
 * Clear the resource cache.
 */
App.prototype.clearCache = function() {
    PhoneGap.exec(null, null, "App", "clearCache", []);
};

/**
 * Load the url into the webview.
 *
 * @param url           The URL to load
 * @param props         Properties that can be passed in to the activity:
 *      wait: int                           => wait msec before loading URL
 *      loadingDialog: "Title,Message"      => display a native loading dialog
 *      hideLoadingDialogOnPage: boolean    => hide loadingDialog when page loaded instead of when deviceready event occurs.
 *      loadInWebView: boolean              => cause all links on web page to be loaded into existing web view, instead of being loaded into new browser.
 *      loadUrlTimeoutValue: int            => time in msec to wait before triggering a timeout error
 *      errorUrl: URL                       => URL to load if there's an error loading specified URL with loadUrl().  Should be a local URL such as file:///android_asset/www/error.html");
 *      keepRunning: boolean                => enable app to keep running in background
 *
 * Example:
 *      App app = new App();
 *      app.loadUrl("http://server/myapp/index.html", {wait:2000, loadingDialog:"Wait,Loading App", loadUrlTimeoutValue: 60000});
 */
App.prototype.loadUrl = function(url, props) {
    PhoneGap.exec(null, null, "App", "loadUrl", [url, props]);
};

/**
 * Cancel loadUrl that is waiting to be loaded.
 */
App.prototype.cancelLoadUrl = function() {
    PhoneGap.exec(null, null, "App", "cancelLoadUrl", []);
};

/**
 * Clear web history in this web view.
 * Instead of BACK button loading the previous web page, it will exit the app.
 */
App.prototype.clearHistory = function() {
    PhoneGap.exec(null, null, "App", "clearHistory", []);
};

/**
 * Go to previous page displayed.
 * This is the same as pressing the backbutton on Android device.
 */
App.prototype.backHistory = function() {
    PhoneGap.exec(null, null, "App", "backHistory", []);
};

/**
 * Override the default behavior of the Android back button.
 * If overridden, when the back button is pressed, the "backKeyDown" JavaScript event will be fired.
 *
 * Note: The user should not have to call this method.  Instead, when the user
 *       registers for the "backbutton" event, this is automatically done.
 *
 * @param override		T=override, F=cancel override
 */
App.prototype.overrideBackbutton = function(override) {
    PhoneGap.exec(null, null, "App", "overrideBackbutton", [override]);
};

/**
 * Exit and terminate the application.
 */
App.prototype.exitApp = function() {
	return PhoneGap.exec(null, null, "App", "exitApp", []);
};

/**
 * Add entry to approved list of URLs (whitelist) that will be loaded into PhoneGap container instead of default browser.
 * 
 * @param origin		URL regular expression to allow
 * @param subdomains	T=include all subdomains under origin
 */
App.prototype.addWhiteListEntry = function(origin, subdomains) {
	return PhoneGap.exec(null, null, "App", "addWhiteListEntry", [origin, subdomains]);	
};

PhoneGap.addConstructor(function() {
    navigator.app = new App();
    navigator.app.origHistoryBack = window.history.back;
    window.history.back = navigator.app.backHistory;
});
}());
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("battery")) {
PhoneGap.addResource("battery");

/**
 * This class contains information about the current battery status.
 * @constructor
 */
var Battery = function() {
    this._level = null;
    this._isPlugged = null;
    this._batteryListener = [];
    this._lowListener = [];
    this._criticalListener = [];
};

/**
 * Registers as an event producer for battery events.
 * 
 * @param {Object} eventType
 * @param {Object} handler
 * @param {Object} add
 */
Battery.prototype.eventHandler = function(eventType, handler, add) {
    var me = navigator.battery;
    if (add) {
        // If there are no current registered event listeners start the battery listener on native side.
        if (me._batteryListener.length === 0 && me._lowListener.length === 0 && me._criticalListener.length === 0) {
            PhoneGap.exec(me._status, me._error, "Battery", "start", []);
        }
        
        // Register the event listener in the proper array
        if (eventType === "batterystatus") {
            var pos = me._batteryListener.indexOf(handler);
            if (pos === -1) {
            	me._batteryListener.push(handler);
            }
        } else if (eventType === "batterylow") {
            var pos = me._lowListener.indexOf(handler);
            if (pos === -1) {
            	me._lowListener.push(handler);
            }
        } else if (eventType === "batterycritical") {
            var pos = me._criticalListener.indexOf(handler);
            if (pos === -1) {
            	me._criticalListener.push(handler);
            }
        }
    } else {
        // Remove the event listener from the proper array
        if (eventType === "batterystatus") {
            var pos = me._batteryListener.indexOf(handler);
            if (pos > -1) {
                me._batteryListener.splice(pos, 1);        
            }
        } else if (eventType === "batterylow") {
            var pos = me._lowListener.indexOf(handler);
            if (pos > -1) {
                me._lowListener.splice(pos, 1);        
            }
        } else if (eventType === "batterycritical") {
            var pos = me._criticalListener.indexOf(handler);
            if (pos > -1) {
                me._criticalListener.splice(pos, 1);        
            }
        }
        
        // If there are no more registered event listeners stop the battery listener on native side.
        if (me._batteryListener.length === 0 && me._lowListener.length === 0 && me._criticalListener.length === 0) {
            PhoneGap.exec(null, null, "Battery", "stop", []);
        }
    }
};

/**
 * Callback for battery status
 * 
 * @param {Object} info			keys: level, isPlugged
 */
Battery.prototype._status = function(info) {
	if (info) {
		var me = this;
		if (me._level != info.level || me._isPlugged != info.isPlugged) {
			// Fire batterystatus event
			PhoneGap.fireWindowEvent("batterystatus", info);

			// Fire low battery event
			if (info.level == 20 || info.level == 5) {
				if (info.level == 20) {
					PhoneGap.fireWindowEvent("batterylow", info);
				}
				else {
					PhoneGap.fireWindowEvent("batterycritical", info);
				}
			}
		}
		me._level = info.level;
		me._isPlugged = info.isPlugged;	
	}
};

/**
 * Error callback for battery start
 */
Battery.prototype._error = function(e) {
    console.log("Error initializing Battery: " + e);
};

PhoneGap.addConstructor(function() {
    if (typeof navigator.battery === "undefined") {
        navigator.battery = new Battery();
        PhoneGap.addWindowEventHandler("batterystatus", navigator.battery.eventHandler);
        PhoneGap.addWindowEventHandler("batterylow", navigator.battery.eventHandler);
        PhoneGap.addWindowEventHandler("batterycritical", navigator.battery.eventHandler);
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("camera")) {
PhoneGap.addResource("camera");

/**
 * This class provides access to the device camera.
 *
 * @constructor
 */
var Camera = function() {
    this.successCallback = null;
    this.errorCallback = null;
    this.options = null;
};

/**
 * Format of image that returned from getPicture.
 *
 * Example: navigator.camera.getPicture(success, fail,
 *              { quality: 80,
 *                destinationType: Camera.DestinationType.DATA_URL,
 *                sourceType: Camera.PictureSourceType.PHOTOLIBRARY})
 */
Camera.DestinationType = {
    DATA_URL: 0,                // Return base64 encoded string
    FILE_URI: 1                 // Return file uri (content://media/external/images/media/2 for Android)
};
Camera.prototype.DestinationType = Camera.DestinationType;

/**
 * Encoding of image returned from getPicture.
 *
 * Example: navigator.camera.getPicture(success, fail,
 *              { quality: 80,
 *                destinationType: Camera.DestinationType.DATA_URL,
 *                sourceType: Camera.PictureSourceType.CAMERA,
 *                encodingType: Camera.EncodingType.PNG})
*/
Camera.EncodingType = {
    JPEG: 0,                    // Return JPEG encoded image
    PNG: 1                      // Return PNG encoded image
};
Camera.prototype.EncodingType = Camera.EncodingType;

/**
 * Type of pictures to select from.  Only applicable when
 *      PictureSourceType is PHOTOLIBRARY or SAVEDPHOTOALBUM
 *
 * Example: navigator.camera.getPicture(success, fail,
 *              { quality: 80,
 *                destinationType: Camera.DestinationType.DATA_URL,
 *                sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
 *                mediaType: Camera.MediaType.PICTURE})
 */
Camera.MediaType = {
       PICTURE: 0,      // allow selection of still pictures only. DEFAULT. Will return format specified via DestinationType
       VIDEO: 1,        // allow selection of video only, ONLY RETURNS URL
       ALLMEDIA : 2     // allow selection from all media types
};
Camera.prototype.MediaType = Camera.MediaType;


/**
 * Source to getPicture from.
 *
 * Example: navigator.camera.getPicture(success, fail,
 *              { quality: 80,
 *                destinationType: Camera.DestinationType.DATA_URL,
 *                sourceType: Camera.PictureSourceType.PHOTOLIBRARY})
 */
Camera.PictureSourceType = {
    PHOTOLIBRARY : 0,           // Choose image from picture library (same as SAVEDPHOTOALBUM for Android)
    CAMERA : 1,                 // Take picture from camera
    SAVEDPHOTOALBUM : 2         // Choose image from picture library (same as PHOTOLIBRARY for Android)
};
Camera.prototype.PictureSourceType = Camera.PictureSourceType;

/**
 * Gets a picture from source defined by "options.sourceType", and returns the
 * image as defined by the "options.destinationType" option.

 * The defaults are sourceType=CAMERA and destinationType=DATA_URL.
 *
 * @param {Function} successCallback
 * @param {Function} errorCallback
 * @param {Object} options
 */
Camera.prototype.getPicture = function(successCallback, errorCallback, options) {

    // successCallback required
    if (typeof successCallback !== "function") {
        console.log("Camera Error: successCallback is not a function");
        return;
    }

    // errorCallback optional
    if (errorCallback && (typeof errorCallback !== "function")) {
        console.log("Camera Error: errorCallback is not a function");
        return;
    }
    
    if (options === null || typeof options === "undefined") {
        options = {};
    }
    if (options.quality === null || typeof options.quality === "undefined") {
        options.quality = 80;
    }
    if (options.maxResolution === null || typeof options.maxResolution === "undefined") {
    	options.maxResolution = 0;
    }
    if (options.destinationType === null || typeof options.destinationType === "undefined") {
        options.destinationType = Camera.DestinationType.DATA_URL;
    }
    if (options.sourceType === null || typeof options.sourceType === "undefined") {
        options.sourceType = Camera.PictureSourceType.CAMERA;
    }
    if (options.encodingType === null || typeof options.encodingType === "undefined") {
        options.encodingType = Camera.EncodingType.JPEG;
    }
    if (options.mediaType === null || typeof options.mediaType === "undefined") {
        options.mediaType = Camera.MediaType.PICTURE;
    }
    if (options.targetWidth === null || typeof options.targetWidth === "undefined") {
        options.targetWidth = -1;
    } 
    else if (typeof options.targetWidth == "string") {
        var width = new Number(options.targetWidth);
        if (isNaN(width) === false) {
            options.targetWidth = width.valueOf();
        }
    }
    if (options.targetHeight === null || typeof options.targetHeight === "undefined") {
        options.targetHeight = -1;
    } 
    else if (typeof options.targetHeight == "string") {
        var height = new Number(options.targetHeight);
        if (isNaN(height) === false) {
            options.targetHeight = height.valueOf();
        }
    }
    
    PhoneGap.exec(successCallback, errorCallback, "Camera", "takePicture", [options]);
};

PhoneGap.addConstructor(function() {
    if (typeof navigator.camera === "undefined") {
        navigator.camera = new Camera();
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("capture")) {
PhoneGap.addResource("capture");
	
/**
 * Represents a single file.
 *
 * name {DOMString} name of the file, without path information
 * fullPath {DOMString} the full path of the file, including the name
 * type {DOMString} mime type
 * lastModifiedDate {Date} last modified date
 * size {Number} size of the file in bytes
 */
var MediaFile = function(name, fullPath, type, lastModifiedDate, size){
	this.name = name || null;
	this.fullPath = fullPath || null;
	this.type = type || null;
	this.lastModifiedDate = lastModifiedDate || null;
	this.size = size || 0;
};

/**
 * Launch device camera application for recording video(s).
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 */
MediaFile.prototype.getFormatData = function(successCallback, errorCallback){
	PhoneGap.exec(successCallback, errorCallback, "Capture", "getFormatData", [this.fullPath, this.type]);
};

/**
 * MediaFileData encapsulates format information of a media file.
 *
 * @param {DOMString} codecs
 * @param {long} bitrate
 * @param {long} height
 * @param {long} width
 * @param {float} duration
 */
var MediaFileData = function(codecs, bitrate, height, width, duration){
	this.codecs = codecs || null;
	this.bitrate = bitrate || 0;
	this.height = height || 0;
	this.width = width || 0;
	this.duration = duration || 0;
};

/**
 * The CaptureError interface encapsulates all errors in the Capture API.
 */
var CaptureError = function(){
	this.code = null;
};

// Capture error codes
CaptureError.CAPTURE_INTERNAL_ERR = 0;
CaptureError.CAPTURE_APPLICATION_BUSY = 1;
CaptureError.CAPTURE_INVALID_ARGUMENT = 2;
CaptureError.CAPTURE_NO_MEDIA_FILES = 3;
CaptureError.CAPTURE_NOT_SUPPORTED = 20;

/**
 * The Capture interface exposes an interface to the camera and microphone of the hosting device.
 */
var Capture = function(){
	this.supportedAudioModes = [];
	this.supportedImageModes = [];
	this.supportedVideoModes = [];
};

/**
 * Launch audio recorder application for recording audio clip(s).
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 * @param {CaptureAudioOptions} options
 */
Capture.prototype.captureAudio = function(successCallback, errorCallback, options){
	PhoneGap.exec(successCallback, errorCallback, "Capture", "captureAudio", [options]);
};

/**
 * Launch camera application for taking image(s).
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 * @param {CaptureImageOptions} options
 */
Capture.prototype.captureImage = function(successCallback, errorCallback, options){
	PhoneGap.exec(successCallback, errorCallback, "Capture", "captureImage", [options]);
};

/**
 * Launch camera application for taking image(s).
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 * @param {CaptureImageOptions} options
 */
Capture.prototype._castMediaFile = function(pluginResult){
	var mediaFiles = [];
	var i;
	for (i = 0; i < pluginResult.message.length; i++) {
		var mediaFile = new MediaFile();
		mediaFile.name = pluginResult.message[i].name;
		mediaFile.fullPath = pluginResult.message[i].fullPath;
		mediaFile.type = pluginResult.message[i].type;
		mediaFile.lastModifiedDate = pluginResult.message[i].lastModifiedDate;
		mediaFile.size = pluginResult.message[i].size;
		mediaFiles.push(mediaFile);
	}
	pluginResult.message = mediaFiles;
	return pluginResult;
};

/**
 * Launch device camera application for recording video(s).
 *
 * @param {Function} successCB
 * @param {Function} errorCB
 * @param {CaptureVideoOptions} options
 */
Capture.prototype.captureVideo = function(successCallback, errorCallback, options){
	PhoneGap.exec(successCallback, errorCallback, "Capture", "captureVideo", [options]);
};

/**
 * Encapsulates a set of parameters that the capture device supports.
 */
var ConfigurationData = function(){
	// The ASCII-encoded string in lower case representing the media type. 
	this.type = null;
	// The height attribute represents height of the image or video in pixels. 
	// In the case of a sound clip this attribute has value 0. 
	this.height = 0;
	// The width attribute represents width of the image or video in pixels. 
	// In the case of a sound clip this attribute has value 0
	this.width = 0;
};

/**
 * Encapsulates all image capture operation configuration options.
 */
var CaptureImageOptions = function(){
	// Upper limit of images user can take. Value must be equal or greater than 1.
	this.limit = 1;
	// The selected image mode. Must match with one of the elements in supportedImageModes array.
	this.mode = null;
};

/**
 * Encapsulates all video capture operation configuration options.
 */
var CaptureVideoOptions = function(){
	// Upper limit of videos user can record. Value must be equal or greater than 1.
	this.limit = 1;
	// Maximum duration of a single video clip in seconds.
	this.duration = 0;
	// The selected video mode. Must match with one of the elements in supportedVideoModes array.
	this.mode = null;
};

/**
 * Encapsulates all audio capture operation configuration options.
 */
var CaptureAudioOptions = function(){
	// Upper limit of sound clips user can record. Value must be equal or greater than 1.
	this.limit = 1;
	// Maximum duration of a single sound clip in seconds.
	this.duration = 0;
	// The selected audio mode. Must match with one of the elements in supportedAudioModes array.
	this.mode = null;
};

PhoneGap.addConstructor(function(){
	if (typeof navigator.device === "undefined") {
		navigator.device = window.device = new Device();
	}
	if (typeof navigator.device.capture === "undefined") {
		navigator.device.capture = window.device.capture = new Capture();
	}
});
}/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("compass")) {
PhoneGap.addResource("compass");

CompassError = function(){
    this.code = null;
};

// Capture error codes
CompassError.COMPASS_INTERNAL_ERR = 0;
CompassError.COMPASS_NOT_SUPPORTED = 20;

CompassHeading = function() {
    this.magneticHeading = null;
    this.trueHeading = null;
    this.headingAccuracy = null;
    this.timestamp = null;
};

/**
 * This class provides access to device Compass data.
 * @constructor
 */
var Compass = function() {
    /**
     * The last known Compass position.
     */
    this.lastHeading = null;

    /**
     * List of compass watch timers
     */
    this.timers = {};
};

Compass.ERROR_MSG = ["Not running", "Starting", "", "Failed to start"];

/**
 * Asynchronously aquires the current heading.
 *
 * @param {Function} successCallback The function to call when the heading data is available
 * @param {Function} errorCallback The function to call when there is an error getting the heading data. (OPTIONAL)
 * @param {PositionOptions} options The options for getting the heading data such as timeout. (OPTIONAL)
 */
Compass.prototype.getCurrentHeading = function(successCallback, errorCallback, options) {

    // successCallback required
    if (typeof successCallback !== "function") {
        console.log("Compass Error: successCallback is not a function");
        return;
    }

    // errorCallback optional
    if (errorCallback && (typeof errorCallback !== "function")) {
        console.log("Compass Error: errorCallback is not a function");
        return;
    }

    // Get heading
    PhoneGap.exec(successCallback, errorCallback, "Compass", "getHeading", []);
};

/**
 * Asynchronously aquires the heading repeatedly at a given interval.
 *
 * @param {Function} successCallback    The function to call each time the heading data is available
 * @param {Function} errorCallback      The function to call when there is an error getting the heading data. (OPTIONAL)
 * @param {HeadingOptions} options      The options for getting the heading data such as timeout and the frequency of the watch. (OPTIONAL)
 * @return String                       The watch id that must be passed to #clearWatch to stop watching.
 */
Compass.prototype.watchHeading= function(successCallback, errorCallback, options) {

    // Default interval (100 msec)
    var frequency = (options !== undefined) ? options.frequency : 100;

    // successCallback required
    if (typeof successCallback !== "function") {
        console.log("Compass Error: successCallback is not a function");
        return;
    }

    // errorCallback optional
    if (errorCallback && (typeof errorCallback !== "function")) {
        console.log("Compass Error: errorCallback is not a function");
        return;
    }

    // Make sure compass timeout > frequency + 10 sec
    PhoneGap.exec(
        function(timeout) {
            if (timeout < (frequency + 10000)) {
                PhoneGap.exec(null, null, "Compass", "setTimeout", [frequency + 10000]);
            }
        },
        function(e) { }, "Compass", "getTimeout", []);

    // Start watch timer to get headings
    var id = PhoneGap.createUUID();
    navigator.compass.timers[id] = setInterval(
        function() {
            PhoneGap.exec(successCallback, errorCallback, "Compass", "getHeading", []);
        }, (frequency ? frequency : 1));

    return id;
};


/**
 * Clears the specified heading watch.
 *
 * @param {String} id       The ID of the watch returned from #watchHeading.
 */
Compass.prototype.clearWatch = function(id) {

    // Stop javascript timer & remove from timer list
    if (id && navigator.compass.timers[id]) {
        clearInterval(navigator.compass.timers[id]);
        delete navigator.compass.timers[id];
    }
};

Compass.prototype._castDate = function(pluginResult) {
    if (pluginResult.message.timestamp) {
        var timestamp = new Date(pluginResult.message.timestamp);
        pluginResult.message.timestamp = timestamp;
    }
    return pluginResult;
};

PhoneGap.addConstructor(function() {
    if (typeof navigator.compass === "undefined") {
        navigator.compass = new Compass();
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("contact")) {
PhoneGap.addResource("contact");

/**
* Contains information about a single contact.
* @constructor
* @param {DOMString} id unique identifier
* @param {DOMString} displayName
* @param {ContactName} name
* @param {DOMString} nickname
* @param {Array.<ContactField>} phoneNumbers array of phone numbers
* @param {Array.<ContactField>} emails array of email addresses
* @param {Array.<ContactAddress>} addresses array of addresses
* @param {Array.<ContactField>} ims instant messaging user ids
* @param {Array.<ContactOrganization>} organizations
* @param {DOMString} birthday contact's birthday
* @param {DOMString} note user notes about contact
* @param {Array.<ContactField>} photos
* @param {Array.<ContactField>} categories
* @param {Array.<ContactField>} urls contact's web sites
*/
var Contact = function (id, displayName, name, nickname, phoneNumbers, emails, addresses,
    ims, organizations, birthday, note, photos, categories, urls) {
    this.id = id || null;
    this.rawId = null;
    this.displayName = displayName || null;
    this.name = name || null; // ContactName
    this.nickname = nickname || null;
    this.phoneNumbers = phoneNumbers || null; // ContactField[]
    this.emails = emails || null; // ContactField[]
    this.addresses = addresses || null; // ContactAddress[]
    this.ims = ims || null; // ContactField[]
    this.organizations = organizations || null; // ContactOrganization[]
    this.birthday = birthday || null;
    this.note = note || null;
    this.photos = photos || null; // ContactField[]
    this.categories = categories || null; // ContactField[]
    this.urls = urls || null; // ContactField[]
};

/**
 *  ContactError.
 *  An error code assigned by an implementation when an error has occurreds
 * @constructor
 */
var ContactError = function() {
    this.code=null;
};

/**
 * Error codes
 */
ContactError.UNKNOWN_ERROR = 0;
ContactError.INVALID_ARGUMENT_ERROR = 1;
ContactError.TIMEOUT_ERROR = 2;
ContactError.PENDING_OPERATION_ERROR = 3;
ContactError.IO_ERROR = 4;
ContactError.NOT_SUPPORTED_ERROR = 5;
ContactError.PERMISSION_DENIED_ERROR = 20;

/**
* Removes contact from device storage.
* @param successCB success callback
* @param errorCB error callback
*/
Contact.prototype.remove = function(successCB, errorCB) {
    if (this.id === null) {
        var errorObj = new ContactError();
        errorObj.code = ContactError.UNKNOWN_ERROR;
        errorCB(errorObj);
    }
    else {
        PhoneGap.exec(successCB, errorCB, "Contacts", "remove", [this.id]);
    }
};

/**
* Creates a deep copy of this Contact.
* With the contact ID set to null.
* @return copy of this Contact
*/
Contact.prototype.clone = function() {
    var clonedContact = PhoneGap.clone(this);
    var i;
    clonedContact.id = null;
    clonedContact.rawId = null;
    // Loop through and clear out any id's in phones, emails, etc.
    if (clonedContact.phoneNumbers) {
        for (i = 0; i < clonedContact.phoneNumbers.length; i++) {
            clonedContact.phoneNumbers[i].id = null;
        }
    }
    if (clonedContact.emails) {
        for (i = 0; i < clonedContact.emails.length; i++) {
            clonedContact.emails[i].id = null;
        }
    }
    if (clonedContact.addresses) {
        for (i = 0; i < clonedContact.addresses.length; i++) {
            clonedContact.addresses[i].id = null;
        }
    }
    if (clonedContact.ims) {
        for (i = 0; i < clonedContact.ims.length; i++) {
            clonedContact.ims[i].id = null;
        }
    }
    if (clonedContact.organizations) {
        for (i = 0; i < clonedContact.organizations.length; i++) {
            clonedContact.organizations[i].id = null;
        }
    }
    if (clonedContact.tags) {
        for (i = 0; i < clonedContact.tags.length; i++) {
            clonedContact.tags[i].id = null;
        }
    }
    if (clonedContact.photos) {
        for (i = 0; i < clonedContact.photos.length; i++) {
            clonedContact.photos[i].id = null;
        }
    }
    if (clonedContact.urls) {
        for (i = 0; i < clonedContact.urls.length; i++) {
            clonedContact.urls[i].id = null;
        }
    }
    return clonedContact;
};

/**
* Persists contact to device storage.
* @param successCB success callback
* @param errorCB error callback
*/
Contact.prototype.save = function(successCB, errorCB) {
    PhoneGap.exec(successCB, errorCB, "Contacts", "save", [this]);
};

/**
* Contact name.
* @constructor
* @param formatted
* @param familyName
* @param givenName
* @param middle
* @param prefix
* @param suffix
*/
var ContactName = function(formatted, familyName, givenName, middle, prefix, suffix) {
    this.formatted = formatted || null;
    this.familyName = familyName || null;
    this.givenName = givenName || null;
    this.middleName = middle || null;
    this.honorificPrefix = prefix || null;
    this.honorificSuffix = suffix || null;
};

/**
* Generic contact field.
* @constructor
* @param {DOMString} id unique identifier, should only be set by native code
* @param type
* @param value
* @param pref
*/
var ContactField = function(type, value, pref) {
	this.id = null;
    this.type = type || null;
    this.value = value || null;
    this.pref = pref || null;
};

/**
* Contact address.
* @constructor
* @param {DOMString} id unique identifier, should only be set by native code
* @param formatted
* @param streetAddress
* @param locality
* @param region
* @param postalCode
* @param country
*/
var ContactAddress = function(pref, type, formatted, streetAddress, locality, region, postalCode, country) {
	this.id = null;
    this.pref = pref || null;
    this.type = type || null;
    this.formatted = formatted || null;
    this.streetAddress = streetAddress || null;
    this.locality = locality || null;
    this.region = region || null;
    this.postalCode = postalCode || null;
    this.country = country || null;
};

/**
* Contact organization.
* @constructor
* @param {DOMString} id unique identifier, should only be set by native code
* @param name
* @param dept
* @param title
* @param startDate
* @param endDate
* @param location
* @param desc
*/
var ContactOrganization = function(pref, type, name, dept, title) {
	this.id = null;
    this.pref = pref || null;
    this.type = type || null;
    this.name = name || null;
    this.department = dept || null;
    this.title = title || null;
};

/**
* Represents a group of Contacts.
* @constructor
*/
var Contacts = function() {
    this.inProgress = false;
    this.records = [];
};
/**
* Returns an array of Contacts matching the search criteria.
* @param fields that should be searched
* @param successCB success callback
* @param errorCB error callback
* @param {ContactFindOptions} options that can be applied to contact searching
* @return array of Contacts matching search criteria
*/
Contacts.prototype.find = function(fields, successCB, errorCB, options) {
    if (successCB === null) {
        throw new TypeError("You must specify a success callback for the find command.");
    }
    if (fields === null || fields === "undefined" || fields.length === "undefined" || fields.length <= 0) {
        if (typeof errorCB === "function") {
            errorCB({"code": ContactError.INVALID_ARGUMENT_ERROR});
        }
    } else {
        PhoneGap.exec(successCB, errorCB, "Contacts", "search", [fields, options]);        
    }
};

/**
* This function creates a new contact, but it does not persist the contact
* to device storage. To persist the contact to device storage, invoke
* contact.save().
* @param properties an object who's properties will be examined to create a new Contact
* @returns new Contact object
*/
Contacts.prototype.create = function(properties) {
    var i;
	var contact = new Contact();
    for (i in properties) {
        if (contact[i] !== 'undefined') {
            contact[i] = properties[i];
        }
    }
    return contact;
};

/**
* This function returns and array of contacts.  It is required as we need to convert raw
* JSON objects into concrete Contact objects.  Currently this method is called after
* navigator.contacts.find but before the find methods success call back.
*
* @param jsonArray an array of JSON Objects that need to be converted to Contact objects.
* @returns an array of Contact objects
*/
Contacts.prototype.cast = function(pluginResult) {
	var contacts = [];
	var i;
	for (i=0; i<pluginResult.message.length; i++) {
		contacts.push(navigator.contacts.create(pluginResult.message[i]));
	}
	pluginResult.message = contacts;
	return pluginResult;
};

/**
 * ContactFindOptions.
 * @constructor
 * @param filter used to match contacts against
 * @param multiple boolean used to determine if more than one contact should be returned
 */
var ContactFindOptions = function(filter, multiple) {
    this.filter = filter || '';
    this.multiple = multiple || false;
};

/**
 * Add the contact interface into the browser.
 */
PhoneGap.addConstructor(function() {
    if(typeof navigator.contacts === "undefined") {
        navigator.contacts = new Contacts();
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

// TODO: Needs to be commented

if (!PhoneGap.hasResource("crypto")) {
PhoneGap.addResource("crypto");

/**
* @constructor
*/
var Crypto = function() {
};

Crypto.prototype.encrypt = function(seed, string, callback) {
    this.encryptWin = callback;
    PhoneGap.exec(null, null, "Crypto", "encrypt", [seed, string]);
};

Crypto.prototype.decrypt = function(seed, string, callback) {
    this.decryptWin = callback;
    PhoneGap.exec(null, null, "Crypto", "decrypt", [seed, string]);
};

Crypto.prototype.gotCryptedString = function(string) {
    this.encryptWin(string);
};

Crypto.prototype.getPlainString = function(string) {
    this.decryptWin(string);
};

PhoneGap.addConstructor(function() {
    if (typeof navigator.Crypto === "undefined") {
        navigator.Crypto = new Crypto();
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("device")) {
PhoneGap.addResource("device");

/**
 * This represents the mobile device, and provides properties for inspecting the model, version, UUID of the
 * phone, etc.
 * @constructor
 */
var Device = function() {
    this.available = PhoneGap.available;
    this.platform = null;
    this.version = null;
    this.name = null;
    this.uuid = null;
    this.phonegap = null;

    var me = this;
    this.getInfo(
        function(info) {
            me.available = true;
            me.platform = info.platform;
            me.version = info.version;
            me.name = info.name;
            me.uuid = info.uuid;
            me.phonegap = info.phonegap;
            PhoneGap.onPhoneGapInfoReady.fire();
        },
        function(e) {
            me.available = false;
            console.log("Error initializing PhoneGap: " + e);
            alert("Error initializing PhoneGap: "+e);
        });
};

/**
 * Get device info
 *
 * @param {Function} successCallback The function to call when the heading data is available
 * @param {Function} errorCallback The function to call when there is an error getting the heading data. (OPTIONAL)
 */
Device.prototype.getInfo = function(successCallback, errorCallback) {

    // successCallback required
    if (typeof successCallback !== "function") {
        console.log("Device Error: successCallback is not a function");
        return;
    }

    // errorCallback optional
    if (errorCallback && (typeof errorCallback !== "function")) {
        console.log("Device Error: errorCallback is not a function");
        return;
    }

    // Get info
    PhoneGap.exec(successCallback, errorCallback, "Device", "getDeviceInfo", []);
};

/*
 * DEPRECATED
 * This is only for Android.
 *
 * You must explicitly override the back button.
 */
Device.prototype.overrideBackButton = function() {
	console.log("Device.overrideBackButton() is deprecated.  Use App.overrideBackbutton(true).");
	navigator.app.overrideBackbutton(true);
};

/*
 * DEPRECATED
 * This is only for Android.
 *
 * This resets the back button to the default behaviour
 */
Device.prototype.resetBackButton = function() {
	console.log("Device.resetBackButton() is deprecated.  Use App.overrideBackbutton(false).");
	navigator.app.overrideBackbutton(false);
};

/*
 * DEPRECATED
 * This is only for Android.
 *
 * This terminates the activity!
 */
Device.prototype.exitApp = function() {
	console.log("Device.exitApp() is deprecated.  Use App.exitApp().");
	navigator.app.exitApp();
};

PhoneGap.addConstructor(function() {
    if (typeof navigator.device === "undefined") {
        navigator.device = window.device = new Device();
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("file")) {
PhoneGap.addResource("file");

/**
 * This class provides some useful information about a file.
 * @constructor
 */
var FileProperties = function(filePath) {
    this.filePath = filePath;
    this.size = 0;
    this.lastModifiedDate = null;
};

/**
 * Represents a single file.
 *
 * @constructor
 * @param name {DOMString} name of the file, without path information
 * @param fullPath {DOMString} the full path of the file, including the name
 * @param type {DOMString} mime type
 * @param lastModifiedDate {Date} last modified date
 * @param size {Number} size of the file in bytes
 */
var File = function(name, fullPath, type, lastModifiedDate, size) {
    this.name = name || null;
    this.fullPath = fullPath || null;
    this.type = type || null;
    this.lastModifiedDate = lastModifiedDate || null;
    this.size = size || 0;
};

/** @constructor */
var FileError = function() {
   this.code = null;
};

// File error codes
// Found in DOMException
FileError.NOT_FOUND_ERR = 1;
FileError.SECURITY_ERR = 2;
FileError.ABORT_ERR = 3;

// Added by this specification
FileError.NOT_READABLE_ERR = 4;
FileError.ENCODING_ERR = 5;
FileError.NO_MODIFICATION_ALLOWED_ERR = 6;
FileError.INVALID_STATE_ERR = 7;
FileError.SYNTAX_ERR = 8;
FileError.INVALID_MODIFICATION_ERR = 9;
FileError.QUOTA_EXCEEDED_ERR = 10;
FileError.TYPE_MISMATCH_ERR = 11;
FileError.PATH_EXISTS_ERR = 12;

//-----------------------------------------------------------------------------
// File Reader
//-----------------------------------------------------------------------------

/**
 * This class reads the mobile device file system.
 *
 * For Android:
 *      The root directory is the root of the file system.
 *      To read from the SD card, the file name is "sdcard/my_file.txt"
 * @constructor
 */
var FileReader = function() {
    this.fileName = "";

    this.readyState = 0;

    // File data
    this.result = null;

    // Error
    this.error = null;

    // Event handlers
    this.onloadstart = null;    // When the read starts.
    this.onprogress = null;     // While reading (and decoding) file or fileBlob data, and reporting partial file data (progess.loaded/progress.total)
    this.onload = null;         // When the read has successfully completed.
    this.onerror = null;        // When the read has failed (see errors).
    this.onloadend = null;      // When the request has completed (either in success or failure).
    this.onabort = null;        // When the read has been aborted. For instance, by invoking the abort() method.
};

// States
FileReader.EMPTY = 0;
FileReader.LOADING = 1;
FileReader.DONE = 2;

/**
 * Abort reading file.
 */
FileReader.prototype.abort = function() {
    var evt;
    this.readyState = FileReader.DONE;
    this.result = null;

    // set error
    var error = new FileError();
    error.code = error.ABORT_ERR;
    this.error = error;

    // If error callback
    if (typeof this.onerror === "function") {
        this.onerror({"type":"error", "target":this});
    }
    // If abort callback
    if (typeof this.onabort === "function") {
        this.onabort({"type":"abort", "target":this});
    }
    // If load end callback
    if (typeof this.onloadend === "function") {
        this.onloadend({"type":"loadend", "target":this});
    }
};

/**
 * Read text file.
 *
 * @param file          {File} File object containing file properties
 * @param encoding      [Optional] (see http://www.iana.org/assignments/character-sets)
 */
FileReader.prototype.readAsText = function(file, encoding) {
    this.fileName = "";
    if (typeof file.fullPath === "undefined") {
        this.fileName = file;
    } else {
        this.fileName = file.fullPath;
    }

    // LOADING state
    this.readyState = FileReader.LOADING;

    // If loadstart callback
    if (typeof this.onloadstart === "function") {
        this.onloadstart({"type":"loadstart", "target":this});
    }

    // Default encoding is UTF-8
    var enc = encoding ? encoding : "UTF-8";

    var me = this;

    // Read file
    PhoneGap.exec(
        // Success callback
        function(r) {
            var evt;

            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileReader.DONE) {
                return;
            }

            // Save result
            me.result = r;

            // If onload callback
            if (typeof me.onload === "function") {
                me.onload({"type":"load", "target":me});
            }

            // DONE state
            me.readyState = FileReader.DONE;

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend({"type":"loadend", "target":me});
            }
        },
        // Error callback
        function(e) {
            var evt;
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileReader.DONE) {
                return;
            }

            // Save error
            me.error = e;

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror({"type":"error", "target":me});
            }

            // DONE state
            me.readyState = FileReader.DONE;

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend({"type":"loadend", "target":me});
            }
        }, "File", "readAsText", [this.fileName, enc]);
};


/**
 * Read file and return data as a base64 encoded data url.
 * A data url is of the form:
 *      data:[<mediatype>][;base64],<data>
 *
 * @param file          {File} File object containing file properties
 */
FileReader.prototype.readAsDataURL = function(file) {
    this.fileName = "";
    if (typeof file.fullPath === "undefined") {
        this.fileName = file;
    } else {
        this.fileName = file.fullPath;
    }

    // LOADING state
    this.readyState = FileReader.LOADING;

    // If loadstart callback
    if (typeof this.onloadstart === "function") {
        this.onloadstart({"type":"loadstart", "target":this});
    }

    var me = this;

    // Read file
    PhoneGap.exec(
        // Success callback
        function(r) {
            var evt;

            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileReader.DONE) {
                return;
            }

            // Save result
            me.result = r;

            // If onload callback
            if (typeof me.onload === "function") {
                me.onload({"type":"load", "target":me});
            }

            // DONE state
            me.readyState = FileReader.DONE;

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend({"type":"loadend", "target":me});
            }
        },
        // Error callback
        function(e) {
            var evt;
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileReader.DONE) {
                return;
            }

            // Save error
            me.error = e;

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror({"type":"error", "target":me});
            }

            // DONE state
            me.readyState = FileReader.DONE;

            // If onloadend callback
            if (typeof me.onloadend === "function") {
                me.onloadend({"type":"loadend", "target":me});
            }
        }, "File", "readAsDataURL", [this.fileName]);
};

/**
 * Read file and return data as a binary data.
 *
 * @param file          {File} File object containing file properties
 */
FileReader.prototype.readAsBinaryString = function(file) {
    // TODO - Can't return binary data to browser.
    this.fileName = file;
};

/**
 * Read file and return data as a binary data.
 *
 * @param file          {File} File object containing file properties
 */
FileReader.prototype.readAsArrayBuffer = function(file) {
    // TODO - Can't return binary data to browser.
    this.fileName = file;
};

//-----------------------------------------------------------------------------
// File Writer
//-----------------------------------------------------------------------------

/**
 * This class writes to the mobile device file system.
 *
 * For Android:
 *      The root directory is the root of the file system.
 *      To write to the SD card, the file name is "sdcard/my_file.txt"
 *
 * @constructor
 * @param file {File} File object containing file properties
 * @param append if true write to the end of the file, otherwise overwrite the file
 */
var FileWriter = function(file) {
    this.fileName = "";
    this.length = 0;
    if (file) {
        this.fileName = file.fullPath || file;
        this.length = file.size || 0;
    }
    // default is to write at the beginning of the file
    this.position = 0;

    this.readyState = 0; // EMPTY

    this.result = null;

    // Error
    this.error = null;

    // Event handlers
    this.onwritestart = null;   // When writing starts
    this.onprogress = null;     // While writing the file, and reporting partial file data
    this.onwrite = null;        // When the write has successfully completed.
    this.onwriteend = null;     // When the request has completed (either in success or failure).
    this.onabort = null;        // When the write has been aborted. For instance, by invoking the abort() method.
    this.onerror = null;        // When the write has failed (see errors).
};

// States
FileWriter.INIT = 0;
FileWriter.WRITING = 1;
FileWriter.DONE = 2;

/**
 * Abort writing file.
 */
FileWriter.prototype.abort = function() {
    // check for invalid state
    if (this.readyState === FileWriter.DONE || this.readyState === FileWriter.INIT) {
        throw FileError.INVALID_STATE_ERR;
    }

    // set error
    var error = new FileError(), evt;
    error.code = error.ABORT_ERR;
    this.error = error;

    // If error callback
    if (typeof this.onerror === "function") {
        this.onerror({"type":"error", "target":this});
    }
    // If abort callback
    if (typeof this.onabort === "function") {
        this.onabort({"type":"abort", "target":this});
    }

    this.readyState = FileWriter.DONE;

    // If write end callback
    if (typeof this.onwriteend == "function") {
        this.onwriteend({"type":"writeend", "target":this});
    }
};

/**
 * Writes data to the file
 *
 * @param text to be written
 */
FileWriter.prototype.write = function(text) {
    // Throw an exception if we are already writing a file
    if (this.readyState === FileWriter.WRITING) {
        throw FileError.INVALID_STATE_ERR;
    }

    // WRITING state
    this.readyState = FileWriter.WRITING;

    var me = this;

    // If onwritestart callback
    if (typeof me.onwritestart === "function") {
        me.onwritestart({"type":"writestart", "target":me});
    }

    // Write file
    PhoneGap.exec(
        // Success callback
        function(r) {
            var evt;
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileWriter.DONE) {
                return;
            }

            // position always increases by bytes written because file would be extended
            me.position += r;
            // The length of the file is now where we are done writing.
            me.length = me.position;

            // If onwrite callback
            if (typeof me.onwrite === "function") {
                me.onwrite({"type":"write", "target":me});
            }

            // DONE state
            me.readyState = FileWriter.DONE;

            // If onwriteend callback
            if (typeof me.onwriteend === "function") {
                me.onwriteend({"type":"writeend", "target":me});
            }
        },
        // Error callback
        function(e) {
            var evt;

            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileWriter.DONE) {
                return;
            }

            // Save error
            me.error = e;

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror({"type":"error", "target":me});
            }

            // DONE state
            me.readyState = FileWriter.DONE;

            // If onwriteend callback
            if (typeof me.onwriteend === "function") {
                me.onwriteend({"type":"writeend", "target":me});
            }
        }, "File", "write", [this.fileName, text, this.position]);
};

/**
 * Moves the file pointer to the location specified.
 *
 * If the offset is a negative number the position of the file
 * pointer is rewound.  If the offset is greater than the file
 * size the position is set to the end of the file.
 *
 * @param offset is the location to move the file pointer to.
 */
FileWriter.prototype.seek = function(offset) {
    // Throw an exception if we are already writing a file
    if (this.readyState === FileWriter.WRITING) {
        throw FileError.INVALID_STATE_ERR;
    }

    if (!offset) {
        return;
    }

    // See back from end of file.
    if (offset < 0) {
        this.position = Math.max(offset + this.length, 0);
    }
    // Offset is bigger then file size so set position
    // to the end of the file.
    else if (offset > this.length) {
        this.position = this.length;
    }
    // Offset is between 0 and file size so set the position
    // to start writing.
    else {
        this.position = offset;
    }
};

/**
 * Truncates the file to the size specified.
 *
 * @param size to chop the file at.
 */
FileWriter.prototype.truncate = function(size) {
    // Throw an exception if we are already writing a file
    if (this.readyState === FileWriter.WRITING) {
        throw FileError.INVALID_STATE_ERR;
    }

    // WRITING state
    this.readyState = FileWriter.WRITING;

    var me = this;

    // If onwritestart callback
    if (typeof me.onwritestart === "function") {
        me.onwritestart({"type":"writestart", "target":this});
    }

    // Write file
    PhoneGap.exec(
        // Success callback
        function(r) {
            var evt;
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileWriter.DONE) {
                return;
            }

            // Update the length of the file
            me.length = r;
            me.position = Math.min(me.position, r);

            // If onwrite callback
            if (typeof me.onwrite === "function") {
                me.onwrite({"type":"write", "target":me});
            }

            // DONE state
            me.readyState = FileWriter.DONE;

            // If onwriteend callback
            if (typeof me.onwriteend === "function") {
                me.onwriteend({"type":"writeend", "target":me});
            }
        },
        // Error callback
        function(e) {
            var evt;
            // If DONE (cancelled), then don't do anything
            if (me.readyState === FileWriter.DONE) {
                return;
            }

            // Save error
            me.error = e;

            // If onerror callback
            if (typeof me.onerror === "function") {
                me.onerror({"type":"error", "target":me});
            }

            // DONE state
            me.readyState = FileWriter.DONE;

            // If onwriteend callback
            if (typeof me.onwriteend === "function") {
                me.onwriteend({"type":"writeend", "target":me});
            }
        }, "File", "truncate", [this.fileName, size]);
};

/**
 * Information about the state of the file or directory
 *
 * @constructor
 * {Date} modificationTime (readonly)
 */
var Metadata = function() {
    this.modificationTime=null;
};

/**
 * Supplies arguments to methods that lookup or create files and directories
 *
 * @constructor
 * @param {boolean} create file or directory if it doesn't exist
 * @param {boolean} exclusive if true the command will fail if the file or directory exists
 */
var Flags = function(create, exclusive) {
    this.create = create || false;
    this.exclusive = exclusive || false;
};

/**
 * An interface representing a file system
 *
 * @constructor
 * {DOMString} name the unique name of the file system (readonly)
 * {DirectoryEntry} root directory of the file system (readonly)
 */
var FileSystem = function() {
    this.name = null;
    this.root = null;
};

/**
 * An interface that lists the files and directories in a directory.
 * @constructor
 */
var DirectoryReader = function(fullPath){
    this.fullPath = fullPath || null;
};

/**
 * Returns a list of entries from a directory.
 *
 * @param {Function} successCallback is called with a list of entries
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryReader.prototype.readEntries = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "readEntries", [this.fullPath]);
};

/**
 * An interface representing a directory on the file system.
 *
 * @constructor
 * {boolean} isFile always false (readonly)
 * {boolean} isDirectory always true (readonly)
 * {DOMString} name of the directory, excluding the path leading to it (readonly)
 * {DOMString} fullPath the absolute full path to the directory (readonly)
 * {FileSystem} filesystem on which the directory resides (readonly)
 */
var DirectoryEntry = function() {
    this.isFile = false;
    this.isDirectory = true;
    this.name = null;
    this.fullPath = null;
    this.filesystem = null;
};

/**
 * Copies a directory to a new location
 *
 * @param {DirectoryEntry} parent the directory to which to copy the entry
 * @param {DOMString} newName the new name of the entry, defaults to the current name
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.copyTo = function(parent, newName, successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "copyTo", [this.fullPath, parent, newName]);
};

/**
 * Looks up the metadata of the entry
 *
 * @param {Function} successCallback is called with a Metadata object
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.getMetadata = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "getMetadata", [this.fullPath]);
};

/**
 * Gets the parent of the entry
 *
 * @param {Function} successCallback is called with a parent entry
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.getParent = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "getParent", [this.fullPath]);
};

/**
 * Moves a directory to a new location
 *
 * @param {DirectoryEntry} parent the directory to which to move the entry
 * @param {DOMString} newName the new name of the entry, defaults to the current name
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.moveTo = function(parent, newName, successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "moveTo", [this.fullPath, parent, newName]);
};

/**
 * Removes the entry
 *
 * @param {Function} successCallback is called with no parameters
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.remove = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "remove", [this.fullPath]);
};

/**
 * Returns a URI that can be used to identify this entry.
 *
 * @param {DOMString} mimeType for a FileEntry, the mime type to be used to interpret the file, when loaded through this URI.
 * @return uri
 */
DirectoryEntry.prototype.toURI = function(mimeType) {
    return "file://" + this.fullPath;
};

/**
 * Creates a new DirectoryReader to read entries from this directory
 */
DirectoryEntry.prototype.createReader = function(successCallback, errorCallback) {
    return new DirectoryReader(this.fullPath);
};

/**
 * Creates or looks up a directory
 *
 * @param {DOMString} path either a relative or absolute path from this directory in which to look up or create a directory
 * @param {Flags} options to create or excluively create the directory
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.getDirectory = function(path, options, successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "getDirectory", [this.fullPath, path, options]);
};

/**
 * Creates or looks up a file
 *
 * @param {DOMString} path either a relative or absolute path from this directory in which to look up or create a file
 * @param {Flags} options to create or excluively create the file
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.getFile = function(path, options, successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "getFile", [this.fullPath, path, options]);
};

/**
 * Deletes a directory and all of it's contents
 *
 * @param {Function} successCallback is called with no parameters
 * @param {Function} errorCallback is called with a FileError
 */
DirectoryEntry.prototype.removeRecursively = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "removeRecursively", [this.fullPath]);
};

/**
 * An interface representing a directory on the file system.
 *
 * @constructor
 * {boolean} isFile always true (readonly)
 * {boolean} isDirectory always false (readonly)
 * {DOMString} name of the file, excluding the path leading to it (readonly)
 * {DOMString} fullPath the absolute full path to the file (readonly)
 * {FileSystem} filesystem on which the directory resides (readonly)
 */
var FileEntry = function() {
    this.isFile = true;
    this.isDirectory = false;
    this.name = null;
    this.fullPath = null;
    this.filesystem = null;
};

/**
 * Copies a file to a new location
 *
 * @param {DirectoryEntry} parent the directory to which to copy the entry
 * @param {DOMString} newName the new name of the entry, defaults to the current name
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.copyTo = function(parent, newName, successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "copyTo", [this.fullPath, parent, newName]);
};

/**
 * Looks up the metadata of the entry
 *
 * @param {Function} successCallback is called with a Metadata object
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.getMetadata = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "getMetadata", [this.fullPath]);
};

/**
 * Gets the parent of the entry
 *
 * @param {Function} successCallback is called with a parent entry
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.getParent = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "getParent", [this.fullPath]);
};

/**
 * Moves a directory to a new location
 *
 * @param {DirectoryEntry} parent the directory to which to move the entry
 * @param {DOMString} newName the new name of the entry, defaults to the current name
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.moveTo = function(parent, newName, successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "moveTo", [this.fullPath, parent, newName]);
};

/**
 * Removes the entry
 *
 * @param {Function} successCallback is called with no parameters
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.remove = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "remove", [this.fullPath]);
};

/**
 * Returns a URI that can be used to identify this entry.
 *
 * @param {DOMString} mimeType for a FileEntry, the mime type to be used to interpret the file, when loaded through this URI.
 * @return uri
 */
FileEntry.prototype.toURI = function(mimeType) {
    return "file://" + this.fullPath;
};

/**
 * Creates a new FileWriter associated with the file that this FileEntry represents.
 *
 * @param {Function} successCallback is called with the new FileWriter
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.createWriter = function(successCallback, errorCallback) {
    this.file(function(filePointer) {
        var writer = new FileWriter(filePointer);
    
        if (writer.fileName === null || writer.fileName === "") {
            if (typeof errorCallback == "function") {
                errorCallback({
                    "code": FileError.INVALID_STATE_ERR
                });
            }
        }
    
        if (typeof successCallback == "function") {
            successCallback(writer);
        }       
    }, errorCallback);
};

/**
 * Returns a File that represents the current state of the file that this FileEntry represents.
 *
 * @param {Function} successCallback is called with the new File object
 * @param {Function} errorCallback is called with a FileError
 */
FileEntry.prototype.file = function(successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "getFileMetadata", [this.fullPath]);
};

/** @constructor */
var LocalFileSystem = function() {
};

// File error codes
LocalFileSystem.TEMPORARY = 0;
LocalFileSystem.PERSISTENT = 1;
LocalFileSystem.RESOURCE = 2;
LocalFileSystem.APPLICATION = 3;

/**
 * Requests a filesystem in which to store application data.
 *
 * @param {int} type of file system being requested
 * @param {Function} successCallback is called with the new FileSystem
 * @param {Function} errorCallback is called with a FileError
 */
LocalFileSystem.prototype.requestFileSystem = function(type, size, successCallback, errorCallback) {
    if (type < 0 || type > 3) {
        if (typeof errorCallback == "function") {
            errorCallback({
                "code": FileError.SYNTAX_ERR
            });
        }
    }
    else {
        PhoneGap.exec(successCallback, errorCallback, "File", "requestFileSystem", [type, size]);
    }
};

/**
 *
 * @param {DOMString} uri referring to a local file in a filesystem
 * @param {Function} successCallback is called with the new entry
 * @param {Function} errorCallback is called with a FileError
 */
LocalFileSystem.prototype.resolveLocalFileSystemURI = function(uri, successCallback, errorCallback) {
    PhoneGap.exec(successCallback, errorCallback, "File", "resolveLocalFileSystemURI", [uri]);
};

/**
* This function returns and array of contacts.  It is required as we need to convert raw
* JSON objects into concrete Contact objects.  Currently this method is called after
* navigator.service.contacts.find but before the find methods success call back.
*
* @param a JSON Objects that need to be converted to DirectoryEntry or FileEntry objects.
* @returns an entry
*/
LocalFileSystem.prototype._castFS = function(pluginResult) {
    var entry = null;
    entry = new DirectoryEntry();
    entry.isDirectory = pluginResult.message.root.isDirectory;
    entry.isFile = pluginResult.message.root.isFile;
    entry.name = pluginResult.message.root.name;
    entry.fullPath = pluginResult.message.root.fullPath;
    pluginResult.message.root = entry;
    return pluginResult;
};

LocalFileSystem.prototype._castEntry = function(pluginResult) {
    var entry = null;
    if (pluginResult.message.isDirectory) {
        console.log("This is a dir");
        entry = new DirectoryEntry();
    }
    else if (pluginResult.message.isFile) {
        console.log("This is a file");
        entry = new FileEntry();
    }
    entry.isDirectory = pluginResult.message.isDirectory;
    entry.isFile = pluginResult.message.isFile;
    entry.name = pluginResult.message.name;
    entry.fullPath = pluginResult.message.fullPath;
    pluginResult.message = entry;
    return pluginResult;
};

LocalFileSystem.prototype._castEntries = function(pluginResult) {
    var entries = pluginResult.message;
    var retVal = [];
    for (var i=0; i<entries.length; i++) {
        retVal.push(window.localFileSystem._createEntry(entries[i]));
    }
    pluginResult.message = retVal;
    return pluginResult;
};

LocalFileSystem.prototype._createEntry = function(castMe) {
    var entry = null;
    if (castMe.isDirectory) {
        console.log("This is a dir");
        entry = new DirectoryEntry();
    }
    else if (castMe.isFile) {
        console.log("This is a file");
        entry = new FileEntry();
    }
    entry.isDirectory = castMe.isDirectory;
    entry.isFile = castMe.isFile;
    entry.name = castMe.name;
    entry.fullPath = castMe.fullPath;
    return entry;
};

LocalFileSystem.prototype._castDate = function(pluginResult) {
    if (pluginResult.message.modificationTime) {
        var modTime = new Date(pluginResult.message.modificationTime);
        pluginResult.message.modificationTime = modTime;
    }
    else if (pluginResult.message.lastModifiedDate) {
        var file = new File();
        file.size = pluginResult.message.size;
        file.type = pluginResult.message.type;
        file.name = pluginResult.message.name;
        file.fullPath = pluginResult.message.fullPath;
        file.lastModifiedDate = new Date(pluginResult.message.lastModifiedDate);
        pluginResult.message = file;
    }
    return pluginResult;
};

/**
 * Add the FileSystem interface into the browser.
 */
PhoneGap.addConstructor(function() {
    var pgLocalFileSystem = new LocalFileSystem();
    // Needed for cast methods
    if(typeof window.localFileSystem == "undefined") window.localFileSystem  = pgLocalFileSystem;
    if(typeof window.requestFileSystem == "undefined") window.requestFileSystem  = pgLocalFileSystem.requestFileSystem;
    if(typeof window.resolveLocalFileSystemURI == "undefined") window.resolveLocalFileSystemURI = pgLocalFileSystem.resolveLocalFileSystemURI;
});
}/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("filetransfer")) {
PhoneGap.addResource("filetransfer");

/**
 * FileTransfer uploads a file to a remote server.
 * @constructor
 */
var FileTransfer = function() {};

/**
 * FileUploadResult
 * @constructor
 */
var FileUploadResult = function() {
    this.bytesSent = 0;
    this.responseCode = null;
    this.response = null;
};

/**
 * FileTransferError
 * @constructor
 */
var FileTransferError = function() {
    this.code = null;
};

FileTransferError.FILE_NOT_FOUND_ERR = 1;
FileTransferError.INVALID_URL_ERR = 2;
FileTransferError.CONNECTION_ERR = 3;

/**
* Given an absolute file path, uploads a file on the device to a remote server
* using a multipart HTTP request.
* @param filePath {String}           Full path of the file on the device
* @param server {String}             URL of the server to receive the file
* @param successCallback (Function}  Callback to be invoked when upload has completed
* @param errorCallback {Function}    Callback to be invoked upon error
* @param options {FileUploadOptions} Optional parameters such as file name and mimetype
*/
FileTransfer.prototype.upload = function(filePath, server, successCallback, errorCallback, options, debug) {

    // check for options
    var fileKey = null;
    var fileName = null;
    var mimeType = null;
    var params = null;
    var chunkedMode = true;
    if (options) {
        fileKey = options.fileKey;
        fileName = options.fileName;
        mimeType = options.mimeType;
        if (options.chunkedMode != null || typeof options.chunkedMode != "undefined") {
            chunkedMode = options.chunkedMode;
        }
        if (options.params) {
            params = options.params;
        }
        else {
            params = {};
        }
    }

    PhoneGap.exec(successCallback, errorCallback, 'FileTransfer', 'upload', [filePath, server, fileKey, fileName, mimeType, params, debug, chunkedMode]);
};

/**
 * Options to customize the HTTP request used to upload files.
 * @constructor
 * @param fileKey {String}   Name of file request parameter.
 * @param fileName {String}  Filename to be used by the server. Defaults to image.jpg.
 * @param mimeType {String}  Mimetype of the uploaded file. Defaults to image/jpeg.
 * @param params {Object}    Object with key: value params to send to the server.
 */
var FileUploadOptions = function(fileKey, fileName, mimeType, params) {
    this.fileKey = fileKey || null;
    this.fileName = fileName || null;
    this.mimeType = mimeType || null;
    this.params = params || null;
};
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("geolocation")) {
PhoneGap.addResource("geolocation");

/**
 * This class provides access to device GPS data.
 * @constructor
 */
var Geolocation = function() {

    // The last known GPS position.
    this.lastPosition = null;

    // Geolocation listeners
    this.listeners = {};
};

/**
 * Position error object
 *
 * @constructor
 * @param code
 * @param message
 */
var PositionError = function(code, message) {
    this.code = code;
    this.message = message;
};

PositionError.PERMISSION_DENIED = 1;
PositionError.POSITION_UNAVAILABLE = 2;
PositionError.TIMEOUT = 3;

/**
 * Asynchronously aquires the current position.
 *
 * @param {Function} successCallback    The function to call when the position data is available
 * @param {Function} errorCallback      The function to call when there is an error getting the heading position. (OPTIONAL)
 * @param {PositionOptions} options     The options for getting the position data. (OPTIONAL)
 */
Geolocation.prototype.getCurrentPosition = function(successCallback, errorCallback, options) {
    if (navigator._geo.listeners.global) {
        console.log("Geolocation Error: Still waiting for previous getCurrentPosition() request.");
        try {
            errorCallback(new PositionError(PositionError.TIMEOUT, "Geolocation Error: Still waiting for previous getCurrentPosition() request."));
        } catch (e) {
        }
        return;
    }
    var maximumAge = 10000;
    var enableHighAccuracy = false;
    var timeout = 10000;
    if (typeof options !== "undefined") {
        if (typeof options.maximumAge !== "undefined") {
            maximumAge = options.maximumAge;
        }
        if (typeof options.enableHighAccuracy !== "undefined") {
            enableHighAccuracy = options.enableHighAccuracy;
        }
        if (typeof options.timeout !== "undefined") {
            timeout = options.timeout;
        }
    }
    navigator._geo.listeners.global = {"success" : successCallback, "fail" : errorCallback };
    PhoneGap.exec(null, null, "Geolocation", "getCurrentLocation", [enableHighAccuracy, timeout, maximumAge]);
};

/**
 * Asynchronously watches the geolocation for changes to geolocation.  When a change occurs,
 * the successCallback is called with the new location.
 *
 * @param {Function} successCallback    The function to call each time the location data is available
 * @param {Function} errorCallback      The function to call when there is an error getting the location data. (OPTIONAL)
 * @param {PositionOptions} options     The options for getting the location data such as frequency. (OPTIONAL)
 * @return String                       The watch id that must be passed to #clearWatch to stop watching.
 */
Geolocation.prototype.watchPosition = function(successCallback, errorCallback, options) {
    var maximumAge = 10000;
    var enableHighAccuracy = false;
    var timeout = 10000;
    if (typeof options !== "undefined") {
        if (typeof options.frequency  !== "undefined") {
            maximumAge = options.frequency;
        }
        if (typeof options.maximumAge !== "undefined") {
            maximumAge = options.maximumAge;
        }
        if (typeof options.enableHighAccuracy !== "undefined") {
            enableHighAccuracy = options.enableHighAccuracy;
        }
        if (typeof options.timeout !== "undefined") {
            timeout = options.timeout;
        }
    }
    var id = PhoneGap.createUUID();
    navigator._geo.listeners[id] = {"success" : successCallback, "fail" : errorCallback };
    PhoneGap.exec(null, null, "Geolocation", "start", [id, enableHighAccuracy, timeout, maximumAge]);
    return id;
};

/*
 * Native callback when watch position has a new position.
 * PRIVATE METHOD
 *
 * @param {String} id
 * @param {Number} lat
 * @param {Number} lng
 * @param {Number} alt
 * @param {Number} altacc
 * @param {Number} head
 * @param {Number} vel
 * @param {Number} stamp
 */
Geolocation.prototype.success = function(id, lat, lng, alt, altacc, head, vel, stamp) {
    var coords = new Coordinates(lat, lng, alt, altacc, head, vel);
    var loc = new Position(coords, stamp);
    try {
        if (lat === "undefined" || lng === "undefined") {
            navigator._geo.listeners[id].fail(new PositionError(PositionError.POSITION_UNAVAILABLE, "Lat/Lng are undefined."));
        }
        else {
            navigator._geo.lastPosition = loc;
            navigator._geo.listeners[id].success(loc);
        }
    }
    catch (e) {
        console.log("Geolocation Error: Error calling success callback function.");
    }

    if (id === "global") {
        delete navigator._geo.listeners.global;
    }
};

/**
 * Native callback when watch position has an error.
 * PRIVATE METHOD
 *
 * @param {String} id       The ID of the watch
 * @param {Number} code     The error code
 * @param {String} msg      The error message
 */
Geolocation.prototype.fail = function(id, code, msg) {
    try {
        navigator._geo.listeners[id].fail(new PositionError(code, msg));
    }
    catch (e) {
        console.log("Geolocation Error: Error calling error callback function.");
    }
};

/**
 * Clears the specified heading watch.
 *
 * @param {String} id       The ID of the watch returned from #watchPosition
 */
Geolocation.prototype.clearWatch = function(id) {
    PhoneGap.exec(null, null, "Geolocation", "stop", [id]);
    delete navigator._geo.listeners[id];
};

/**
 * Force the PhoneGap geolocation to be used instead of built-in.
 */
Geolocation.usingPhoneGap = false;
Geolocation.usePhoneGap = function() {
    if (Geolocation.usingPhoneGap) {
        return;
    }
    Geolocation.usingPhoneGap = true;

    // Set built-in geolocation methods to our own implementations
    // (Cannot replace entire geolocation, but can replace individual methods)
    navigator.geolocation.setLocation = navigator._geo.setLocation;
    navigator.geolocation.getCurrentPosition = navigator._geo.getCurrentPosition;
    navigator.geolocation.watchPosition = navigator._geo.watchPosition;
    navigator.geolocation.clearWatch = navigator._geo.clearWatch;
    navigator.geolocation.start = navigator._geo.start;
    navigator.geolocation.stop = navigator._geo.stop;
};

PhoneGap.addConstructor(function() {
    navigator._geo = new Geolocation();

    // No native geolocation object for Android 1.x, so use PhoneGap geolocation
    if (typeof navigator.geolocation === 'undefined') {
        navigator.geolocation = navigator._geo;
        Geolocation.usingPhoneGap = true;
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("media")) {
PhoneGap.addResource("media");

/**
 * This class provides access to the device media, interfaces to both sound and video
 *
 * @constructor
 * @param src                   The file name or url to play
 * @param successCallback       The callback to be called when the file is done playing or recording.
 *                                  successCallback() - OPTIONAL
 * @param errorCallback         The callback to be called if there is an error.
 *                                  errorCallback(int errorCode) - OPTIONAL
 * @param statusCallback        The callback to be called when media status has changed.
 *                                  statusCallback(int statusCode) - OPTIONAL
 * @param positionCallback      The callback to be called when media position has changed.
 *                                  positionCallback(long position) - OPTIONAL
 */
var Media = function(src, successCallback, errorCallback, statusCallback, positionCallback) {

    // successCallback optional
    if (successCallback && (typeof successCallback !== "function")) {
        console.log("Media Error: successCallback is not a function");
        return;
    }

    // errorCallback optional
    if (errorCallback && (typeof errorCallback !== "function")) {
        console.log("Media Error: errorCallback is not a function");
        return;
    }

    // statusCallback optional
    if (statusCallback && (typeof statusCallback !== "function")) {
        console.log("Media Error: statusCallback is not a function");
        return;
    }

    // statusCallback optional
    if (positionCallback && (typeof positionCallback !== "function")) {
        console.log("Media Error: positionCallback is not a function");
        return;
    }

    this.id = PhoneGap.createUUID();
    PhoneGap.mediaObjects[this.id] = this;
    this.src = src;
    this.successCallback = successCallback;
    this.errorCallback = errorCallback;
    this.statusCallback = statusCallback;
    this.positionCallback = positionCallback;
    this._duration = -1;
    this._position = -1;
};

// Media messages
Media.MEDIA_STATE = 1;
Media.MEDIA_DURATION = 2;
Media.MEDIA_POSITION = 3;
Media.MEDIA_ERROR = 9;

// Media states
Media.MEDIA_NONE = 0;
Media.MEDIA_STARTING = 1;
Media.MEDIA_RUNNING = 2;
Media.MEDIA_PAUSED = 3;
Media.MEDIA_STOPPED = 4;
Media.MEDIA_MSG = ["None", "Starting", "Running", "Paused", "Stopped"];

// TODO: Will MediaError be used?
/**
 * This class contains information about any Media errors.
 * @constructor
 */
var MediaError = function() {
    this.code = null;
    this.message = "";
};

MediaError.MEDIA_ERR_ABORTED        = 1;
MediaError.MEDIA_ERR_NETWORK        = 2;
MediaError.MEDIA_ERR_DECODE         = 3;
MediaError.MEDIA_ERR_NONE_SUPPORTED = 4;

/**
 * Start or resume playing audio file.
 */
Media.prototype.play = function() {
    PhoneGap.exec(null, null, "Media", "startPlayingAudio", [this.id, this.src]);
};

/**
 * Stop playing audio file.
 */
Media.prototype.stop = function() {
    return PhoneGap.exec(null, null, "Media", "stopPlayingAudio", [this.id]);
};

/**
 * Seek or jump to a new time in the track..
 */
Media.prototype.seekTo = function(milliseconds) {
    PhoneGap.exec(null, null, "Media", "seekToAudio", [this.id, milliseconds]);
};

/**
 * Pause playing audio file.
 */
Media.prototype.pause = function() {
    PhoneGap.exec(null, null, "Media", "pausePlayingAudio", [this.id]);
};

/**
 * Get duration of an audio file.
 * The duration is only set for audio that is playing, paused or stopped.
 *
 * @return      duration or -1 if not known.
 */
Media.prototype.getDuration = function() {
    return this._duration;
};

/**
 * Get position of audio.
 */
Media.prototype.getCurrentPosition = function(success, fail) {
    PhoneGap.exec(success, fail, "Media", "getCurrentPositionAudio", [this.id]);
};

/**
 * Start recording audio file.
 */
Media.prototype.startRecord = function() {
    PhoneGap.exec(null, null, "Media", "startRecordingAudio", [this.id, this.src]);
};

/**
 * Stop recording audio file.
 */
Media.prototype.stopRecord = function() {
    PhoneGap.exec(null, null, "Media", "stopRecordingAudio", [this.id]);
};

/**
 * Release the resources.
 */
Media.prototype.release = function() {
    PhoneGap.exec(null, null, "Media", "release", [this.id]);
};

/**
 * Adjust the volume.
 */
Media.prototype.setVolume = function(volume) {
    PhoneGap.exec(null, null, "Media", "setVolume", [this.id, volume]);
};

/**
 * List of media objects.
 * PRIVATE
 */
PhoneGap.mediaObjects = {};

/**
 * Object that receives native callbacks.
 * PRIVATE
 * @constructor
 */
PhoneGap.Media = function() {};

/**
 * Get the media object.
 * PRIVATE
 *
 * @param id            The media object id (string)
 */
PhoneGap.Media.getMediaObject = function(id) {
    return PhoneGap.mediaObjects[id];
};

/**
 * Audio has status update.
 * PRIVATE
 *
 * @param id            The media object id (string)
 * @param status        The status code (int)
 * @param msg           The status message (string)
 */
PhoneGap.Media.onStatus = function(id, msg, value) {
    var media = PhoneGap.mediaObjects[id];
    // If state update
    if (msg === Media.MEDIA_STATE) {
        if (value === Media.MEDIA_STOPPED) {
            if (media.successCallback) {
                media.successCallback();
            }
        }
        if (media.statusCallback) {
            media.statusCallback(value);
        }
    }
    else if (msg === Media.MEDIA_DURATION) {
        media._duration = value;
    }
    else if (msg === Media.MEDIA_ERROR) {
        if (media.errorCallback) {
            media.errorCallback(value);
        }
    }
    else if (msg == Media.MEDIA_POSITION) {
        media._position = value;
    }
};
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("network")) {
PhoneGap.addResource("network");

/**
 * This class contains information about the current network Connection.
 * @constructor
 */
var Connection = function() {
    this.type = null;
    this._firstRun = true;
    this._timer = null;
    this.timeout = 500;

    var me = this;
    this.getInfo(
        function(type) {
            // Need to send events if we are on or offline
            if (type == "none") {
                // set a timer if still offline at the end of timer send the offline event
                me._timer = setTimeout(function(){
                    me.type = type;
                    PhoneGap.fireDocumentEvent('offline');
                    me._timer = null;
                    }, me.timeout);
            } else {
                // If there is a current offline event pending clear it
                if (me._timer != null) {
                    clearTimeout(me._timer);
                    me._timer = null;
                }
                me.type = type;
                PhoneGap.fireDocumentEvent('online');
            }
            
            // should only fire this once
            if (me._firstRun) {
                me._firstRun = false;
                PhoneGap.onPhoneGapConnectionReady.fire();
            }            
        },
        function(e) {
            // If we can't get the network info we should still tell PhoneGap
            // to fire the deviceready event.
            if (me._firstRun) {
                me._firstRun = false;
                PhoneGap.onPhoneGapConnectionReady.fire();
            }            
            console.log("Error initializing Network Connection: " + e);
        });
};

Connection.UNKNOWN = "unknown";
Connection.ETHERNET = "ethernet";
Connection.WIFI = "wifi";
Connection.CELL_2G = "2g";
Connection.CELL_3G = "3g";
Connection.CELL_4G = "4g";
Connection.NONE = "none";

/**
 * Get connection info
 *
 * @param {Function} successCallback The function to call when the Connection data is available
 * @param {Function} errorCallback The function to call when there is an error getting the Connection data. (OPTIONAL)
 */
Connection.prototype.getInfo = function(successCallback, errorCallback) {
    // Get info
    PhoneGap.exec(successCallback, errorCallback, "Network Status", "getConnectionInfo", []);
};


PhoneGap.addConstructor(function() {
    if (typeof navigator.network === "undefined") {
        navigator.network = new Object();
    }
    if (typeof navigator.network.connection === "undefined") {
        navigator.network.connection = new Connection();
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("notification")) {
PhoneGap.addResource("notification");

/**
 * This class provides access to notifications on the device.
 * @constructor
 */
var Notification = function() {
};

/**
 * Open a native alert dialog, with a customizable title and button text.
 *
 * @param {String} message              Message to print in the body of the alert
 * @param {Function} completeCallback   The callback that is called when user clicks on a button.
 * @param {String} title                Title of the alert dialog (default: Alert)
 * @param {String} buttonLabel          Label of the close button (default: OK)
 */
Notification.prototype.alert = function(message, completeCallback, title, buttonLabel) {
    var _title = (title || "Alert");
    var _buttonLabel = (buttonLabel || "OK");
    PhoneGap.exec(completeCallback, null, "Notification", "alert", [message,_title,_buttonLabel]);
};

/**
 * Open a native confirm dialog, with a customizable title and button text.
 * The result that the user selects is returned to the result callback.
 *
 * @param {String} message              Message to print in the body of the alert
 * @param {Function} resultCallback     The callback that is called when user clicks on a button.
 * @param {String} title                Title of the alert dialog (default: Confirm)
 * @param {String} buttonLabels         Comma separated list of the labels of the buttons (default: 'OK,Cancel')
 */
Notification.prototype.confirm = function(message, resultCallback, title, buttonLabels) {
    var _title = (title || "Confirm");
    var _buttonLabels = (buttonLabels || "OK,Cancel");
    PhoneGap.exec(resultCallback, null, "Notification", "confirm", [message,_title,_buttonLabels]);
};

/**
 * Start spinning the activity indicator on the statusbar
 */
Notification.prototype.activityStart = function() {
    PhoneGap.exec(null, null, "Notification", "activityStart", ["Busy","Please wait..."]);
};

/**
 * Stop spinning the activity indicator on the statusbar, if it's currently spinning
 */
Notification.prototype.activityStop = function() {
    PhoneGap.exec(null, null, "Notification", "activityStop", []);
};

/**
 * Display a progress dialog with progress bar that goes from 0 to 100.
 *
 * @param {String} title        Title of the progress dialog.
 * @param {String} message      Message to display in the dialog.
 */
Notification.prototype.progressStart = function(title, message) {
    PhoneGap.exec(null, null, "Notification", "progressStart", [title, message]);
};

/**
 * Set the progress dialog value.
 *
 * @param {Number} value         0-100
 */
Notification.prototype.progressValue = function(value) {
    PhoneGap.exec(null, null, "Notification", "progressValue", [value]);
};

/**
 * Close the progress dialog.
 */
Notification.prototype.progressStop = function() {
    PhoneGap.exec(null, null, "Notification", "progressStop", []);
};

/**
 * Causes the device to blink a status LED.
 *
 * @param {Integer} count       The number of blinks.
 * @param {String} colour       The colour of the light.
 */
Notification.prototype.blink = function(count, colour) {
    // NOT IMPLEMENTED
};

/**
 * Causes the device to vibrate.
 *
 * @param {Integer} mills       The number of milliseconds to vibrate for.
 */
Notification.prototype.vibrate = function(mills) {
    PhoneGap.exec(null, null, "Notification", "vibrate", [mills]);
};

/**
 * Causes the device to beep.
 * On Android, the default notification ringtone is played "count" times.
 *
 * @param {Integer} count       The number of beeps.
 */
Notification.prototype.beep = function(count) {
    PhoneGap.exec(null, null, "Notification", "beep", [count]);
};

PhoneGap.addConstructor(function() {
    if (typeof navigator.notification === "undefined") {
        navigator.notification = new Notification();
    }
});
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

if (!PhoneGap.hasResource("position")) {
PhoneGap.addResource("position");

/**
 * This class contains position information.
 * @param {Object} lat
 * @param {Object} lng
 * @param {Object} acc
 * @param {Object} alt
 * @param {Object} altacc
 * @param {Object} head
 * @param {Object} vel
 * @constructor
 */
var Position = function(coords, timestamp) {
	this.coords = coords;
	this.timestamp = (timestamp !== 'undefined') ? timestamp : new Date().getTime();
};

/** @constructor */
var Coordinates = function(lat, lng, alt, acc, head, vel, altacc) {
	/**
	 * The latitude of the position.
	 */
	this.latitude = lat;
	/**
	 * The longitude of the position,
	 */
	this.longitude = lng;
	/**
	 * The accuracy of the position.
	 */
	this.accuracy = acc;
	/**
	 * The altitude of the position.
	 */
	this.altitude = alt;
	/**
	 * The direction the device is moving at the position.
	 */
	this.heading = head;
	/**
	 * The velocity with which the device is moving at the position.
	 */
	this.speed = vel;
	/**
	 * The altitude accuracy of the position.
	 */
	this.altitudeAccuracy = (altacc !== 'undefined') ? altacc : null;
};

/**
 * This class specifies the options for requesting position data.
 * @constructor
 */
var PositionOptions = function() {
	/**
	 * Specifies the desired position accuracy.
	 */
	this.enableHighAccuracy = true;
	/**
	 * The timeout after which if position data cannot be obtained the errorCallback
	 * is called.
	 */
	this.timeout = 10000;
};

/**
 * This class contains information about any GSP errors.
 * @constructor
 */
var PositionError = function() {
	this.code = null;
	this.message = "";
};

PositionError.UNKNOWN_ERROR = 0;
PositionError.PERMISSION_DENIED = 1;
PositionError.POSITION_UNAVAILABLE = 2;
PositionError.TIMEOUT = 3;
}
/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2010-2011, IBM Corporation
 */

/*
 * This is purely for the Android 1.5/1.6 HTML 5 Storage
 * I was hoping that Android 2.0 would deprecate this, but given the fact that
 * most manufacturers ship with Android 1.5 and do not do OTA Updates, this is required
 */

if (!PhoneGap.hasResource("storage")) {
PhoneGap.addResource("storage");

/**
 * SQL result set object
 * PRIVATE METHOD
 * @constructor
 */
var DroidDB_Rows = function() {
    this.resultSet = [];    // results array
    this.length = 0;        // number of rows
};

/**
 * Get item from SQL result set
 *
 * @param row           The row number to return
 * @return              The row object
 */
DroidDB_Rows.prototype.item = function(row) {
    return this.resultSet[row];
};

/**
 * SQL result set that is returned to user.
 * PRIVATE METHOD
 * @constructor
 */
var DroidDB_Result = function() {
    this.rows = new DroidDB_Rows();
};

/**
 * Storage object that is called by native code when performing queries.
 * PRIVATE METHOD
 * @constructor
 */
var DroidDB = function() {
    this.queryQueue = {};
};

/**
 * Callback from native code when query is complete.
 * PRIVATE METHOD
 *
 * @param id                Query id
 */
DroidDB.prototype.completeQuery = function(id, data) {
    var query = this.queryQueue[id];
    if (query) {
        try {
            delete this.queryQueue[id];

            // Get transaction
            var tx = query.tx;

            // If transaction hasn't failed
            // Note: We ignore all query results if previous query
            //       in the same transaction failed.
            if (tx && tx.queryList[id]) {

                // Save query results
                var r = new DroidDB_Result();
                r.rows.resultSet = data;
                r.rows.length = data.length;
                try {
                    if (typeof query.successCallback === 'function') {
                        query.successCallback(query.tx, r);
                    }
                } catch (ex) {
                    console.log("executeSql error calling user success callback: "+ex);
                }

                tx.queryComplete(id);
            }
        } catch (e) {
            console.log("executeSql error: "+e);
        }
    }
};

/**
 * Callback from native code when query fails
 * PRIVATE METHOD
 *
 * @param reason            Error message
 * @param id                Query id
 */
DroidDB.prototype.fail = function(reason, id) {
    var query = this.queryQueue[id];
    if (query) {
        try {
            delete this.queryQueue[id];

            // Get transaction
            var tx = query.tx;

            // If transaction hasn't failed
            // Note: We ignore all query results if previous query
            //       in the same transaction failed.
            if (tx && tx.queryList[id]) {
                tx.queryList = {};

                try {
                    if (typeof query.errorCallback === 'function') {
                        query.errorCallback(query.tx, reason);
                    }
                } catch (ex) {
                    console.log("executeSql error calling user error callback: "+ex);
                }

                tx.queryFailed(id, reason);
            }

        } catch (e) {
            console.log("executeSql error: "+e);
        }
    }
};

/**
 * SQL query object
 * PRIVATE METHOD
 *
 * @constructor
 * @param tx                The transaction object that this query belongs to
 */
var DroidDB_Query = function(tx) {

    // Set the id of the query
    this.id = PhoneGap.createUUID();

    // Add this query to the queue
    droiddb.queryQueue[this.id] = this;

    // Init result
    this.resultSet = [];

    // Set transaction that this query belongs to
    this.tx = tx;

    // Add this query to transaction list
    this.tx.queryList[this.id] = this;

    // Callbacks
    this.successCallback = null;
    this.errorCallback = null;

};

/**
 * Transaction object
 * PRIVATE METHOD
 * @constructor
 */
var DroidDB_Tx = function() {

    // Set the id of the transaction
    this.id = PhoneGap.createUUID();

    // Callbacks
    this.successCallback = null;
    this.errorCallback = null;

    // Query list
    this.queryList = {};
};

/**
 * Mark query in transaction as complete.
 * If all queries are complete, call the user's transaction success callback.
 *
 * @param id                Query id
 */
DroidDB_Tx.prototype.queryComplete = function(id) {
    delete this.queryList[id];

    // If no more outstanding queries, then fire transaction success
    if (this.successCallback) {
        var count = 0;
        var i;
        for (i in this.queryList) {
            if (this.queryList.hasOwnProperty(i)) {
                count++;
            }
        }
        if (count === 0) {
            try {
                this.successCallback();
            } catch(e) {
                console.log("Transaction error calling user success callback: " + e);
            }
        }
    }
};

/**
 * Mark query in transaction as failed.
 *
 * @param id                Query id
 * @param reason            Error message
 */
DroidDB_Tx.prototype.queryFailed = function(id, reason) {

    // The sql queries in this transaction have already been run, since
    // we really don't have a real transaction implemented in native code.
    // However, the user callbacks for the remaining sql queries in transaction
    // will not be called.
    this.queryList = {};

    if (this.errorCallback) {
        try {
            this.errorCallback(reason);
        } catch(e) {
            console.log("Transaction error calling user error callback: " + e);
        }
    }
};

/**
 * Execute SQL statement
 *
 * @param sql                   SQL statement to execute
 * @param params                Statement parameters
 * @param successCallback       Success callback
 * @param errorCallback         Error callback
 */
DroidDB_Tx.prototype.executeSql = function(sql, params, successCallback, errorCallback) {

    // Init params array
    if (typeof params === 'undefined') {
        params = [];
    }

    // Create query and add to queue
    var query = new DroidDB_Query(this);
    droiddb.queryQueue[query.id] = query;

    // Save callbacks
    query.successCallback = successCallback;
    query.errorCallback = errorCallback;

    // Call native code
    PhoneGap.exec(null, null, "Storage", "executeSql", [sql, params, query.id]);
};

var DatabaseShell = function() {
};

/**
 * Start a transaction.
 * Does not support rollback in event of failure.
 *
 * @param process {Function}            The transaction function
 * @param successCallback {Function}
 * @param errorCallback {Function}
 */
DatabaseShell.prototype.transaction = function(process, errorCallback, successCallback) {
    var tx = new DroidDB_Tx();
    tx.successCallback = successCallback;
    tx.errorCallback = errorCallback;
    try {
        process(tx);
    } catch (e) {
        console.log("Transaction error: "+e);
        if (tx.errorCallback) {
            try {
                tx.errorCallback(e);
            } catch (ex) {
                console.log("Transaction error calling user error callback: "+e);
            }
        }
    }
};

/**
 * Open database
 *
 * @param name              Database name
 * @param version           Database version
 * @param display_name      Database display name
 * @param size              Database size in bytes
 * @return                  Database object
 */
var DroidDB_openDatabase = function(name, version, display_name, size) {
    PhoneGap.exec(null, null, "Storage", "openDatabase", [name, version, display_name, size]);
    var db = new DatabaseShell();
    return db;
};

/**
 * For browsers with no localStorage we emulate it with SQLite. Follows the w3c api.
 * TODO: Do similar for sessionStorage.
 */

/**
 * @constructor
 */
var CupcakeLocalStorage = function() {
		try {

			this.db = openDatabase('localStorage', '1.0', 'localStorage', 2621440);
			var storage = {};
			this.length = 0;
			function setLength (length) {
				this.length = length;
				localStorage.length = length;
			}
			this.db.transaction(
				function (transaction) {
				    var i;
					transaction.executeSql('CREATE TABLE IF NOT EXISTS storage (id NVARCHAR(40) PRIMARY KEY, body NVARCHAR(255))');
					transaction.executeSql('SELECT * FROM storage', [], function(tx, result) {
						for(var i = 0; i < result.rows.length; i++) {
							storage[result.rows.item(i)['id']] =  result.rows.item(i)['body'];
						}
						setLength(result.rows.length);
						PhoneGap.initializationComplete("cupcakeStorage");
					});

				},
				function (err) {
					alert(err.message);
				}
			);
			this.setItem = function(key, val) {
				if (typeof(storage[key])=='undefined') {
					this.length++;
				}
				storage[key] = val;
				this.db.transaction(
					function (transaction) {
						transaction.executeSql('CREATE TABLE IF NOT EXISTS storage (id NVARCHAR(40) PRIMARY KEY, body NVARCHAR(255))');
						transaction.executeSql('REPLACE INTO storage (id, body) values(?,?)', [key,val]);
					}
				);
			};
			this.getItem = function(key) {
				return storage[key];
			};
			this.removeItem = function(key) {
				delete storage[key];
				this.length--;
				this.db.transaction(
					function (transaction) {
						transaction.executeSql('CREATE TABLE IF NOT EXISTS storage (id NVARCHAR(40) PRIMARY KEY, body NVARCHAR(255))');
						transaction.executeSql('DELETE FROM storage where id=?', [key]);
					}
				);
			};
			this.clear = function() {
				storage = {};
				this.length = 0;
				this.db.transaction(
					function (transaction) {
						transaction.executeSql('CREATE TABLE IF NOT EXISTS storage (id NVARCHAR(40) PRIMARY KEY, body NVARCHAR(255))');
						transaction.executeSql('DELETE FROM storage', []);
					}
				);
			};
			this.key = function(index) {
				var i = 0;
				for (var j in storage) {
					if (i==index) {
						return j;
					} else {
						i++;
					}
				}
				return null;
			};

		} catch(e) {
			alert("Database error "+e+".");
		    return;
		}
};

PhoneGap.addConstructor(function() {
    var setupDroidDB = function() {
        navigator.openDatabase = window.openDatabase = DroidDB_openDatabase;
        window.droiddb = new DroidDB();
    }
    if (typeof window.openDatabase === "undefined") {
        setupDroidDB();
    } else {
        window.openDatabase_orig = window.openDatabase;
        window.openDatabase = function(name, version, desc, size){
            // Some versions of Android will throw a SECURITY_ERR so we need 
            // to catch the exception and seutp our own DB handling.
            var db = null;
            try {
                db = window.openDatabase_orig(name, version, desc, size);
            } 
            catch (ex) {
                db = null;
            }

            if (db == null) {
                setupDroidDB();
                return DroidDB_openDatabase(name, version, desc, size);
            }
            else {
                return db;
            }
        }
    }
    
    if (typeof window.localStorage === "undefined") {
        navigator.localStorage = window.localStorage = new CupcakeLocalStorage();
        PhoneGap.waitForInitialization("cupcakeStorage");
    }
});
}
// file: 001-NumberFormat.js
/**  domain Javascript formatting utility.
Published in http://www.mredkj.com/javascript/nfdocs.html as a public domain work (no Copyright)
9-November-2005 - Modified by Tersus (Youval Bronicki):
	Move properties to the prototype to prevent each instance from being a big object
12-May-2008  - Modified by Tersus (Youval Bronicki):
    Added support for setting decimal point without separators
16-Nov-2008  - Modified by Tersus (Youval Bronicki):
    Changed toPercentage to use same formatting as toFormatted 
*/
/*
 * NumberFormat 1.5.3
 * v1.5.3 - 29-September-2004
 * v1.5.2 - 27-August-2004
 * v1.5.1 - 13-February-2004
 * v1.5.0 - 20-December-2002
 * v1.0.3 - 23-March-2002
 * v1.0.2 - 13-March-2002
 * v1.0.1 - 20-July-2001
 * v1.0.0 - 13-April-2000
 * http://www.mredkj.com
 */
  
/*
 * NumberFormat -The constructor
 * num - The number to be formatted.
 *  Also refer to setNumber
 * inputDecimal - (Optional) The decimal character for the input
 *  Also refer to setInputDecimal
 */
function NumberFormat() {};
	// constants
	NumberFormat.prototype.COMMA = ',';
	NumberFormat.prototype.PERIOD = '.';
	NumberFormat.prototype.DASH = '-'; // v1.5.0 - new - used internally
	NumberFormat.prototype.LEFT_PAREN = '('; // v1.5.0 - new - used internally
	NumberFormat.prototype.RIGHT_PAREN = ')'; // v1.5.0 - new - used internally
	NumberFormat.prototype.LEFT_OUTSIDE = 0; // v1.5.0 - new - currency
	NumberFormat.prototype.LEFT_INSIDE = 1;  // v1.5.0 - new - currency
	NumberFormat.prototype.RIGHT_INSIDE = 2;  // v1.5.0 - new - currency
	NumberFormat.prototype.RIGHT_OUTSIDE = 3;  // v1.5.0 - new - currency
	NumberFormat.prototype.LEFT_DASH = 0; // v1.5.0 - new - negative
	NumberFormat.prototype.RIGHT_DASH = 1; // v1.5.0 - new - negative
	NumberFormat.prototype.PARENTHESIS = 2; // v1.5.0 - new - negative
	NumberFormat.prototype.NO_ROUNDING = -1 // v1.5.1 - new

	// member variables
	NumberFormat.prototype.num = null;
	NumberFormat.prototype.numOriginal = null;
	NumberFormat.prototype.hasSeparators = null;  // v1.5.0 - new
	NumberFormat.prototype.separatorValue = NumberFormat.prototype.COMMA;  // v1.5.0 - new
	NumberFormat.prototype.inputDecimalValue = NumberFormat.prototype.PERIOD; // v1.5.0 - new
	NumberFormat.prototype.decimalValue = NumberFormat.prototype.PERIOD;  // v1.5.0 - new
	NumberFormat.prototype.negativeFormat = NumberFormat.prototype.LEFT_DASH; // v1.5.0 - new
	NumberFormat.prototype.negativeRed = false; // v1.5.0 - new
	NumberFormat.prototype.hasCurrency = false;  // v1.5.0 - modified
	NumberFormat.prototype.currencyPosition = NumberFormat.prototype.LEFT_INSIDE;  // v1.5.0 - new
	NumberFormat.prototype.currencyValue = '$';  // v1.5.0 - modified
	NumberFormat.prototype.places = NumberFormat.prototype.NO_ROUNDING;
	NumberFormat.prototype.roundToPlaces = null; // v1.5.1 - new

	// external methods
	NumberFormat.prototype.setNumber = setNumberNF;
	NumberFormat.prototype.toUnformatted = toUnformattedNF;
	NumberFormat.prototype.setInputDecimal = setInputDecimalNF; // v1.5.0 - new
	NumberFormat.prototype.setSeparators = setSeparatorsNF; // v1.5.0 - new - for separators and decimals
	NumberFormat.prototype.setCommas = setCommasNF;
	NumberFormat.prototype.setNegativeFormat = setNegativeFormatNF; // v1.5.0 - new
	NumberFormat.prototype.setNegativeRed = setNegativeRedNF; // v1.5.0 - new
	NumberFormat.prototype.setCurrency = setCurrencyNF;
	NumberFormat.prototype.setCurrencyPrefix = setCurrencyPrefixNF;
	NumberFormat.prototype.setCurrencyValue = setCurrencyValueNF; // v1.5.0 - new - setCurrencyPrefix uses this
	NumberFormat.prototype.setCurrencyPosition = setCurrencyPositionNF; // v1.5.0 - new - setCurrencyPrefix uses this
	NumberFormat.prototype.setPlaces = setPlacesNF;
	NumberFormat.prototype.toFormatted = toFormattedNF;
	NumberFormat.prototype.toPercentage = toPercentageNF;
	NumberFormat.prototype.getOriginal = getOriginalNF;
	NumberFormat.prototype.moveDecimalRight = moveDecimalRightNF;
	NumberFormat.prototype.moveDecimalLeft = moveDecimalLeftNF;

	// internal methods
	NumberFormat.prototype.getRounded = getRoundedNF;
	NumberFormat.prototype.preserveZeros = preserveZerosNF;
	NumberFormat.prototype.justNumber = justNumberNF;
	NumberFormat.prototype.expandExponential = expandExponentialNF;
	NumberFormat.prototype.getZeros = getZerosNF;
	NumberFormat.prototype.moveDecimalAsString = moveDecimalAsStringNF;
	NumberFormat.prototype.moveDecimal = moveDecimalNF;
	NumberFormat.prototype.addSeparators = addSeparatorsNF;



/*
 * setInputDecimal
 * val - The decimal value for the input.
 *
 * v1.5.0 - new
 */
function setInputDecimalNF(val)
{
	this.inputDecimalValue = val;
}

/*
 * setNumber - Sets the number
 * num - The number to be formatted
 * inputDecimal - (Optional) The decimal character for the input
 *  Also refer to setInputDecimal
 * 
 * If there is a non-period decimal format for the input,
 * setInputDecimal should be called before calling setNumber.
 *
 * v1.5.0 - modified
 */
function setNumberNF(num, inputDecimal)
{
	if (inputDecimal != null) {
		this.setInputDecimal(inputDecimal); // v.1.5.1 - new
	}
	
	this.numOriginal = num;
	this.num = this.justNumber(num);
}

/*
 * toUnformatted - Returns the number as just a number.
 * If the original value was '100,000', then this method will return the number 100000
 * v1.0.2 - Modified comments, because this method no longer returns the original value.
 */
function toUnformattedNF()
{
	return (this.num);
}

/*
 * getOriginal - Returns the number as it was passed in, which may include non-number characters.
 * This function is new in v1.0.2
 */
function getOriginalNF()
{
	return (this.numOriginal);
}

/*
 * setNegativeFormat - How to format a negative number.
 * 
 * format - The format. Use one of the following constants.
 * LEFT_DASH   example: -1000
 * RIGHT_DASH  example: 1000-
 * PARENTHESIS example: (1000)
 *
 * v1.5.0 - new
 */
function setNegativeFormatNF(format)
{
	this.negativeFormat = format;
}

/*
 * setNegativeRed - Format the number red if it's negative.
 * 
 * isRed - true, to format the number red if negative, black if positive;
 *  false, for it to always be black font.
 *
 * v1.5.0 - new
 */
function setNegativeRedNF(isRed)
{
	this.negativeRed = isRed;
}

/*
 * setSeparators - One purpose of this method is to set a
 *  switch that indicates if there should be separators between groups of numbers.
 *  Also, can use it to set the values for the separator and decimal.
 *  For example, in the value 1,000.00
 *   The comma (,) is the separator and the period (.) is the decimal.
 *
 * Both separator or decimal are not required.
 * The separator and decimal cannot be the same value. If they are, decimal with be changed.
 * Can use the following constants (via the instantiated object) for separator or decimal:
 *  COMMA
 *  PERIOD
 * 
 * isC - true, if there should be separators; false, if there should be no separators
 * separator - the value of the separator.
 * decimal - the value of the decimal.
 *
 * v1.5.0 - new
 */
function setSeparatorsNF(isC, separator, decimal)
{
	this.hasSeparators = isC;
	
	// Make sure a separator was passed in
	if (separator == null) separator = this.COMMA;
	
	// Make sure a decimal was passed in
	if (decimal == null) decimal = this.PERIOD;
	
	// Additionally, make sure the values aren't the same.
	//  When the separator and decimal both are periods, make the decimal a comma.
	//  When the separator and decimal both are any other value, make the decimal a period.
	if (separator == decimal) {
		this.decimalValue = (decimal == this.PERIOD) ? this.COMMA : this.PERIOD;
	} else {
		this.decimalValue = decimal;
	}
	
	// Since the decimal value changes if decimal and separator are the same,
	// the separator value can keep its setting.
	this.separatorValue = separator;
}

/*
 * setCommas - Sets a switch that indicates if there should be commas.
 * The separator value is set to a comma and the decimal value is set to a period.
 * isC - true, if the number should be formatted with separators (commas); false, if no separators
 *
 * v1.5.0 - modified
 */
function setCommasNF(isC)
{
	this.setSeparators(isC, this.COMMA, this.PERIOD);
}

/*
 * setCurrency - Sets a switch that indicates if should be displayed as currency
 * isC - true, if should be currency; false, if not currency
 */
function setCurrencyNF(isC)
{
	this.hasCurrency = isC;
}

/*
 * setCurrencyPrefix - Sets the symbol for currency.
 * val - The symbol
 */
function setCurrencyValueNF(val)
{
	this.currencyValue = val;
}

/*
 * setCurrencyPrefix - Sets the symbol for currency.
 * The symbol will show up on the left of the numbers and outside a negative sign.
 * cp - The symbol
 *
 * v1.5.0 - modified - This now calls setCurrencyValue and setCurrencyPosition(this.LEFT_OUTSIDE)
 */
function setCurrencyPrefixNF(cp)
{
	this.setCurrencyValue(cp);
	this.setCurrencyPosition(this.LEFT_OUTSIDE);
}

/*
 * setCurrencyPosition - Sets the position for currency,
 *  which includes position relative to the numbers and negative sign.
 * cp - The position. Use one of the following constants.
 *  This method does not automatically put the negative sign at the left or right.
 *  They are left by default, and would need to be set right with setNegativeFormat.
 *	LEFT_OUTSIDE  example: $-1.00
 *	LEFT_INSIDE   example: -$1.00
 *	RIGHT_INSIDE  example: 1.00$-
 *	RIGHT_OUTSIDE example: 1.00-$
 *
 * v1.5.0 - new
 */
function setCurrencyPositionNF(cp)
{
	this.currencyPosition = cp
}

/*
 * setPlaces - Sets the precision of decimal places
 * p - The number of places.
 *  -1 or the constant NO_ROUNDING turns off rounding to a set number of places.
 *  Any other number of places less than or equal to zero is considered zero.
 *
 * v1.5.1 - modified
 */
function setPlacesNF(p)
{
	this.roundToPlaces = !(p == this.NO_ROUNDING); // v1.5.1
	this.places = (p < 0) ? 0 : p; // v1.5.1 - Don't leave negatives.
}

/*
 * v1.5.2 - new
 *
 * addSeparators
 * The value to be formatted shouldn't have any formatting already.
 *
 * nStr - A number or number as a string
 * inD - Input decimal (string value). Example: '.'
 * outD - Output decimal (string value). Example: '.'
 * sep - Output separator (string value). Example: ','
 */
function addSeparatorsNF(nStr, inD, outD, sep)
{
	nStr += '';
	var dpos = nStr.indexOf(inD);
	var nStrEnd = '';
	if (dpos != -1) {
		nStrEnd = outD + nStr.substring(dpos + 1, nStr.length);
		nStr = nStr.substring(0, dpos);
	}
	var rgx = /(\d+)(\d{3})/;
	while (rgx.test(nStr)) {
		nStr = nStr.replace(rgx, '$1' + sep + '$2');
	}
	return nStr + nStrEnd;
}

/*
 * toFormatted - Returns the number formatted according to the settings (a string)
 *
 * v1.5.0 - modified
 * v1.5.1 - modified
 */
function toFormattedNF(v)
{	
	var pos;
	var nNum = v!=null ? v : this.num; // v1.0.1 - number as a number
	var nStr;            // v1.0.1 - number as a string
	var splitString = new Array(2);   // v1.5.0
	
	// round decimal places - modified v1.5.1
	// Note: Take away negative temporarily with Math.abs
	if (this.roundToPlaces) {
		nNum = this.getRounded(nNum);
		nStr = this.preserveZeros(Math.abs(nNum)); // this step makes nNum into a string. v1.0.1 Math.abs
	} else {
		nStr = this.expandExponential(Math.abs(nNum)); // expandExponential is called in preserveZeros, so call it here too
	}
	
	// v1.5.3 - lost the if in 1.5.2, so putting it back
	if (this.hasSeparators) {
		// v1.5.2
		// Note that the argument being passed in for inD is this.PERIOD
		//  That's because the toFormatted method is working with an unformatted number
		nStr = this.addSeparators(nStr, this.PERIOD, this.decimalValue, this.separatorValue);
	}
	else
	{
		// still need to set the decimal period (which could have been overridden)
		nStr = nStr.replace(/\./g,this.decimalValue);
	}
	
	// negative and currency
	// $[c0] -[n0] $[c1] -[n1] #.#[nStr] -[n2] $[c2] -[n3] $[c3]
	var c0 = '';
	var n0 = '';
	var c1 = '';
	var n1 = '';
	var n2 = '';
	var c2 = '';
	var n3 = '';
	var c3 = '';
	var negSignL = (this.negativeFormat == this.PARENTHESIS) ? this.LEFT_PAREN : this.DASH;
	var negSignR = (this.negativeFormat == this.PARENTHESIS) ? this.RIGHT_PAREN : this.DASH;
		
	if (this.currencyPosition == this.LEFT_OUTSIDE) {
		// add currency sign in front, outside of any negative. example: $-1.00	
		if (nNum < 0) {
			if (this.negativeFormat == this.LEFT_DASH || this.negativeFormat == this.PARENTHESIS) n1 = negSignL;
			if (this.negativeFormat == this.RIGHT_DASH || this.negativeFormat == this.PARENTHESIS) n2 = negSignR;
		}
		if (this.hasCurrency) c0 = this.currencyValue;
	} else if (this.currencyPosition == this.LEFT_INSIDE) {
		// add currency sign in front, inside of any negative. example: -$1.00
		if (nNum < 0) {
			if (this.negativeFormat == this.LEFT_DASH || this.negativeFormat == this.PARENTHESIS) n0 = negSignL;
			if (this.negativeFormat == this.RIGHT_DASH || this.negativeFormat == this.PARENTHESIS) n3 = negSignR;
		}
		if (this.hasCurrency) c1 = this.currencyValue;
	}
	else if (this.currencyPosition == this.RIGHT_INSIDE) {
		// add currency sign at the end, inside of any negative. example: 1.00$-
		if (nNum < 0) {
			if (this.negativeFormat == this.LEFT_DASH || this.negativeFormat == this.PARENTHESIS) n0 = negSignL;
			if (this.negativeFormat == this.RIGHT_DASH || this.negativeFormat == this.PARENTHESIS) n3 = negSignR;
		}
		if (this.hasCurrency) c2 = this.currencyValue;
	}
	else if (this.currencyPosition == this.RIGHT_OUTSIDE) {
		// add currency sign at the end, outside of any negative. example: 1.00-$
		if (nNum < 0) {
			if (this.negativeFormat == this.LEFT_DASH || this.negativeFormat == this.PARENTHESIS) n1 = negSignL;
			if (this.negativeFormat == this.RIGHT_DASH || this.negativeFormat == this.PARENTHESIS) n2 = negSignR;
		}
		if (this.hasCurrency) c3 = this.currencyValue;
	}

	nStr = c0 + n0 + c1 + n1 + nStr + n2 + c2 + n3 + c3;
	
	// negative red
	if (this.negativeRed && nNum < 0) {
		nStr = '<font color="red">' + nStr + '</font>';
	}

	return (nStr);
}

/*
 * 2008-11-16 (Tersus - uses same formatting as toFormatted)
 * toPercentage - Format the current number as a percentage.
 * This is separate from most of the regular formatting settings.
 * The exception is the number of decimal places.
 * If a number is 0.123 it will be formatted as 12.3%
 *
 * !! This is an initial version, so it doesn't use many settings.
 * !! should use some of the formatting settings that toFormatted uses.
 * !! probably won't want to use settings like currency.
 *
 * v1.5.0 - new
 */
function toPercentageNF()
{
	nNum = this.num * 100;
	
	// round decimal places
	nNum = this.toFormatted(nNum);
	
	return nNum + '%';
}

/*
 * Return concatenated zeros as a string. Used to pad a number.
 * It might be extra if already have many decimal places
 * but is needed if the number doesn't have enough decimals. 
 */
function getZerosNF(places)
{
		var extraZ = '';
		var i;
		for (i=0; i<places; i++) {
			extraZ += '0';
		}
		return extraZ;
}

/*
 * Takes a number that JavaScript expresses in notational format
 * and makes it the full number (as a string).
 * e.g. Makes -1e-21 into -0.000000000000000000001
 *
 * If the value passed in is not a number (as determined by isNaN),
 * this function just returns the original value.
 *
 * Exponential number formats can include 1e21 1e+21 1e-21
 *  where 1e21 and 1e+21 are the same thing.
 *
 * If an exponential number is evaluated by JavaScript,
 * it will change 12.34e-9 to 1.234e-8,
 * which is a benefit to this method, because
 * it prevents extra zeros that occur for certain numbers
 * when using moveDecimalAsString
 *
 * Returns a string.
 *
 * v1.5.1 - new
 */
function expandExponentialNF(origVal)
{
	if (isNaN(origVal)) return origVal;

	var newVal = parseFloat(origVal) + ''; // parseFloat to let JavaScript evaluate number
	var eLoc = newVal.toLowerCase().indexOf('e');

	if (eLoc != -1) {
		var plusLoc = newVal.toLowerCase().indexOf('+');
		var negLoc = newVal.toLowerCase().indexOf('-', eLoc); // search for - after the e
		var justNumber = newVal.substring(0, eLoc);
		
		if (negLoc != -1) {
			// shift decimal to the left
			var places = newVal.substring(negLoc + 1, newVal.length);
			justNumber = this.moveDecimalAsString(justNumber, true, parseInt(places));
		} else {
			// shift decimal to the right
			// Check if there's a plus sign, and if not refer to where the e is.
			// This is to account for either formatting 1e21 or 1e+21
			if (plusLoc == -1) plusLoc = eLoc;
			var places = newVal.substring(plusLoc + 1, newVal.length);
			justNumber = this.moveDecimalAsString(justNumber, false, parseInt(places));
		}
		
		newVal = justNumber;
	}

	return newVal;
} 

/*
 * Move decimal right.
 * Returns a number.
 *
 * v1.5.1 - new
 */
function moveDecimalRightNF(val, places)
{
	var newVal = '';
	
	if (places == null) {
		newVal = this.moveDecimal(val, false);
	} else {
		newVal = this.moveDecimal(val, false, places);
	}
	
	return newVal;
}

/*
 * Move decimal left.
 * Returns a number.
 *
 * v1.5.1 - new
 */
function moveDecimalLeftNF(val, places)
{
	var newVal = '';
	
	if (places == null) {
		newVal = this.moveDecimal(val, true);
	} else {
		newVal = this.moveDecimal(val, true, places);
	}
	
	return newVal;
}

/*
 * moveDecimalAsString
 * This is used by moveDecimal, and does not run parseFloat on the return value.
 * 
 * Normally a decimal place is moved by multiplying by powers of 10
 * Multiplication and division in JavaScript can result in floating point limitations.
 * So use this method to move a decimal place left or right.
 *
 * Parameters:
 * val - The value to be shifted. Can be a number or a string,
 *  but don't include special characters. It should evaluate to a number.
 * left - If true, then move decimal left. If false, move right.
 * places - (optional) If not included, then use the objects this.places
 *  The purpose is so this method can be used independent of the state of the object.
 *
 * The regular expressions:
 * re1
 * Pad with zeros in case there aren't enough numbers to cover the spaces shift.
 * A left shift pads to the left, and a right shift pads to the right.
 * Can't just concatenate. There might be a negative sign or the value could be an exponential.
 *
 * re2
 * Switch the decimal.
 * Need the first [0-9]+ to force the search to start rightmost.
 * The \.? and [0-9]{} criteria are the pieces that will be switched
 *
 * Other notes:
 * This method works on exponential numbers, e.g. 1.7e-12
 * because the regular expressions only modify the number and decimal parts.
 *
 * Mozilla can't handle [0-9]{0} in the regular expression.
 *  Fix: Since nothing changes when the decimal is shifted zero places, return the original value.
 *
 * IE is incorrect if exponential ends in .
 *  e.g. -8500000000000000000000. should be -8.5e+21
 *  IE counts it as -8.5e+22
 *	Fix: Replace trailing period, if there is one, using replace(/\.$/, '').
 *
 * Netscape 4.74 cannot handle a leading - in the string being searched for the re2 expressions.
 *  e.g. /([0-9]*)(\.?)([0-9]{2})/ should match everything in -100.00 except the -
 *  but it matches nothing using Netscape 4.74.
 *  It might be a combination of the * ? special characters.
 *  Fix: (-?) was added to each of the re2 expressions to look for - one or zero times.
 *
 * Returns a string.
 *
 * v1.5.1 - new
 * v1.5.2 - modified
 */
function moveDecimalAsStringNF(val, left, places)
{
	var spaces = (arguments.length < 3) ? this.places : places;
	if (spaces <= 0) return val; // to avoid Mozilla limitation
			
	var newVal = val + '';
	var extraZ = this.getZeros(spaces);
	var re1 = new RegExp('([0-9.]+)');
	if (left) {
		newVal = newVal.replace(re1, extraZ + '$1');
		var re2 = new RegExp('(-?)([0-9]*)([0-9]{' + spaces + '})(\\.?)');		
		newVal = newVal.replace(re2, '$1$2.$3');
	} else {
		var reArray = re1.exec(newVal); // v1.5.2
		if (reArray != null) {
			newVal = newVal.substring(0,reArray.index) + reArray[1] + extraZ + newVal.substring(reArray.index + reArray[0].length); // v1.5.2
		}
		var re2 = new RegExp('(-?)([0-9]*)(\\.?)([0-9]{' + spaces + '})');
		newVal = newVal.replace(re2, '$1$2$4.');
	}
	newVal = newVal.replace(/\.$/, ''); // to avoid IE flaw
	
	return newVal;
}

/*
 * moveDecimal
 * Refer to notes in moveDecimalAsString
 * parseFloat is called here to clear away the padded zeros.
 *
 * Returns a number.
 *
 * v1.5.1 - new
 */
function moveDecimalNF(val, left, places)
{
	var newVal = '';
	
	if (places == null) {
		newVal = this.moveDecimalAsString(val, left);
	} else {
		newVal = this.moveDecimalAsString(val, left, places);
	}
	
	return parseFloat(newVal);
}

/*
 * getRounded - Used internally to round a value
 * val - The number to be rounded
 * 
 *  To round to a certain decimal precision,
 *  all that should need to be done is
 *  multiply by a power of 10, round, then divide by the same power of 10.
 *  However, occasional numbers don't get exact results in most browsers.
 *  e.g. 0.295 multiplied by 10 yields 2.9499999999999997 instead of 2.95
 *  Instead of adjusting the incorrect multiplication,
 *  this function uses string manipulation to shift the decimal.
 *
 * Returns a number.
 *
 * v1.5.1 - modified
 */
function getRoundedNF(val)
{
	val = this.moveDecimalRight(val);
	val = Math.round(val);
	val = this.moveDecimalLeft(val);
	
	return val;
}

/*
 * preserveZeros - Used internally to make the number a string
 * 	that preserves zeros at the end of the number
 * val - The number
 */
function preserveZerosNF(val)
{
	var i;

	// make a string - to preserve the zeros at the end
	val = this.expandExponential(val);
	
	if (this.places <= 0) return val; // leave now. no zeros are necessary - v1.0.1 less than or equal
	
	var decimalPos = val.indexOf('.');
	if (decimalPos == -1) {
		val += '.';
		for (i=0; i<this.places; i++) {
			val += '0';
		}
	} else {
		var actualDecimals = (val.length - 1) - decimalPos;
		var difference = this.places - actualDecimals;
		for (i=0; i<difference; i++) {
			val += '0';
		}
	}
	
	return val;
}

/*
 * justNumber - Used internally to parse the value into a floating point number.
 * Replace all characters that are not 0-9, a decimal point, or a negative sign.
 *
 *  A number can be entered using special notation.
 *  For example, the following is a valid number: 0.0314E+2
 *
 * v1.0.2 - new
 * v1.5.0 - modified
 * v1.5.1 - modified
 * v1.5.2 - modified
 */
function justNumberNF(val)
{
	newVal = val + '';
	
	var isPercentage = false;
	
	// check for percentage
	// v1.5.0
	if (newVal.indexOf('%') != -1) {
		newVal = newVal.replace(/\%/g, '');
		isPercentage = true; // mark a flag
	}
		
	// Replace everything but digits - + ( ) e E
	var re = new RegExp('[^\\' + this.inputDecimalValue + '\\d\\-\\+\\(\\)eE]', 'g');	// v1.5.2	
	newVal = newVal.replace(re, '');
	// Replace the first decimal with a period and the rest with blank
	// The regular expression will only break if a special character
	//  is used as the inputDecimalValue
	//  e.g. \ but not .
	var tempRe = new RegExp('[' + this.inputDecimalValue + ']', 'g');
	var treArray = tempRe.exec(newVal); // v1.5.2
	if (treArray != null) {
	  var tempRight = newVal.substring(treArray.index + treArray[0].length); // v1.5.2
		newVal = newVal.substring(0,treArray.index) + this.PERIOD + tempRight.replace(tempRe, ''); // v1.5.2
	}
	
	// If negative, get it in -n format
	if (newVal.charAt(newVal.length - 1) == this.DASH ) {
		newVal = newVal.substring(0, newVal.length - 1);
		newVal = '-' + newVal;
	}
	else if (newVal.charAt(0) == this.LEFT_PAREN
	 && newVal.charAt(newVal.length - 1) == this.RIGHT_PAREN) {
		newVal = newVal.substring(1, newVal.length - 1);
		newVal = '-' + newVal;
	}
	
	newVal = parseFloat(newVal);
	
	if (!isFinite(newVal)) {
		newVal = 0;
  }
	
	// now that it's a number, adjust for percentage, if applicable.
  // example. if the number was formatted 24%, then move decimal left to get 0.24
  // v1.5.0 - updated v1.5.1
  if (isPercentage) {
  	newVal = this.moveDecimalLeft(newVal, 2);
  }
		
	return newVal;
}
// file: 002-cookies.js
/* cookies.js */
/*
     Example File From "JavaScript and DHTML Cookbook"
     Published by O'Reilly & Associates
     Copyright 2003 Danny Goodman
*/

// utility function to retrieve a future expiration date in proper format;
// pass three integer parameters for the number of days, hours,
// and minutes from now you want the cookie to expire; all three
// parameters required, so use zeros where appropriate
function getExpDate(days, hours, minutes) {
    var expDate = new Date();
    if (typeof days == "number" && typeof hours == "number" && typeof hours == "number") {
        expDate.setDate(expDate.getDate() + parseInt(days));
        expDate.setHours(expDate.getHours() + parseInt(hours));
        expDate.setMinutes(expDate.getMinutes() + parseInt(minutes));
        return expDate.toGMTString();
    }
}

// utility function called by getCookie()
function getCookieVal(offset) {
    var endstr = document.cookie.indexOf (";", offset);
    if (endstr == -1) {
        endstr = document.cookie.length;
    }
    return unescape(document.cookie.substring(offset, endstr));
}

// primary function to retrieve cookie by name
function getCookie(name) {
    var arg = name + "=";
    var alen = arg.length;
    var cookie = document.cookie;
    var clen = cookie.length;
    var i = 0;
    while (i < clen) {
        var j = i + alen;
        if (cookie.substring(i, j) == arg) {
            return getCookieVal(j);
        }
        i = cookie.indexOf(" ", i) + 1;
        if (i == 0) break; 
    }
    return null;
}

// store cookie value with optional details as needed
function setCookie(name, value, expires, path, domain, secure) {
    document.cookie = name + "=" + escape (value) +
        ((expires) ? "; expires=" + expires : "") +
        ((path) ? "; path=" + path : "") +
        ((domain) ? "; domain=" + domain : "") +
        ((secure) ? "; secure" : "");
}

// remove the cookie by setting ancient expiration date
function deleteCookie(name,path,domain) {
    if (getCookie(name)) {
        document.cookie = name + "=" +
            ((path) ? "; path=" + path : "") +
            ((domain) ? "; domain=" + domain : "") +
            "; expires=Thu, 01-Jan-70 00:00:01 GMT";
    }
}

// file: 005-tersus.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 if (!window.tersus) window.tersus  = {};
if (!tersus.settings) tersus.settings = {};
window.tlog = null;
tersus.isLocalHost = function(host)
{
	return host == 'localhost' || host =='127.0.0.1' || host == '0:0:0:0:0:0:0:1%0';
};
tersus.isNative=false;//Overridden in native context
tersus.useIFrames=true;
tersus.ANYTHING = 'Common/Data Types/Anything';
tersus.loadScript = function loadScript(url,w)
{
	if (!w)
		w = window;
    var loadScript = w.document.createElement("script");
    
    // Add script object attributes
    loadScript.setAttribute("type", "text/javascript");
    loadScript.setAttribute("charset", "utf-8");
    loadScript.setAttribute("src", url);
	var head = w.document.getElementsByTagName("head").item(0);
	head.appendChild(loadScript);

};
tersus.topZindex = 10;
tersus.SERVICE_MAX_RETRIES = 2;
tersus.SERVICE_RETRY_DELAY = 2000;
tersus.alertMode = {interactive:true};
tersus.invokeAction=function(eventName,parameters, domEvent, targetNode)
{
	if (tlog) tlog('invokeAction called ['+eventName+']');
	if (!targetNode)
	targetNode = window.currentRootDisplayNode;
	if (targetNode)
	{
		var eventAttributes = null;
		if (domEvent)
			eventAttributes = getEventAttributes(domEvent);

		var eventSpec = null;
		for (var eventKey in Events)
		{
			if (Events[eventKey].elementName == eventName)
			eventSpec = Events[eventKey];
		}
		if (!eventSpec)
			eventSpec = new tersus.Event(eventName); // Ad-hoc event
		if (tlog)
			tlog('Queuing Event ..');	
		//window.engine.handleEvent(targetNode,eventSpec,parameters,eventAttributes);
		targetNode.queueEvent(eventSpec,  parameters, domEvent);
	}
};
window.tersus.clearLog = function()
{
	window.tersus._log = [];
	window.tlog  = window.tersus.log;
};
tersus.formatTime = function(time)
{
	var p2 = tersus.DateAndTime.pad2;
	return  p2(time.getHours())+':'+p2(time.getMinutes())+':'+p2(time.getSeconds());
};

tersus.useXMLHTTPRequest = true;
tersus.autoLogout = true;
window.tersus.log = function(msg1, msg2, msg3)
{
	var now=new Date();
	var time = now.getTime();
	if (l = window.tersus._log)
	{
		l.push(time);
		l.push('\t');
		l.push(tersus.escapeHTML(msg1));
		if (msg2 != null)
		{
			l.push('\t');
			l.push(msg2);
		}			
		if (msg3 != null)
		{
			l.push('\t');
			l.push(msg3);
		}		
		l.push('\n');	
	}
};
window.tersus.showLog = function()
{
	if (window.tersus._log)
	{
		var w = window.open();
		w.document.write('<pre>');
		w.document.write(window.tersus._log.join(''));
		w.document.close();
	}
	window.tersus.clearLog();
};
window.tersus.escapeHTML = function(s)
{
	if (s == null)
		return '';
	if (typeof(s) != 'string')
		s = s.toString();
	s = s.replace(/&/g, '&amp;');
	s = s.replace(/</g, '&lt;');
	s = s.replace(/>/g, '&gt;');
	s = s.replace(/\n/g, '<br/>');
	s = s.replace(/["]/g, '&quot;');
	return s;	
};
tersus.onUnload = function()
{
	if (tersus.Subscribe && tersus.Subscribe.xhr)
	{
		tersus.Subscribe.xhr.abort();
	}
	if (! tersus.reloading) // the onunload event should not be fired when we're reloading
	{
		tersus.isUnloading = true;
		if (window.rootDisplayNode)
			window.engine.handleEvent(window.rootDisplayNode,Events.ON_UNLOAD);
		if (tersus.autoLogout)
			tersus.logout();
	}
};
tersus._disabled = false;
tersus.createClassNameList = function()
{
	var list = [];
	var sheets = document.styleSheets;
	var count = 0;
	for (var i=0;i<sheets.length;i++)
	{
		var sheet = sheets[i];
		var rules = sheet.cssRules;
		if (rules == undefined)
			rules = sheet.rules; //IE
		if (rules == undefined)
			break;
		for (var j=0;j<rules.length;j++)
		{
			var rule = rules[j];
			var selectorText = rule.selectorText; 
			/* Adding ' ' to make sure we have somthing before the .,
			 so we can always ignore the first part after splitting */
			var classNames = (' '+selectorText).split('.');
			classNames.shift();
			list = list.concat(classNames);
		}
	}
	tersus.styleClassNames  = new Set(list);
};
tersus.logout = function logout()
{
	var req = createXMLHTTPRequest();
	
	var url=tersus.rootURL+'Logout';
	req.open('GET', url, false);
	req.send(null);
};
tersus.KEEP_ALIVE_INTERVAL=600000; //10 minutes
tersus.DEV_UPDATE_CHECK_INTERVAL = 2000; // 2 seconds
tersus.PROD_UPDATE_CHECK_INTERVAL = 60000; // 1 minute
tersus.init = function()
{
	window.onfocus = tersus.checkUpdate;
	if (tersus.useIFrames)
		tersus.async.exec(tersus.initServiceResponseFrames);
	if (window.browser == 'IE')
		window.document.body.onunload = tersus.onUnload;
	else
		window.onunload = tersus.onUnload;
	tersus.async.timer = setInterval(tersus.async.worker, tersus.async.interval);
	tersus.keepAliveTimer = setInterval(tersus.keepAlive, tersus.KEEP_ALIVE_INTERVAL);
		
	tersus.createClassNameList();
	tersus.repository = new ConstructorRepository();
	window.onresize = tersus.onResize;
	if (tersus.settings.dev_mode)
		tersus.updateCheckInterval = tersus.DEV_UPDATE_CHECK_INTERVAL;
	else
		tersus.updateCheckInterval = tersus.PROD_UPDATE_CHECK_INTERVAL;
//	tersus.clearLog();
};
tersus.noop = function()
{
};
tersus.keepAliveReq = null;
tersus.keepAlive = function()
{
	var req = tersus.keepAliveReq;
	if (!req)
		req = tersus.keepAliveReq = createXMLHTTPRequest();
	var url = tersus.rootURL+'blank.html?'+(new Date()).getTime();
	req.open('GET', url, true);
	req.onreadystatechange = tersus.noop;
	req.send(null);
};
tersus.getTimestampReq = null;
tersus.lastCheckUpdateTime = 0;
tersus.checkUpdate = function()
{
	var time = (new Date()).getTime();
	if (time < tersus.lastCheckUpdateTime + tersus.updateCheckInterval)
		return; // We don't want to check too often
	tersus.getTimestamp(tersus.handleCheckUpdateResponse);
}
tersus.getTimestamp = function (callback)
{
	var time = (new Date()).getTime();
	tersus.lastCheckUpdateTime = time;
	var path = 'Timestamp';
 	var url = tersus.rootURL+path;
	var req = tersus.getTimestampReq;
	if (!req)
		req = tersus.getTimestampReq = createXMLHTTPRequest();
	req.open('POST',url, true);
	req.onreadystatechange = callback;
	req.send(null);
	
}
tersus.parseTimestampResponse = function (req)
{
	return eval ('x='+req.responseText+';x;');
}
if (tersus.lastTimestamp == undefined)
	tersus.lastTimestamp = 0;
tersus.setLastTimestamp = function(timestamp)
{
	if (timestamp > tersus.lastTimestamp)
		tersus.lastTimestamp = timestamp;
};
tersus.handleCheckUpdateResponse = function(responseText)
{
	if (!window.tersus)
		return; // This can happen, at least in Firefox, when navigating away from the application
	var req = tersus.getTimestampReq;
	if (req.readyState != 4)
		return;
	if (req.status == 200 || req.responseText)
	{
		var r = tersus.parseTimestampResponse(req);
		var currentTimestamp = r.timestamp;
		window.userName = r.userName;
		if (tersus.lastTimestamp && currentTimestamp && currentTimestamp > tersus.lastTimestamp)
		{
			if (tlog) tlog("Application Updated -> Reloading");
			tersus.progress.set('Reloading Application ...');
//			tersus.progress.show();
			setTimeout(tersus.reloadApplication,1000);
		}
		else
			if (tlog) tlog("No update");
		tersus.setLastTimestamp(currentTimestamp);
	}
};
tersus.reloadApplication = function()
{
	tersus.reloading = true;
	var windows = tersus.getWindows();
	for (var i=0;i<windows.length;i++)
	{
		if (window != windows[i])
			windows[i].close();
	}
	location.replace('reload.html?'+encodeURIComponent(location.href));
};
tersus.initServiceResponseFrames = function ()
{
	tersus.serviceResponseFrames = [];
	for (var i=0; i<10; i++)
	{
		var f = tersus.serviceResponseFrames[i] = createAnIFrame(window,'_service_'+i);
		f.src  = 'blank.html';
	};
	tersus.nextResponseFrameIndex = 0;
};

tersus.nextServiceResponseFrame = function ()
{
	var iframe = tersus.serviceResponseFrames[tersus.nextResponseFrameIndex++];
	if (tersus.nextResponseFrameIndex >= tersus.serviceResponseFrames.length)
		tersus.nextResponseFrameIndex = 0;
	return iframe;
};
tersus.windows = [self];
tersus.getWindows = function()
{
	if (tersus.windows.length > 1)
		tersus.unregisterClosedWindows();
	return tersus.windows;
};
tersus.registerWindow = function (w)
{
	tersus.windows.push(w);
	if (tlog) tlog("registerWindow:"+w.name+" current:"+tersus.windows.length);
};
tersus.unregisterWindow = function (w)
{
	for (var i=0; i <tersus.windows.length; i++)
	{
		if (tersus.windows[i] == w)
		{
			tersus.windows.splice(i,1); // remove
			if (tlog) tlog("unregisterWindow:"+w.name+" index:"+i);
			return;
		}
		if (tlog) tlog("unregisterWindow:"+w.name+" ** not found **");
	}
	
}
tersus.popupNodes = {}; 
tersus.checkWindow = function (w)
{
	try
	{
		return w && !w.closed;
	}
	catch (e)
	{
		return false;
	}
}
tersus.unregisterPopupNodes = function()
{
	for (var id in tersus.popupNodes)
	{
		var node = tersus.popupNodes[id];
		var alive = false;
		try
		{
			alive = node.currentWindow && ! node.currentWindow.closed && (node.isPopIn || node.isDialog || (id == node.currentWindow.popupId));
		}
		catch (e)
		{
		}
		if (! alive)
		{
			delete tersus.popupNodes[id];
		}
	};
};
tersus.unregisterClosedWindows = function ()
{
	if (tlog) tlog("unregisterClosedWindows - before:"+tersus.windows.length);
	var temp = [];
	for (var i=0;i<tersus.windows.length;i++)
	{
		var w = tersus.windows[i];
		try
		{
			if ((! w.closed) && w.document && w.document.body)
				temp.push(w);
			else
				if (tlog) tlog(w.name + ' closed:'+w.closed + ' body:'+w.document.body);
		}
		catch (e)
		{
			if (tlog) tlog(e);
		};
	}
	tersus.windows = temp;
	if (tlog) tlog("unregisterClosedWindows - after:"+tersus.windows.length);
};
tersus.progress = {};
tersus.progress.setStatus = function (msg)
{
	var windows = tersus.getWindows();
	for (var i=0;i<windows.length;i++)
		windows[i].status = msg;
};
/** Threshold for forcing the progress bar to open when handling a service response */
tersus.progress.responseLengthThreshold = 10000;
tersus.progress.set = function (msg, cancellationFunc)
{
	if (tlog) tlog("tersus.progress.set('"+msg+"')");
	
	var p = tersus.progress;
	setTimeout(function(){tersus.progress.setStatus(msg);},0);
	var windows = tersus.getWindows();
	for (var i=0;i<windows.length;i++)
	{
		// Create or Update a progress bar for each window of the applications
		var w = windows[i];
		var body;
		try
		{
			var doc = w.document;
			var body = doc.body;
		}
		catch (e)
		{
			body = null;
		}
		if (body)
		{
			if (window.browser == 'IE')
				body.style.cursor='wait';
			if (!body.className)
				body.className = 'progress';
			else if (body.className.search(/progress/) < 0)
				body.className += ' progress';
	/*		var b = doc.getElementById('progressBar');
			var m = doc.getElementById('progressMessage');
			var c = doc.getElementById('progressCancel');
			if (!b)
			{
				b = doc.createElement('div');
				b.id = 'progressBar';
				b.className = 'progressBar';
				b.style.display='none';
				doc.body.appendChild(b); 
				var img = doc.createElement('img');
				img.id='progressAnimation';
				img.src="images/progress.gif";
				b.appendChild(img);
				m = doc.createElement('div');
				m.id = 'progressMessage';
				b.appendChild(m);
				c = doc.createElement('a');
				c.innerHTML='Cancel';
				c.href='javascript:tersus.progress.cancel()';
				c.id='progressCancel';
				b.appendChild(c);
			}
			m.innerHTML = tersus.escapeHTML(msg);
			if (cancellationFunc)
				c.style.display = '';
			else
				c.style.display='none';
			tersus.progress.cancel = cancellationFunc; */
		}
	}
	tersus.progress.message = msg;
	p.active = true; 
};
tersus.progress.delay = 1000; // 1000 milliseconds until progress bar appears

tersus.progress.clear = function()
{
	if (tlog) tlog("tersus.progress.clear()");
	var p = tersus.progress;
	if (isBusy())
	{
		if (tlog) tlog("tersus.progress.clear() : still busy ..");
		return;
	}
	setTimeout(function(){tersus.progress.setStatus('Done');},0);
	// Hide progress bar in each window
	var windows = tersus.getWindows();
	var bodies = [];
	for (var i=0;i<windows.length;i++)
	{
		var w = windows[i];
		var body;
		try
		{
			body = w.document.body;
		}
		catch (e)
		{
			body = null; // Exception means window is not accessible
		}
		if (body)
		{
			bodies.push(body);
			var c = body.className;
			if (c == 'progress')
				body.className = '';
			else
				body.className = c.replace(/ progress/, '');
			if (window.browser == 'IE')
				body.style.cursor = '';	
			var b = w.document.getElementById('progressBar');
			if (b)
			{
				b.style.display = 'none';
				if (tlog) tlog("hiding progress bar in "+w.name);
			}
		}
		
	}
//	w.setTimeout(function(){for (var i=0;i<bodies.length;i++) bodies[i].style.cursor='';},0);
	
	p.active = false;
};

tersus.gotoURL = function(url, replace, reload)
{
	if (tlog) tlog("gotoURL: '"+url+"' reload="+reload);
	if (replace)
		window.location.replace(url)
	else
		window.location.href=url;
	if (!reload)
	{
		tersus.history.location = url;
		tersus.loadParams();
	}
	if (window.browser=='IE' && url.indexOf('mailto:')!=0) // In IE, we add the new URL to the history stack using a special IFRAME
	{
		var historyFrame = document.getElementById('_history');
		if (historyFrame)
		{
			var src = 'history.html?'+encodeURIComponent(url);
			if (replace)
				getIFrameWindow(historyFrame).location.replace(src);
			else
				historyFrame.src =src ;
		}
	}
	else
	{
		// The following code loads a dummy page - this makes firefox "understand" that there is nothing more to wait for
		// (the behavior looks like a Firefox bug, but hasn't been fully analyzed)
		var f= tersus.nextServiceResponseFrame()
		if (f)
		{
			getIFrameWindow(f).location.replace('blank.html?u='+encodeURIComponent(url));
		}
	}
};


tersus.history = {};
tersus.history.fixedURL = null;
tersus.history._checkLocation = function()
{
	var current = window.location.href;
	if (current != tersus.history.location)
	{
		if (tlog) tlog("New location:"+current);
		tersus.history.location = current;
		tersus.loadParams();
		tersus.history.locationChanged();
	};
};
tersus.history.checkLocation = function()
{
	if (window.location.href != tersus.history.location)
 		tersus.async.exec(tersus.history._checkLocation, false /* don't wait */);
};


tersus.history.locationChanged = function()
{
	if (tersus.history.fixedURL) /* The 'fixedURL' property is used to disable navigation */
	{
		tersus.gotoURL(tersus.history.fixedURL, false/*replace*/, false/*reload*/);
		return;
	}
	var newPerspectiveName = tersus.searchParams[tersus.PERSPECTIVE];
	if (newPerspectiveName == null && window.perspectives && window.perspectives.length > 0)
		newPerspectiveName = window.perspectives[0].name;
	var newViewName = tersus.searchParams[tersus.VIEW];
	if (newViewName == null && newPerspectiveName)
	{
		var p = getPerspective(newPerspectiveName);
		if (p && p.views && p.views.length>0)
			newViewName = p.views[0].name;
    }

	var perspectiveChanged = window.perspectives && window.perspectives.length > 1 && window.perspective.name != newPerspectiveName;
	var viewChanged = window.view == null || window.view.name != newViewName;
	if (perspectiveChanged || viewChanged)
	{
			window.engine.handleEvent(window.rootDisplayNode,Events.ON_LEAVE_VIEW);
			queueEvent(tersus, tersus.reloadView, [], null,null, null, -1);
	}
	else
		tersus.updateView();
};

tersus.locationChangeListeners = [];
tersus.updateView = function()
{
	if (tersus.locationChangeListeners.length > 0)
	{
		for (var i =0 ; i< tersus.locationChangeListeners.length; i++)
		{
			var listenerId = tersus.locationChangeListeners[i];
			var listenerNode = tersus.findNode(listenerId);
			if (listenerNode)
				window.engine.handleEvent(listenerNode,Events.ON_LOCATION_CHANGE);
		}
	}
	else
	{
		window.rootDisplayNode.refresh();
		resumeRoot();
	}

}
// Extract search parameters
tersus.loadParams = function()
{
	tersus.searchParams = {};
	var terms = [];
	var search = window.location.search;
	if (search && search.length > 1)
		terms = terms.concat(search.substring(1).split('&'));
		
	// Extract the part that follows the hash ('#').  
	// We can't use window.location.hash because some bad decoding happens there
	var hashIndex = location.href.indexOf('#');
	if (hashIndex >=0)
	{
		var hash = location.href.substring(hashIndex+1);
		terms = terms.concat(hash.split('&'));	
	}
	for (var i=0; i< terms.length; i++)
	{
		var term = terms[i];
		var keyValuePair = term.split('=');
		var key = decodeURIComponent(keyValuePair[0]);
		var value = decodeURIComponent(keyValuePair[1]);
		tersus.searchParams[key]=value;
	}
}

tersus.history.init = function ()
{
	tersus.history.location = window.location.href;
	tersus.loadParams();
	tersus.history.timer = setInterval(tersus.history.checkLocation,100);
}

tersus.getInternalParameters = function()
{
	var params = {};
	for (var p in tersus.searchParams)
		params[p] = tersus.searchParams[p];
	return params;
};

tersus.history.init();	

// tersus.async - a mechanism for preventing concurrency
tersus.async = {};
tersus.async.queue = [];
tersus.async.interval=100;
tersus.async.isBusy = function () { return tersus.async.queue.length > 0;};
tersus.async.nowait = function()
{
	// Returns true if any of the items on queue requires waiting (i.e. is a blocking operation)
	for (var i=0;i<tersus.async.queue.length;i++)
		if (tersus.async.queue[i].wait)
			return false;
	return true;
};
tersus.async.exec = function(f, waitIfBusy)
{
	var item = {func:f, wait:waitIfBusy?true:false};
	if (item.wait && ! tersus.progress.active)
		tersus.progress.set('..');
		
	tersus.async.queue.push(item);
};

tersus.async.worker = function()
{
	while (tersus.async.queue.length > 0 || window.rootDisplayNode && window.rootDisplayNode.status == FlowStatus.READY_TO_RESUME)
	{
		if (window.rootDisplayNode && window.rootDisplayNode.status == FlowStatus.READY_TO_RESUME)
			resumeRoot();
		else if (tersus.async.queue.length > 0)
		{
			var busy = isBusy(); // Root system busy
			if (! busy)
			{
				var item = tersus.async.queue.shift();
				try
				{
					if (tlog) tlog("tersus.async.worker: start");
					item.func();
				}
				catch (e)
				{
					if (tlog)
						tlog(e);
				}
				finally
				{
					if (tlog) tlog("tersus.async.worker: end");
				}
			}
			else // if the root system is busy, we search for items on the queue that don't need to wait
			{
				var found = false;
				for (var i=0;i<tersus.async.queue.length;i++)
				{
					var item = tersus.async.queue[i];
					if (!item.wait)
					{
						tersus.async.queue.splice(i,1);
						try
						{
							if (tlog) tlog("tersus.async.worker: start");
							item.func();
						}
						catch (e)
						{
							if (tlog)
								tlog(e)
						}
						finally
						{
							if (tlog) tlog("tersus.async.worker: end");
						}
						found = true;
					}
				}
				if (!found)
					break; // Try again later
			}
		}
	}
	//Since we're here - any queue are waiting for the root system to complete. So if the root system is not busy it means that the queue is empty and we can clear progress
	if (tersus.progress.active)
	{
		if ( ! isBusy())
		{
			debugAssertion(tersus.async.queue.length == 0, 'tersus.async.queue expected to be empty when root system not busy');
			tersus.progress.clear();
		}
	}
	
};
tersus.unescapeString = function unescape_string(s)
{
	return s.replace(/\\n/g, '\n').replace(/\\t/g,'\t').replace(/\\\\/g,'\\');
};
tersus.setDirection = function tersus_setDirection(direction)
{
	var windows = tersus.getWindows();
	window.textDirection = direction;
	for (var i=0; i< windows.length;i++)
	{
		windows[i].document.body.className=direction;
	}
	
};
tersus.getDirection = function tersus_getDirection()
{
	return window.textDirection == 'rtl'? 'rtl':'ltr';
};
tersus.translateResource = function (s)
{
	if (tersus.resourceMap && tersus.resourceMap[s])
		return tersus.resourceMap[s];
	else
		return s;
};
tersus.Conventions = {};
tersus.Conventions.isEventHandler = function (subflow)
{
	return subflow && (subflow.role.match(/^<On .*>$/) || subflow.role == '<Open>') ;
};
window.engine = {};
FlowStatus = {};
FlowStatus.NOT_READY_TO_START = 'Not Ready to Start';
FlowStatus.READY_TO_START = 'Ready to Start';
FlowStatus.READY_TO_RESUME = 'Ready to Resume';
FlowStatus.WAITING_FOR_INPUT = 'Waiting for Input';
FlowStatus.STARTED = 'Started';
FlowStatus.RESUMED = 'Resumed';
FlowStatus.PAUSED = 'Paused';
FlowStatus.DONE = 'Done';

BuiltinNames = {};
BuiltinNames.DONE_EXIT_NAME = '<Done>';

tersus.Event = function(elementName, jsName, methodName, name)
{
	this.elementName = elementName;
	this.jsName = jsName;
	this.methodName = methodName;
	this.name=name;
};
Events = {};
Events.ON_CLICK = new tersus.Event('<On Click>', 'onclick', 'onClick', 'click');
Events.ON_CONTEXT_MENU = new tersus.Event('<On Context Menu>', 'oncontextmenu' , 'onContextMenu', 'contextmenu');
Events.ON_FOCUS = new tersus.Event('<On Focus>', 'onfocus', 'onFocus', 'focus');
Events.ON_BLUR = new tersus.Event('<On Blur>', 'onblur','onBlur', 'blur');
Events.ON_MOUSEOVER = new tersus.Event('<On Mouse Over>','onmouseover','onMouseOver', 'mouseover');
Events.ON_MOUSEOUT = new tersus.Event('<On Mouse Out>','onmouseout','onMouseOut', 'mouseout');
Events.ON_MOUSEDOWN = new tersus.Event('<On Mouse Down>','onmousedown','onMouseDown', 'mousedown');
Events.ON_MOUSEUP = new tersus.Event('<On Mouse Up>','onmouseup','onMouseUp', 'mouseup');
Events.ON_MOUSEMOVE = new tersus.Event('<On Mouse Move>','onmousemove','onMouseMove', 'mousemove');
Events.ON_KEYPRESS = new tersus.Event('<On Key Press>','onkeypress','onKeyPress', 'keypress');
Events.ON_KEYDOWN = new tersus.Event('<On Key Down>','onkeydown','onKeyDown', 'keydown');
Events.ON_KEYUP = new tersus.Event('<On Key Up>','onkeyup','onKeyUp', 'keyup');
Events.ON_DOUBLE_CLICK = new tersus.Event('<On Double Click>','ondblclick','onDoubleClick', 'doubleclick');
Events.ON_TOUCHSTART = new tersus.Event('<On Touch Start>','ontouchstart','onTouchStart', 'touchstart');
Events.ON_TOUCHMOVE = new tersus.Event('<On Touch Move>','ontouchmove','onTouchMove', 'touchmove');
Events.ON_TOUCHEND = new tersus.Event('<On Touch End>','ontouchend','onTouchEnd', 'touchend');
Events.ON_BROWSE = new tersus.Event('<On Browse>',null,null);
Events.ON_CHANGE = new tersus.Event('<On Change>','onchange','onChange', 'change');
Events.ON_EXPAND = new tersus.Event('<On Expand>');
Events.ON_COLLAPSE = new tersus.Event('<On Collapse>');
Events.ON_SELECT = new tersus.Event('<On Select>');
Events.ON_LOCATION_CHANGE = new tersus.Event('<On Location Change>');
Events.ON_UNLOAD = new tersus.Event('<On Unload>');
Events.ON_LEAVE_VIEW = new tersus.Event('<On Leave View>');
Events.ON_SORT = new tersus.Event('<On Sort>');
Events.OPEN = new tersus.Event('<Open>');
Events.ON_DELETE = new tersus.Event('<On Delete>');
Events.ON_ELEMENT_CHANGE = new tersus.Event('<On Element Change>');
Events.ON_ORIENTATION_CHANGE = new tersus.Event('<On Orientation Change>');
Events.ON_TAP = new tersus.Event('<On Tap>', 'ontap', 'onTap', 'tap');
Events.ON_SUBMIT = new tersus.Event('<On Submit>', 'onsubmit', 'onSubmit', 'submit');

BuiltinModels = {};
BuiltinModels.TEXT_ID='Common/Data Types/Text';
BuiltinModels.NUMBER_ID='Common/Data Types/Number';
BuiltinModels.NOTHING_ID='Common/Data Types/Nothing';

function compareValues(value1, value2, caseInsensitive, recursionMap)
{
	//recursionMap holds a list all comparisons already made (or in progress) - we don't want to comapre the same objects twice
	if (value1 && value1.isDataNode)
		value1 = value1.value;
	if (value2 && value2.isDataNode)
		value2 = value2.value;

	if( value1==value2)
		return true;
	if (! value1 || ! value2)
		return false;
	
	if (! value1.isNode || !value2.isNode)
		return false;
	if (value1.constructor != value2.constructor && ! (value1.isLeaf && value2.isLeaf && value1.pluginFunc == value2.pluginFunc))
		return false;
	if (! value1.elementList)
	{
		var l1 = value1.leafValue;
		var l2 = value2.leafValue;
		var c1 = 0;
		var c2 = 0;
		if (value1.isMap && value2.isMap)
		{
			var keys2 = new Set();
			for (var key in l2)
			{
				++c2;
				keys2.add(key);
			}
			for (var key in l1)
			{
				if (! keys2.contains(key))
					return false;
				if (! compareValues(l1[key],l2[key]), caseInsensitive, recursionMap)
					return false;
				++c1;
			}
			if (c1 != c2)
				return false; //Different number of memebers
			return true;
		}
		if (value1.isText && caseInsensitive)
		{
			l1 = l1.toUpperCase();
			l2 = l2.toUpperCase();
		}
		return l1 == l2 || l1 && l1.equals && l1.equals(l2);
	}
	
	var id1  = value1.getNodeId();
	var id2 = value2.getNodeId();
	var key = id1+':'+id2;
	if (!recursionMap)
		recursionMap = {};
	if (recursionMap[key])
		return true; // already compared (or comparison in progress) - no need to comapre again
	recursionMap[key]=true;

	for (var i=0; i<value1.elementList.length; i++)
	{
		var e = value1.elementList[i];
//		if (e.isDisplayElement || e.isDataElement || e.isMethodElement)
		{
			var children1;
			var children2;
			if (e.isRepetitive)
			{
				children1 = e.getChildren(value1);
				children2 = e.getChildren(value2);
			}
			else
			{
				children1 = [e.getChild(value1)];
				children2 = [e.getChild(value2)];
			}
			if (children1.length != children2.length)
				return false;
			for (var j=0; j<children1.length; j++)
			{
				if (! compareValues(children1[j], children2[j], caseInsensitive, recursionMap))
					return false;
			}
		}
	}
	return true;
};
function escapeHTML(value)
{
	return value.replace(/&/g, '&amp;').replace(/>/g,'&gt;').replace(/</g,'&lt;').replace(/\n/g, '<br>');
};
HTMLProperties = {};


// Creates a set

function loadBooleanOption(name)
{
	var value = getCookie(name);
	window[name] = (value == 'true');
//	alert('Option:'+name+' value:'+value+'('+window[name]+')');
};
function loadOptions()
{
	loadBooleanOption('autoReload');
	loadBooleanOption('serverTracing');
	loadBooleanOption('showServiceInvocations');
	loadBooleanOption('clientTracing');
};

tersus.loadFile = function(path, callback, ignoreErrors)
{
	var storageKey = null;
	if (window.localStorage && tersus.useLocalStorage)
	{
		storageKey = "File?"+path;
		var text = localStorage.getItem(storageKey);
		if (text != null)
		{
			if (callback)
			{
				setTimeout(function() {callback(text);},0);
				return;
			}
			else
				return text;
		}
	}
	return tersus.loadURL(tersus.rootURL+path, callback, ignoreErrors, null/*error callback*/, storageKey);
};
tersus.loadURL = function(url, callback, ignoreErrors, errorCallback, storageKey)
{
	var req = createXMLHTTPRequest();
	
	if (callback)
	{
		req.open('GET',url, true);
		req.onreadystatechange = function()
		{
			var responseText = null;
			var status = null;
			var statusText = null;
			try
			{
				if (req.readyState != 4)
					return;
				var status = req.status;
				var statusText = req.statusText;
				if (status == 200)
				{
					responseText = req.responseText;
					if (storageKey && window.localStorage)
					{
						if (tlog)
							tlog('Storing '+storageKey+ ' in local storage.');
						localStorage.setItem(storageKey, responseText);
					}
				}
			}
			catch (e)
			{
				if (ignoreErrors)
					callback(null);
				else if (errorCallback)
					errorCallback(status, statusText, e);
				else
					alert('Failed to load "'+url+'"');
				return;
				
			}
			if (status == 200)
				 callback(responseText);
			else if (ignoreErrors)
				callback(null);
			else if (errorCallback)
					errorCallback(status, statusText, null);
			else
				alert('Failed to load "'+url+'"');
		};
		req.send(null);
		return req;
	}
	else
	{
		var method = 'GET';
		if (tersus.useLocalStorage && isWebKit && navigator.onLine == true)
			method = 'POST'; // For some reason 'GET' doesn't work in Safari cached mode (and so offline-enabled applications can't load templates like 'popin.html' even when online)
		req.open(method, url, false); 
		req.send(null);
		if (req.status != 200)
		{
			alert('Missing resource '+url+ '\nerror code='+req.status+' error message='+req.statusText);
			if (parent && parent.notLoaded)
				parent.notLoaded();
		
			return;
		}
		if (storageKey && window.localStorage)
		{
			if (tlog)
				tlog('Storing '+storageKey+ ' in local storage.');
			localStorage.setItem(storageKey, req.responseText);
		}
		return req.responseText;
	}
};
tersus.getUserName = function()
{
	return window.userName;
};
tersus.setUserName = function(name)
{
	window.userName = name;
};
tersus.getCurrentViewURL = function()
{
	if (tersus.firstURL.match('\\?minify=false#?$'))
		return tersus.firstURL;
	else
		return tersus.rootURL;
};
function addHiddenField(form, name, value)
{
	
	var field = form.ownerDocument.createElement('input');
	field.type = 'hidden';
	field.value =value;
	field.name = name;
	form.appendChild(field);
	return field;
}

loadOptions();
function set(list)
{
	var out = {};
	for (var i=0; i< list.length; i++)
	{
		out[list[i]] = list[i];
	}
	return out;
}
function Set(list)
{
	this.members = {};
	if (list) 
	{
		for (var i=0; i< list.length; i++)
		{
			this.members[list[i]] = true;
		}
	}
};
Set.prototype.contains = function (member) 
{
	return this.members[member];
};
Set.prototype.add = function(obj)
{
	this.members[obj]=true;
};
Set.prototype.toList = function()
{
	var list = [];
	for (var memeber in this.members)
		list.push(memeber);
	return list;
};

Set.prototype.remove = function(member)
{
	delete this.members[member];
};
HTMLProperties['table']=set(['width']);
HTMLProperties['TABLE']=set(['width']);
STYLE_PROPERTIES = set(['width', 'height', 'overflow']);

Operation = {};
Operation.REPLACE = 'Replace';
Operation.ADD = 'Add';
Operation.REMOVE = 'Remove';
function internalError(message)
{
		alert ('Error:\n'+message);
		this.undefined();
};
function modelError(message, model)
{
	if (model && model.modelId)
		message = 'Invalid Model: '+model.modelId+'\n'+message;
	if (tlog)
		tlog(message);
	throw new tersus.Exception(message);
};
function debugAssertion(expression, message)
{
	//Default implemenation is a no-op. Should be overridden in test settings
}
function modelExecutionError(message, node)
{
	if (node)
		message += '\nLocation:'+new tersus.RuntimeLocation(node);
	alert ('Model Execution Error:\n'+message);
};

function makeAssertion(flag, description)
{
	if (! description)
		var description = '';
	if ( !flag)
	{
		alert ('Assertion Failed ' + description);
		this.undefined();
	}
};

function debugMessage(message)
{
	if (window.clientTracing)
		alert('DEBUG: '+message);
}

tersus.RuntimeLocation = function(node,element, suffix)
{
	var segments = [];
	// Collecting path segments in reverse order
	if (suffix)
		segments.push(suffix);
	if (element)
		segments.push(element.role);
	var current = node;
	while (current)
	{
		if (!current.isBackground)
		{
			var currentModelId = current.getModelId();
			if (! current.element  || current.element.modelId != currentModelId)
			{
				this.baseModelId = currentModelId;
				if (!currentModelId)
					error("No currentModelId id");
				break;
			}
			else
				segments.push(current.element.role);
		}
		current = current.parent;
	}
	segments.reverse();
	this.path = new Path();
	this.path.setSegments(segments);
};
tersus.RuntimeLocation.prototype.toString = function()
{
	return this.baseModelId + '['+this.path.str+']';
};
/*
	Handles a user event (mouse click etc.)
	displayNode: the displayNode on which the event was registered
	eventType: the type of the event (see Events.ON_XXX)
	parameters: parameters that will be passed to the event handler action
*/
window.engine.handleEvent = function handleEvent(displayNode, eventType , parameters, domEventAttributes) 
{
	if (tlog) tlog ('handleEvent '+eventType.elementName + ' on '+displayNode.modelName + ' ( '+displayNode.getPath().str+')');
	tersus.currentEventAttributes = domEventAttributes;

	var subFlow = displayNode.getEventHandler(eventType);
	if (!subFlow)
		return;
	var child = subFlow.createChild(displayNode);
	if (parameters)
	{
		for (var parameterName in parameters)
		{
			var trigger = child.getElement(parameterName);
			if (trigger)
			{
				var value = parameters[parameterName];
				if (value !== undefined)
				{
					if (trigger.isRepetitive)
					{
						var v = value.length? value : [value];
						for (var i=0;i<v.length;i++)
						{
							trigger.addAChild(child,trigger.getValueNode(v[i]));
						}
					}
					else
						trigger.replaceChild(child, trigger.getValueNode(value));
				}
			}
		}
	}
	if (subFlow.isRepetitive)
		subFlow.addAChild(displayNode,child);
	else
		subFlow.replaceChild(displayNode,child);
	child.readyToStart();
	resumeRoot();
};

function isBusy()
{
	return(window.rootDisplayNode && window.rootDisplayNode.status != FlowStatus.WAITING_FOR_INPUT)
};
function checkBusy()
{
	if (isBusy())
	{
		if (tersus.alertMode.interactive)
			alert('The application is busy, please wait (or refresh)');
		return true;
	}
	else
		return false;
};

function resumeRoot()
{
	var start = new Date();
	if (tlog) tlog('resumeRoot');
	if (! tersus.progress.active)
		tersus.progress.set('Busy...');
	if (tlog) tlog ('resumeRoot: resuming root node');
	window.rootDisplayNode.doResume();
	if (window.trace)
	{
		if (tlog) tlog("resumeRoot: flushing trace");
		window.trace.flush();
	}
	var end = new Date();
	$(window).trigger('ter-process-end');
	if (tlog) tlog("resumeRoot: duration: " + (end - start) + " ms.");
	window.duration = "Action duration: " + (end - start) + " ms.";

};

tersus.queueEventDelay = 5000;
function queueEvent(object, method, args, startTime,description, callback, maxDelay)
{
	if (!maxDelay)
		maxDelay=tersus.queueEventDelay;
	if (isBusy())
	{
		var now = new Date();
		var delay = now - startTime;
		if (tlog) 
		{
			if (!description)
				description = method.name;
			tlog('QE: Busy ('+delay+' ms) - '+description);
		}
		if (delay < maxDelay || maxDelay<0)
		{
			tersus.progress.set('Busy ...');
			setTimeout(function() {queueEvent(object,method, args, startTime, description, callback, maxDelay);},100);
			return;
		}
	}
	//TODO - provide a nice visual indication that the system is waiting for something, and 
	// a way for the user to abort the running operation (or refresh the screen).
	// In the meantime, we use 'checkBusy()', that shows a warning message.
	if (checkBusy())
	{
		if (tlog)
			tlog('QE: Giving up - '+description);
		return;
	}
	if (tlog)
		tlog('QE: Invoking - '+description);
	/* TODO prevent concurrent operation:
	
	-- If we have a single user gesture triggering multiple events, we want them all to run, even if the first one takes a long time
	-- However, we don't want them to run "in parallel" (in simulated parallel)
	-- Therefore, putting them in the async queue is not good enough (they will run "in parallel")
	-- We may need to add a pre-queue (maybe a variant of queueEvent that runs for ever)
	
	*/
	tersus.async.exec (function () {method.apply(object,args);if (callback)
		tersus.async.exec(callback,true);}, true /* wait if busy */);
};

tersus.traverseHierarchy = function traverseHierarchy(root, iterator, operation)
{
	var alreadyScanned = {};
	var iteration = function(node)
	{
		var id = node.getNodeId();
		if (alreadyScanned[id])
			return;
		alreadyScanned[id] = true;
		var stop = operation(node);
		if (stop == true)
			return true;
		iterator(node,iteration);
	}
	iteration(root);
}
tersus.scanHierarchy = function scanHierarchy(root, iterator, filter)
{
	var alreadyScanned = {};
	var out = [];
	var operation = function(node)
	{
		if (filter(node))
			out.push(node);
	};
	traverseHierarchy(root,iterator, operation);
	return out;
}
tersus.sqlize = function(str)
{
	return str.replace(/[^a-zA-Z0-9]/g, "_");
};



function stats()
{
	var packages = {};
	var nModels = 0;
	var nPackages = 0;
	for (var modelId in tersus.repository.constructors)
	{
		++nModels;
		var packageId = tersus.getPackageId(modelId);
		if (packages[packageId])
			++packages[packageId];
		else
		{
			packages[packageId]=1;
			++nPackages;
		}
	}
	var message = 'Total models:'+nModels;
	message += 'Total packages:'+nPackages;
	for (var packageId in packages)
	{
		message += '\n'+packageId+':'+packages[packageId];
	}
	var w = window.open();
	w.document.write('<pre>');
	w.document.write(message);
	w.document.write('</pre>');
	w.document.close();

}
tersus.getPackageId = function(modelId)
{
	var i = modelId.lastIndexOf('/');
	return modelId.substring(0,i);
};
Function.prototype.__subclass = function(name, c)
{
	if (!c)
		c = function(){};
	c.name=name;
	c.prototype = new this();
	c.prototype.constructor = c;
	c.prototype.parentPrototype = this.prototype;
	c.prototype.supr = function(name, args)
	{
		if (args === undefined)
			args = [];
		var p = this.parentPrototype;
		var f = this[name];
		while (p && p[name]===f)
			p = p.parentPrototype;
		return p[name].apply(this,args);
	};
	return c;
};

tersus.newAction = function(name, startFunc)
{
	var c = ActionNode.__subclass('tersus.'+name);
	c.prototype.start = startFunc;
	tersus[name]=c;
};

tersus.findChildNodesByClassName = function(node, className, out)
{
	if (!out) out = [];
	if (node.className && node.className==className)
		out.push(node);
	var children = node.childNodes;
	for( var i = 0; i<children.length; i++)
	{
		tersus.findChildNodesByClassName(children[i], className, out);
	}
	return out;
};
tersus.adjustHeight = function (domNode, delta)
{
	var currentHeight =  getVisibleSize(domNode).height ;
	if (window.browser != 'IE')
	{
		var paddingBottom = parseInt(getActualStyle(domNode,'padding-bottom'));
		var paddingTop = parseInt(getActualStyle(domNode,'padding-top'));
		currentHeight -= (paddingBottom + paddingTop);
	}
	domNode.style.height = (currentHeight + delta) + 'px';
}

tersus.foreach = function(array, f)
{
	if (array && array.length)
		for (var i=0;i<array.length; i++)
		{
			f(array[i]);
		}
}

tersus.preventDefaultEventHandling = function()
{
	var ev = tersus.currentEvent;
	if (ev)
	{
		if (ev.preventDefault)
			ev.preventDefault();
		else
			ev.returnValue=false;
			
	    if (ev.stopPropagation)
            ev.stopPropagation();
		else
            ev.cancelBubble = true;
	}		
}
tersus.isIPhone =  navigator.userAgent.toLowerCase().indexOf('iphone') >=0 || tersus.searchParams.UserAgent == 'iPhone';
tersus.isIPhoneSafari =  navigator.userAgent.toLowerCase().indexOf('iphone') >=0 && navigator.userAgent.toLowerCase().indexOf('applewebkit') >=0 ;
if (tersus.isPhone)
	tersus.useIFrames = false;

tersus.foreach = function(a, func, contextObj)
{
	if (a == null)
		return;
	for (var i=0;i<a.length;i++)
	{
		func.call(contextObj, a[i], i);
	};
};
tersus.addAll = function (a1, a2)
{
	tersus.foreach(a2, function(x) {a1.push(x);});
};
// file: 006-errors.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.Exception = function(message,cause)
{
	this.message = message;
	this.cause = cause;
};
tersus.Exception.prototype = new tersus.Exception(null,null);
tersus.Exception.prototype.toString = function()
{
	return this.message;
}
tersus.errors = null;
tersus.error = function(m)
{
	if (window.tlog) tlog(m);
	if (tersus.errors)
		tersus.errors.push(m);
	else
		alert(m);
};

tersus.getErrors = function()
{	
	if (tersus.errors == null || tersus.errors.length == 0)
		return null;
	else
		return tersus.errors.join('\n');
};

tersus.clearErrors = function()
{
	tersus.errors=[];
};

tersus.serverError = function(msg)
{
	tersus.error(msg);
	if (tersus.pendingServiceCalls)
	{
		var list = tersus.pendingServiceCalls.toList();
		if (list.length == 1)
		{
			var node = tersus.findNode(list[0]);
			if (node && node.handleResponse)
			{
				node.handleResponse({});
			}
		}
		else if (length > 1)
		{
			tersus.error("Can't identify caller.  Please refresh.");
		}
	}
};// file: 009-Repository.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function ConstructorRepository()
{
	this.constructors = {};
	this.htmlTemplates = {};
	this.preloaded = {};
	this.loadedPackages = [];
	this.packageTexts = {};
	this.parsedPackages = {};
};
ConstructorRepository.prototype.add = function add_prototype(c)
{
	this.constructors[c.prototype.modelId]=c;
};

ConstructorRepository.prototype.get = function get(id)
{
	if (!this.constructors[id])
	{
		this.parsePackage(tersus.getPackageId(id));
	}
	if (! this.constructors[id])
	{
		if (tlog) tlog('Not preloaded:'+id);
		this.loadPackages([tersus.getPackageId(id)]);
		this.parsePackage(tersus.getPackageId(id));
	}
	var c =  this.constructors[id];
	return c;
};
ConstructorRepository.prototype.isLoaded = function(modelId)
{
	this.parsePackage(tersus.getPackageId(modelId));
	// Check again - maybe the package was parsed and now the constructor is there
	return (this.constructors[modelId]!=undefined);
};
ConstructorRepository.prototype.parsePackage = function(packageId)
{
	if (this.packageTexts[packageId] && !this.parsedPackages[packageId])
	{
		var t0 = new Date();
		this.parsedPackages[packageId] = true;
		var text = this.packageTexts[packageId];
		var models = eval(text);
		var t1 = new Date();
		for (var i=0;i<models.length;i++)
		{
			this.loadModel(packageId, models[i]);
		}
		var t2 = new Date();
		if (tlog)
			tlog('LOADER: ' + packageId+ ': '+models.length+' models loaded in '+(t2-t0) + ' ms ( eval time='+(t1-t0) + ' ms)');
	}
	
};

ConstructorRepository.prototype.loadModel = function (packageId, m)
{
	var c = function(){this.init();};
	var p = c.prototype = eval ('new '+m.c + '()');
	p.constructor = c;
	p.modelId = packageId + '/'+m.n;
	p.modelName = m.n;
	p.plugin = m.p;
	if (p.initModel)
		p.initModel();
	if (m.sp != null) p.sharedProperties = m.sp;
	if (m.pv != null) p.pluginVersion = m.pv;
	if (m.cc != null) p.cancellable = m.cc;
	if (m.sc != null) p.secure = m.sc;
	if (m.runImmediately != null) p.runImmediately=m.runImmediately;
	if (m.pm != null) p.progressMessage = m.pm;
	if (m.xmlNS != null)
		p.xmlNamespace = m.xmlNS;
	if (m.format != null)
		p.format = m.format;
	if (m.formatAsGMT != null)
		p.formatAsGMT = m.formatAsGMT;
	if (m.progressMessage != null) p.progressMessage = m.progressMessage;
	if (m.v != null)
	{
		try
		{
			p.parseString(m.v);
		}
		catch (e)
		{
			throw {message:'Invalid value/format for '+p.modelId + ' details='+e.message};
		}
	}
	if (m.tableName)
		p.tableName=m.tableName;
	if (m.e)
	{
		for (var i=0;i<m.e.length;i++)
		{
			var me = m.e[i];
			var e = null;
			switch (me.t)
			{
				case 'sf':
				  e = p.addSubFlow(me.r,me.ref);
				  break;
				case 'l':
			      e = p.addLink(me.r, me.src, me.tgt);
			      break;
			    case 'ctx':
			      e = p.addAncestorReference(me.r,me.ref);
			      break;
			    case 'const':
			      e = p.addConstant(me.r,me.ref);
			      break;
			    case 'var':
			      e = p.addIntermediateDataElement(me.r,me.ref);
			      break;
			    case 'da':
			      e = p.addDataElement(me.r,me.ref);
			      break;
			    case 'di':
			      e = p.addDisplayElement(me.r,me.ref,me.ep);
			      break;
			    case 'tr':
			      e = p.addTrigger(me.r, me.ref);
			      break;
			    case 'ex':
			      e = p.addExit(me.r,me.ref);
			      break;
			    case 'er':
			      e = p.addErrorExit(me.r,me.ref);
			      break;
			    default:
			      throw new tersus.Exception('Invalid Model ('+p.modelId+'): unknown element type "'+me.t+'"');
			      
			}
			if (e)
			{
				if (me.rep != null)	e.isRepetitive = me.rep;
				if (me.rk != null)	e.dependencyRank = me.rk;
				if (me.m != null)	e.mandatory = me.m;
				if (me.ac != null)	e.alwaysCreate = me.ac;
				if (me.v != null)	e.valueStr = me.v;
				if (me.ct != null)	e.isConstantTrigger = me.ct;
				if (me.op != null)	e.operation = me.op;
				if (me.sr != null)	e.isSelfReady = me.sr;
				if (me.primaryKey != null) e.isPrimaryKey = me.primaryKey;
				if (me.columnName != null) e.columnName = me.columnName;
				if (me.xmlNS != null) e.xmlNamespace = me.xmlNS;
				if (me.format != null) e.format = me.format;
				if (me.formatAsGMT != null) e.formatAsGMT = me.formatAsGMT;
			}
		}
	}
	this.add(c);
	if (p.onload)
		p.onload();
	
	
};
ConstructorRepository.prototype.checkLoaded = function(modelId)
{
	if (this.isLoaded(modelId))
		return true;
	for (var i=0;i<this.loadedPackages.length;i++)
	{
		var packageId = this.loadedPackages[i];
		if (modelId.substring(0,packageId.length) == packageId
			&& modelId.charAt(packageId.length) == '/')
		{
			modelError("Missing model "+modelId + "\n(pakcage "+packageId + " already loaded)");
			break;
		}
	}
	return false;
		
}

var MAX_URL_LENGTH=1024;
tersus.packageLoader = function(){};
tersus.packageLoader.prototype.onload = function(packages)
{
	tersus[this.scriptCallback] = null; //Drop reference to allow garbage collection
	for (var id in packages)
	{
		tersus.repository.packageTexts[id] = packages[id];
	}
	this.runAsync();
};
tersus.packageLoader.error = function(msg)
{
	alert('An error has occurred while loading the application:\n'+msg);
};
tersus.nextPackageLoaderId = 1;
tersus.packageLoader.prototype.runAsync = function runAsync()
{
	var userName = tersus.getUserName();
	var loader=this;
	if (loader.currentIndex == loader.packageIds.length)
	{
		setTimeout(loader.callback,0);
		return;
	}
	if (window.localStorage && tersus.useLocalStorage) // new mechanism - packages stored in local storage 
	{
		var url = tersus.rootURL+'Code';
        var paramString='packages=';
        while (this.currentIndex<this.packageIds.length)
        {
        	paramString += encodeURIComponent(':'+loader.packageIds[this.currentIndex]);
        	++ this.currentIndex;
        }
        var tsKey = tersus.baseURL+':LAST_TIMESTAMP';
        var storedTimestamp = localStorage.getItem(tsKey);
        var timestamp = Math.max(tersus.lastTimestamp, storedTimestamp);
        var storageKey=tersus.baseURL+':Code?timestamp='+timestamp+'&user='+encodeURIComponent(userName)+'&'+paramString;
        if (timestamp == storedTimestamp)
        {
        	var text = localStorage.getItem(storageKey);
        	if (text != null)
        	{
        		var packages = eval(text);
        		loader.onload(packages);
        		if (tlog) tlog('Loaded from local Storage');
       			return; 	
        	}
        }
        else
        {
        	for (var i=localStorage.length -1; i>=0 ;i--)
        	{
        		var key=localStorage.key(i);
        		if (key.search(/^(Code|File)\?/) >= 0)
        		{
	       			localStorage.removeItem(key);
	       			if (tlog) tlog('Removing '+key + ' from local storage');
	       		}
        	}
        	localStorage.removeItem(tsKey);
        	
        }
 		var req = createXMLHTTPRequest();
        req.onreadystatechange= function(){
        	if (req.readyState ==4)
        	{
        		if (req.status==200)
        		{
        			var text = req.responseText;
	        		var packages=eval(text);
	        		try
	        		{
	        			localStorage.setItem(storageKey, text);
	        			localStorage.setItem(tsKey,timestamp);
	        		}
	        		catch (e)
	        		{
	        			if (localStorage.length > 0)
	        			{
	        				localStorage.clear();
	        				tersus.reloadApplication();
	        				return;
	        			}
	        			else 
	        			{
	        				alert("Can't load application - local storage error:\n"+e);
	        				return;
	        			}
	        			
	        		}
	        		loader.onload(packages);
	        		if (tlog) tlog('Loaded from server');
	        	}
	        	else
	        	{
	        		tersus.error("Failed to load models from server");
	        	}
	        }
	        	
        };
        req.open('POST', url, true);
        req.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
        req.send(paramString);
		return;
	}
	else // old mode
	{
		loader.scriptCallback = 'loadPackages'+(tersus.nextPackageLoaderId ++);
		tersus[loader.scriptCallback] = function(constructors) { loader.onload(constructors);};
		var url = tersus.rootURL+'Code?timestamp='+tersus.lastTimestamp+'&_js_callback='+'tersus.'+loader.scriptCallback+'&user='+encodeURIComponent(userName)+'&packages=';
			
		// We may need to split the request into multiple requests if the url ends up too long
		for (var nextIndex=loader.currentIndex;nextIndex<loader.packageIds.length;nextIndex++)
		{
			var nextURL = url+encodeURIComponent(':'+loader.packageIds[nextIndex]);
			if (nextURL.length > MAX_URL_LENGTH)
				break;
			else
				url = nextURL;
		}		
		if (nextIndex==loader.currentIndex)
		{
			modelError("Package id "+loader.packageIds[nextIndex] + " is too long to load");
			return;
		}
		if (tersus.progress.active)
			tersus.progress.set("Loading application components");
		loader.currentIndex = nextIndex;
		tersus.loadScript(url);
	}
};
tersus.packageLoader.prototype.runSync = function runSync()
{
	var loader = this;
	var url = tersus.rootURL+'Code?timestamp='+tersus.lastTimestamp+'&user='+encodeURIComponent(tersus.getUserName())+'&packages=';
	
	// We work hard to retrieve packages using 'GET', as this will enable caching
	// This means we sometimes have to split the list into multiple requests
	if (tersus.progress.active)
		tersus.progress.set("Loading application components");
	while (loader.currentIndex < loader.packageIds.length)
	{
		for (var nextIndex=loader.currentIndex;nextIndex<loader.packageIds.length;nextIndex++)
		{
			var nextURL = url+encodeURIComponent(':'+loader.packageIds[nextIndex]);
			if (nextURL.length > MAX_URL_LENGTH)
				break;
			else
				url = nextURL;
		}		
		if (nextIndex==loader.currentIndex)
		{
			modelError("Package id "+loader.packageIds[nextIndex] + " is too long to load");
			return;
		}
		loader.currentIndex = nextIndex;
		var jsText = tersus.loadURL(url);
		if (jsText)
		{
			var packages = eval(jsText);	
			for (var id in packages)
			{
				tersus.repository.packageTexts[id] = packages[id];
			}
		}
	}
};
ConstructorRepository.prototype.loadPackages = function loadPackages(packageIds, callback) 
{
	if (tlog) tlog('Loading packages: '+packageIds.join(';'));
	for (var i=0;i<packageIds.length;i++)
	{
		this.loadedPackages.push(packageIds[i]);
	}

	var loader = new tersus.packageLoader();
	
	loader.packageIds = packageIds;
	loader.callback = callback;
	loader.currentIndex = 0;
	if (loader.callback)
		loader.runAsync();
	else
		loader.runSync();
};


ConstructorRepository.prototype.content = function content()
{
	var content = null;
	for (var id in this.constructors)
	{
		if (! content)
			content = id;
		else
			content = content + '\n'+id;
	}
	return content;
};
ConstructorRepository.prototype.templateNumber = 0;

//TODO - change loading mechanism so that html snippets are loaded with model
tersus.preloadedTemplates = {};
tersus.preloadTemplate = function(name,text)
{
	tersus.preloadedTemplates[name]=text;
};
ConstructorRepository.prototype.extractTemplate = function(path,html)
{
	var $t0=$(html);
	var $t = $t0.filter('#content').add($t0.find('#content')); // content cound be top level or lower level
	if (! $t[0]) // Create a container for all content nodes of the template
	{
		var $t = $('<div>');
		$t.append($t0.filter('*').not('title,script,meta'));//filter('*') excludes text nodes
	}
	this.saveTemplate(path, $t[0]);
	return $t[0];
};
ConstructorRepository.prototype.loadHTMLTemplate = function (path)
{
	if (this.htmlTemplates[path])
		return; //Already loaded
	if (tersus.preloadedTemplates[path] != null)
	{
		var text = tersus.preloadedTemplates[path];
		var template = document.createElement('DIV');
		template.innerHTML = text;
		this.saveTemplate(path, template);
		return;
	}
	if (tersus.progress.active)
	{
		var message;
		if (tersus.progressMessage)
			message = tersus.progressMessage;
		else
			message = 'Loading template '+path + '...';
		tersus.progress.set(message);
	}
	var html = tersus.loadFile(path);
	
	this.extractTemplate(path,html);
	
	/*
	
	// Old mechanism (no jQuery)
	if (tersus.progress.active)
		tersus.progress.set('Template loaded.');
	var name = 'templateFrame_'+ (++this.templateNumber);
	var iframe = window.createAnIFrame(window, name);
	var iframeNode = window.document.getElementById(name);
	var repository=this;
	var count=0;
	var cont1 = function()
	{
		++count;
		var d = iframeNode.contentDocument;
		var w = iframeNode.contentWindow;
		if ( d == null && w == null)
		{
			if (count < 50)
				setTimeout(cont1,50);
			else
				alert('Failed to load template '+path);
			return;
		}
		if (d == null)
		{
			d = iframeNode.contentWindow.document;
		}
		d.write(html);
		d.close();
		var cont2 = function() 
		{
			var templateNode = d.getElementById('content');
			if (!templateNode)
				templateNode = d.body;
			templateNode.id='';
			var template;
			if (templateNode.tagName == 'BODY')
			{
				template = document.createElement('DIV');
				template.innerHTML = templateNode.innerHTML;
			}
			else
			{
			  template =  importNode(window.document,templateNode);
			}
			repository.processTemplate(template);
			iframeNode.parentNode.removeChild(iframeNode);
			repository.htmlTemplates[path]=template;
		};
		setTimeout(cont2,50);
	}
	setTimeout(cont1,50);
	*/
};
ConstructorRepository.prototype.saveTemplate = function saveTemplate(path,template)
{
	this.processTemplate(template);
	this.htmlTemplates[path]=template;
};
//TODO move this mechanism to "Embedded HTML" so that it isn't loaded unnecessarity
ConstructorRepository.prototype.getHTMLTemplate = function (path)
{
	var template = this.htmlTemplates[path];
	if (!template)
	{
		var wasInProgress = tersus.progress.active;
		var progressMessage;
		if (tersus.progressMessage)
			message = tersus.progressMessage;
		else
			message = 'Loading template '+path + '...';
		
		tersus.progress.set(message);
		var html = tersus.loadFile(path);
		if (wasInProgress)
			tersus.progress.set('Template loaded.');
		else
			tersus.progress.clear();
			
		/*
		// Old mechanism
		
		var name = 'templateFrame';
		var iframe = window.createAnIFrame(window, name);
		var iframeNode = window.document.getElementById(name);
		var d = iframeNode.contentWindow.document;
		d.write(html);
		d.close();
		var templateNode = d.getElementById('content');
		if (!templateNode)
			templateNode = d.body;
		var template;
		if (templateNode.tagName == 'BODY')
		{
			template = document.createElement('DIV');
			template.innerHTML = templateNode.innerHTML;
		}
		else
		{
		  template =  importNode(window.document,templateNode);
		}
		this.processTemplate(template);
		iframeNode.parentNode.removeChild(iframeNode);
		this.htmlTemplates[path]=template;
		*/
		
		var template = this.extractTemplate(path,html);
	}
	return template;
};
/**
 * Perform the following processing for a template:
 *
 * 1) Remove sample nodes
 * 2) Hide repetitive template nodes
*/

ConstructorRepository.prototype.processTemplate = function(template)
{
	var $t=$(template);
	$t.attr('id','');
	$t.find("[repetitive=sample]").remove();
	$t.find("[repetitive=true]").css('display','none');

/*
	var elements = template.getElementsByTagName('*');
	for (var i=0; i<elements.length; i++)
	{
		var e = elements[i];
		var repetitive  = e.getAttribute('repetitive');
		if (repetitive == 'sample')
		{
			if (e.parentNode)
				e.parentNode.removeChild(e);
		}
		else if (repetitive == true)
			e.style.display='none';
	}
	*/
};
// file: 011-Path.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
//Path
function Path(pathStr)
{
	if (pathStr != null && pathStr != undefined)
	{
		this.str = pathStr;
	 	if (pathStr.length == 0)
			this.segments = [];
		else
			this.segments = pathStr.split('/');
	}
};

Path.prototype.setSegments = function(segments)
{
	this.segments = segments;
	this.str = segments.join('/');
};
Path.prototype.toString = function toString() {return this.str;};
Path.prototype.addSegment = function addSegment(role) 
{
	this.segments.push(role);
	this.str += '/' + role;
};

Path.prototype.getElement = function (parent)
{
	var currentModel = parent;
	var currentElement = null;
	for (var i=0; i<this.segments.length;i++)
	{
		if (currentModel == null)
			return null;
		currentElement = currentModel.getElement(this.segments[i]);
		if (currentElement == null)
			return null;
		if (currentElement.modelId)
			currentModel = tersus.repository.get(currentElement.modelId).prototype;
		else
			currentModel = null;
		
	};
	return currentElement;
};
Path.prototype.clone = function()
{
	var clone = new Path();
	clone.segments = this.segments.concat();
	clone.str = this.str;
	return clone;	
};

// file: 012-Node.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 //Generic Runtime Node
tersus.Node = function tersus_Node() {};
tersus.Node.prototype.type='Node';
tersus.Node.prototype.textTranslation = true;
tersus.Node.prototype.pluginVersion = 0;
tersus.Node.prototype.getPrototype = function()
{
	if (this.modelId)
		return tersus.repository.get(this.modelId).prototype;
}
tersus.Node.prototype.getNodeId = function()
{
	if (!this.nodeId)
	{
		this.nodeId = window.nextNodeId ? window.nextNodeId : 1;
		window.nextNodeId = this.nodeId + 1;
	}
	return this.nodeId;
};
tersus.Node.prototype.getModelId = function()
{
	return this.modelId;
};
tersus.Node.prototype.isInstance = function(value)
{
	return value && value.isNode && value.modelId == this.modelId;
};
tersus.Node.prototype.init = function() {}; // Called when a concrete instance is created
tersus.findNodeOld = function findNodeOld(id)
{
	if (tlog) tlog("Trying to find node "+id);

	var node = window.rootDisplayNode.findById(id);
	if (node)
	{
		if (tlog) tlog("Node "+id + " found.");
		return node;
	}
	tersus.unregisterPopupNodes();
	for (var popupId in tersus.popupNodes)
	{
		node = tersus.popupNodes[popupId].findById(id);
		if (node)
		{
			if (tlog) tlog("Node "+id + " found.");
			return node;
		}
	}
	return null;
		
};
tersus.Node.prototype.findById = function(id, alreadyScanned, depth)
{
	if (!depth)
	   depth=1;
		
	if (this.nodeId == id)
		return this;
	if (! this.elementList)
		return null;
	if (!alreadyScanned)
		alreadyScanned = {};
	var thisNodeId = this.getNodeId();
	if (alreadyScanned[thisNodeId])
		return null;
	alreadyScanned[thisNodeId]=true;
	for (var i=0; i<this.elementList.length; i++)
	{
		var e = this.elementList[i];
		if (e.isRepetitive)
		{
			var children = e.getChildren(this);
			for (var j=0; j<children.length;j++)
			{
				var child = children[j];
				if (child)
				{
					var node = child.findById(id, alreadyScanned, depth+1);
					if (node)
						return node;
				}
			}
		}
		else
		{
			var child = e.getChild(this);
			if (child)
			{
				var node = child.findById(id, alreadyScanned, depth+1);
				if (node)
					return node;
			}
		}
	}
	return null;
};

tersus.Node.prototype.scanChildren = function(filter, alreadyScanned, outputList)
{
	var id = this.getNodeId();
	if (alreadyScanned[id])
		return;
	alreadyScanned[id]=true;
	if (this.currentWindow && this.currentWindow.closed)
		return;
	if (filter.match(this))
		outputList.push(this);
	if (! this.elementList)
		return null;
	for (var i=0; i<this.elementList.length; i++)
	{
		var e = this.elementList[i];
		if (e.isDisplayElement || e.isDataElement || e.isMethodElement)
		{
			if (e.isRepetitive)
			{
				var children = e.getChildren(this);
				for (var j=0; j<children.length;j++)
				{
					var child = children[j];
					if (child)
					{
						child.scanChildren(filter, alreadyScanned, outputList);
					}
				}
			}
			else
			{
				var child = e.getChild(this);
				if (child)
				{
					child.scanChildren(filter, alreadyScanned, outputList);
				}
			}
		}
	}
	return null;
};
tersus.Node.prototype.countHierarchy = function()
{
	var count = 1
	if (this.elementList)
	{
		for (var i=0; i<this.elementList.length; i++)
		{
			var e = this.elementList[i];
			if (e.isRepetitive)
			{
				var children = e.getChildren(this);
				for (var j=0; j<children.length;j++)
				{
					var child = children[j];
					if (child)
					{
						count += child.countHierarchy();
					}
				}
			}
		
			else
			{
				var child = e.getChild(this);
				if (child)
				{
					count += child.countHierarchy();
				}
			}
		}
	}
	return count;
};

tersus.Node.prototype.invokeLinks = function invokeLinks()
{
	var links = this.element.outgoingLinks;
	if (links)
	{
		for (var i=0; i<links.length; i++)
		{
			var link = links[i];
			link.invoke(this);		
		}
	}
};
tersus.Node.prototype.isNode = true;
tersus.Node.prototype.isReady = function isReady()
{
	return true;
};

tersus.Node.prototype.addElement = function addElement(element)
{
	if (!this.elements)
		this.elements = {};
	this.elements[element.role]=element;
	if (!this.elementList)
		this.elementList = [];
	this.elementList.push(element);
	element.owner = this;
	return element;
};

tersus.Node.prototype.getElement = function getElement(role)
{
	var element = null;
	if (this.elements)
		element = this.elements[role];
	if (element)
		return element;
	else
		return null;
};
tersus.Node.prototype.addGetter = function(role, getter)
{
	this['getter_'+role] = getter;
};
tersus.Node.prototype.addSetter = function(role, setter)
{
	this['setter_'+role] = setter;
};
tersus.Node.prototype.addRemover = function(role, remover)
{
	this['remover_'+role] = remover;
};

tersus.Node.prototype.getGetter = function getGetter(role)
{
	if (this['getter_'+role])
		return this['getter_'+role];
	else if (role.charAt(0)=='<')
		return this['get'+role];// Obsolete mode (still widely used as of June 2009)
	else return null;
};
tersus.Node.prototype.getSetter = function getSetter(role)
{
	if (this['setter_'+role])
		return this['setter_'+role];
	else if (role.charAt(0)=='<') 
		return this['set'+role]; // Obsolete mode (still widely used as of June 2009)
	else
		return null;
};
tersus.Node.prototype.getAdder = function getAdder(role)
{
	return this['add'+role];
};
tersus.Node.prototype.getRemover = function getRemover(role)
{
	if ( this['remover_'+role])
		return this['remover_'+role];
	else if (role.charAt(0)=='<') 
		return this['remove'+role]; // Obsolete mode (still widely used as of June 2009)
	else
		return null;
};

tersus.Node.prototype.getChild = function getChild(role)
{
	var element = this.getElement(role);
	if (element)
		return element.getChild(this);
	else
		return null;
};

tersus.Node.prototype.getChildren = function getChildren(role)
{
	var element = this.getElement(role);
	if (element)
		return element.getChildren(this);
	else
		return [];
};
/*
	Returns an array of all values at given path.
	path - a Path containing the required path
	index - an optional number indicating which segment in the path should be deemed first 
	         (the first 'index' segments are ignored.)
*/
tersus.Node.prototype.getValues = function getValues(path, index)
{
	if (!index)
		var index = 0;
	var values = [];
	this._getValues(path, index, values);
	return values;
};


/*
	Adds all values at a given path to a given array of values
	This method is used internally to implement 'getValues'/
	path - a Path containing the required path
	index - an optional number indicating which segment in the path should be deemed first 
	         (the first 'index' segments are ignored.)
	values - an array to which all values will be added.
	 
*/
tersus.Node.prototype._getValues = function _getValues(path, index, values)
{
	if (path.segments.length < index)
	{
		internalError('Bad path in _getValues');
		return;
	}
	var role = path.segments[index];
	if (path.segments.length == index )
	{ 
		values.push(this);
	}
	else
	{
		var element = this.getElement(role);
		if (!element)
			return; // The path links to a non-existant element (as of Feb 26, this can happen because of the way the permission mechanism works)
		if (element.isRepetitive)
		{
			var children = element.getChildren(this);
			for (var i=0; i<children.length; i++)
			{
				var child = children[i];
				if (child)
					child._getValues(path, index+1, values);
			}
		}
		else
		{
			var child = element.getChild(this);
			if (child)
				child._getValues(path, index+1, values);
			
		}
	}
}
/**
	Sets the value of a descendant, specified by path
	The index specifies a number of initial segments of the path that should be ignored
	(index = 0 means using the whole path, index = 1 means starting from the second segment).
*/
tersus.Node.prototype.setValue = function setValue(path, value, operation, index)
{
	if (! operation)
	{
		internalError('Operation not specified');
	}
	if (! index)
		var index = 0;
	if (path.segments.length <= index)
	{
		internalError('Bad path in setValue');
	}
	var role = path.segments[index];
	if (path.segments.length-1 == index)
	{
		var element = this.getElement(role);
		if (!element)
			return;// The path links to a non-existant element (as of 0.9.49, this can happen because of the way the permission mechanism works)
		element.setChild(this, value, operation);
	}
	else if (path.segments.length > index)
	{
		var element = this.getElement(role);
		if (! element)
			return; // The path links to a non-existant element (as of 0.9.49, this can happen because of the way the permission mechanism works)
		var child = element.getChild(this);
		if (! child)
			child = element.createChild(this); // Creating an intermediate level
		child.setValue(path, value, operation, index+1);
	}
};


tersus.Node.prototype.getPath = function getPath()
{
	if (this.parent)
	{
		var path = this.parent.getPath();
		path.addSegment(this.element.role);
		return path;
	}
	else
		return new Path(tersus.viewPath);
};
tersus.Node.prototype.setAlwaysCreate = function setAlwaysCreate(role, alwaysCreate)
{
	this.elements[role].alwaysCreate = alwaysCreate;
};
// Performs any operation needed when this node is deleted
tersus.Node.prototype.onDelete = function onDelete() 
{
};

// Performs a 'set' operation with a leafValue
// Returns true if the operation is sucessful, false if there is no element with the given role
tersus.Node.prototype.setLeafChild = function setLeafChild(role, leafValue, operation)
{
	if (leafValue == null)
		return;
	var element = this.getElement(role);
	if (!element)
		return false;
	var value = element.createChildInstance();
	value.setLeaf(leafValue);
	element.setChild(this, value, operation);
	return true;
};

tersus.Node.prototype.getLeafChild = function getLeafChild(role)
{
	var child = this.getChild(role);
	if (child)
		return child.leafValue;
	else
		return null;
};
tersus.Node.prototype.setChild = function setChild(role, value, operation)
{
	var element = this.getElement(role);
	if (!element)
		return false;
	element.setChild(this, value, operation);
	return true;
};
tersus.Node.prototype.setLeaf = function setLeaf(value)
{
	this.leafValue = value;
};

tersus.Node.prototype.parseString = function(str)
{
	this.leafValue = str;
};
tersus.Node.prototype.copyObject = function copyObject(value)
{
	if (this.isLeaf)
		this.setLeaf(value);
	else if (this.elements)
	{
		for (var elementRole in this.elements)
		{
			var element = this.getElement(elementRole);
			if (element.isRepetitive)
			{
				var values = value[elementRole];
				if (values)
				{
					for (var i=0; i< values.length; i++)
					{
						var child = element.createChild(this);
						child.copyObject(values[i]);
					}
				}
			}
			else
			{
				var childValue = value[elementRole];
				if (childValue || childValue == 0 || childValue == '')
				{
					if (childValue ==0)
						var x=0;
					var child = element.createChild(this);
					child.copyObject(childValue);
				}
			}
		}
	}
};
tersus.Node.prototype.getTranslation = function(key)
{
	if (tersus.labelMap)
	{
		var v = tersus.labelMap[key];
		if (v && v.leafValue)
			return v.leafValue;
		else
			return v;
	}
};

tersus.Node.prototype.translate = function (s)
{
	if (! this.textTranslation)
		return s;
	return this.forceTranslate(s);
};
tersus.Node.prototype.forceTranslate = function (s)
{
	var t = this.getTranslation(s);
	if (t != undefined)
		return t;
	else
		return s;
};
tersus.Node.prototype.preloadOnStart = true;
tersus.Node.prototype.preload = function(callback)
{
	if (tersus.repository.preloaded[this.modelId])
		return true;
	var packageSet = new Set();
	if (tlog) var startTime = new Date();
	var	alreadyChecked = new Set();
	this.addPreloadPackages(packageSet,alreadyChecked);
	this.addMissingLinkTargets(packageSet,alreadyChecked);
	if (tlog)
	{
		var endTime = new Date();
		if (tlog) tlog("Package scan took "+(endTime-startTime) +" milliseconds");
	}
	var packageList = packageSet.toList();
	if (packageList.length == 0)
	{
		tersus.repository.preloaded[this.modelId] = true;
		return true;
	}
	else
	{
		if (tlog) tlog('Preloading '+this.modelId);
		packageList.sort();
		var mainPackages = [];
		var last = null;
		for (var i=0;i<packageList.length;i++)
		{
			var current = packageList[i];
			if (last && last == current.substring(0,last.length))
				continue;
			mainPackages.push(current);
			last = current;
		}
		this.pause();
		tersus.repository.loadPackages(mainPackages, callback);
		return false;
	}
};
tersus.Node.prototype.addPreloadPackages = function (packageSet, alreadyChecked)
{
	if (tersus.repository.preloaded[this.modelId])
		return;
	if (alreadyChecked.contains(this.modelId))
		return;
	alreadyChecked.add(this.modelId);
	if (this.elementList)
	{
		for (var i=0;i<this.elementList.length;i++)
		{
			var e = this.elementList[i];
			if (e.modelId)
			{
				if (!tersus.repository.checkLoaded(e.modelId))
				{
					var packageId = tersus.getPackageId(e.modelId);
					packageSet.add(packageId);
				}
				else 
				{
					if (e.isDisplayElement || e.isFlowDataElement || ! this.isDisplayNode )
					{
						var childConstructor = tersus.repository.get(e.modelId);
						var childModel = childConstructor.prototype;
//						if (childModel.preloadOnStart)
//							childModel.addPreloadPackages(packageSet, alreadyChecked);
					}
				}
			}
		}
	}
};
tersus.Node.prototype.addMissingLinkTargets = function (packageSet, alreadyChecked)
{
	if (!this.links)
		return;
	for (var i=0;i<this.links.length; i++)
	{
		var link = this.links[i];
		var currentModel = this;
		var segments = link.target.segments;
		for (var j=0; j< segments.length; j++)
		{
			var element = currentModel.getElement(segments[j]);
			if (!element)
				continue; // This can happen if there is no permission
			var id = element.modelId;
			if (!id)
				continue; 
			if (! tersus.repository.checkLoaded(id))
			{
				if (tlog) tlog('Need to preload '+id);
				var packageId = tersus.getPackageId(id);
				packageSet.add(packageId);
				break;
			}
			else
			{
				currentModel = tersus.repository.get(id).prototype;	
				if (currentModel.preloadOnStart)
					currentModel.addPreloadPackages(packageSet,alreadyChecked);
			}
		}
	}
	return true;
};	
tersus.Node.prototype.destroyHierarchy = function()
{
	this.destroyChildren();
        if (this.destroy)
                this.destroy();
};
tersus.Node.prototype.destroyChildren = function()
{
        tersus.Node.subFlowIterator(this,function(node) {node.destroyHierarchy()});
};

tersus.Node.childIterator = function (node,iteration)
{
	if (node && node.elements)
	{
		for (var i=0; i<node.elementList.length; i++)
		{
			var e = node.elementList[i];
			if ( e.isRepetitive)
			{
				var children  = e.getChildren(node);
				for (var j=0;j<children.length;j++)
				{
					var stop = iteration(children[j]);
					if (stop == true)
						return;
				}
			}
			else
			{
				var child = e.getChild(node);
				if (child)
				{
					var stop = iteration(child);
					if (stop == true)
						return;
				}
			}
		}
	}
	
};
tersus.Node.subFlowIterator = function (node,iteration)
{
	if (node && node.subFlowList)
	{
		for (var i=0; i<node.subFlowList.length; i++)
		{
			var e = node.subFlowList[i];
			if ( e.isRepetitive)
			{
				var children  = e.getChildren(node);
				for (var j=0;j<children.length;j++)
				{
					var stop = iteration(children[j]);
					if (stop == true)
						return;
				}
			}
			else
			{
				var child = e.getChild(node);
				if (child)
				{
					var stop = iteration(child);
					if (stop == true)
						return;
				}
			}
		}
	}
	
};

tersus.compare = function (obj1, obj2)
{
	if (obj1 == null)
		return obj2 == null ? 0 : -1;
	return obj1.compare(obj2);
};
tersus.Node.prototype.compare = function (obj2)
{
	if (obj2 == null)
		return 1;
	var obj1 = this;
    if (!this.isInstance(obj2))
        throw "compare: Can't compare "+obj1.modelId + " and "+ obj2.modelId;
        
    if (this.isLeaf)
    	return obj1.leafValue > obj2.leafValue ? 1 : obj1.leafValue == obj2.leafValue ? 0 : -1;

    for (var i = 0; i < this.elementList.length; i++)
    {
        var e = this.elementList[i];
        if (e.isRepetitive)
        {
            var list1 = e.getChildren(obj1);
            var list2 = e.getChildren(obj2);
            var commonSize = Math.min(list1.length, list2.length);
            for (var j=0; j<commonSize;j++)
            {
                var item1 = list1[j]
                var item2 = list2[j];
                
               	var comparison = tersus.compare(item1, item2);
                if (comparison != 0)
                    return comparison;
            }
            return list1.length - list2.length;
        }
        else
        {
            var v1 = e.getChild(obj1);
            var v2 = e.getChild(obj2);
            var comparison = tersus.compare(v1, v2);
            if (comparison != 0)
                return comparison;
        }
    }
    return 0;
};

// file: 013-Element.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Element() {};
/**
	createChildInstance - creates an instance of the model refered by this element.
	 
	Note: the instance is not added to the parent node (which is not specified).

	
*/
Element.prototype.dependencyRank = -1;
Element.prototype.isRemovable = true;
Element.prototype.alwaysCreate = true;
Element.prototype.getChildConstructor = function (explicitModelId)
{
	var modelId = explicitModelId ? explicitModelId : this.modelId;
	var childConstructor;
	if (modelId)
		childConstructor = tersus.repository.get(modelId);
	else
		childConstructor = DataValue;
	if (! childConstructor)
	{
		var message ="'"+ this.modelId + "' not found in constructor repository.\n";
		message += "Existing consructors:\n";
		for (var modelId in tersus.repository.constructors)
		{
			message += "'" + modelId + "'\n";
		}
		tersus.error(message);
	}
	return childConstructor;
};
Element.prototype.createChildInstance = function createChildInstance(modelId)
{
	var childConstructor = this.getChildConstructor(modelId);
	var childInstance = new childConstructor();
	childInstance.prototype = childConstructor.prototype;
	return childInstance;
}

/*
	Creates a child instance, and stores it as a child.
	(This implementation is used by DataElements and Exits. Display Elements have a different implementation).
*/
Element.prototype.createChild = function createChild(parent)
{
	var childInstance = this.createChildInstance();
	this.addChild(parent, childInstance);
	return childInstance;
};	
Element.prototype.addChild = function addChild(parent, child)
{
	if (! parent.children)
		parent.children = {};
	if (this.isRepetitive)
	{
		if (! parent.children[this.role])
			parent.children[this.role] = [];
		parent.children[this.role].push(child);
	}
	else
	{
		parent.children[this.role] = child;
	}
};
Element.prototype.isEmpty = function isEmpty(parent)
{
	if (this.isRepetitive)
	{
		var children = this.getChildren(parent);
		return (! children || children.length ==0);
	}
	else
	{
		var child = this.getChild(parent);
		return (! child);
	}
};
Element.prototype.getChild = function getChild(parent)
{
	if ( this.isRepetitive)
		internalError("getChild(parent) called on repetitive element");
	var child = null;
	if (parent.children)
		child = parent.children[this.role];
	return child;
};

/*
	returns a list of children (child instances) of a given parent.
	If there are no such children, an empty list is returned.
	IMPORTANT:  The caller must not modify the returned list 
*/
Element.prototype.getChildren = function getChildren(parent)
{
	if ( ! this.isRepetitive)
		internalError("getChildren(parent) called on non repetitive element");
	var children = null;
	if (parent.children)
		children = parent.children[this.role];
	if (! children)
		children = [];
	return children;
};

Element.prototype.removeChildren = function removeChildren(parent)
{
	makeAssertion( this.isRepetitive);
	var children = this.getChildren(parent);
	for (var i=0; i< children.length; i++)
	{
		var child = children[i];
		if (child && child.parent == parent)
			child.onDelete();
	}
	if (parent.children)
		parent.children[this.role] = [];
};

Element.prototype.removeNumberedChild = function removeNumberedChild(parent, index)
{
	makeAssertion( this.isRepetitive);
	var children = this.getChildren(parent);
	children.splice(index - 1, 1);
};

Element.prototype.setChildren = function setChildren(parent, values, operation)
{
	if ( ! this.isRepetitive)
		internalError("setChildren(parent, values, operation) called on non repetitive element");
	if (operation ==  Operation.REPLACE)
		this.removeChildren(parent);
	for (var i=0; i<values.length; i++)
	{
		var value = values[i];
		this.setChild(parent, value, Operation.ADD);
	}
};


Element.prototype.addAChild = function addAChild(parent,value)
{
	if (! parent.children)
		parent.children = {};
	makeAssertion( this.isRepetitive);
	if (! parent.children[this.role])
		parent.children[this.role] = [];
	parent.children[this.role].push(value);
};
Element.prototype.replaceChild = function replaceChild(parent, value)
{
	makeAssertion( ! this.isRepetitive);
	if (! parent.children)
		parent.children = {};
	if (this.modelId && tersus.repository.get(this.modelId).prototype.isNothing && ! value.isNothing)
		this.createChild(parent); // Creates a new instance of Nothing
	else
		parent.children[this.role]=value;	
}
Element.prototype.removeChild = function removeChildren(parent)
{
	makeAssertion( ! this.isRepetitive);
	if (! parent.children)
		return;
	var child = this.getChild(parent);
	if (child != null)
	{
		if (parent.hasElementChangeHandler)
		{
			var oldValue = child.isDataNode ? child.value : child;
			parent.handleElementChange(this.role, oldValue, null);
		}
		delete parent.children[this.role];
		if (child && child.parent == parent)
			child.onDelete();
	}
};



Element.prototype.setChild = function setChild(parent, value, operation)
{
	if (! parent.children)
		parent.children = {};
	if (this.isRepetitive)
	{
		if (operation == Operation.ADD)
			this.addAChild(parent, value);
		else if (operation == Operation.REMOVE)
			this.removeChildren(parent);
		else
			internalError('Operation '+operation+' not supported for repetitive elements');
		
	}
	else
	{
		if (operation == Operation.REPLACE)
			this.replaceChild(parent, value,true);
		else if (operation == Operation.REMOVE)
			this.removeChild(parent);
		else
			
			internalError('Operation '+operation+' not supported for non-repetitive elements');
	}
};
Element.prototype.moveChild = function (parent, fromIndex, toIndex)
{
	var children = this.getChildren(parent);
	var child = children.splice(fromIndex, 1)[0];
	children.splice(toIndex,0,child);
	return child;
};
Element.prototype.getValueNode = function(value)
{
	if (value == null)
		return null;
	var valueNode;
	if (value.isNode)
		valueNode=value;
	else
	{
		valueNode=this.createChildInstance();
		valueNode.setLeaf(value);
	}
	return valueNode;
};// file: 015-DataNode.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
//DataNode (of a Flow)
//A DataNode holds a reference to some value, and participates in the flow
// (it can be invoked).
function DataNode() {};
DataNode.prototype = new tersus.Node();
DataNode.prototype.constructor = DataNode;
DataNode.prototype.isDataNode = true;
DataNode.prototype.invoke = function()
{
	if (window.trace)
		window.trace.activated(this);
	this.invokeLinks();
}
DataNode.prototype.toString = function toString()
{
	return this.leafValue + '['+this.modelId +']';
};
DataNode.prototype.copyObject = function copyObject(obj)
{
	this.value.copyObject(obj);
};
DataNode.prototype._getValues = function _getValues(path, index, values)
{
	if (index == path.segments.length)
		values.push(this.value);
	else
		return this.value._getValues(path, index, values);
}
DataNode.prototype.setValue = function setValue(path, value, operation, index)
{
	this.value.setValue(path, value, operation, index);
	//TODO we probably need a special case for leaf data nodes
};
DataNode.prototype.scanChildren = function(filter, alreadyScanned, outputList)
{
// Do nothing - we don't want to scan variables 
//	this.value.scanChildren(filter, alreadyScanned, outputList);
};
DataNode.prototype.getModelId = function()
{
	return this.value.getModelId();
};// file: 020-FlowNode.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
//FlowNode - an instance of FlowModel (implements composite behavior)

function FlowNode() {};
FlowNode.prototype = new tersus.Node();
FlowNode.prototype.constructor = FlowNode;
FlowNode.prototype.errorExit = null;
FlowNode.prototype.addSubFlow = function addSubFlow(role, modelId)
{
	// Adds a representation of the sub-flow both to a list of sub-flows
	// and to a map of sub-flows
	var subFlow = new SubFlow(role, modelId);
	this.registerSubFlow(subFlow);
	return this.addElement(subFlow);
}
FlowNode.prototype.registerSubFlow = function registerSubFlow(subFlow)
{
	if (! this.subFlowMap)
		this.subFlowMap = {};
	this.subFlowMap[subFlow.role] = subFlow;
	if (! this.subFlowList)
		this.subFlowList = [];
	this.subFlowList.push(subFlow);
};
/**
	Returns the sole DisplayElement of this Flow.
	Returns null if there are 0 or multiple DisplayElements.
*/
FlowNode.prototype.addAncestorReference = function addAncestorReference(role, modelId)
{
	var reference = new FlowDataElement(role, modelId);
	reference.isAncestorReference = true;
	if (! this.ancestorReferenceList)
		this.ancestorReferenceList = [];
	
	this.ancestorReferenceList.push(reference);
	return this.addElement(reference);
};

FlowNode.prototype.addIntermediateDataElement = function addIntermediateDataElement(role, modelId)
{
	if (this.getGetter(role))
	{
		var element = new MethodElement(this, role, modelId);
		element.modelId = modelId;
	}
	else
		var element = new FlowDataElement(role, modelId);
	return this.addElement(element);
};

FlowNode.prototype.addConstant = function addConstant(role, modelId)
{
	var constantReference = new FlowDataElement(role, modelId);
	constantReference.type = 'Constant';
	if (!this.constantList)
		this.constantList = [];
	this.constantList.push(constantReference);
	return this.addElement(constantReference);
};

FlowNode.prototype.addLink = function addLink(role, source, target)
{
	if (!this.elements)
		return null; // If there are no elements, there is nothing to link. This can happen because of permission issues  
		
	var link = new Link(role, source,target);
	var sourceElement;
	if (!this.links)
		this.links = [];
	this.links.push(link);
	var sourceElement = this.elements[link.source.segments[0]];
	if (! sourceElement)
		return;
	if (! sourceElement.outgoingLinks)
		sourceElement.outgoingLinks = [];
	sourceElement.outgoingLinks.push(link);
	return link;
};

FlowNode.prototype.getLink = function getLink(role)
{
	if (! this.links)
		return null;
	for (var i=0; i<this.links.length; i++)
	{
		var link = this.links[i];
		if (link.role == role)
			return link;
	}
	return null;
};

/*
	Called before a FlowNode is started
*/
FlowNode.prototype.started = function started()
{
	this.setStatus(FlowStatus.STARTED);
	if (window.trace)
		window.trace.started(this);
};
FlowNode.prototype.resumed = function ()
{
	this.setStatus(FlowStatus.RESUMED);
};

FlowNode.prototype.finished = function ()
{
	/* Because actions can contain pop-up windows, finish() may be called more than once,
	 * but we don't want the <Done> exit to be charged again, or the trace to contain an additional event*/
	
	if (! this.alreadyFinished)
	{
		if (!this.error)
		{
			var doneExit = this.getElement(BuiltinNames.DONE_EXIT_NAME);
			if (doneExit)
				this.chargeEmptyExit(doneExit);
		}
		if (window.trace)
			window.trace.finished(this);
		if (this.whenDone)
			this.whenDone();
	}
	this.setStatus(FlowStatus.DONE);
	this.alreadyFinished = true;
}
FlowNode.prototype.waiting = function ()
{
	this.setStatus(FlowStatus.WAITING_FOR_INPUT);
	if (window.trace)
		window.trace.waiting(this);
}
FlowNode.prototype.continueExecution = function()
{
		var executionContext = this.getExecutionContext();
		this.resumeAfterPause();
		executionContext.doResume();
		if (window.trace)
			window.trace.flush();
		if (this == window.rootDisplayNode)
			tersus.progress.clear();
		$(window).trigger('ter-process-end');
};
FlowNode.prototype.doStart = function()
{
	var node = this;
	var	resume = function()
	{
		if (tlog) tlog(node.modelName,'resumed after preload', node.modelId);
		node.resumeAfterPause();
		node.setStatus(FlowStatus.READY_TO_START);
		var executionContext = node.getExecutionContext();
		executionContext.doResume();
		if (window.trace)
			window.trace.flush();
	};
	if (! this.preloadOnStart || this.preload(resume))
	{
		if (tlog) tlog(node.modelName,'started (already loaded)', node.modelId);
	
		var isAtomic = this.isAction && !this.subFlowMap;
		var isDisplay = this.isDisplayNode;
		var isRootAction = this.isAction && (!this.parent || !this.parent.isAction); //Can be optimized - we can know this in advance
		if (isAtomic || isDisplay || this.errorExit || (isRootAction&&!tersus.failOnScriptErrors))
		{
			try
			{
				this.started();
				this.start();
				if (tlog) tlog(node.modelName,'started done', node.modelId);
			}
			catch (e)
			{
				if (this.errorExit || isRootAction || isDisplay)
					this.handleException(e);
				else
					throw this.convertException(e);
			}
		}
		else
		{
			this.started();
			this.start();
			if (tlog) tlog(node.modelName,'started done', node.modelId);
		}
		if (this.status != FlowStatus.PAUSED)
		{
			if (this.isAction)
				this.finished();
			else
				this.waiting();
		}
	}
	else
		if (tlog) tlog(node.modelName,'paused(waiting for models to load)', node.modelId);
};
FlowNode.prototype.convertException = function(e)
{
	if (typeof e === 'object' && 'getMessage' in e) // Java exception
	{
		e = {message: e.getClass().getName() + ' - ' + e.getMessage()};
	}
	else if (typeof e != 'object')
		e = {message:''+e};
	if (!e.errorPath)
		e.errorPath = this.getPath().str;
	return e;
}
FlowNode.prototype.handleException = function (e)
{
	e = this.convertException(e);
	var msg = e.message?e.message:''+e;
	
	if (this.errorExit)
	{
		var error = this.errorExit.createChildInstance();
		error.setLeafChild('Message', msg, Operation.REPLACE);
		error.setLeafChild('Location', e.errorPath, Operation.REPLACE);
		this.chargeExit(this.errorExit,error);
		this.error = error;
	}
	else
	{
		//TODO  - Throw if below event level, Show message box if at event level
		var m ="An error has occurred: "+e.message;
		if (e.details)
			m+='Details:'+e.details;
		if (e.errorPath)
			m+='\nPath:'+e.errorPath;
		
		if (!tersus.checkWindow(this.currentWindow))
		{
			m += "\n\nThe problem may have been caused by closing a pop-up window while it was being populated."
		}
		tersus.error(m);
	}
};
FlowNode.prototype.doResume = function doResume(message)
{
	this.resumed();
	var isAtomic = this.isAction && !this.subFlowMap;
	var isDisplay = this.isDisplayNode;
	var isRootAction = this.isAction && (!this.parent || !this.parent.isAction); //Can be optimized - we can know this in advance
	if (isAtomic || isDisplay || this.errorExit || (isRootAction&&!tersus.failOnScriptErrors))
	{
		try
		{
			this.resume();
		}
		catch (e)
		{
			if (this.errorExit || isRootAction || isDisplay)
				this.handleException(e);
			else
				throw this.convertException(e);
		}
	}
	else
	{
		this.resume();
	}
	if (this.status != FlowStatus.PAUSED)
	{
		if (this.isAction)
			this.finished();
		else
			this.waiting();
	}
};

FlowNode.prototype.start = function()
{
	var element; //represents a model element
	var child; // represents a child instance (sub-flow instance)
	var role;
	// add trigger values to the ready queue
	if (this.triggers)
		for (var i = 0; i < this.triggers.length; i++)
		{
			var trigger = this.triggers[i];
			
			if (trigger.isRepetitive)
			{
				var values = trigger.getChildren(this);
				trigger.removeChildren(this);
				for (var j=0; j< values.length; j++)
				{
					var value = values[j];
					this.createReference(trigger, value);
				}
			}		
			else
			{
				var value = this.getChild(trigger.role);
				if (value)
					this.createReference(trigger, value);
			}
		}
			
			
		//create instances for constants
		if (this.constantList)
		{
			for (var i=0; i< this.constantList.length; i++)
			{
				element = this.constantList[i];
				if (element.isSelfReady)
					child = element.createChild(this);
			}
		}
		
		if (this.ancestorReferenceList)
		{
			var ancestor;
			var ancestorData;
			var referenceNode;
			for (var i=0; i< this.ancestorReferenceList.length; i++)
			{
				element = this.ancestorReferenceList[i];
				role = element.role;
				ancestor = this.parent;
				var child = null;
				while (ancestor)
				{
					var parentElement = ancestor.getElement(role);
					if ( parentElement && parentElement.modelId == element.modelId) // In the direct ancestor we will find the referring element
					{
						var parentObj = parentElement.getChild(ancestor);
						if (parentObj)
							child = this.createReference(element, parentObj);
						ancestor = null;
					}
					else if (ancestor.modelId == element.modelId)
					{
						child = this.createReference(element, ancestor);
						ancestor = null; //No need to continue searching
					}
					else
					{
						ancestor = ancestor.parent;
					}
				}
				if (! child)
					if (tlog)
						tlog("Missing ancestor reference for "+role + " in "+ this.modelId);

			}
		}
		this.createSubFlows();
		this.runLoop();
	};
FlowNode.prototype.createSubFlows = function()
{
	//initializes sub flows
	if (this.subFlowList)
	{
		for (var i=0;i<this.subFlowList.length; i++)
		{
			var subFlow = this.subFlowList[i];
			subFlow.init(this);
		}
	}
};


FlowNode.prototype.createReference = function createReference(element, value)
{
	var child = new DataNode();
	child.type = element.type;
	child.mainWindow = this.mainWindow;
	child.currentWindow = this.currentWindow;
	child.parent = this;
	child.element = element;
	if (value.isDataNode)
		child.value = value.value;
	else
		child.value = value;
	element.addChild(this, child);
	if (child.isReady())
		this.addReadyElement(child);
	return child;
};




FlowNode.prototype.readyToResume = function readyToResume()
{
		this.setStatus(FlowStatus.READY_TO_RESUME);
		var parent = this.getParentContext();
		if (parent == null)
			return;
		parent.addReadyElement(this);
		if (parent.status == FlowStatus.WAITING_FOR_INPUT || parent.status == FlowStatus.DONE)
			parent.readyToResume();
};

/*
	Marks this FlowNode and its ancestors as ready to resume from the point of pause:
	(The paused processes are added to the beginning of the run-queue)
	
*/
FlowNode.prototype.resumeAfterPause = function resumeAfterPause()
{
	this.setStatus(FlowStatus.READY_TO_RESUME);
	var parent = this.getParentContext();
	if (parent == null)
		return;
	if ( ! parent.readyElements)
		parent.readyElements = [];
	parent.readyElements.splice(0,0, this); // insert at the beginning
	parent.resumeAfterPause();
}	

FlowNode.prototype.pause = function pause()
{
	this.setStatus(FlowStatus.PAUSED);
	if (window.trace)
		window.trace.paused(this);
	var parent = this.getParentContext();
	if (parent)
		parent.pause();
};

FlowNode.prototype.readyToStart = function readyToStart()
{
		this.setStatus(FlowStatus.READY_TO_START);
		var parent = this.getParentContext();
		if (parent == null)
			return;
		parent.addReadyElement(this);
		if (parent.status == FlowStatus.WAITING_FOR_INPUT || parent.status == FlowStatus.DONE)
			parent.readyToResume();
};
FlowNode.prototype.setInitialStatus = function setInitialStatus()
{
	this.setStatus(FlowStatus.NOT_READY_TO_START);
	this.checkReady();
};

FlowNode.prototype.checkReady = function checkReady()
{
	if (this.status == FlowStatus.NOT_READY_TO_START && this.isReady())
	{
		this.readyToStart();
	}
		
};

/*
	Adds 'child' to the run-queue, while preserving the dependency rank order
	('child' is added at the largest index that will result in a non-decreasing 
	dependency rank order)
*/
FlowNode.prototype.addReadyElement = function addReadyElement(child)
{
	
	
	
	var childRank = child.element.dependencyRank;
	
	
	if (!this.readyElements)
		this.readyElements = [];
	var targetIndex = this.readyElements.length;
	if( child.element === this.initElement) // <Init>
	{
		var parent = this;
		// After <Init> is done, we need to create child elements
		child.whenDone = function()
		{
			parent.createChildren();
		};
		// <Init> must be executed first
		targetIndex = 0;
	}
	else
	{
		while (targetIndex > 0)
		{
			var node = this.readyElements[targetIndex-1];
			var nodeRank = node.element.dependencyRank;
			if (nodeRank <= childRank)
			{
				// child should be inserted at targetIndex
				break;
			}
			--targetIndex;
		}
	}
	// Calling splice adds 'child' at targetIndex
	this.readyElements.splice(targetIndex, 0, child);
	
};

FlowNode.prototype.setStatus = function setStatus(status)
{
	if (tlog)
		tlog('STATUS ['+this.getNodeId()+' - '+this.modelName+']: '+this.status+' => '+status);
	this.status = status;
};


FlowNode.prototype.resume = function resume(message)
{

	this.runLoop();

	if (this == window.rootDisplayNode && 	this.status == FlowStatus.WAITING_FOR_INPUT)
		if (tersus.async.nowait())
			tersus.progress.clear();
};
FlowNode.prototype.isReady = function isReady()
{
	if (this.triggers)
	{
		for (var i=0; i<this.triggers.length; i++)
		{
			var trigger = this.triggers[i];
			if (trigger.mandatory)
			{
				if (trigger.isEmpty(this))
				{
					return false;
				}
			}
		}
	}
	return true;
};
FlowNode.prototype.runLoop = function run()
{
	var child;
//		createParentElementReferences(context, flow);
	while (this.status != FlowStatus.PAUSED && this.hasReadyElements())
	{
		child = this.readyElements.shift();
		child.invoke(); //Start or resume
	}

};

FlowNode.prototype.hasReadyElements = function hasReadyElements()
{
	return (this.readyElements && this.readyElements.length > 0);
};

FlowNode.prototype.invoke = function invoke ()
{
	switch(this.status)
	{
		case FlowStatus.READY_TO_START:
			this.doStart();
			break;
		case FlowStatus.READY_TO_RESUME:
			this.doResume();
			break;
		default:
			internalError('Child with invalid status invoked '+ this.status);
	}
	if (this.status != FlowStatus.PAUSED)
		this.invokeLinks();
	//TODO - remove exit values / input values
};

FlowNode.prototype.chargeEmptyExit = function chargeEmptyExit(exit)
{
	if (! exit.isExit)
		exit = this.getExit(exit); // exit given by name
	if (exit)
	{
		exit.createChild(this);
		if (window.trace)
		{
			window.trace.charged(this, exit);
		};
	}
};

FlowNode.prototype.getExit = function (name)
{
	if (!this.exits)
		return null;
	var exit = this.getElement(name);
	if (exit && exit.isExit)
		return exit;
	else if (name.search(/^<.*>$/)>=0) // Distinguished name - allow perfixes
	{
		var l = name.length;
		for (var i=0; i<this.exits.length; i++)
		{
			var current = this.exits[i];
			var r=current.role;
			if (r.substr(-l) == name && r.indexOf('<')==r.length-l)
			{
				if (exit)
					return null; //ambigous
				exit = current;
			}
		}
	}
	return exit;
};

FlowNode.prototype.getTrigger = function (name)
{
	if (!this.triggers)
		return null;
	var trigger = this.getElement(name);
	if (trigger)
		return trigger;
	else if (name.search(/^<.*>$/)>=0) // Distinguished name - allow perfixes
	{
		var l = name.length;
		for (var i=0; i<this.triggers.length; i++)
		{
			var current = this.triggers[i];
			var r=current.role;
			if (r.substr(-l) == name && r.indexOf('<')==r.length-l)
			{
				if (trigger)
					return null; //ambigous
				trigger = current;
			}
		}
	}
	return trigger;
};
FlowNode.prototype.getTriggerLeafValue = function(role)
{
	var trigger = this.getTrigger(role);
	var value = null;
	if (trigger)
	{
		value = trigger.getChild(this);
		if (value != null)
			value = value.leafValue;
	}
	return value;
};
FlowNode.prototype.chargeExit = function (exit, value)
{
	if (! exit.isExit)
		exit = this.getExit(exit);
	if (exit)
	{
		exit.replaceChild(this,value);
		if (window.trace)
			window.trace.charged(this, exit, value);
	}
};
FlowNode.prototype.accumulateExitValue = function(exit, value)
{
	if (! exit.isExit)
		exit = this.getExit(exit);
	if (exit)
	{
		exit.addAChild(this,value);
		if (window.trace)
			window.trace.charged(this, exit, value);
	}
};

FlowNode.prototype.chargeLeafExit = function chargeLeafExit(exit, leafValue)
{

	if (! exit.isExit)
		exit = this.getExit(exit); // exit given by name
	if (exit && (leafValue != null) )
	{
		var child = exit.createChild(this);
		child.setLeaf(leafValue);
		if (window.trace)
			window.trace.charged(this, exit, child);
		
	}
};

FlowNode.prototype.getExecutionContext = function getExecutionContext()
{
	if (this.getParentContext())
		return this.getParentContext().getExecutionContext();
	else
		return this;
};

FlowNode.prototype.getParentContext = function getParentContext()
{
	return this.parent;
};

FlowNode.prototype.addTrigger = function addTrigger(role, modelId)
{
	if (! this.triggers)
		this.triggers = [];
	var trigger = new Trigger(role, modelId);
	this.triggers.push( trigger);
	return this.addElement(trigger);
};

FlowNode.prototype.addExit = function addExit(role, modelId)
{
	if (! this.exits)
		this.exits = [];
	var exit = new Exit(role, modelId);
	this.exits.push( exit);
	return this.addElement(exit);
};
FlowNode.prototype.addErrorExit = function addErrorExit(role, modelId)
{
	var e = this.addExit(role,modelId);
	if (this.errorExit)
		modelError('Multiple Error exits',this);
	this.errorExit = e;
	e.isError=true;
	return e;
};

FlowNode.prototype.getTriggerValues = function getTriggerValues(excludedTriggerName)
{
	var values = [];
	for (var i=0; i<this.triggers.length; i++)
	{
		var trigger = this.triggers[i];
		if (trigger.modelId != BuiltinModels.NOTHING_ID && trigger.role != excludedTriggerName )
		{
			if (trigger.isRepetitive)
			{
				values = values.concat(trigger.getChildren(this));
			}
			else
			{
				var value = trigger.getChild(this);
				if (value != null && value != undefined)
					values.push(value);	
			}
			
		}
	}
	return values;
};

FlowNode.prototype.getEventHandler = function(eventType)
{
	if (! this.subFlowMap)
		return null;
	var handler = this.subFlowMap[eventType.elementName];
	if (handler == undefined)
		return null;
	return handler;
};
FlowNode.prototype.hasEventHandler = function(eventType)
{
	return (this.getEventHandler(eventType) != null);
};

FlowNode.prototype.queueEvent = function (eventType, params, domEvent, callback)
{
	var eventAttrbutes = domEvent ? getEventAttributes(domEvent) : tersus.currentEventAttributes;
	var description = null;
	if (tlog)
	{
		description = eventType.methodName + ' ['+ this.getPath().str+']';
		tlog('queuing event:'+eventType.jsName ? eventType.jsName: eventType.elementName);
		if (eventAttrbutes)
			tlog('keyCode='+eventAttrbutes.keyCode+' type='+eventAttrbutes.type);
	}
	var handler = this.getEventHandler(eventType);
	if (handler)
	{
		var c = handler.getChildConstructor();
		
		if (c && c.prototype.runImmediately)
		{
			// Run a "background process" in current thread
			var process = new BackgroundProcess(this, handler, params);
			process.whenDone(callback);
			process.start(); 
		}
		else
			queueEvent(window.engine, window.engine.handleEvent, [this,eventType,  params, eventAttrbutes], new Date(), description, callback);
	}
	else if (callback)
   		tersus.async.exec(callback,true);

};

FlowNode.map = {};
FlowNode.prototype.getNodeId = function()
{
        if (!this.nodeId)
        {
                this.nodeId = window.nextNodeId ? window.nextNodeId : 1;
                FlowNode.map[this.nodeId]=this;
                window.nextNodeId = this.nodeId + 1;
        }
        return this.nodeId;
};
FlowNode.prototype.destroy = function()
{
	if (this.nodeId)
		delete FlowNode.map[this.nodeId]
}
tersus.findNode = function findNode(id)
{
	return FlowNode.map[id];
}
// file: 021-ActionNode.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function ActionNode() {};
ActionNode.prototype = new FlowNode();
ActionNode.prototype.constructor = ActionNode;
ActionNode.prototype.isAction = true;
ActionNode.prototype.type="Action";
ActionNode.prototype.runImmediately=false;// file: 024-BackgroundProcess.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 

function BackgroundProcess (parent, subflow, params)
{
	if (parent && subflow)
	{
		this.parent = parent;
		this.process = subflow.createChildInstance();
		this.context = new BackgroundProcessContext(parent);
		this.process.mainWindow = parent.mainWindow;
		this.process.currentWindow = parent.currentWindow;
		this.process.parent = this.context;
		this.process.element = subflow;
		if (params)
		{
			for (var key in params)
			{
				this.process.setChild(key, params[key], Operation.REPLACE);
			} 
		}
	}
};

BackgroundProcess.prototype.setTriggerValue = function setTriggerValue(role, value)
{
	this.process.setChild(role,value, Operation.REPLACE);
};
BackgroundProcess.prototype.setTriggerLeafValue = function setTriggerValue(role, value)
{
	this.process.setLeafChild(role,value, Operation.REPLACE);
};


BackgroundProcess.prototype.getExitValues = function getExitValues(role)
{
	return this.process.getChildren(role);
};

BackgroundProcess.prototype.start = function()
{
	if (this.process.isReady())
		this.process.doStart();
	if (window.trace)
		window.trace.flush();
	
};
BackgroundProcess.prototype.whenDone = function whenDone(callback)
{
	this.process.whenDone = callback;
};
function BackgroundProcessContext(context)
{
	this.context = context;
	this.parent = context;
	this.setStatus(FlowStatus.WAITING_FOR_INPUT);
};
BackgroundProcessContext.prototype = new FlowNode();
BackgroundProcessContext.prototype.constructor = BackgroundProcessContext;
BackgroundProcessContext.prototype.getPath = function getPath()
{
	return this.context.getPath();
};
BackgroundProcessContext.prototype.getParentContext = function getParentContext()
{
	return null;
};
BackgroundProcessContext.prototype.isBackground = true;
// file: 025-DisplayNode.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at http://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
// DisplayNode (a FlowNode that displays soemthing)
function DisplayNode()
{
};


DisplayNode.prototype = new FlowNode();
DisplayNode.prototype.visible = true;
DisplayNode.prototype.constructor = DisplayNode;
DisplayNode.prototype.type='Display';
DisplayNode.prototype.isDisplayNode=true;
  
DisplayNode.prototype.explicitReadOnly = false;
DisplayNode.prototype.VALIDATION_MESSAGE = '<Validation Message>';

DisplayNode.prototype.setViewNode = function DisplayNode_setViewNode(viewNode)
{
	this.viewNode = viewNode;
};
DisplayNode.prototype.getValidationContext = function DisplayNode_getValidationContext()
{
	if (this.parent && this.parent.getValidationContext)
		return this.parent.getValidationContext();
	else
		return this;
};
DisplayNode.findNodeForDocumentNode = function(viewNode)
{
	var nodeId = null;
	while (viewNode && viewNode.getAttribute && !nodeId)
	{
		nodeId = viewNode.getAttribute("lid"); // logic node id
		if (! nodeId)
			nodeId = viewNode.id;
		if (nodeId)
			return tersus.findNode(nodeId);
		viewNode = viewNode.parentNode;
	}
	return null; 
};
DisplayNode.prototype.escapeHTML = function(s)
{
	if (this.sharedProperties.allowHTMLTags != true && ! this.allowHTMLTags)
		return tersus.escapeHTML(s);
	else
		return s;
};
DisplayNode.prototype._onFocus = function()
{
	var parent = this.parent;
	while (parent)
	{
		if (parent.onChildFocus)
			parent.onChildFocus();
		parent = parent.parent;
	}
};

tersus.createEventHandler = function(eventType)
{
	// Default event handler method on DisplayNode
	if (eventType.methodName)
	{
		DisplayNode.prototype[eventType.methodName] = function(domNode, callback)
		{
			if (window.trace)
			{
				window.trace.event(this, eventType);
			}
			var baseMethodName = '_'+eventType.methodName;
			if (this[baseMethodName])
				this[baseMethodName]();
			this.queueEvent(eventType, null, null, callback);
		
		}
	}
};

for (var key in Events)
{
	tersus.createEventHandler(Events[key]);
};

tersus.createEventDispatcher = function(eventType, w)
{
	// Event dispatching global function
	if (eventType.jsName && eventType.methodName)
	{
		w.tersus_dispatchers[eventType.jsName] = function(e)
		{
			var eName = eventType.name;
			var $this = $(this);
			if ($this.hasClass(eName))
			{
				if (tlog) tlog ('Duplicate ' + eName + ' ignored  on ' + this.tagName);
				return;
			}
			$this.addClass(eName);
			var clear = function()
			{
				$this.removeClass(eName);
			};
			var ev = e;
			if (ev == null)
			{
				var w = getParentWindow(this);
				ev = w.event;
			}
			if (tersus.disabled == true || tersus.disabled && tersus.disabled[eventType.jsName] == true)
				return true;
			if (ev)
				tersus.currentEventAttributes = getEventAttributes(ev);
			tersus.currentEvent = ev;
			try
			{
				var node = DisplayNode.findNodeForDocumentNode(this);
				if (tlog)
				{
					var tagDesc = this.tagName;
					if (this.className)
						tagDesc+='.'+this.className;
					tlog('Captured event: '+ eventType.methodName + ' on '+tagDesc+' ['+ node.getPath().str+']');
				}
				
				if (node)
				{
					return node[eventType.methodName].call(node, this, clear);
				}
				else
					return true;
			}
			finally
			{
				tersus.currentEvent = null;
			}
		};
	}
};


tersus.createEventDispatchers = function t_addEvents(w)
{
	w.tersus_dispatchers = {};
	for (var key in Events)
	{
		tersus.createEventDispatcher(Events[key], w);
	}
};

tersus.createEventDispatchers(window);
// New implementation based on jQuery and jQuery Mobile
DisplayNode.prototype.registerEventHandler = function(eventType,domNode)
{
  	if (!domNode)
    {
            if (this.eventTargets)
                    domNode = this.eventTargets[eventType.jsName];
            if (!domNode)
                    domNode = this.viewNode;
    }
    domNode.setAttribute('lid',this.getNodeId());
    if (!eventType.jsName)
    {
            internalError('No event dispatcher defined for '+eventType.elementName);
            return;
    }
    
    var dispatcher = this.currentWindow.tersus_dispatchers[eventType.jsName];
    if (dispatcher)
    {
    	var jqName = eventType.jsName.substr(2);
    	
    	var doc = domNode.ownerDocument;
		var w = doc.defaultView || doc.parentWindow;
    	if (w == window && jqName == 'click' && $.event.special.vclick) // Virtual events don't work in pop-up
    		jqName = 'vclick';
        $(domNode).unbind(jqName,dispatcher);
        $(domNode).bind(jqName,dispatcher);
    }
    else
            internalError('No event dispatcher defined for '+eventType.jsName);
    
};

/* Old implementation */
/*
DisplayNode.prototype.registerEventHandler = function(eventType, domNode)
{
	if (!domNode)
	{
		if (this.eventTargets)
			domNode = this.eventTargets[eventType.jsName];
		if (!domNode)
			domNode = this.viewNode;
	 }
	domNode.setAttribute('lid',this.getNodeId());
	if (!eventType.jsName)
	{
		internalError('No event dispatcher defined for '+eventType.elementName);
		return;
	}
	
	var dispatcher = this.currentWindow.tersus_dispatchers[eventType.jsName];
	if (dispatcher)
		domNode[eventType.jsName] = dispatcher;
	else
		internalError('No event dispatcher defined for '+eventType.jsName);
	
};
*/
DisplayNode.prototype.checkBooleanProperty = function (name)
{
	return this.sprop(name) == true;
}
DisplayNode.prototype.sprop = function (name) // Can be made redundant if we prevent blanks on loading
{
	if (!this.sharedProperties) 
		return null;
	var p = this.sharedProperties[name];
	return (p !== ' ' ? p :null);
}
DisplayNode.prototype.copyValue = function(value)
{
	this.deepCopy(value);
};
DisplayNode.prototype.create = function DisplayNode_create(value)
{
	if (! (tersus.preferHTMLCreation && this.checkBooleanProperty('optimizeHTML')) )
	{
		this.createViewNode();
		this.attachViewNode();
		this.initViewNode();
		if (!this.initElement)
			this.createChildren();
	}
	if (value != null)
		this.copyValue(value);
	
};
DisplayNode.prototype.createChildren = function DisplayNode_createChildren()
{
	if (this.displayElements)
	{
		for (var i=0; i<this.displayElements.length; i++)
		{
			var e = this.displayElements[i];
			if ( !e.isRepetitive && ! e.isMethodElement && e.alwaysCreate)
			{
				if (!e.getChild(this))
					e.createChild(this);
			}
		}
	}
	
};
DisplayNode.prototype.setDisplayOrder = function setDisplayOrder()
{
	var displayOrder = this.element? this.element.displayOrder:1;
	this.viewNode.setAttribute('display_order',displayOrder);
};
DisplayNode.prototype.attachViewNode = function DisplayNode_attachViewNode()
{
	if (! this.viewNode)
		return;
	if (this.parent)
	{
		this.setDisplayOrder();
		this.parent.appendChildViewNode(this, this.element);
	}
};
DisplayNode.prototype.initViewNode = function DisplayNode_initViewNode()
{
	if (this.defaultStyleClass)
		this.setStyleClass(this.defaultStyleClass);
	this.setProperties();
	this.setEventHandlers();

}
tersus.removeEventHandlers = function(domNode)
{
	var doc = domNode.ownerDocument;
	var w = doc.defaultView || doc.parentWindow;
	for (key in Events)
	{
		var eventType = Events[key];
		if (eventType.jsName)
		{
			var dispatcher = w.tersus_dispatchers[eventType.jsName];
    		if (dispatcher)
		    {
		    	var jqName = eventType.jsName.substr(2);
		    	if (w == window && jqName == 'click' && $.event.special.vclick) // Virtual events don't work in pop-up
		    		jqName = 'vclick';
		        $(domNode).unbind(jqName,dispatcher);
		    }
		}
	}
	
};
DisplayNode.prototype.setEventHandlers = function DisplayNode_setEventHandlers()
{
	for (var key in Events)
	{
		var eventType = Events[key];
		if (this.hasEventHandler(eventType) && eventType.jsName)
		{
			if (this.element)
				this.registerEventHandler(eventType);
			else
				this.registerEventHandler(eventType,document.body);
		}
	}
	if (this.hasEventHandler(Events.ON_LOCATION_CHANGE))
		tersus.locationChangeListeners.push(this.getNodeId());
}
DisplayNode.prototype.removeChildViewNode = function(child)
{
	if (child)
	{
		var top = child.wrapperNode ? child.wrapperNode : child.viewNode;
		if (top && top.parentNode)
			top.parentNode.removeChild(top);
	}
};
DisplayNode.prototype.appendChildViewNode = function appendChildViewNode(child, childElement)
{
	var childNode;
	if (child.wrapperNode)
	{
		childNode = child.wrapperNode; // Child wrapper already created - no need to wrap again 
	}
	else if (child.sharedProperties && (child.sharedProperties.wrapperTag != null) && (child.sharedProperties.wrapperTag != ' ') )
	{
		childNode = this.createHTMLElement(child.sharedProperties.wrapperTag, child.sharedProperties.wrapperStyleClass);
		childNode.setAttribute('display_order', child.viewNode.getAttribute('display_order'));
		childNode.appendChild(child.viewNode);
		child.wrapperNode = childNode;
	}
	else
		childNode = child.viewNode;
	if (child.placeHolderNode)
	{
		this.viewNode.replaceChild(childNode, child.placeHolderNode);
		child.placeHolderNode = null;
	}
	else	
	{
		DisplayNode.appendChildNode(this.viewNode, childNode);
	}
		
};

DisplayNode.appendChildNode = function(parent, newChild)
{
	var displayOrder = parseFloat(newChild.getAttribute('display_order'));
	var children = parent.childNodes;
	for( var i = 0; i<children.length; i++)
	{
		var child = children[i];
		if (child.getAttribute && parseFloat(child.getAttribute('display_order')) > displayOrder)
		{
			parent.insertBefore(newChild, child);
			return;
		}
	}
	parent.appendChild(newChild);
};
DisplayNode.prototype.forDisplayChildren = function (f)
{
	if (this.displayElements)
	{
		for (var i=0;i<this.displayElements.length;i++)
		{
			var e = this.displayElements[i];
			if (e.isRepetitive)
			{
				var list = e.getChildren(this);
				for (var j=0;j<list.length;j++)
				{
					var child = list[j];
					if (child)
						if (f.call(null,child))
							reutrn;
				}
			}
			else
			{
				child = e.getChild(this);
				if (child)
					if (f.call(null,child))
						return;
			}
		}
	}
}	

DisplayNode.prototype.deleteMe = function()
{
	var element = this.element;
	var parent= this.parent;
	if (! parent)
		return;
	if (element.isRepetitive)
	{
		var siblings = element.getChildren(parent);
		for (var i=0;i<siblings.length;i++)
		{
			if (siblings[i] == this)
			{
				siblings.splice(i,1); // removes element at index i
				this.onDelete();
				return;
			}
		}
	}
	else
		element.removeChild(parent);
};
DisplayNode.prototype.destroy = function()
{
	FlowNode.prototype.destroy.call(this);
	// Remove event listener
	if (this.hasEventHandler(Events.ON_LOCATION_CHANGE))
	{
		// Remove this node's id from the tersus.locationChangeListeners
		var listeners = [];
		for (var i=0;i<tersus.locationChangeListeners.length;i++)
		{
			if (tersus.locationChangeListeners[i] != this.getNodeId())
				listeners.push(tersus.locationChangeListeners[i]);
		}
		tersus.locationChangeListeners = listeners;
	};
	
	if (this.hasEventHandler(Events.ON_DELETE))
	{
		var process = new BackgroundProcess(this, this.getEventHandler(Events.ON_DELETE));
		tersus.async.exec(function(){process.start()}, false /* don't wait */); 
	};
};
DisplayNode.prototype.onDelete = function onDelete()
{
	this.destroyHierarchy();
	if (this.parent)
		this.parent.removeChildViewNode(this);
};

DisplayNode.prototype.createHTMLElement = function createHTMLElement(tag, styleClass)
{
	var element = this.currentWindow.document.createElement(tag);
	if (styleClass && styleClass != ' ')
		element.className=styleClass;
	return element;
};
DisplayNode.prototype.createHTMLChildElement = function createHTMLChildElement(parentNode,tag, styleClass)
{
	var childNode = this.createHTMLElement(tag, styleClass);
	parentNode.appendChild(childNode);
	return childNode;
};
DisplayNode.prototype.getTag = function getTag()
{
	var t = this.sprop('tag');
	if (!t)
		t = this.defaultTag;
	return t;
};
DisplayNode.prototype.createViewNode = function DisplayNode_createViewNode()
{
	var tag = this.getTag();
	this.viewNode = this.createHTMLElement(tag);
	this.viewNode.setAttribute('lid',this.getNodeId());
};
DisplayNode.prototype.setProperties = function setProperties()
{
	var props = this.sharedProperties;
	if (!props)
		return;
	if (props.styleClass && props.styleClass != ' ')
		this.setStyleClass(props.styleClass);
	else if (this.defaultStyleClass)
		this.setStyleClass(this.defaultStyleClass);
	this.setStyle(props.style);
	if (props.label)
		this.viewNode.innerHTML = props.label;
	if (props.textTranslation != undefined)
	{
		if (props.textTranslation == 'false')
			this.textTranslation = false;
		else
			this.textTranslation = true;
	}
	var tag = this.viewNode.tagName;
	for (var p in props)
	{
		var value = props[p];
		if ( value != ' ')
		{
			if ((p == 'width'|| p=='height') && value.search(/^[0-9]+$/)>=0)
				value = value+'px';
				
			if (HTMLProperties[tag] && HTMLProperties[tag][p])
				this.viewNode[p]=value;
			else if (STYLE_PROPERTIES[p])
				this.viewNode.style[STYLE_PROPERTIES[p]]=value;
		}
	}

};
DisplayNode.prototype.removeAll = function DisplayNode_removeAll()
{
	if (this.elements)
	{
		for (var i=0; i<this.elementList.length; i++)
		{
			var e = this.elementList[i];
			if (e.isRemovable)
			{
				if (e.isRepetitive)
				{
					e.removeChildren(this);
				}
				else
				{
					e.removeChild(this);
				}
			}
		}
	}
}
DisplayNode.prototype.sharedProperties = DisplayNode.prototype.NO_PROPERTIES = {};
DisplayNode.prototype.addDisplayElement = function addElement(role, modelId, properties)
{
	if (this.getGetter(role))
	{
		var element = new MethodElement(this, role, modelId);
		element.modelId = modelId;
	}
	else
		var element = new DisplayElement(role, modelId);
	if (! this.displayElements)
		this.displayElements = [element];
	else
		this.displayElements.push(element);
	element.displayOrder = this.displayElements.length;
	this.registerSubFlow(element);
	element.properties = properties ? properties : this.NO_PROPERTIES;
	return this.addElement(element);
};
DisplayNode.prototype.optimizeFormatString = false;
DisplayNode.prototype.onload = function()
{
	this.optimizeFormatString = 
		(this.sprop('decimalPlaces') == null || this.sprop('decimalPlaces') == -1)
		&&
	 	this.sprop('useThousandsSeparator') == null &&
	 	this.sprop('currencySymbol') == null;
	 this.htmlTagSuffix = '</'+this.getTag()+'>';
	 this.hasElementChangeHandler =  this.hasEventHandler(Events.ON_ELEMENT_CHANGE);
	 this.initElement = this.getElement('<Init>');
}
DisplayNode.prototype.handleElementChange = function (role, oldValue, newValue)
{
	var parameters = {'<Element Name>':role, '<Previous Value>':oldValue, '<New Value>':newValue};
	window.engine.handleEvent(this,Events.ON_ELEMENT_CHANGE,parameters,null /* No GUI Attributes */);
	
}

DisplayNode.prototype['set<Value>'] = function setValue(value)
{
	this.value = value;
	this.setValueString();
};
DisplayNode.prototype['set<Value>'].requiresNodes = true;
DisplayNode.prototype.setValueString = function()
{
	this.valueStr = '';
	if (this.value != null)
	{
		if (this.optimizeFormatString)
			this.valueStr = this.value.leafValue;
		else if (this.value.formatString)
			this.valueStr = this.value.formatString(this.sharedProperties);
		else
			this.valueStr = this.value.toString();
	}
	if (this.viewNode)
		this.viewNode.innerHTML = this.escapeHTML(this.valueStr);
};
DisplayNode.prototype['remove<Value>'] = function removeVaule()
{
	this.value = null;
	if (this.viewNode)
		this.viewNode.innerHTML='';
};
DisplayNode.prototype['set<Labels>'] = function setLabels(map)
{
	tersus.labelMap = map;
	var op = function(node)
	{
		if (node.captionChanged && node.viewNode)
			node.captionChanged();
	};
	tersus.traverseHierarchy(window.rootDisplayNode, tersus.Node.childIterator, op);
};
DisplayNode.prototype['get<Labels>'] = function getLabels()
{
	return tersus.labelMap;
};
DisplayNode.prototype['set<Resources>'] = function setLabels(map)
{
	tersus.resourceMap = map;
};
DisplayNode.prototype['get<Resources>'] = function getLabels()
{
	return tersus.resourceMap;
};
DisplayNode.prototype['set<Direction>'] = function DisplayNode_setDirection(direction)
{
	tersus.setDirection(direction);
};
DisplayNode.prototype['get<Direction>'] = function getDirection()
{
	return tersus.getDirection();
};

DisplayNode.prototype['get<Visible>'] = function ()
{
	return this.visible;
};

DisplayNode.prototype['set<Visible>'] = function (visible)
{
	this.visible = visible;
	this.refreshVisibility();
};
DisplayNode.prototype.isInvisible = function()
{
	var visible = this.visible;
	var parent = this.parent;
	while (visible && parent && parent.isGroup)
	{
		visible = parent.visible;
		parent = parent.parent;
	}
	return !visible;	
}
DisplayNode.prototype.refreshVisibility = function()
{
	var domNode = this.wrapperNode ? this.wrapperNode : this.viewNode;
	if (domNode)
	{
		if (this.isInvisible())
			domNode.style.display='none';
		else
		{
			domNode.style.display='';
			this.onResize();
		}
	}
};

DisplayNode.prototype.styleClass = '';
// baseClassName - the class name defined by the application, not taking into account special conditions like selection
DisplayNode.prototype.baseClassName = ''; 
DisplayNode.prototype.extraClassName = ''; 
DisplayNode.prototype.setStyleClass = function setStyleClass(value)
{
	if (this.viewNode)
	{
		var className = value;
		var rtlClassName = value+'_rtl';
		if (window.textDirection == 'rtl' && tersus.styleClassNames.contains(rtlClassName))
			className = rtlClassName;
		$(this.viewNode).removeClass(this.baseClassName).addClass(className);
		this.baseClassName = className;
	}
	this.styleClass = value;
};
DisplayNode.prototype['set<Style Class>'] = function(value)
{
	this.setStyleClass(value);
};
 DisplayNode.prototype['get<Style Class>'] = function getStyleClass()
{
	return this.styleClass;
};
DisplayNode.prototype.setStyle =  DisplayNode.prototype['set<Style>'] = function(value)
{
	if (this.viewNode && value != null)
	{
		try
		{
			return this.viewNode.style.cssText = value;
		}
		catch (e)
		{
			modelExecutionError('Failed to set style "'+value+'"  (HTML tag='+this.viewNode.tagName+')',this);
		}
	}
};
DisplayNode.prototype['get<Style>'] = function getStyleClass()
{
	if (this.viewNode)
		return this.viewNode.style.cssText;
};
DisplayNode.prototype['get<Value>'] = function getValue()
{
	return this.value;
}
DisplayNode.prototype.getBaseCaption = DisplayNode.prototype['get<Caption>'] = function()
{
	if (this.caption != undefined)
		return this.caption;
	var c = this.sprop('caption');
	if (c != null)
		return c;
	else
		return this.element.role;
	
};
DisplayNode.prototype.getCaption = function()
{
	var idTranslation = this.getTranslation(this.modelId);
	if(idTranslation != undefined)
		return idTranslation; 
	else
		return this.translate(this.getBaseCaption());
};
DisplayNode.prototype.captionChanged = function() {};
DisplayNode.prototype['set<Caption>'] = function(caption)
{
	this.caption = caption;
	this.captionChanged();
};
DisplayNode.prototype['get<Read Only>'] = function ()
{
	return this.explicitReadOnly;
};
DisplayNode.prototype['set<Read Only>'] = function (readOnly)
{
	if (this.explicitReadOnly != readOnly)
	{
		var originalState = this.computedReadOnly();
		this.explicitReadOnly = readOnly;
		var newState = this.computedReadOnly();
		if (this.newState != originalState) 
		{
			this.updateSelfReadOnly(newState);
			this.updateChildrenReadOnly(newState);
		}
		
	}
};
DisplayNode.prototype.updateSelfReadOnly = function ()
{
};
DisplayNode.prototype.updateChildrenReadOnly = function DisplayNode_updateChildrenReadOnly(readOnly)
{
	if (! this.displayElements)
		return;
	for (var i=0; i<this.displayElements.length; i++)
	{
		var element = this.displayElements[i];
		if (element.isRepetitive)
		{
			var children = element.getChildren(this);
			for (var j=0; j<children.length;j++)
			{
				var child = children[j];
				if (! child.explicitReadOnly) // If the child is explicitly read only, its state won't change
					child.parentReadOnlyChanged(readOnly);
			}
		}
		else
		{
			var child = element.getChild(this);
			if (child && ! child.explicitReadOnly) // If the child is explicitly read only, its state won't change
				child.parentReadOnlyChanged(readOnly);
		}
	}
};
DisplayNode.prototype.parentReadOnlyChanged = function(readOnly)
{
	this.updateSelfReadOnly(readOnly);
	this.updateChildrenReadOnly(readOnly)
};
/**
 * Computes the read only state of this node.  A node is in read only mode if it's explicitly
 * marked as read only, or if its ancestor is read only
 */	
DisplayNode.prototype.computedReadOnly = function DisplayNode_computedReadOnly()
{
	if (this.explicitReadOnly)
		return true;
	else if (this.parent && this.parent.computedReadOnly)
		return this.parent.computedReadOnly();
	else
		return false;
};
DisplayNode.prototype.onResizeChildren = function onResizeChildren()
{
	if (this.displayElements)
	{
		for (var i=0; i<this.displayElements.length; i++)
		{
			var element = this.displayElements[i];
			if (element.isRepetitive)
			{
				var children = element.getChildren(this);
				for (var j=0; j<children.length;j++)
				{
					var child = children[j];
					child.onResize();
				}
			}
			else
			{
				var child = element.getChild(this);
				if (child)
					child.onResize();
			}
		}
	}
};
DisplayNode.prototype.onResize = DisplayNode.prototype.onResizeChildren;

DisplayNode.prototype.isDisplayNode = true;
/******************************************************************
 *  reset() is called before replacing a display element, and does not trigger initialization processes
 *  refresh() is called when refreshing a display element, and includes initiailization processes (createSubFlows).
 *
 *  If reset triggered initialization processes, we might end up in strange situations, like double initialization
 *   (e.g. If a chooser initializes itself, and then a different chooser is copied onto it)
 *
 *
 ****************************************************************** */
DisplayNode.prototype.reset = function DisplayNode_reset()
{
	this.removeAll();
	this.initViewNode();
	this.createChildren(); // TODO(x) eliminate redundant creation of children (not sure whether this really is redundant as reset() is overridden in HTMLTag)
};
DisplayNode.prototype.refresh = function DisplayNode_refresh()
{
	this.reset();
	this.createSubFlows();
};


DisplayNode.prototype.deepCopy = function deepCopy(value)
{
	if (tlog) tlog("start deepCopy:"+value.modelId);
	for (var elementRole in value.elements)
	{
		var element = this.getElement(elementRole);
		if (element)
		{
			if (element.isRepetitive)
			{
				var childValues = value.getChildren(elementRole);
				element.setChildren(this, childValues, Operation.REPLACE);
			}
			else
			{
				var childValue = value.getChild(elementRole);
				if (childValue != null)
				{
					element.replaceChild(this, childValue);
				}
			}
		}
	}
	if (tlog) tlog("end deepCopy:"+value.modelId);
};
DisplayNode.prototype.tooltip = null;
DisplayNode.prototype.setTooltip = DisplayNode.prototype['set<Tooltip>'] = function (tooltip)
{
	this.tooltip = tooltip;
	if (this.viewNode && tooltip != undefined && tooltip != null)
	{
		this.viewNode.title = this.translate(tooltip);
	};
};
DisplayNode.prototype.getTooltip = DisplayNode.prototype['get<Tooltip>'] = function ()
{
	return this.tooltip;
};
DisplayNode.prototype.setValidationMessage = DisplayNode.prototype['set<Validation Message>']= function (message)
{
	this.validationMessage = message;
	if (this.currentWindow && this.viewNode)
	{
		if (message != null)
		{
			this.viewNode.className = this.styleClass + ' invalid';
			this.viewNode.title = this.forceTranslate(message);
		}
		else
		{
			this.viewNode.className = this.styleClass;		
			this.viewNode.title='';
		}
	}
};
DisplayNode.prototype['remove<Validation Message>']= function ()
{
	this.setValidationMessage(null);
};
DisplayNode.prototype.getValidationMessage = DisplayNode.prototype['get<Validation Message>']= function (message)
{
	return this.validationMessage;
};

DisplayNode.prototype.showValidationMessage = function (message)
{
	this.currentWindow.alert(message);
};
DisplayNode.prototype.validate = function validate()
{
	var message = this.getValidationMessage();
	if (message != null)
	{
		this.showValidationMessage(this.forceTranslate(message));
		return false;
	}
	if (! this.elementList)
		return true;
	for (var j=0; j<this.elementList.length; j++)
	{
		var element = this.elementList[j];
		if (element.isRepetitive)
		{
			var values = element.getChildren(this);
			for (var i=0; i< values.length; i++)
			{
				var value = values[i];
				if (value && value.validate && ! value.validate())
					return false;
				
					
			}
		}
		else
		{
			var value = element.getChild(this);
			if (value && value.validate && ! value.validate())
				return false;
		}
	}
	return true;
};
DisplayNode.prototype.findDisplayElementIndex = function (role)
{
	for (var i=0; i<this.displayElements.length;i++)
	{
		var e = this.displayElements[i];
		if (e.role ==role)
			return i;
	}
	return null;
};
DisplayNode.prototype.findPreviousSibling = function (child)
{
	var role = child.element.role;
	
	// Find the last child among the preceding display elements
	var index = this.findDisplayElementIndex(role);
	if (index == null)
		return null;
	for (var i=index;i>=0;i--)
	{
		var e = this.displayElements[i];
		if (e.isRepetitive)
		{
			var children = e.getChildren(this);
			for (var j=children.length-1; j>=0; j--)
			{
				var sibling = children[j];
				if (sibling && sibling != child)
					return sibling;
			}
		}
		else
		{
			var sibling = e.getChild(this);
			if (sibling && sibling != child)
				return sibling;
		}
	}
	return null;
};
DisplayNode.prototype.focusOnFirstField = function()
{
	var elements = this.viewNode.getElementsByTagName('*');
	for (var i=0; i<elements.length; i++)
	{
		var e = elements[i];
		if (e.tagName == 'INPUT' || e.tagName == 'TEXTAREA' || e.tagName == 'SELECT')
		{
			if (isVisible(e) && !e.disabled)
			{
				this.currentWindow.setTimeout(function(){if (isVisible(e) && !e.disabled) e.focus();},0);
				break;
			}
		}
	}
};


DisplayNode.prototype.addGetter('<Screen X>', function()
{
	if (!this.viewNode)
		return null;
	return getScreenPosition(this.viewNode).left;
});
DisplayNode.prototype.addSetter('<Screen X>', function(x)
{
	this.viewNode.style.position='absolute';
	this.viewNode.style.left = (x - getScroll(this.viewNode).left)+'px';
});

DisplayNode.prototype.addGetter('<Screen Y>', function()
{
	if (!this.viewNode)
		return null;
	if (this.viewNode.tagName == 'SPAN')
	{
		if (this.inputField) // This happens in input fields
			return getScreenPosition(this.inputField).top;
		else
			internalError("Getting the vertical position of 'SPAN' elements not supported yet");
	}
	else
		return getScreenPosition(this.viewNode).top;
});
DisplayNode.prototype.addSetter('<Screen Y>', function(y)
{
	this.viewNode.style.position='absolute';
	this.viewNode.style.top = (y - getScroll(this.viewNode).top)+'px';
});

DisplayNode.prototype.addGetter('<URL>', function()
{
	if (this.viewNode)
		return this.viewNode.href;
});
DisplayNode.prototype.addSetter('<URL>', function(url)
{
	if (this.viewNode)
		this.viewNode.href = url;
});
DisplayNode.prototype.addGetter('<Target Window>', function()
{
	if (this.viewNode)
		return this.viewNode.target;
});
DisplayNode.prototype.addSetter('<Target Window>', function(target)
{
	if (this.viewNode)
		this.viewNode.target = target;
});
DisplayNode.prototype.addGetter('<Width>', function()
{
	if (this.viewNode)
		return getVisibleSize(this.viewNode).width;
	else
		return this.width;
});
DisplayNode.prototype.addSetter('<Width>', function(width)
{
	this.width = width;
	if (this.viewNode)
		this.viewNode.style.width = width  + 'px';
});
DisplayNode.prototype.addGetter('<Height>', function()
{
	if (! this.viewNode)
		return this.height;
	if (this.viewNode.tagName == 'SPAN')
	{
		if (this.inputField) // This happens in input fields
			return getVisibleSize(this.inputField).height;
		else
			internalError("Getting the height of 'SPAN' elements not supported yet");
	}
	else
		return getVisibleSize(this.viewNode).height;
});
DisplayNode.prototype.addSetter('<Height>', function(height)
{
	this.height = height;
	if (this.viewNode)
		this.viewNode.style.height = height+'px';
});
DisplayNode.prototype.addGetter('<Col Span>', function()
{
	if (this.viewNode && this.viewNode.parentNode && this.viewNode.parentNode.colSpan)
		return this.viewNode.parentNode.colSpan;
	else
		return this.colSpan;
});
DisplayNode.prototype.addSetter('<Col Span>', function(span)
{
	this.colSpan = span;
	if (this.viewNode && this.viewNode.parentNode && this.viewNode.parentNode.colSpan)
		this.viewNode.parentNode.colSpan = span;
});
DisplayNode.prototype.addGetter('<Row Span>', function()
{
	if (this.viewNode && this.viewNode.parentNode && this.viewNode.parentNode.rowSpan)
		return this.viewNode.parentNode.rowSpan;
	else
		return this.rowSpan;
});
DisplayNode.prototype.addSetter('<Row Span>', function(span)
{
	this.rowSpan = span;
	if (this.viewNode && this.viewNode.parentNode && this.viewNode.parentNode.rowSpan)
		this.viewNode.parentNode.rowSpan = span;
});

DisplayNode.displayChildrenIterator = function (node,iteration)
{
	if (node && node.displayElements)
	{
		for (var i=0; i<node.displayElements.length; i++)
		{
			var e = node.displayElements[i];
			if ( e.isRepetitive)
			{
				var children  = e.getChildren(node);
				for (var j=0;i<children.length;j++)
				{
					var stop = iteration(children[j]);
					if (stop == true)
						return;
				}
			}
			else
			{
				var child = e.getChild(node);
				if (child)
				{
					var stop = iteration(child);
					if (stop == true)
						return;
				}
			}
		}
	}
	
};
// file: 026-DataValue.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function DataValue() {};
DataValue.prototype = new tersus.Node();
DataValue.prototype.constructor = DataValue;
DataValue.prototype.isDataValue = true;
DataValue.prototype.sqlType = 'NONE';
DataValue.prototype.toString = function toString()
{
	if (this.leafValue)
	{
		return ''+this.leafValue;
	}
	else
		return 'Composite Value: '+this.modelId;
};
DataValue.prototype.formatString = function (props)
{
	return this.toString();
};
DataValue.prototype.formatInput = function (props)
{
	return this.formatString(props);
};
DataValue.prototype.parseInput = function()
{
	internalError("parseInput not implemented for "+this.constructor.name);
};
DataValue.prototype.addDataElement = function addDataElement(role,modelId)
{
	var element;
	var getter = this.getGetter(role);
	if (getter)
		element = new MethodElement(this, role, modelId);
	else
		element = new DataElement(role, modelId);
	return this.addElement(element);
};

// file: 026-ServiceNode.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 

//Service Node

function ServiceNode() {};
ServiceNode.prototype = new FlowNode();
ServiceNode.prototype.constructor = ServiceNode;
ServiceNode.prototype.type = 'Service';
ServiceNode.prototype.secure = false; // true means service must be invoked under HTTPS
ServiceNode.prototype.excludedTriggers=new Set(['<URL>']);
ServiceNode.prototype.start = function startService()
{	
	if (this.secure && !tersus.rootURL.match(/^https:/))
	{
		tersus.error('The service "'+this.modelName+'" must  be used in a secure context');
		return;
	} 
	this.pause();
	this.retries = 0;
	this.callService();
};
ServiceNode.prototype.started = function serviceStarted()
{
	this.setStatus(FlowStatus.STARTED);
	if (window.trace)
	{
		window.trace.invoked(this);
		window.trace.flush();
	}
};
ServiceNode.prototype.isService = true;
/*
	Invokes a service that contains file arguments.
	This is a special case for the following reasons:
	(1) We must use the orginial "File Input Node" to send the file 
	(2) For this reason, we use the "Form" node that is already part of the document
	(3) We need to use "form/multipart" encoding
	
	
	In this case, we can't create a separate form, 
	because there is no way to copy the FileInput node's content to another
	file input node (it is possible in Mozilla, but not in IE)
	
	Using XMLHTTPRequest is out of the question because JavaScript can't read
	local files.
*/

ServiceNode.prototype.callService = function  callService()
{
	this.explicitURL = this.getLeafChild('<URL>');
	var baseURL = tersus.serverURL ? tersus.serverURL+'/':tersus.rootURL;
	this.url = this.explicitURL ? this.explicitURL : baseURL+'Service';
	
	var host = document.domain;
 	var urlHost = this.url.match(/\/\/([^:/]+)[:/]/)[1];
 	if (urlHost != null)
 		host=urlHost;
    if (host == '')
    	modelError('Missing <URL> for service invocation, or attempt to access server-side database in native application',this);
	if (navigator.onLine == false)
	{
		var host = document.domain;
		if(! tersus.isLocalHost(host))
		{
			this.serviceError = "Offline - can't access server";
			this.handleResponse({});
			return;
		}
	}
	if (tlog)
		tlog("Calling service:"+this.modelName);
	if (this.findFileTrigger() || !tersus.useXMLHTTPRequest) 
		this.callFileService();
	else
		this.callRegularService();
};
ServiceNode.prototype.cancellable = true;
ServiceNode.prototype.getCancellationFunc = function getCancellationFunc()
{
	if (this.cancellable == 'false' || !this.cancellable)
		return null;
	var serviceNode = this;
	var cancellation = function () { 
			tersus.progress.set('Request cancelled.');
			serviceNode.cancel();
			};
	return cancellation;
};
ServiceNode.prototype.cancel = function cancel()
{
	if (this.responseReceived)
		return;
	this.abort();
	if (this.responseReceived)
		return;
	this.handleResponse({});
	this.cancelled = true;
};
ServiceNode.prototype.abort = function abort()
{
	var req = createXMLHTTPRequest();
	var serviceNode = this;
	var url = tersus.rootURL+'Abort';
	req.open('POST', url, true);
	req.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
	req.send('_userRequestId='+window.encodeURIComponent(this.getNodeId()));
};
ServiceNode.prototype.setProgressMessage = function()
{
	if (tersus.progress.active)
	{
		var message = null;
		if (this.progressMessage && this.progressMessage != ' ')
			message = this.progressMessage;
		else if (tersus.progressMessage)
			message = tersus.progressMessage;
		else
			message = 'Service: '+this.modelName;
		tersus.progress.set(message, this.getCancellationFunc());
	}
};
ServiceNode.prototype.callRegularService = function getService()
{
	var paramString = this.getParamString();
	this.nodesToRemove = [];
	
	// We wrap the call to XMLHTTPRequest in setTimeout to avoid a firefox bug:
	// see http://www.highdots.com/forums/javascript/xmlhttprequest-firefox-problem-577724.html
	// However, we suppress this behavior if we're inside 'onUnload' (or the service will never be invoked)
	var serviceNode = this;
	var f = function() {
		serviceNode.setProgressMessage();
		serviceNode.callXMLHTTPService(paramString);
	};
	if (tersus.isUnloading)
		f();
	else
		tersus.async.exec(f, false/* don't wait */);
};

ServiceNode.prototype.callXMLHTTPService = function callXMLHTTPService(paramString)
{
	var req = createXMLHTTPRequest();
	var nodeId = this.getNodeId();
	var modelName = this.modelName;
	var url = this.url;
	if (! tersus.isUnloading) // If we're unloading we won't be able to process the response
	{
		req.onreadystatechange= function(){
			ServiceNode.handleXMLHTTPResponse(req, nodeId, url, modelName);
		};
	}
	req.open('POST', this.url, true);
	req.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
	req.send(paramString);
};
ServiceNode.dummy = function () {};
ServiceNode.handleXMLHTTPResponse = function(req, nodeId, url, modelName)
{
	if (req.readyState != 4)
		return;
	req.onreadystatechange = ServiceNode.dummy;
	var status  = 'Error';
	var statusText = 'Unknown';
	try
	{
		status = req.status;
		statusText = req.statusText;
	}
	catch (e)
	{
	}
	var serviceNode = tersus.findNode(nodeId);
	if (!serviceNode)
		return;
	if (serviceNode.cancelled)
		return;
	delete FlowNode.map[nodeId]; 
	if (status == 400 && serviceNode.retries < tersus.SERVICE_MAX_RETRIES)
	{
		serviceNode.retries++;
		setTimeout(function(){serviceNode.callRegularService();}, tersus.SERVICE_RETRY_DELAY);
		return;
	}
	if (status != 200)
	{
		var message;
		if (status == 400)
		{
			message = 'The server has reported "BAD REQUEST".  This is possibly a result of a temporary network problem.';
			if (statusText != 'OK') // Safari bug - reports 'OK' instead of server message
				message += '\n\nOriginal server message: '+statusText;
		}
		else
			message = 'Failed to execute service (url='+url+', modelName='+modelName +',status='+status+',message='+statusText+')';
		serviceNode.serviceError = message;
		setTimeout(function() {serviceNode.handleResponse({});},0);
		return;
	}

	if (tersus.progress.active)
		tersus.progress.set('Processing service response ...');
	var responseText = req.responseText;
	
	/* If there is a sizeable response, we expect processing to take significant time
	   (during which there is no opportunity for the progress bar to appear, so we make
	   force progress bar to be visible  */
//	if (responseText.length > tersus.progress.responseLengthThreshold)
//		tersus.progress.show();
	setTimeout( function()
		{
			if (tlog) tlog('Parsing response ('+responseText.length+' characters)');
			var exitValues = eval("var output = "+responseText+";output;");
			if (tlog) tlog('Done parsing response');
			if (exitValues._ERROR)
			{
				serviceNode.serviceError=exitValues._ERROR;
				serviceNode.handleResponse({});
			}
			else
			{
				if (exitValues['<Done>'] == null)
					exitValues['<Done>']={};
				serviceNode.handleResponse(exitValues);
			}
		},0);
};

	
ServiceNode.prototype.getParamString = function()
{
	var location = new tersus.RuntimeLocation(this);
	var serializer = new Serializer();
	if (!this.explicitURL)
	{
		var paramString = "_baseModelId="+encodeURIComponent(location.baseModelId);
		paramString += "&_path="+encodeURIComponent(location.path.str);
		paramString+='&_userRequestId='+this.getNodeId();
	}
	else
	{
		var paramString = '_output=json';
	}
	if (window.trace)
	{
		paramString+='&_traceFileName='+encodeURIComponent(this.getTraceFileName());
	}	
	if (this.triggers)
	{
		for (var i=0; i< this.triggers.length; i++)
		{
			var trigger = this.triggers[i];
			var paramName = trigger.role;
			var paramValue = null;
			if (trigger.isRepetitive)
			{
				var values = trigger.getChildren(this);
				if (values.length > 0)
				{
					paramValue = serializer.serialize(values); 
				}
			}
			else
			{
				var value = trigger.getChild(this);
				if (value != null)
				{
					if (value.isLeaf)
						paramValue = value.serialize ? value.serialize() : value.leafValue;
					else
						paramValue = serializer.serialize(value);
				}
			}
			if (paramValue != null)
			{	
				if (paramString.length > 0)
					paramString+= '&';
				paramString += encodeURIComponent(paramName);
				paramString += '=';
				paramString += encodeURIComponent(paramValue);
			}
		}
	}
	if (!this.explicitURL)
	{
		paramString += '&_sig=END';
	}
	if (tlog)
		tlog('Done creating paramString');
	return paramString;
};
ServiceNode.prototype.callFileService = function callFileService()
{
	var form = this.prepareForm();
	if ( ! form)
	{
		this.continueExecution();
		return;
	};
	this.addTersusParameters(form);
	
	var iframe = this.getResponseFrame();
	
	form.action = this.url;
	form.method = 'POST';
	form.target  = iframe.name;
	this.callbackName = iframe.name;
	var serviceNode = this;
	this.mainWindow[this.callbackName] = function callback(exitValues)
	{
		serviceNode.mainWindow[serviceNode.callbackName] = null;
		if (serviceNode.cancelled)
			return;
		tersus.progress.set('Processing service response ...');
		setTimeout( function()
		{
			if (exitValues._ERROR)
			{
				serviceNode.serviceError=exitValues._ERROR;
				serviceNode.handleResponse({});
			}
			else
				serviceNode.handleResponse(exitValues);
		},0);
	};
	var callbackField = addHiddenField(form, '_js_callback', 'parent.'+iframe.name);
	form.appendChild(callbackField);
	this.nodesToRemove.push(callbackField);
	if (tersus.IFRAME_SERVICE_TIMEOUT)
	{
		setTimeout(function (){
			if (!serviceNode.responseReceived)
			{
				serviceNode.currentWindow.alert('Server Error: Timeout');
				serviceNode.cancel();
			}
		}
		,tersus.IFRAME_SERVICE_TIMEOUT);
	};
	var sigField = addHiddenField(form, '_sig', 'END');
	form.appendChild(sigField);
	this.nodesToRemove.push(sigField);
	try
	{
		var nodeId = this.getNodeId();
		if (!tersus.pendingServiceCalls)
			tersus.pendingServiceCalls = new Set([nodeId]);
		else
			tersus.pendingServiceCalls.add(nodeId);
		form.submit();
		this.setProgressMessage();	
	}
	catch (e)
	{
		var msg;
		var fileTrigger = this.findFileTrigger();
		var fileName = this.getChild(fileTrigger.role).getFileName();
		if (e.name=='TypeError') // This happens in IE when a user types an invalid file name
			msg = 'Please check the file name "'+fileName + '"';
		else
			msg = 'An error occurred when invoke the service ('+e.message+').';
		alert(msg);
		this.continueExecution();
	}
	for (var i=0; i< this.nodesToRemove.length; i++)
	{
		var node = this.nodesToRemove[i];
		node.parentNode.removeChild(node);
	}
};
ServiceNode.prototype.prepareForm = function prepareForm()
{
	var serializer = new Serializer();
	var form;
	var fileTrigger = this.findFileTrigger();
	this.nodesToRemove = [];
	
	if (fileTrigger)
	{
		var fileInputDisplayNode = this.getChild(fileTrigger.role).inputFieldDisplayNode;
		if ( fileInputDisplayNode.currentWindow.closed)
		{
			alert('Tersus Application Error: Reference to a file input field in closed popup window.\n');
			return null;
		}
		
		form = fileInputDisplayNode.viewNode;
		setEncType(form,'multipart/form-data'); 
		var fileInputNode = fileInputDisplayNode.fileNode;
		fileInputNode.name = fileTrigger.role;
	}
	else
	{
		form = this.mainWindow.document.createElement('form');
		if (! window.showServiceInvocations)
		{
			this.nodesToRemove.push(form);
		}
		this.mainWindow.document.body.appendChild(form);
	}

	if (this.triggers)
	{
		for (var i=0; i< this.triggers.length; i++)
		{
			var trigger = this.triggers[i];
			if (this.excludedTriggers.contains(trigger.role))
				continue;
			if (trigger != fileTrigger) // If fileTrigger is non-null, it is already part of the form
			{
				if (trigger.isRepetitive)
				{
					var values = trigger.getChildren(this);
					if (values.length > 0)
					{
						var serialization = serializer.serialize(values); 
						var inputField = form.ownerDocument.createElement('input');
						inputField.type = 'hidden';
						inputField.name = trigger.role;
						inputField.value = serialization;
						form.appendChild(inputField);
						if (fileTrigger)
							this.nodesToRemove.push(inputField);
					}
				}
				else
				{
					var value = trigger.getChild(this);
					if (value != null)
					{
						var serialization;
						if (value.isLeaf)
							serialization = value.serialize ? value.serialize() : value.leafValue;
						else
							serialization = serializer.serialize(value);
						var inputField = form.ownerDocument.createElement('input');
						inputField.type = 'hidden';
						inputField.name = trigger.role;
						inputField.value = serialization;
						form.appendChild(inputField);
						if (fileTrigger)
							this.nodesToRemove.push(inputField);
					}
				}	
			}
		}
	}	
	return form;
};

ServiceNode.prototype.addTersusParameters = function(form)
{
	var location = new tersus.RuntimeLocation(this);
	if (this.explicitURL)
	{
		var outputField = addHiddenField(form, '_output', 'json');
		form.appendChild(outputField);
		this.nodesToRemove.push(outputField);
	
	}
	else
	{
		var baseModelIdField = addHiddenField(form, '_baseModelId', location.baseModelId);
		form.appendChild(baseModelIdField);
		this.nodesToRemove.push(baseModelIdField);
		var pathField = addHiddenField(form, '_path', location.path.str);
		form.appendChild(pathField);
		this.nodesToRemove.push(pathField);
	}
	if (window.trace)
	{
		window.trace.flush();
		var traceFileField = addHiddenField(form, '_traceFileName', this.getTraceFileName());
		form.appendChild(traceFileField);
		var requestIdField = addHiddenField(form, '_userRequestId', this.getNodeId());
		form.appendChild(traceFileField);
		this.nodesToRemove.push(traceFileField);
		this.nodesToRemove.push(requestIdField);
	}
};
ServiceNode.prototype.getTraceFileName = function()
{
	return window.trace.filename;
};

ServiceNode.prototype.handleResponse = function handleResponse(response)
{
	if (tlog) tlog('Processing response for service: '+	this.modelName);
	if (tersus.pendingServiceCalls)
		tersus.pendingServiceCalls.remove(this.getNodeId());
	var start = new Date();
	this.responseReceived = true;
	if (window.trace)
		window.trace.responded(this);
		
	for (var exitRole in response)
	{
        var exit = this.getElement(exitRole);
        if (!exit)
        	continue; // It's possible that the exit is missing because of permission issues
        if (exit.isRepetitive)
        {
        	var values = response[exitRole];
        	for (var i=0; i< values.length; i++)
        	{
        		var dataValue  = exit.createChild(this);
        	    dataValue.copyObject(values[i]);
        	}
        }
        else
        {
        	var value = response[exitRole];
        	var dataValue  = exit.createChild(this);
       	    dataValue.copyObject(value);
        }
        			
	};
	if (this.status != FlowStatus.PAUSED)
		debugMessage('Unexpected status "'+this.status+'" for ' + this.getPath());
	this.continueExecution();
	var end = new Date();
	window.duration = "Response processed in " + (end - start) + " ms";
	
};

ServiceNode.prototype.resume = function resume()
{
	if (this.serviceError)
		throw this.serviceError;
};
/* Creates an iframe for invoking the service */

ServiceNode.prototype.getResponseFrame = function getResponseFrame()
{
	var iframe = tersus.nextServiceResponseFrame();
	this.iframeId =iframe.id;
	return iframe;
};
/*
	returns the unique trigger that holds a 'File Input' value
	If there is no such trigger, returns null.
	If there is more than one trigger, an error is reported.
*/
ServiceNode.prototype.findFileTrigger = function findFileTrigger()
{
	var fileTrigger = null;
	if (this.triggers)
	{
		for (var i=0; i<this.triggers.length; i++)
		{
			//TODO consider using the modelId of the trigger to identify the file
			var trigger = this.triggers[i];
			if (trigger.isRepetitive)
				continue;
			var child = this.getChild(trigger.role); 
			if (child && child.isFile)
			{
				if (fileTrigger)
				{
					modelError("Too many file input values");
					return null;
				}
				else
					fileTrigger = trigger;
			}
		}
	}
	return fileTrigger;
};
// file: 030-DataElement.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function DataElement(role, modelId)
{
	this.role = role;
	if (modelId)
		this.modelId = modelId;
};
DataElement.prototype = new Element();
DataElement.prototype.constructor = DataElement;
DataElement.prototype.type = 'Data Element';
DataElement.prototype.isDataElement = true;// file: 031-SubFlow.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function SubFlow(role, modelId)
{
	this.role = role;
	this.modelId = modelId;
};
SubFlow.prototype = new Element();
SubFlow.prototype.constructor = SubFlow;
SubFlow.prototype.isSubFlow = true;
SubFlow.prototype.createChild = function createChild(parent, modelId)
{
	debugAssertion(modelId == null, "Expecting SubFlow.createChild to always receive null modelId");
	var child = this.createChildInstance(modelId);
	child.mainWindow = parent.mainWindow;
	child.currentWindow = parent.currentWindow;
	child.parent = parent;
	child.element = this;
	if (child.initialize)
		child.initialize();
	return child;
};
SubFlow.prototype.init = function SubFlow_init(parent)
{
	if (! this.isRepetitive && ! tersus.Conventions.isEventHandler(this))
	{
		var child = this.createChild(parent);
		this.addChild(parent, child);
		child.setInitialStatus();
	}
};// file: 032-DisplayElement.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function DisplayElement(role, modelId)
{
	this.role = role;
	this.modelId = modelId;
}
DisplayElement.prototype = new SubFlow();
DisplayElement.prototype.constructor = DisplayElement;
DisplayElement.prototype.isDisplayElement = true;
DisplayElement.prototype.type = 'Display Element';
DisplayElement.prototype.addAChild = function addChild(parent, value)
{
	if (parent.currentWindow)
	{
		this.createChild(parent, value.modelId, value);
	}
	else
	{
		Element.prototype.addAChild.call(this, parent, value);
	}
};

DisplayElement.prototype.replaceChild = function replaceChild(parent, value, reset)
{
	if (parent.currentWindow) // parent is a real display node
	{
		var oldChild = this.getChild(parent);
		if (oldChild)
		{
			if (oldChild.modelId == value.modelId)
			{
				if (reset)
					oldChild.reset();
				child = oldChild;
				child.copyValue(value);
			}
			else
			{
				var oldViewNode = null;
				if (oldChild != null)
				{
					var placeHolder = oldChild.wrapperNode ? oldChild.wrapperNode: oldChild.viewNode;
					oldChild.viewNode = null;
					placeHolder.style.display = 'none';
					this.removeChild(parent);
				}
				child = this.createChild(parent,value.modelId, value, placeHolder);
			}
		}
		else
			child = this.createChild(parent,value.modelId, value);
	}
	else
	{
		Element.prototype.replaceChild.call(this, parent, value);
	}
}
DisplayElement.prototype.moveChild = function (parent, fromIndex, toIndex)
{
	var children = this.getChildren(parent);
	var currentChild = null;
	if (children.length > 0)
		currentChild = children[toIndex];
		
	var child = Element.prototype.moveChild.call(this,parent,fromIndex,toIndex);
	if (child.viewNode && currentChild)
	{
		// Move the new child before the current child
		child.viewNode.parentNode.insertBefore(child.viewNode, currentChild.viewNode);
	}
	return child;
};

DisplayElement.prototype.removeNumberedChild = function removeNumberedChild(parent, index)
{
	makeAssertion( this.isRepetitive);
	var children = this.getChildren(parent);
	children[index - 1].deleteMe();
};


// As opposed to regular SubFlows, DisplayElements are added to the parent node
DisplayElement.prototype.createChildWithoutVisuals = function createChildWithoutVisuals(parent, modelId)
{
	var child = SubFlow.prototype.createChild.call(this, parent, modelId);
	this.addChild(parent, child);
	if (parent.currentWindow && ! child.isPopup)
	{
		child.setInitialStatus();
	}
	return child;
};
DisplayElement.prototype.createChild = function createChild(parent, modelId, value, placeHolderNode)
{
	var child = this.createChildWithoutVisuals(parent,modelId);
	if (parent.currentWindow && ! child.isPopup)
	{
		child.placeHolderNode = placeHolderNode;
		child.create(value);
	}
	return child;
};
DisplayElement.prototype.init = function DisplayElement_init(parent)
{
	// There is no need to initialize display elements, as the whole  hierarchy is created in advance
	// Except for popups (which are more like regular subflows)
	// TODO clean this up (the generic DisplayElement should not know about popups)
	if (tersus.repository.get(this.modelId).prototype.isPopup)
	{
		SubFlow.prototype.init.call(this,parent);
	}
	
};

DisplayElement.prototype.prop = function(name)
{
	
	 var local = this.properties[name];
	 if (local != null)
	 	return local;
	 var childConstructor = this.getChildConstructor();
	 if (!childConstructor)
	 	return null;
	 return childConstructor.prototype.sharedProperties[name];
};
// file: 033-Trigger.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function Trigger(role, modelId)
{
	this.role = role;
	if (modelId)
		this.modelId = modelId;
};
Trigger.prototype = new Element();
Trigger.prototype.constructor = Trigger;
Trigger.prototype.type='Trigger';

Trigger.prototype.replaceChild = function(parent, value)
{
	if (parent.isInputSet) // Special case: trigger of a repetitive sub-process - we accumulate the value
	{
		if (! parent.children[this.role])
			parent.children[this.role] = [];
		parent.children[this.role].push(child);
		
	}
	else
		Element.prototype.replaceChild.call(this, parent, value);
};
Trigger.prototype.removeChildren = function(parent)
{
	makeAssertion( this.isRepetitive);
	if (parent.children)
		parent.children[this.role] = [];
};// file: 034-Exit.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function Exit(role, modelId)
{
	this.role = role;
	if (modelId)
		this.modelId = modelId;
};
Exit.prototype = new Element();
Exit.prototype.constructor = Exit;
Exit.prototype.isExit=true;
Exit.prototype.isError=false;
Exit.prototype.type='Exit';// file: 035-FlowDataElement.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function FlowDataElement(role, modelId)
{
	this.role = role;
	if (modelId)
		this.modelId = modelId;
};
FlowDataElement.prototype = new Element();
FlowDataElement.prototype.isFlowDataElement = true;
FlowDataElement.prototype.constructor = FlowDataElement;
FlowDataElement.prototype.type = 'Data Element';
FlowDataElement.prototype.createChild = function createChild(parent, modelId)
{
	var dataValue = this.createChildInstance(modelId);
	return parent.createReference(this, dataValue);
};
	
FlowDataElement.prototype.replaceChild = function replaceChild(parent, value)
{
	var oldValue;
	if (parent.hasElementChangeHandler)
		oldValue = this.getChild(parent);
	parent.createReference(this, value);
	if (parent.hasElementChangeHandler)
	{
		if (oldValue && oldValue.isDataNode)
			oldValue = oldValue.value;
		if (oldValue != value)
		{
			// handle the case of different leaf instances with same leaf value
			if (oldValue == null || value == null || !oldValue.isLeaf || ! value.isLeaf || (oldValue.leafValue != value.leafValue && ! (oldValue.leafValue != null && oldValue.leafValue.equals && oldValue.leafValue.equals(value.leafValue))))
				parent.handleElementChange(this.role, oldValue, value);
		}
	}
};
FlowDataElement.prototype.addAChild = function addAChild(parent, value)
{
	var reference = parent.createReference(this,value);
}
// file: 036-InputSet.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function InputSet(parent, subFlow)
{
	this.parent = parent;
	this.subFlow=subFlow;
	this.element=subFlow;
	this.childConstructor = tersus.repository.get(subFlow.modelId);
	this.triggers = this.childConstructor.prototype.triggers;
	this.setStatus(FlowStatus.NOT_READY_TO_START);
	
};
InputSet.prototype = new FlowNode();
InputSet.prototype.constructor = InputSet;
InputSet.prototype.invoke = function ()
{
	this.explode();
};
InputSet.prototype.createInstance = function()
{
	var instance =  this.subFlow.createChild(this.parent);
	instance.setStatus(FlowStatus.NOT_READY_TO_START);
	return instance;
};
InputSet.prototype.explode = function ()
{
	var instances = [];
	instances.push(this.createInstance()); 
	for (var i=0; i<this.triggers.length; i++)
	{
		var trigger = this.triggers[i];
		var values = this.children[trigger.role];
		var nPrototypes = instances.length;
		for (var j=0; j<nPrototypes; j++)
		{
			//TODO handle the case of repetitive triggers (no cartesian product is needed)
			var prototypeInstance = instances[j];
			if (values)
			{
				for (var k=0; k<values.length; k++)
				{
					var value = values[k];
					if (k==0)
					{
						trigger.replaceChild(prototypeInstance, value);
					}
					else
					{
						var newInstance = this.createInstance();
						for (var l=0; l<i; l++)
						{
							// copy values of preceding triggers
							var previousTrigger = this.triggers[l];
							previousTrigger.replaceChild(newInstance, previousTrigger.getChild(prototypeInstance));
						}
						// set current value
						trigger.replaceChild(newInstance, value);
						instances.push(newInstance);
					}
				}
			}	
		}
	}
	for (var i=0; i<instances.length; i++)
		instances[i].checkReady();
};
InputSet.prototype.addValue = function (role, value)
{
	if (! this.children)
		this.children = {};
	if (! this.children[role])
		this.children[role]= [];
	this.children[role].push(value);
};// file: 037-Link.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function Link(role, source, target)
{	
	this.role = role;
	this.source = new Path(source);
	this.target = new Path(target);
};
Link.prototype.type= 'Flow';
Link.prototype.invoke = function invoke(sourceNode)
{
	var values;
	var parent = sourceNode.parent;
	// get source value
	values = sourceNode.getValues(this.source, 1);
	var targetElement = parent.getElement(this.target.segments[0]);
	if (! targetElement)
		return; // The target element may be null if acccess control excluded it from the user's application
	if (window.trace && values.length > 0)
			window.trace.flow(this, parent, values.length + " values");
	for (var i=0; i<values.length; i++)
	{
		var value = values[i];
		
		if (this.isConstantTrigger)
		{
			var constantElement = targetElement;
			constantElement.createChild(parent);
		}
		else
		{
			if (window.trace)
			{
				window.trace.flowValue(this, parent, value);
			}
			if (targetElement.isSubFlow && targetElement.isRepetitive)
			{
				var inputSet = parent.children[targetElement.role];
				if (!inputSet)
				{
					inputSet = new InputSet(parent, targetElement);
					parent.children[targetElement.role] = inputSet;
				}
				makeAssertion(this.target.segments.length == 2);
				var triggerRole = this.target.segments[1];
				inputSet.addValue(triggerRole, value);
				inputSet.checkReady();
			}
			else
			{
				// set target value
				sourceNode.parent.setValue(this.target, value, this.operation);
			
				if (! targetElement.isRepetitive)
				{
					var targetNode = targetElement.getChild(parent);
					if (targetNode && targetNode.checkReady)
						targetNode.checkReady();
				}
			}
		}
		
	}
};
// file: 038-MethodElement.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 
function MethodElement(owner, role, modelId)
{
	this.owner = owner;
	this.role = role;
	debugAssertion(modelId != null, "Expecting MethodElements to always have a model Id");
	if (modelId)
		this.modelId = modelId;
	else
		this.modelId = BuiltinModels.TEXT_ID; //TODO - remove this once we're sure that modelId is always given
	if (owner.getAdder(role)) // TODO - this should be a validation
		this.isRepetitive = true;
	else
		this.isRepetitive = false;
	
	if (! owner.getRemover(this.role))
		this.isRemovable = false;
};
MethodElement.prototype = new Element();
MethodElement.prototype.constructor = MethodElement;
MethodElement.prototype.type = 'Data Element';
MethodElement.prototype.isMethodElement = true;
MethodElement.prototype.replaceChild = function replaceChild(parent, value)
{
	if (parent.currentWindow) // parent is a real display node
	{
		var setter = parent.getSetter(this.role);
		if (setter == null)
			return;
		if (setter.requiresNodes)
			setter.call(parent, value);
		else
		{
			var  leafValue = value.leafValue;
			setter.call(parent, leafValue);
		}
	}
	else
		Element.prototype.replaceChild.call(this, parent,value);
	
};
MethodElement.prototype.addAChild = function addAChild(parent, value)
{
	if (parent.currentWindow) // parent is a real display node
	{
		var adder = parent.getAdder(this.role);
		if (adder.requiresNodes)
			adder.call(parent, value);
		else
		{
			var  leafValue = value.leafValue;
			adder.call(parent, leafValue);
		}
	}
	else
		Element.prototype.addAChild.call(this, parent,value);
	
};
MethodElement.prototype.removeChild = function removeChild(parent)
{
	if (parent.currentWindow) // parent is a real display node
	{
		var remover = parent.getRemover(this.role);
		if (remover)
			remover.call(parent);
	}
	else
		Element.prototype.removeChild.call(this, parent);
};

MethodElement.prototype.getChild = function getChild(parent)
{
	if (parent.currentWindow || parent.isDataValue) // parent is a real display node or a DataValue
	{
		var getter = parent.getGetter(this.role);
		var value = getter.call(parent);
		if (value || value == 0 || value == '')
		{
			if (value.isNode)
				return value;
			else
			{
				//This is a primitive value - wrap it as a DataValue
				var dataValue = this.createChildInstance();
				dataValue.setLeaf(value);
				return dataValue;
			}
		}
		else
		return null;
	}
	else // parent is a 'temporary' display-data node
	{
		return Element.prototype.getChild.call(this, parent);
	}
};

MethodElement.prototype.getChildren = function getChildren(parent)
{
	if (parent.currentWindow || parent.isDataValue) // parent is a real display node
	{
		var getter = parent.getGetter(this.role);
		var values = getter.call(parent);
		if (values.length > 0 && ! values[0].isNode) // The getter returned raw(leaf) values, not nodes
		{
			var leafValues = values
			values = [];
			if (leafValues.length)
			{
				for (var i=0; i<leafValues.length; i++)
				{
					var leafValue = leafValues[i];
					if (leafValue || leafValue == 0 || leafValue == '')
					{
						var dataValue = this.createChildInstance();
						dataValue.setLeaf(leafValue);
						values.push(dataValue);
					}
				}
			}
		}
		return values;
	}
	else // parent is a 'temporary' display-data node
	{
		return Element.prototype.getChildren.call(this, parent);
	}
	
};
MethodElement.prototype.removeChildren = function removeChildren(parent)
{
	if (parent.currentWindow) // parent is a real display node
	{
		var remover = parent.getRemover(this.role);
		remover.call(parent);
	}
	else // parent is a 'temporary' display-data node
	{
		Element.prototype.removeChildren.call(this, parent);
	}
		
};

// file: 040-DataTypes.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 DATE_FORMAT='mmmm dd yyyy';
if (! tersus) tersus = {};
tersus.DECIMAL_POINT='.';
tersus.THOUSANDS_SEPARATOR=',';
if (! tersus.Messages) tersus.Messages = {};
tersus.Messages.INVALID_NUMBER = 'Please type a valid number';
tersus.Messages.INVALID_DATE = 'Please type a valid date in format '+DATE_FORMAT;

tersus.Nothing = function tersus_Nothing() {};
tersus.Nothing.prototype = new DataValue();
tersus.Nothing.prototype.constructor = tersus.Nothing;
tersus.Nothing.prototype.pluginFunc = tersus.Nothing;
tersus.Nothing.prototype.isLeaf = true;
tersus.Nothing.prototype.isNothing = true;
tersus.Nothing.prototype.toString = function toString()
{
	return '<Nothing>';
}
tersus.Nothing.prototype.serialize = function() 
{
	return 'Nothing';
};
tersus.Nothing.prototype.isInstance = function(value)
{
	return value != null && value.isNothing;
};

tersus.Text = function tersus_Text() {};
tersus.Text.prototype = new DataValue();
tersus.Text.prototype.constructor = tersus.Text;
tersus.Text.prototype.isLeaf = true;
tersus.Text.prototype.pluginFunc = tersus.Text;
tersus.Text.prototype.isText = true;
tersus.Text.prototype.sqlType = 'TEXT';
tersus.Text.prototype.parseInput = function tersus_Text_parseInput(str)
{
	if (str == '')
		return null;
	
	var constructor = this.constructor;
	var text = new constructor();
	text.leafValue = str.replace (/\r/g,'');
	return text;
};
tersus.Text.prototype.isInstance = function(value)
{
	return value != null && value.isText;
};
tersus.Text.prototype.formatString = function tersus_Text_formatString(properties)
{
	if (this.leafValue == null)
		return '';
	var s = this.leafValue.toString();
	return s;
};
tersus.Text.prototype.formatInput = function tersus_Text_formatInput(properties)
{
	if (this.leafValue == null)
		return '';
	var s = this.leafValue.toString();
	return s;
};
tersus.Text.prototype.setLeaf = function(value)
{
	if (value == null)
		this.leafValue == null;
	else this.leafValue = ''+value;
};
tersus.Text.prototype.parseString = tersus.Text.prototype.setLeaf;
tersus.Text.prototype.toString=tersus.Text.prototype.serialize= function(){return this.leafValue;}
tersus.SecretText = tersus.Text.__subclass('tersus.SecretText');
tersus.SecretText.prototype.isSecret=true;

tersus.notEmpty = function (value)
{
	return value != null && value != undefined && value != ' ';
};
tersus.isNumber = function tersus_isNumber(value)
{
	if (value == null)
		return false;
	var str = value.toString();
	var re = /^\s*[-+]?(\d+(\.\d*)?|(\.\d+))([eE][-+]?\d+)?\s*$/;
	return str.match(re);
};
tersus.isNumberWithSeparators = function tersus_isNumber(value)
{
	if (value == null)
		return false;
	var str = value.toString();
	var reStr = '^[-]?((\\d{1,3}(['+tersus.THOUSANDS_SEPARATOR+']\\d{3})*)|\\d*)(['+tersus.DECIMAL_POINT+']\\d*)?$';
	var re = new RegExp(reStr);
	return str.match(re);
};

tersus.Number = function tersus_Number() {};
tersus.Number.prototype = new DataValue();
tersus.Number.prototype.constructor = tersus.Number;
tersus.Number.prototype.isLeaf = true;
tersus.Number.prototype.sqlType = 'NUMERIC';
tersus.Number.prototype.pluginFunc = tersus.Number;
tersus.Number.prototype.isNumber = true;
tersus.Number.prototype.isInstance = function(value)
{
	return value != null && value.isNumber;
};
tersus.Number.prototype.parseInput = function(s, props) 
{
	if (s == null || s == '')
		return null;
	if (props.currencySymbol)
		s = s.replace(props.currencySymbol, '');
	if (s == '')
		return null;
	var s1 = null;
	if (tersus.permissiveNumberInput)
	{
		var s0 = s.replace(/[,]/,'.');
		if (tersus.isNumber(s0))
			s1 = s0;
	}
	
	if (s1 == null) // Not permissive or permissive format doesn't match
	{
		s1 = s.replace(new RegExp('['+tersus.THOUSANDS_SEPARATOR+']','g'),'');
		s1 = s1.replace(new RegExp('['+tersus.DECIMAL_POINT+']','g'),'.');
	}
	var x = parseFloat(s1);
	if (isNaN(x))
		return null;
	var number = new (this.constructor)();
	number.leafValue = x;
	return number;
};

tersus.Number.prototype.serialize = tersus.Number.prototype.toString = function()
{
	if (this.leafValue == null || this.leafValue == undefined)
		return null;
	else
		return this.leafValue.toString();
}

tersus.Number.prototype.validateInput = function (s, props)
{
	if ( s == null || s == '')
		return null;
	if (props.currencySymbol)
		s = s.replace(props.currencySymbol,'');
	if (tersus.permissiveNumberInput)
	{
		var s0 = s.replace(/[,]/,'.');
		if (tersus.isNumber(s0))
			return null;
	}
	
	if (tersus.isNumberWithSeparators(s))
		return null;
	else
		return tersus.Messages.INVALID_NUMBER;
};
tersus.Number.prototype.formatInput = function tersus_Number_formatInput(props)
{
	return this.formatString(props, true);
};
tersus.Number.prototype.formatString = function tersus_Number_formatString(props, noRounding)
{
//	return ''+this.leafValue;
	var formatter = new NumberFormat();
	if (props)
	{
		if (!noRounding && tersus.isNumber(props.decimalPlaces))
			formatter.setPlaces(props.decimalPlaces);
		var useSeparators =  (true == props.useThousandsSeparator);
		formatter.setSeparators(useSeparators, tersus.THOUSANDS_SEPARATOR, tersus.DECIMAL_POINT);
		if (props.currencySymbol && props.currencySymbol != ' ')
		{
			formatter.setCurrencyValue(props.currencySymbol);
			formatter.setCurrency(true);
		}
		if (props.currencyPosition && NumberFormat.prototype[props.currencyPosition])
			formatter.setCurrencyPosition(NumberFormat.prototype[props.currencyPosition]);
	}
	formatter.setNumber(this.leafValue);
	if (props && props.showPercentage)
		return formatter.toPercentage();
	else
		return formatter.toFormatted();
	
	
};
tersus.Number.prototype.parseString = function tersus_Number_parseString (str, element)
{
	var format;
	if (element) format = element.format;
	if (!format) format = this.format;
	if (format)
	{
		try
		{
		 	this.leafValue = gwtParseNumber(str, format);
		}
		catch (e)
		{
			throw {message:'Invalid number format. Format="'+format+'" text="'+str+'"'};
		}
		
	}
	else
	{
		if (! tersus.isNumber(str))
			throw {message:'Invalid number format: text="'+str+'"'}; 
		this.leafValue = parseFloat(str);
	}
};
tersus.Boolean = function tersus_Boolean() {};
tersus.Boolean.prototype = new DataValue();
tersus.Boolean.prototype.constructor = tersus.Boolean;
tersus.Boolean.prototype.isLeaf = true;
tersus.Boolean.prototype.pluginFunc = tersus.Boolean;
tersus.Boolean.prototype.YES_STR = 'Yes';
tersus.Boolean.prototype.TRUE_STR = 'true';
tersus.Boolean.prototype.NO_STR = 'No';
tersus.Boolean.prototype.leafValue = null;
tersus.Boolean.prototype.sqlType = 'CHAR(1)';
tersus.Boolean.prototype.setLeaf = function (v)
{
	if (v == null)
		this.leafValue = null;
	else if (v == true || v == this.YES_STR || v == this.TRUE_STR)
		this.leafValue = true;
	else
		this.leafValue = false;
};
tersus.Boolean.prototype.parseString = function tersus_Boolean_parseString(str, element)
{
	this.leafValue = (str == this.YES_STR || str == this.TRUE_STR);
};

tersus.Boolean.prototype.serialize = function ()
{
	if (this.leafValue == null)
		return 'null';
	else
		return this.leafValue? this.YES_STR: this.NO_STR;
		
};
tersus.Boolean.prototype.toString = tersus.Boolean.prototype.serialize;
tersus.Boolean.prototype.isBooleanValue = true;
tersus.Boolean.prototype.isInstance = function(value)
{
	return value != null && value.isBooleanValue;
};


tersus.DateValue = function tersus_DateValue(dateStr)
{
	if (dateStr)
	{
		var parts = dateStr.split('-');
		this.year = parseInt(parts[0],10);
		this.month = parseInt(parts[1],10);
		this.day = parseInt(parts[2],10);
	}
};
tersus.DateValue.prototype.formatString = function formatString()
{
	return formatDate(this.year, this.month, this.day, DATE_FORMAT);
}
tersus.DateValue.prototype.toString = function toString()
{
	var p2 = tersus.DateAndTime.pad2;
	return this.year+'-'+p2(this.month)+'-'+p2(this.day);
}
tersus.DateValue.prototype.isTersusDate = true;
tersus.DateValue.prototype.toLocalDate = function()
{
	var d= new Date(this.year, this.month-1,this.day);
	return d;
};

tersus.DateValue.prototype.equals = function (other)
{
	return other && other.isTersusDate && other.year == this.year && other.month == this.month && other.day == this.day;
};
function createDate(year, month, day)
{
	var date = new tersus.DateValue();
	date.year = year;
	date.month = month;
	date.day = day;
	return date;
};

tersus.Date = function tersus_Date() {};
tersus.Date.prototype = new DataValue();
tersus.Date.prototype.constructor = tersus.Date;
tersus.Date.prototype.isLeaf = true;
tersus.Date.prototype.sqlType = 'Date';
tersus.Date.prototype.pluginFunc = tersus.Date;
tersus.Date.prototype.isDate = true;
tersus.Date.prototype.isInstance = function(value)
{
	return value != null && value.isDate;
};
tersus.Date.prototype.setLeaf = function tersus_Date_setLeaf(value)
{
	if (value.isTersusDate)
		this.leafValue = value;
	else
		this.leafValue = new tersus.DateValue(value);
};

tersus.Date.prototype.format = 'yyyy-MM-dd';
tersus.Date.prototype.parseString = function tersus_Date_parseString(str, element)
{
	//TODO: implement date format
	var format;
	if (element)
		format = element.format;
	if (!format)
		format = this.format;	
	var time =  gwtParseDateAndTime(str, format, -100000); // Using local time zone
	if (time == -1)
		throw {message:'Invalid date format. Format:"'+format+'" text="'+str+'"'};
	var d = new Date(time); // Using 
	var date = new tersus.DateValue();
	date.year=d.getFullYear();
	date.month=d.getMonth()+1;
	date.day=d.getDate();
	this.leafValue=date;	
};

tersus.Date.prototype.getYear = function tersus_Date_getYear()
{
	if (this.leafValue)
		return this.leafValue.year;
	else
		return null;
};
tersus.Date.prototype.getMonth = function tersus_Date_getMonth()
{
	if (this.leafValue)
		return this.leafValue.month;
	else
		return null;
};
tersus.Date.prototype.getDay = function tersus_Date_getDay()
{
	if (this.leafValue)
		return this.leafValue.day;
	else
		return null;
};

tersus.Date.prototype.formatString = function(props)
{
	var dateFormat = null;
	if (props)
	{
		var dateFormat = props.dateFormat;
	}	
	if (! dateFormat)
		dateFormat = DATE_FORMAT;
	var date = this.leafValue;
	var s;
	if (date == null)
		s =  '';
	else if (date.isTersusDate)
		s=formatDate(date.year, date.month, date.day, dateFormat);
	else
		s = date.toString();
	return s;

};

Date.prototype.equals = function (other)
{
	return other.getTime && this.getTime() == other.getTime();
};

tersus.getDate = function(o)
{
	if (o && o.leafValue)
	{
		if (o.leafValue.isTersusDate)
			return o.leafValue.toLocalDate();
		else 
			return o.leafValue;
	}
		
};

tersus.Date.prototype.parseInput = function (str)
{
	if (str == null || str == '')
		return null;
	var d = parseDate(str, DATE_FORMAT);
	if (d == null)
		return null;
	var constructor = this.constructor;
	var node = new constructor();
	var date = new tersus.DateValue();
	date.year=d.y;
	date.month=d.m+1;
	date.day=d.d;
	node.leafValue=date;
	return node;
};

tersus.Date.prototype.validateInput = function (s, props)
{
	if ( s == null || s == '')
		return null;
	if (parseDate(s, DATE_FORMAT))
		return null;
	else
		return tersus.Messages.INVALID_DATE;
};

tersus.DateAndTime = function tersus_DateAndTime() {};
tersus.DateAndTime.prototype = new DataValue();
tersus.DateAndTime.prototype.constructor = tersus.DateAndTime;
tersus.DateAndTime.prototype.isLeaf = true;
tersus.DateAndTime.prototype.pluginFunc = tersus.DateAndTime;
tersus.DateAndTime.prototype.isDateAndTime = true;
tersus.DateAndTime.prototype.sqlType = 'Timestamp';
tersus.DateAndTime.prototype.isInstance = function(value)
{
	return value != null && value.isDateAndTime;
};
tersus.DateAndTime.prototype.formatString = function(props)
{
	var	dateFormat = DATE_FORMAT;
	var date = this.leafValue;
	var s;
	if (date == null)
		s =  '';
	else
	{
		s = formatDate(date.getFullYear(), date.getMonth()+1, date.getDate(), dateFormat);
		var p2 = tersus.DateAndTime.pad2;
		s += ' - '+p2(date.getHours())+':'+p2(date.getMinutes())+':'+p2(date.getSeconds());
	}
	return s;
};
tersus.DateAndTime.prototype.re = new RegExp("([0-9]+)-([0-9]+)-([0-9]+) ([0-9]+):([0-9]+)(:([0-9]+)(:([0-9]+))?)?");
tersus.DateAndTime.pad2 = function(value)
{
	if (value < 10)
		return '0'+value;
	else
		return value;
};
tersus.DateAndTime.pad3 = function(value)
{
	if (value < 10)
		return '00' + value;
	else if (value < 100)
		return '0' + value;
	else
		return value;
};
tersus.DateAndTime.prototype.format='yyyy-MM-dd HH:mm:ss:SSS';
tersus.DateAndTime.prototype.formatAsGMT=false;
tersus.DateAndTime.prototype.W3CFORMATS = ["yyyy-MM-dd'T'HH:mm:ss.SSSSSSZ","yyyy-MM-dd'T'HH:mm:ssZ","yyyy-MM-dd'T'HH:mmZ"];
tersus.DateAndTime.prototype.W3CFORMAT="yyyy-MM-dd'T'HH:mm:ss.SSSSSSSTZD";
tersus.DateAndTime.prototype.setLeaf = function tersus_Date_setLeaf(value)
{
	if (value.constructor==Date)
		this.leafValue =value;
	else
	{
		var d = this.re.exec(value);
		if (d == null)
			internalError("Failed to parse date/time string '"+value+"'");
		var date =  this.leafValue = new Date(0);
		date.setUTCFullYear(d[1]);
		date.setUTCMonth(d[2]-1);
		date.setUTCDate(d[3]);
		date.setUTCHours(d[4]);
		date.setUTCMinutes(d[5]);
		if (d[7])
			date.setUTCSeconds(d[7]);
		if (d[9])
		date.setUTCMilliseconds(d[9]);
	};	
};
tersus.DateAndTime.prototype.parseString = function tesus_DateAndTime_parseString(str, element)
{
	var format = this.format;
	var formatAsGMT = this.formatAsGMT;
	if (element)
	{
		if (element.format != null)
			format = element.format;
		if (element.formatAsGMT != null)
			formatAsGMT = element.formatAsGMT;
	}
	
	var time;
	if (format == this.W3CFORMAT)
	{
		var s1=str.replace(/z/i,'+00:00');
		for (var i=0;i<this.W3CFORMATS.length;i++)
		{
			time =  gwtParseDateAndTime(s1,this.W3CFORMATS[i], 0);
			if (time != -1)
				break;
		}
	}
	else
	{
		time =  gwtParseDateAndTime(str,format, formatAsGMT?0:-100000);
	}
	if (time == -1)
			throw {message:'Invalid date/time format. Format:"'+format+'" text="'+str+'"'};
	this.leafValue = new Date(time);
};

tersus.DateAndTime.prototype.serialize = function(noquote)
{
	var d = this.leafValue;
	var p2 = tersus.DateAndTime.pad2;
	var p3 = tersus.DateAndTime.pad3;
	var q = noquote?'':'"';
	return d.getUTCFullYear()+"-"+p2(d.getUTCMonth()+1)+"-"+p2(d.getUTCDate())+" "+p2(d.getUTCHours())+":"+p2(d.getUTCMinutes())+":"+p2(d.getUTCSeconds())+":"+p3(d.getUTCMilliseconds());
};

File = function File(ownerField)
{
	this.inputFieldDisplayNode = ownerField;
};
File.prototype = new DataValue();
File.prototype.constructor = File;
File.prototype.pluginFunc = File;

File.prototype.isFile = true;
File.prototype.getFileName = function getFileName()
{
	if (! this.inputFieldDisplayNode)
	{
		if (this.children && this.children['File Name'])
			return this.children['File Name'];
		else
			return null;
	}
	if ( this.inputFieldDisplayNode.currentWindow.closed)
	{
		alert('Tersus Application Error: Reference to a file input field in closed popup window.\n');
		return null;
	}
	if (! this.inputFieldDisplayNode.fileNode)
		return null;
	if (this.inputFieldDisplayNode.fileNode.value)
		return this.inputFieldDisplayNode.fileNode.value;
	else if (this.children)
		return this.children['File Name'];
	else
		return null;
	
};

File.prototype.addGetter('File Name',File.prototype.getFileName);
File.prototype.toString = function toString()
{
	return 'Input File '+ this.getFileName();
};// file: 050-serializer.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function Serializer() {};
Serializer.prototype.DEBUG = false;
Serializer.prototype.Tokens = {};
Serializer.prototype.Tokens.START_OBJECT = '{';
Serializer.prototype.Tokens.END_OBJECT = '}';
Serializer.prototype.Tokens.START_ARRAY = '[';
Serializer.prototype.Tokens.END_ARRAY = ']';
Serializer.prototype.Tokens.OBJECT_SEPARATOR = ',';
Serializer.prototype.Tokens.ARRAY_SEPARATOR = ',';
Serializer.prototype.Tokens.ASSIGN = ':';
Serializer.prototype.Tokens.QUOTE = '"';
Serializer.prototype.addWhitespace = true;

Serializer.prototype.serialize = function serialize(value)
{
	var output = [];
	var stack = [];
	if (value.constructor == Array)
		this._serializeList(value, output, stack);
	else
		this._serializeValue(value, output, stack);
	var serialization = output.join('');
	return serialization;
};
Serializer.prototype._serializeList = function _serializeList(values, output, stack)
{
	output.push(this.Tokens.START_ARRAY);
	for (var i=0; i<values.length; i++)
	{
		if (i>0)
		{
			output.push(this.Tokens.ARRAY_SEPARATOR);
			if (this.addWhitespace)
				output.push('\n');
		}
		this._serializeValue(values[i], output, stack);
	};
	output.push(this.Tokens.END_ARRAY);
};
/*
 Serializes a value hierarchy into an output buffer
 value - the value to be serialized
 output - an array of strings/primitive values serving as the output buffer
*/
Serializer.prototype._serializeValue  = function _serializeValue(value, output, stack)
{
	if (value.isDataNode)
	{
		this._serializeValue(value.value, output, stack);
		return;
	};
	if (value.isNode)
	{
		for (var i=0;i<stack.length;i++)
		{
			var n = stack[i];
			if (n == value)
				return;
		}
	}
	stack.push(value);
	if (value.isSecret && this.hideSecretValues)
	{
		output.push('*****');
	}
	else if (value.isNumber || value.isBooleanValue)
	{
		if (value.leafValue != null)
			output.push(value.leafValue); // No need for quoting or escaping
	}
	else if (value.serialize)
	{
		this._quote(value.serialize(),output);
	}
	else if (value.isNothing)
	{
		output.push(this.Tokens.START_OBJECT);
		output.push(this.Tokens.END_OBJECT);
	}
	else if (value.isLeaf)
	{
		if (value.leafValue || (value.leafValue == 0) || (value.leafValue == '') )
			this._quote(value.leafValue, output);
		else
			output.push('null');
	}
	else
	{
		output.push(this.Tokens.START_OBJECT);
		var childCount = 0;
		if (value.elements)
		{
			for (var role in value.elements)
			{
				var element = value.elements[role];
				if (element.isAncestorReference)
					continue;
				if (element.isSubFlow && ! element.isDisplayElement)
					continue;
				if (element.isRepetitive)
				{
					var childValues = element.getChildren(value);
					if (childValues.length == 0)
						continue;
					if (childCount >0)
						output.push(this.Tokens.OBJECT_SEPARATOR);
					this._quote(element.role, output);
					output.push(this.Tokens.ASSIGN);
					this._serializeList(childValues, output, stack);
				}
				else
				{
					var childValue = element.getChild(value);
					if (! childValue)
						continue;
					if (childValue.isPopup)
						continue; // A pop is realy an action here, and it may be closed.  We don't want to serialize it
					if (childCount >0)
						output.push(this.Tokens.OBJECT_SEPARATOR);
					this._quote(element.role, output);
					output.push(this.Tokens.ASSIGN);
					this._serializeValue(childValue, output, stack);
				}
				childCount++;
			}
		}
		output.push(this.Tokens.END_OBJECT);
	}
	stack.pop();
};

Serializer.prototype._quote= function _quote(value, output)
{
	output.push(this.Tokens.QUOTE);
	var s = value.toString(); 
	var l = s.length;
	var escaped = "";
    for (var i = 0; i < l; i++)
    {
    	c = s.charAt(i);
    	switch (c)
    	{
      		case '\\':
				escaped += '\\\\';
                break;
      		case '\t':
				escaped += '\\t';
                break;
			case '\r':
				escaped += '\\r';
				break;
			case '\n':
				escaped += '\\n';
				break;
			case this.Tokens.QUOTE:
				escaped += "\\"+this.Tokens.QUOTE;
				break;
			default:
				escaped += c;
          }
	}
	//TODO complete escaping (use JSON http://www.crockford.com/JSON)
	
	output.push(escaped);
	output.push(this.Tokens.QUOTE);
};// file: 051-trace.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function startTrace(silent)
{
	closeTrace();
	
	window.trace = new Trace();
	window.trace.open(silent);
};
function showTrace()
{
	var traceWindow = window.open("","traceWindow");
	traceWindow.document.write('<textarea cols="120" rows="80">' + trace.buffer.join('') + '</textarea>');
	window.trace.window = traceWindow;
	setTimeout(function(){traceWindow.focus();}, 100);
};
function stopTrace(silent)
{
	if (!silent)
	{
		if (! window.trace)
			alert('Tracing not started');
		else
			alert("Trace file '"+window.trace.filename+"' closed.");
	}
	closeTrace();
};
function closeTrace()
{
	if (window.trace)
	{
		window.trace.flush();
		if (window.trace.window) window.trace.window.close();
	}
	deleteCookie(Trace.COOKIE_NAME);
	window.trace = null;
};

function initTrace()
{
	var file = getCookie(Trace.COOKIE_NAME);
	if (file)
	{
		alert('Using trace file ' + file);
		window.trace = new Trace();
		window.trace.filename=file;
	}
};
function Trace() 
{

	this.buffer = [];
	this.elementStack = [];
	this.indentation = '';
};

Trace.prototype.serializer = new Serializer();
Trace.prototype.serializer.addWhitespace=false;
Trace.prototype.serializer.hideSecretValues=true;

Trace.COOKIE_NAME='tersus_trace_file';
Trace.prototype.open = function(silent)
{
	this.filename = this.openTraceFile();
	setCookie(Trace.COOKIE_NAME, this.filename);	
	if (!silent)
		alert("Tracing started. The trace file is '"+this.filename +"'");
	
};
Trace.prototype.openTraceFile = function()
{
	var req = createXMLHTTPRequest();
	var url = tersus.rootURL+'/Trace/NewFile';
	req.open('POST', url, false);
	req.send(null);
	if (req.status != 200)
	{
		alert('Failed to create trace file');
		return null;
	}
	else
	{
		var filename=''+req.responseText;
		return filename;
	}
};
Trace.prototype.flush=function()
{
	if (this.buffer.length == 0)
		return;
	var prefix = "// Client side trace\n";
	var content = prefix + this.buffer.join('');
	this.buffer.splice(0, this.buffer.length); // remove all
	var url = tersus.rootURL+'/Trace/'+encodeURIComponent(this.filename);
	var req = createXMLHTTPRequest();
/*	req.onreadystatechange= function()
	{
		if (req.readyState == 4 && req.status != 200)
		{
			alert('Failed to write to trace file');
		}
		
	};
	req.open('POST', url, true);
	*/
	req.open('POST', url, false);
	req.send(content);
	if (req.readyState == 4 && req.status != 200)
	{
		alert('Failed to write to trace file');
	}
	
};
Trace.prototype.close = function()
{
	this.traceWindow.close();
};
Trace.prototype.STARTED= 'Started';
Trace.prototype.FINISHED= 'Finished';
Trace.prototype.WAITING= 'Waiting';
Trace.prototype.RESUMED= 'Resumed';
Trace.prototype.PAUSED = 'Paused';
Trace.prototype.RESPONDED = 'Responded';
Trace.prototype.INVOKED = 'Invoked';
Trace.prototype.VALUE = 'Value';
Trace.prototype.ACTIVATED = 'Activated';
Trace.prototype.SET = 'Set';
Trace.prototype.ACCUMULATED = 'Accumulated';
Trace.prototype.REMOVED = 'Removed';
Trace.prototype.CREATED = 'Created';
Trace.prototype.CLICKED = 'Clicked';
Trace.prototype.SQL = 'SQL';

Trace.prototype.location = function (node, element, suffix)
{
	return new tersus.RuntimeLocation(node,element, suffix);
};
Trace.prototype.started = function(node) 
{
	if (node.isDisplayNode && ! node.isButton)
		return; // We don't really care about 'start' of display nodes
	this.trace(node.type, this.STARTED, this.location(node), this.getTriggerValues(node));
};
Trace.prototype.getTriggerValues = function(node)
{
	var details=[];
	var count = 0;
	if (node.triggers)
	{
		details.push('{');
		for (var i=0; i<node.triggers.length; i++)
		{
			var trigger = node.triggers[i];
			var serialization = null;
			if (trigger.isRepetitive)
			{
				var values  = trigger.getChildren(node);
				if (values.length > 0)
					serialization = this.serializer.serialize(values);
			}
			else
			{
				var value = trigger.getChild(node);
				if (value != null)
					serialization = this.serializer.serialize(value);
			}
			if (serialization != null)
			{
				
				if (count>0)
				{
					details.push(',');
				}
				++count;
				this.serializer._quote(trigger.role, details);
				details.push(':');
				details.push(serialization);
			}
		}
		details.push('}');
	}
	return details.join('');
};
Trace.prototype.resumed = function(node) 
{
	//No tracing for 'resumed' - nothing interesting here (?)
};
Trace.prototype.finished = function(node)
{
	this.trace(node.type, this.FINISHED, this.location(node));
};
Trace.prototype.waiting = function(node)
{
	//No tracing for 'waiting' - nothing interesting here (?)
};
Trace.prototype.charged = function(node, exit, value)
{
	var details = null;
	if (value != null)
	{
		details = this.serializer.serialize(value);
	}
	var event = this.SET;
	if (exit.isRepetitive)
		event = this.ACCUMULATED;
	this.trace(exit.type, event, this.location(node,exit),details,null);
	
};
Trace.prototype.sql = function(node, sql, values)
{
	var l = [];
	if (values)
	{
		l.push(' values: ');
		for (var i=0;i<values.length;i++)
		{
			if (i>0)
				l.push(',');
			var v = values [i];
			if (v == null)
				l.push('null');
			else if (typeof(v) == 'string')
			{
				this.serializer._quote(v,l);
			}
			else
				l.push(v);
		}
	}
	var valuesStr = l.join('');
	this.trace(null,this.SQL,this.location(node),"Executing statement: '" + sql + "'" + valuesStr, null);
}

Trace.prototype.paused = function(node)
{
	//No tracing for 'pause' - we want the trace to simulate the synchronous behavior
};
Trace.prototype.flow = function(flow,parent, details)
{
	this.trace(flow.type, this.ACTIVATED, this.location(parent,flow), details);
};

Trace.prototype.flowValue = function(flow, parent, value)
{
	var lastElement = flow.target.getElement(parent);
	if (!lastElement)
		return;//This can happen when elements are excluded from the client-side model because of permissions
	var eventType = '?';
	if (flow.operation == Operation.REPLACE)
		eventType = this.SET;
	else if (flow.operation == Operation.REMOVE)
		eventType = this.REMOVED;
	else if (flow.operation == Operation.ADD)
		eventType = this.ACCUMULATED;
	var pp = this.location(parent).path.str;
	var displayPath = pp ? pp + "/"+flow.target : flow.target;
	this.trace(lastElement.type, eventType, this.location(parent,flow,"Value"),this.serializer.serialize(value),null,displayPath);
};
Trace.prototype.link = function(flow, filename, comment)
{
	this.trace(null,this.LINK, this.location(flow), "'"+filename+"' - "+ comment);
};

Trace.prototype.invoked = function(node)
{
	var details = null;
	this.trace(node.type, this.INVOKED, this.location(node), this.getTriggerValues(node));
};

Trace.prototype.activated = function(node)
{
	this.trace(node.type, this.ACTIVATED, this.location(node.parent, node.element), this.serializer.serialize(node.value));
}

Trace.prototype.responded = function(node)
{
	this.trace(node.type, this.RESPONDED, this.location(node));
};

Trace.prototype.trace = function(objectType, event, location, details, comment, displayPath)
{
	if (location && (location.baseModelId != this.lastBaseModelId || this.buffer.length==0) )
	{
		// We need to write out the base model id either if this is the first record or if the base model id changed
		this.buffer.push('@');
		this.buffer.push(location.baseModelId);
		this.buffer.push('\n');
		this.lastBaseModelId = location.baseModelId;
	}
	if (!objectType)
		objectType = "-";
	if (!details)
		details = "-";
	if (! comment)
		comment = "-";
	var path = location.path?location.path.str:null;
	if (! path)
		path = "-";
	if (!displayPath)
		displayPath = "-";
	var now=new Date();
	var time = Date.UTC(now.getUTCFullYear(), now.getUTCMonth(), now.getUTCDate(), now.getUTCHours(), now.getUTCMinutes(), now.getUTCSeconds(), now.getUTCMilliseconds());
	
	this.buffer.push(objectType,"\t",event,"\t",path,"\t");
	this.addEscaped(details);
	this.buffer.push("\t",comment,"\t",displayPath,"\t",time,"\n");
};
Trace.prototype.addEscaped = function (s)
{
	var s1 = s.replace(/\n/g,'\\n').replace(/\r/g,'\\r').replace(/\t/g,'\\t');
	this.buffer.push(s1);
};
Trace.prototype.event = function(node, event)
{
	if (event == Events.ON_CLICK)
	{
		var location = this.location(node);
		var displayPath = location.path.str;
		location.path = null; // We send a location with an empty path to 'force' the event to appear as a top level event
		this.trace(node.type,this.CLICKED,location,null/*details*/,null/*comment*/,displayPath); 
	}
};

initTrace();// file: 052-loader.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.PERSPECTIVE = 'Perspective';
tersus.VIEW = 'View';
tersus.preloadAll = true;
tersus.onResize = function()
{
	tersus.async.exec(tersus._onResize, false /* don't wait */);
};
tersus._onResize = function onResize()
{
	if (window.currentRootDisplayNode)
		window.currentRootDisplayNode.onResize();
};
function getPerspective(name)
{
	for (var i=0; i<window.perspectives.length; i++)
	{
		if (window.perspectives[i].name == name)
		{
			return window.perspectives[i];
		}
	}
	return null;
};
function getView(name)
{
	for (var i=0; i< window.perspective.views.length; i++)
	{
		var v = perspective.views[i];
		if (v.name == name)
		{
			return  v;
		}
	}
	return null;
};
tersus.setPerspectiveAndView = function()
{
	var perspectiveName = tersus.searchParams[tersus.PERSPECTIVE];
	var viewName = tersus.searchParams[tersus.VIEW];
	window.perspective = getPerspective(perspectiveName);
	if (!window.perspective)
		window.perspective = window.perspectives[0];
	window.view = getView(viewName);
	if (!window.view)
		window.view = window.perspective.views[0];
};
tersus.load = function load()
{
	var cache = window.applicationCache;
	if (cache) 
	{
		if (tlog) tlog('tersus.load: cacheStatus='+cache.status);
		switch (cache.status)
		{
			case cache.UPDATEREADY:
				tersus.reloadApplication(); 
				return;
			case cache.CHECKING:
			case cache.DOWNLOADING:
			case cache.UPDATING:
			cache.onupdateready = function(){if(tlog) tlog('OnUpdateReady');setTimeout(tersus.reloadApplication,50);};
			cache.oncached=function(){if(tlog) tlog('OnCached');applicationCache.onnoupdate=null;tersus.load();};
			cache.onnoupdate=function(){if(tlog) tlog('OnUpdate');applicationCache.onnoupdate=null;tersus.load();};
			cache.onerror=function(){if(tlog) tlog('OnError');applicationCache.onerror=null;tersus.load();};
			return;	
		}
	}
	if (tlog) tlog('load()');
	tersus.init();
	if (tlog) tlog('init() finished');
	tersus.progress.set('Loading application ...');
	if (tlog) 
		tlog ('tersus.lastTimestamp:'+tersus.lastTimestamp)
	if (!tersus.lastTimestamp)
		tersus.reloadView();
	else if (window.localStorage && tersus.useLocalStorage && navigator.onLine == false && !tersus.isLocalHost(document.domain))
	{
		tersus.reloadView();
	}
	else
	{
		tersus.getTimestamp(tersus.handleLoadTimestampResponse);
	}
	if (tlog) tlog('load() returns');
	
};
tersus.handleLoadTimestampResponse = function()
{
	var req = tersus.getTimestampReq;
	if (tlog) tlog('Ready state:'+req.readyState);
	if (req.readyState != 4)
		return;
	if (tlog) tlog('Status:'+req.status);
		
	if (req.status == 200 || req.responseText)
	{
		var r = tersus.parseTimestampResponse(req);
		var currentTimestamp = r.timestamp;
		window.userName = r.userName;
		tersus.setLastTimestamp(currentTimestamp);
		tersus.reloadView();
	}
	else if(req.status == 0 && window.localStorage && tersus.useLocalStorage)
		tersus.reloadView();
	else if (req.status == 0 && navigator.onLine == false)
		tersus.reloadView();
};
tersus.reloadView = function()
{
	tersus.setPerspectiveAndView();
	tersus.loadView(document.getElementById('tersus.content'),view);
};
tersus.loader = {};
tersus.loadView = function loadView(targetElement, view)
{
	tersus.progress.set('Loading view - '+view.name);
	tersus.async.exec(function(){ tersus._loadView(targetElement, view);},false);
}
tersus._loadView = function loadView(targetElement, view)
{
	tersus.progress.set('Loading view - '+view.name);
	tersus.loader.currentView = view;
	tersus.loader.targetElement = targetElement;
	if (window.rootDisplayNode)
	{
		window.rootDisplayNode.destroyHierarchy();
		window.rootDisplayNode = null;
	}
	if (window.initPopupCalendar && ! tersus.popupCalendarInitialized)
	{
		tersus.popupCalendarInitialized = true;
		initPopupCalendar();
	}
	tersus.viewPath = view.path;
	if (tersus.repository.checkLoaded(view.id))
		tersus.loader.load();
	else
		if (tersus.preloadAll)
			tersus.repository.loadPackages([""],tersus.loader.load);
		else
			tersus.repository.loadPackages([tersus.getPackageId(view.id)],tersus.loader.load);
	tersus.alertMode = {interactive:true};
}
tersus.loader.load = function load()
{
	var view = tersus.loader.currentView;
	var targetElement = tersus.loader.targetElement;
	var root = window.rootDisplayNode;
	try
	{
		targetElement.innerHTML = '';
		if (!tersus.repository.isLoaded(view.id))
		{
			targetElement.innerHTML = 'View '+view.id+' not found';
			tersus.progress.clear();
			return;
		}
		if (! root)
		{
			var constructor = tersus.repository.get(view.id)
			window.currentRootDisplayNode = window.rootDisplayNode = root =  new constructor();
			root.mainWindow = window;
			root.currentWindow = window;
		}
		if (root.preload(tersus.loader.load))
		{
			root.create();
			root.doStart();
			if (window.trace)
				window.trace.flush();
			
			var topNode = root.wrapperNode ? root.wrapperNode : root.viewNode;	
			if (!topNode.parentNode || !topNode.parentNode.tagName)
				targetElement.appendChild(topNode);
			tersus.progress.clear();
			if (!is_touch_device())
				window.setTimeout(function(){window.rootDisplayNode.focusOnFirstField();},0);
		}
	}
	catch (e)
	{
			targetElement.innerHTML = escapeHTML(e.message);
	}
}
tersus.switchView = function(perspective,view,parameters)
{
	if (!perspective && window.perspectives.length > 1)
		perspective = window.perspective.name;
	if (perspective)
		var url = tersus.baseURL+'#Perspective='+encodeURIComponent(perspective)+'&View='+encodeURIComponent(view);
	else
		var url = tersus.baseURL+'#View='+encodeURIComponent(view);
	if (parameters)
		url+='&'+parameters;
	tersus.gotoURL(url, false, true);

};

function openTools()
{
	var w = 250;
	var h = 220;
	var leftPx = window.screenX+window.innerWidth;
	if (leftPx  > screen.availWidth -w -10)
		leftPx = screen.availWidth -w -10;
	if (leftPx<0) leftPx=0;
		
	var topPx = window.screenY;
	var popup = window.open("tools.html","Tools","top="+topPx+",left="+leftPx+"toolbar=no,location=no,status=no,menubar=no,width=" + w + ",height=" + h); 
	popup.focus();
};
function showContent()
{
	var w = window.open("","","");
	w.document.write('<textarea cols="80" rows="50">'+document.getElementById("main.content").innerHTML+"</textarea>");
	setTimeout(function(){w.focus();},0);
};


function removeSuffix(str, separator)
{
	var index = str.indexOf(separator);
	if (index >=0)
		return str.substring(0,index);
	else
		return str;
};
	
// Extract rootURL and baseURL
var fullURL = window.location.href;
var lastIndex = fullURL.lastIndexOf('/');
tersus.rootURL = fullURL.substring(0, lastIndex+1);
tersus.firstURL = fullURL;

tersus.baseURL = removeSuffix(removeSuffix(fullURL,'#'),'?');
// file: 060-sha256.js
/* A JavaScript implementation of the Secure Hash Algorithm, SHA-256
 * Version 0.3 Copyright Angel Marin 2003-2004 - http://anmar.eu.org/
 * Distributed under the BSD License
 * Some bits taken from Paul Johnston's SHA-1 implementation
 */
var chrsz = 8;  /* bits per input character. 8 - ASCII; 16 - Unicode  */
function safe_add (x, y) {
  var lsw = (x & 0xFFFF) + (y & 0xFFFF);
  var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
  return (msw << 16) | (lsw & 0xFFFF);
}
function S (X, n) {return ( X >>> n ) | (X << (32 - n));}
function R (X, n) {return ( X >>> n );}
function Ch(x, y, z) {return ((x & y) ^ ((~x) & z));}
function Maj(x, y, z) {return ((x & y) ^ (x & z) ^ (y & z));}
function Sigma0256(x) {return (S(x, 2) ^ S(x, 13) ^ S(x, 22));}
function Sigma1256(x) {return (S(x, 6) ^ S(x, 11) ^ S(x, 25));}
function Gamma0256(x) {return (S(x, 7) ^ S(x, 18) ^ R(x, 3));}
function Gamma1256(x) {return (S(x, 17) ^ S(x, 19) ^ R(x, 10));}
function core_sha256 (m, l) {
    var K = new Array(0x428A2F98,0x71374491,0xB5C0FBCF,0xE9B5DBA5,0x3956C25B,0x59F111F1,0x923F82A4,0xAB1C5ED5,0xD807AA98,0x12835B01,0x243185BE,0x550C7DC3,0x72BE5D74,0x80DEB1FE,0x9BDC06A7,0xC19BF174,0xE49B69C1,0xEFBE4786,0xFC19DC6,0x240CA1CC,0x2DE92C6F,0x4A7484AA,0x5CB0A9DC,0x76F988DA,0x983E5152,0xA831C66D,0xB00327C8,0xBF597FC7,0xC6E00BF3,0xD5A79147,0x6CA6351,0x14292967,0x27B70A85,0x2E1B2138,0x4D2C6DFC,0x53380D13,0x650A7354,0x766A0ABB,0x81C2C92E,0x92722C85,0xA2BFE8A1,0xA81A664B,0xC24B8B70,0xC76C51A3,0xD192E819,0xD6990624,0xF40E3585,0x106AA070,0x19A4C116,0x1E376C08,0x2748774C,0x34B0BCB5,0x391C0CB3,0x4ED8AA4A,0x5B9CCA4F,0x682E6FF3,0x748F82EE,0x78A5636F,0x84C87814,0x8CC70208,0x90BEFFFA,0xA4506CEB,0xBEF9A3F7,0xC67178F2);
    var HASH = new Array(0x6A09E667, 0xBB67AE85, 0x3C6EF372, 0xA54FF53A, 0x510E527F, 0x9B05688C, 0x1F83D9AB, 0x5BE0CD19);
    var W = new Array(64);
    var a, b, c, d, e, f, g, h, i, j;
    var T1, T2;
    /* append padding */
    m[l >> 5] |= 0x80 << (24 - l % 32);
    m[((l + 64 >> 9) << 4) + 15] = l;
    for ( var i = 0; i<m.length; i+=16 ) {
        a = HASH[0]; b = HASH[1]; c = HASH[2]; d = HASH[3]; e = HASH[4]; f = HASH[5]; g = HASH[6]; h = HASH[7];
        for ( var j = 0; j<64; j++) {
            if (j < 16) W[j] = m[j + i];
            else W[j] = safe_add(safe_add(safe_add(Gamma1256(W[j - 2]), W[j - 7]), Gamma0256(W[j - 15])), W[j - 16]);
            T1 = safe_add(safe_add(safe_add(safe_add(h, Sigma1256(e)), Ch(e, f, g)), K[j]), W[j]);
            T2 = safe_add(Sigma0256(a), Maj(a, b, c));
            h = g; g = f; f = e; e = safe_add(d, T1); d = c; c = b; b = a; a = safe_add(T1, T2);
        }
        HASH[0] = safe_add(a, HASH[0]); HASH[1] = safe_add(b, HASH[1]); HASH[2] = safe_add(c, HASH[2]); HASH[3] = safe_add(d, HASH[3]); HASH[4] = safe_add(e, HASH[4]); HASH[5] = safe_add(f, HASH[5]); HASH[6] = safe_add(g, HASH[6]); HASH[7] = safe_add(h, HASH[7]);
    }
    return HASH;
}
function str2binb (str) {
  var bin = Array();
  var mask = (1 << chrsz) - 1;
  for(var i = 0; i < str.length * chrsz; i += chrsz)
    bin[i>>5] |= (str.charCodeAt(i / chrsz) & mask) << (24 - i%32);
  return bin;
}
function binb2hex (binarray) {
  var hexcase = 0; /* hex output format. 0 - lowercase; 1 - uppercase */
  var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
  var str = "";
  for (var i = 0; i < binarray.length * 4; i++) {
    str += hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8+4)) & 0xF) + hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8  )) & 0xF);
  }
  return str;
}
function hex_sha256(s){return binb2hex(core_sha256(str2binb(s),s.length * chrsz));}
// file: 061-popcalendar.js
//	written	by Tan Ling	Wee	on 2 Dec 2001
//	last updated 28 Jul 2003
//	email : fuushikaden@yahoo.com
//	website : www.pengz.com
//	TabSize: 4
//
//	modified by ALQUANTO 30 July 2003 - german language included.
//									  - modified languageLogic with the ISO-2letter-strings
//									  - changes in in showCalendar: defaultLanguage is already set...
//									  - js and html corrected... more xhtml-compliant... simplier css
//	email: popcalendar@alquanto.de
//
//	modified by PinoToy 25 July 2003  - new logic for multiple languages (English, Spanish and ready for more).
//									  - changes in popUpMonth & popDownMonth methods for hidding	popup.
//									  - changes in popDownYear & popDownYear methods for hidding	popup.
//									  - new logic for disabling dates in	the past.
//									  - new method showCalendar, dynamic	configuration of language, enabling	past & position.
//									  - changes in the styles.
//	email  : pinotoy@yahoo.com
// No Copyright notice - assumed to be public domain
// Modified by Tersus, October 2004 - Image locations + Week starts on Monday
//                       January 2005 - Extract 'formatDate' function
//                                    - Changed 'popupCaledar' function to accept a callback in addition to the target node.
//                       May 2006     - changed 'popupCaledar' function to take into account scrolling and window size (added functions visible_size, screen_position at the end of the file)
//						 Aug 2006     - fixed formatDate(y,m,d, format) - used to add 1 to the month when outputing in numeric format
//						 Jan 2008     - added Hebrew translation
//                                    - added setCalendarStartOnSunday() to enable externalizing the setup.
//						 Jun 2008     - fixed arrows and drop-downs from right-to-left mode
//                       Sep 2008     - Using 'display:none' instead of 'visibility:hidden' to prevent scrolling problems
//						 Aug 2009	  - Refreshing current date and time when showing calendar	
//                                    - Extracted parseDate as a separate function
//						 Sep 2009     - Dutch localization (translation by JP van de Giessen)
//                       Dec 2009     - Adjustments for standards-mode rendering
//						 Jan 2010     - en_GB, en_US, de_DE and pl_PL localization (translation by NeoPrimus sp. z o.o)
//                       Sep 2011     - Fixing positioning of calendar when content is scrolled. Current version depends on Tersus external function getScroll (which in turn depends on jQuery)
	var language = 'en';	// Default Language: en - english ; es - spanish; de - german
	var enablePast = 1;		// 0 - disabled ; 1 - enabled
	var fixedX = -1;		// x position (-1 if to appear below control)
	var fixedY = -1;		// y position (-1 if to appear below control)
	var showWeekNumber = 1;	// 0 - don't show; 1 - show
	var showToday = 1;		// 0 - don't show; 1 - show
	var imgDir = 'images/calendar/';		// directory for images ... e.g. var imgDir="/img/"
	var dayName = '';

	var gotoString = {
		en : 'Go To Current Month',
		en_GB : 'Go To Current Month',
		en_US : 'Go To Current Month',
		es : 'Ir al Mes Actual',
		de : 'Gehe zu aktuellem Monat',
		de_DE : 'Gehe zu aktuellem Monat',
		nl : 'Ga naar huidige Maand',
		he : 'עבור לחודש נוכחי',
		pl_PL : 'Bieżący miesiąc'
	};
	var todayString = {
		en : 'Today is',
		en_GB : 'Today is',
		en_US : 'Today is',
		es : 'Hoy es',
		de : 'Heute ist',
		de_DE : 'Heute ist',
		nl : 'Vandaag is',
		he : 'היום - יום',
		pl_PL : 'Dzisiaj jest'
	};
	var weekString = {
		en : 'Wk',
		en_GB : 'Wk',
		en_US : 'Wk',
		es : 'Sem',
		de : 'KW',
		de_DE : 'KW',
		nl : 'Wk',
		he : 'שבוע',
		pl_PL : 'Tydz'
	};
	var scrollLeftMessage = {
		en : 'Click to scroll to previous month. Hold mouse button to scroll automatically.',
		en_GB : 'Click to scroll to previous month. Hold mouse button to scroll automatically.',
		en_US : 'Click to scroll to previous month. Hold mouse button to scroll automatically.',
		es : 'Presione para pasar al mes anterior. Deje presionado para pasar varios meses.',
		de : 'Klicken um zum vorigen Monat zu gelangen. Gedr?ckt halten, um automatisch weiter zu scrollen.',
		de_DE : 'Klicken um zum vorigen Monat zu gelangen. Gedr?ckt halten, um automatisch weiter zu scrollen.',
		nl : 'Klik om naar de vorige maand te gaan. Ingedrukt houden om automatisch verder te scrollen.',
		he : 'לחץ למעבר לחודש הקודם. לחץ והחזק לגלילה אוטומטית.',
		pl_PL : 'Kliknij aby przejść do poprzedniego miesiąca. Przytrzymaj przycisk myszy aby przewijać automatycznie.'
	};
	var scrollRightMessage = {
		en : 'Click to scroll to next month. Hold mouse button to scroll automatically.',
		en_GB : 'Click to scroll to next month. Hold mouse button to scroll automatically.',
		en_US : 'Click to scroll to next month. Hold mouse button to scroll automatically.',
		es : 'Presione para pasar al siguiente mes. Deje presionado para pasar varios meses.',
		de : 'Klicken um zum nächsten Monat zu gelangen. Gedr?ckt halten, um automatisch weiter zu scrollen.',
		de_DE : 'Klicken um zum nächsten Monat zu gelangen. Gedr?ckt halten, um automatisch weiter zu scrollen.',
		nl : 'Klik om naar de volgende maand te gaan. Ingedrukt houden om automatisch verder te scrollen.',
		he : 'לחץ למעבר לחודש הבא. לחץ והחזק לגלילה אוטומטית.',
		pl_PL : 'Kliknij aby przejść do następnego miesiąca. Przytrzymaj przycisk myszy aby przewijać automatycznie.'
	};
	var selectMonthMessage = {
		en : 'Click to select a month.',
		en_GB : 'Click to select a month.',
		en_US : 'Click to select a month.',
		es : 'Presione para seleccionar un mes.',
		de : 'Klicken um Monat auszuwählen.',
		de_DE : 'Klicken um Monat auszuwählen',
		nl : 'Klik om maand te selecteren.',
		he : 'לחץ לבחירת חודש.',
		pl_PL : 'Kliknij aby wybrać miesiąc.'
	};
	var selectYearMessage = {
		en : 'Click to select a year.',
		en_GB : 'Click to select a year.',
		en_US : 'Click to select a year.',
		es : 'Presione para seleccionar un ano.',
		de : 'Klicken um Jahr auszuwählen.',
		de_DE : 'Klicken um Jahr auszuwählen',
		nl : 'Klik om jaar te selecteren.',
		he : 'לחץ לבחירת שנה.',
		pl_PL : 'Kliknij aby wybrać rok.'
	};
	var selectDateMessage = {		// do not replace [date], it will be replaced by date.
		en : 'Select [date] as date.',
		en_GB : 'Select [date] as date.',
		en_US : 'Select [date] as date.',
		es : 'Seleccione [date] como fecha.',
		de : 'Wähle [date] als Datum.',
		de_DE : 'Wähle [date] als Datum.',
		nl : 'Kies [date] als Datum.',
		he : 'בחבר ב [date] כתאריך.',
		pl_PL : 'Wybierz [date] jako datę.'
	};
	var	monthName = {
		en : new Array('January','February','March','April','May','June','July','August','September','October','November','December'),
		en_GB : new Array('January','February','March','April','May','June','July','August','September','October','November','December'),
		en_US : new Array('January','February','March','April','May','June','July','August','September','October','November','December'),
		es : new Array('Enero','Febrero','Marzo','Abril','Mayo','Junio','Julio','Agosto','Septiembre','Octubre','Noviembre','Diciembre'),
		de : new Array('Januar','Februar','März','April','Mai','Juni','Juli','August','September','Oktober','November','Dezember'),
		de_DE : new Array('Januar','Februar','März','April','Mai','Juni','Juli','August','September','Oktober','November','Dezember'),
		nl : new Array('Januari','Februari','Maart','April','Mei','Juni','Juli','Augustus','September','Oktober','November','December'),
		he : new Array('ינואר','פברואר','מרץ','אפריל','מאי','יוני','יולי','אוגוסט','ספטמבר','אוקטובר','נובמבר','דצמבר'),
		pl_PL : new Array('Styczeń','Luty','Marzec','Kwiecień','Maj','Czerwiec','Lipiec','Sierpień','Wrzesień','Pańdziernik','Listopad','Grudzień')
	};
	var	monthName2 = {
		en : new Array('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'),
		en_GB : new Array('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'),
		en_US : new Array('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'),
		es : new Array('ENE','FEB','MAR','ABR','MAY','JUN','JUL','AGO','SEP','OCT','NOV','DIC'),
		de : new Array('JAN','FEB','MRZ','APR','MAI','JUN','JUL','AUG','SEP','OKT','NOV','DEZ'),
		de_DE : new Array('JAN','FEB','MRZ','APR','MAI','JUN','JUL','AUG','SEP','OKT','NOV','DEZ'),
		nl : new Array('Jan','Feb','Mar','Apr','Mei','Jun','Jul','Aug','Sep','Okt','Nov','Dec'),
		he : new Array('ינו','פבר','מרץ','אפר','מאי','יונ','יול','אוג','ספט','אוק','נוב','דצמ'),
		pl_PL : new Array('Sty','Lut','Mar','Kwi','Maj','Cze','Lip','Sie','Wrz','Paz','Lis','Gru')
	};

	function cal_pixels(n)
	{
		return n + 'px';
	}
	function setCalendarStartOnSunday(startOnSunday)
	{
		if (startOnSunday) {
			startAt = 0;
			dayName = {
				en : new Array('Sun','Mon','Tue','Wed','Thu','Fri','Sat'),
				en_GB : new Array('Sun','Mon','Tue','Wed','Thu','Fri','Sat'),
				en_US : new Array('Sun','Mon','Tue','Wed','Thu','Fri','Sat'),
				es : new Array('Dom','Lun','Mar','Mie','Jue','Vie','Sab'),
				de : new Array('So','Mo','Di','Mi','Do','Fr','Sa'),
				de_DE : new Array('So','Mo','Di','Mi','Do','Fr','Sa'),
				nl : new Array('Zo','Ma','Di','Wo','Do','Vr','Za'),
				he : new Array('א','ב','ג','ד','ה','ו','ש'),
				pl_PL : new Array('N','Pon','Wt','Śr','Czw','Pt','Sob')
			};
		}
		else {
			startAt = 1;
			dayName = {
				en : new Array('Mon','Tue','Wed','Thu','Fri','Sat','Sun'),
				en_GB : new Array('Mon','Tue','Wed','Thu','Fri','Sat','Sun'),
				en_US : new Array('Mon','Tue','Wed','Thu','Fri','Sat','Sun'),
				es : new Array('Lun','Mar','Mie','Jue','Vie','Sab','Dom'),
				de : new Array('Mo','Di','Mi','Do','Fr','Sa','So'),
				de_DE : new Array('Mo','Di','Mi','Do','Fr','Sa','So'),
				nl : new Array('Ma','Di','Wo','Do','Vr','Za','Zo'),
				he : new Array('ב','ג','ד','ה','ו','ש','א'),
				pl_PL : new Array('Pon','Wt','Śr','Czw','Pt','Sob','N')
			};
		}
	};
	setCalendarStartOnSunday(false);

function getCalendarDirection()
{
	var node = document.getElementById('calendar');
	if (node.currentStyle)
		return node.currentStyle['direction'];
	else if (window.getComputedStyle)
		return getComputedStyle(node,null).getPropertyValue('direction');
};
	
function setToday()
{
 	today    = new Date();
 	dateNow  = today.getDate();
	monthNow = today.getMonth();
	yearNow  = today.getFullYear();
};
	var crossobj, crossMonthObj, crossYearObj, monthSelected, yearSelected, dateSelected, omonthSelected, oyearSelected, odateSelected, monthConstructed, yearConstructed, intervalID1, intervalID2, timeoutID1, timeoutID2, targetNode, valueHandler, ctlNow, dateFormat, nStartingYear, selDayAction, isPast;
	var visYear  = 0;
	var visMonth = 0;
	var bPageLoaded = false;
	var ie  = document.all;
	var dom = document.getElementById;
	var ns4 = document.layers;
	setToday();
	
	var imgsrc   = new Array('drop1.gif','drop2.gif','left1.gif','left2.gif','right1.gif','right2.gif');
	var img      = new Array();
	var bShow    = false;

	/* hides <select> and <applet> objects (for IE only) */
	function hideElement( elmID, overDiv ) {
		if(ie) {
			for(i = 0; i < document.all.tags( elmID ).length; i++) {
				obj = document.all.tags( elmID )[i];
				if(!obj || !obj.offsetParent) continue;

				// Find the element's offsetTop and offsetLeft relative to the BODY tag.
				objLeft   = obj.offsetLeft;
				objTop    = obj.offsetTop;
				objParent = obj.offsetParent;

				while(objParent.tagName.toUpperCase() != 'BODY') {
					objLeft  += objParent.offsetLeft;
					objTop   += objParent.offsetTop;
					objParent = objParent.offsetParent;
				}

				objHeight = obj.offsetHeight;
				objWidth  = obj.offsetWidth;

				if((overDiv.offsetLeft + overDiv.offsetWidth) <= objLeft);
				else if((overDiv.offsetTop + overDiv.offsetHeight) <= objTop);
				/* CHANGE by Charlie Roche for nested TDs*/
				else if(overDiv.offsetTop >= (objTop + objHeight + obj.height));
				/* END CHANGE */
				else if(overDiv.offsetLeft >= (objLeft + objWidth));
				else {
					obj.style.display = 'none';
				}
			}
		}
	}

	/*
	* unhides <select> and <applet> objects (for IE only)
	*/
	function showElement(elmID) {
		if(ie) {
			for(i = 0; i < document.all.tags( elmID ).length; i++) {
				obj = document.all.tags(elmID)[i];
				if(!obj || !obj.offsetParent) continue;
				obj.style.display = '';
			}
		}
	}

	function HolidayRec (d, m, y, desc) {
		this.d = d;
		this.m = m;
		this.y = y;
		this.desc = desc;
	}

	var HolidaysCounter = 0;
	var Holidays = new Array();

	function addHoliday (d, m, y, desc) {
		Holidays[HolidaysCounter++] = new HolidayRec (d, m, y, desc);
	}

	if (dom) {
		for	(i=0;i<imgsrc.length;i++) {
			img[i] = new Image;
			img[i].src = imgDir + imgsrc[i];
		}
		document.write ('<div onclick="bShow=true" id="calendar" style="z-index:+999;position:absolute;display:none;"><table width="'+((showWeekNumber==1)?250:220)+'" style="font-family:Arial;font-size:11px;border: 1px solid #A0A0A0;" bgcolor="#ffffff"><tr bgcolor="#000066"><td><table width="'+((showWeekNumber==1)?248:218)+'"><tr><td style="padding:2px;font-family:Arial;font-size:11px;"><font color="#ffffff' + '' /*C9D3E9*/ +'"><b><span id="caption"></span></b></font></td><td align="right"><a href="javascript:hideCalendar()"><img src="'+imgDir+'close.gif" width="15" height="13" border="0" /></a></td></tr></table></td></tr><tr><td style="padding:5px" bgcolor="#ffffff"><span id="content"></span></td></tr>');

		if (showToday == 1) {
			document.write ('<tr bgcolor="#f0f0f0"><td style="padding:5px" align="center"><span id="lblToday"></span></td></tr>');
		}
			
		document.write ('</table></div><div id="selectMonth" style="z-index:+999;position:absolute;display:none;"></div><div id="selectYear" style="z-index:+999;position:absolute;display:none;"></div>');
	}

	var	styleAnchor = 'text-decoration:none;color:black;';
	var	styleLightBorder = 'border:1px solid #a0a0a0;';

	function swapImage(srcImg, destImg) {
		if (ie) document.getElementById(srcImg).setAttribute('src',imgDir + destImg);
	}
	
	
	function initToday()
	{
		setToday();
		if (showToday == 1) {
			document.getElementById('lblToday').innerHTML =	'<font color="#000066">' + todayString[language] + ' <a onmousemove="window.status=\''+gotoString[language]+'\'" onmouseout="window.status=\'\'" title="'+gotoString[language]+'" style="'+styleAnchor+'" href="javascript:monthSelected=monthNow;yearSelected=yearNow;constructCalendar();">'+dayName[language][(today.getDay()-startAt==-1)?6:(today.getDay()-startAt)]+', ' + dateNow + ' ' + monthName[language][monthNow].substring(0,3) + ' ' + yearNow + '</a></font>';
		}
	
	}

	function initPopupCalendar() {
		if (!ns4)
		{

			crossobj=(dom)?document.getElementById('calendar').style : ie? document.all.calendar : document.calendar;
			hideCalendar();

			crossMonthObj = (dom) ? document.getElementById('selectMonth').style : ie ? document.all.selectMonth : document.selectMonth;

			crossYearObj = (dom) ? document.getElementById('selectYear').style : ie ? document.all.selectYear : document.selectYear;

			monthConstructed = false;
			yearConstructed = false;
			initToday();
			if (getCalendarDirection() == 'rtl')
			{
				var next1='left1.gif';var next2='left2.gif';var prev1='right1.gif';var prev2='right2.gif';
			}
			else
			{	
				var prev1='left1.gif';var prev2='left2.gif';var next1='right1.gif';var next2='right2.gif';
			}
			sHTML1 = '<span id="spanLeft" style="border:1px solid #36f;cursor:pointer" onmouseover="swapImage(\'changeLeft\',\''+prev2+'\');this.style.borderColor=\'#8af\';window.status=\''+scrollLeftMessage[language]+'\'" onclick="decMonth()" onmouseout="clearInterval(intervalID1);swapImage(\'changeLeft\',\''+prev1+'\');this.style.borderColor=\'#36f\';window.status=\'\'" onmousedown="clearTimeout(timeoutID1);timeoutID1=setTimeout(\'StartDecMonth()\',500)" onmouseup="clearTimeout(timeoutID1);clearInterval(intervalID1)">&nbsp<img id="changeLeft" src="'+imgDir+prev1+'" width="10" height="11" border="0">&nbsp</span>&nbsp;';
			sHTML1 += '<span id="spanRight" style="border:1px solid #36f;cursor:pointer" onmouseover="swapImage(\'changeRight\',\''+next2+'right2.gif\');this.style.borderColor=\'#8af\';window.status=\''+scrollRightMessage[language]+'\'" onmouseout="clearInterval(intervalID1);swapImage(\'changeRight\',\''+next1+'\');this.style.borderColor=\'#36f\';window.status=\'\'" onclick="incMonth()" onmousedown="clearTimeout(timeoutID1);timeoutID1=setTimeout(\'StartIncMonth()\',500)" onmouseup="clearTimeout(timeoutID1);clearInterval(intervalID1)">&nbsp<img id="changeRight" src="'+imgDir+next1+'" width="10" height="11" border="0">&nbsp</span>&nbsp;';
			sHTML1 += '<span id="spanMonth" style="border:1px solid #36f;cursor:pointer" onmouseover="swapImage(\'changeMonth\',\'drop2.gif\');this.style.borderColor=\'#8af\';window.status=\''+selectMonthMessage[language]+'\'" onmouseout="swapImage(\'changeMonth\',\'drop1.gif\');this.style.borderColor=\'#36f\';window.status=\'\'" onclick="popUpMonth()"></span>&nbsp;';
			sHTML1 += '<span id="spanYear" style="border:1px solid #36f;cursor:pointer" onmouseover="swapImage(\'changeYear\',\'drop2.gif\');this.style.borderColor=\'#8af\';window.status=\''+selectYearMessage[language]+'\'" onmouseout="swapImage(\'changeYear\',\'drop1.gif\');this.style.borderColor=\'#36f\';window.status=\'\'" onclick="popUpYear()"></span>&nbsp;';

			document.getElementById('caption').innerHTML = sHTML1;

			bPageLoaded=true;
		}
	}

	function hideCalendar() {
		crossobj.display = 'none';
		if (crossMonthObj != null) crossMonthObj.display = 'none';
		if (crossYearObj  != null) crossYearObj.display = 'none';
		showElement('SELECT');
		showElement('APPLET');
	}

	function padZero(num) {
		return (num	< 10) ? '0' + num : num;
	}

	function constructDate(d,m,y) {
		return formatDate(y,m+1,d, dateFormat);
	}
	/*
	Creates a date string according to a given format
	y: full year (e.g. 1999, 2004);
	m: month number (1..12)
	d: day in month (1..31)
	*/
	function formatDate(y,m,d, format) {
		sTmp = format;
		sTmp = sTmp.replace ('dd','<e>');
		sTmp = sTmp.replace ('d','<d>');
		sTmp = sTmp.replace ('<e>',padZero(d));
		sTmp = sTmp.replace ('<d>',d);
		sTmp = sTmp.replace ('mmmm','<p>');
		sTmp = sTmp.replace ('mmm','<o>');
		sTmp = sTmp.replace ('mm','<n>');
		sTmp = sTmp.replace ('m','<m>');
		sTmp = sTmp.replace ('<m>',m);
		sTmp = sTmp.replace ('<n>',padZero(m));
		sTmp = sTmp.replace ('<o>',monthName[language][m-1]);
		sTmp = sTmp.replace ('<p>',monthName2[language][m-1]);
		sTmp = sTmp.replace ('yyyy',y);
		return sTmp.replace ('yy',padZero(y%100));
	}

	function closeCalendar() {
		hideCalendar();
		targetNode.value = constructDate(dateSelected,monthSelected,yearSelected);
		if (valueHandler)
			valueHandler(yearSelected, monthSelected+1, dateSelected);
		valueHandler = null; // clean up
	}

	/*** Month Pulldown	***/
	function StartDecMonth() {
		intervalID1 = setInterval("decMonth()",80);
	}

	function StartIncMonth() {
		intervalID1 = setInterval("incMonth()",80);
	}

	function incMonth () {
		monthSelected++;
		if (monthSelected > 11) {
			monthSelected = 0;
			yearSelected++;
		}
		constructCalendar();
	}

	function decMonth () {
		monthSelected--;
		if (monthSelected < 0) {
			monthSelected = 11;
			yearSelected--;
		}
		constructCalendar();
	}

	function constructMonth() {
		popDownYear()
		if (!monthConstructed) {
			sHTML = "";
			for (i=0; i<12; i++) {
				sName = monthName[language][i];
				if (i == monthSelected){
					sName = '<b>' + sName + '</b>';
				}
				sHTML += '<tr><td id="m' + i + '" onmouseover="this.style.backgroundColor=\'#909090\'" onmouseout="this.style.backgroundColor=\'\'" style="cursor:pointer" onclick="monthConstructed=false;monthSelected=' + i + ';constructCalendar();popDownMonth();event.cancelBubble=true"><font color="#000066">&nbsp;' + sName + '&nbsp;</font></td></tr>';
			}

			document.getElementById('selectMonth').innerHTML = '<table width="70" style="font-family:Arial;font-size:11px;border:1px solid #a0a0a0;" bgcolor="#f0f0f0" cellspacing="0" onmouseover="clearTimeout(timeoutID1)" onmouseout="clearTimeout(timeoutID1);timeoutID1=setTimeout(\'popDownMonth()\',100);event.cancelBubble=true">' + sHTML + '</table>';

			monthConstructed = true;
		}
	}

	function popUpMonth() {
		if (visMonth == 1) {
			popDownMonth();
			visMonth--;
		} else {
			constructMonth();
			crossMonthObj.display='';
			if (getCalendarDirection() == 'rtl')
				crossMonthObj.left = cal_pixels(parseInt(crossobj.left) + 132);
			else
				crossMonthObj.left = cal_pixels(parseInt(crossobj.left) + 50);
			crossMonthObj.top =	cal_pixels(parseInt(crossobj.top) + 26);
			hideElement('SELECT', document.getElementById('selectMonth'));
			hideElement('APPLET', document.getElementById('selectMonth'));
			visMonth++;
		}
	}

	function popDownMonth() {
		crossMonthObj.display = 'none';
		visMonth = 0;
	}

	/*** Year Pulldown ***/
	function incYear() {
		for	(i=0; i<7; i++) {
			newYear	= (i + nStartingYear) + 1;
			if (newYear == yearSelected)
				txtYear = '<span style="color:#006;font-weight:bold;">&nbsp;' + newYear + '&nbsp;</span>';
			else
				txtYear = '<span style="color:#006;">&nbsp;' + newYear + '&nbsp;</span>';
			document.getElementById('y'+i).innerHTML = txtYear;
		}
		nStartingYear++;
		bShow=true;
	}

	function decYear() {
		for	(i=0; i<7; i++) {
			newYear = (i + nStartingYear) - 1;
			if (newYear == yearSelected)
				txtYear = '<span style="color:#006;font-weight:bold">&nbsp;' + newYear + '&nbsp;</span>';
			else
				txtYear = '<span style="color:#006;">&nbsp;' + newYear + '&nbsp;</span>';
			document.getElementById('y'+i).innerHTML = txtYear;
		}
		nStartingYear--;
		bShow=true;
	}

	function selectYear(nYear) {
		yearSelected = parseInt(nYear + nStartingYear);
		yearConstructed = false;
		constructCalendar();
		popDownYear();
	}

	function constructYear() {
		popDownMonth();
		sHTML = '';
		if (!yearConstructed) {
			sHTML = '<tr><td align="center" onmouseover="this.style.backgroundColor=\'#909090\'" onmouseout="clearInterval(intervalID1);this.style.backgroundColor=\'\'" style="cursor:pointer" onmousedown="clearInterval(intervalID1);intervalID1=setInterval(\'decYear()\',30)" onmouseup="clearInterval(intervalID1)"><font color="#000066">-</font></td></tr>';

			j = 0;
			nStartingYear =	yearSelected - 3;
			for ( i = (yearSelected-3); i <= (yearSelected+3); i++ ) {
				sName = i;
				if (i == yearSelected) sName = '<b>' + sName + '</b>';
				sHTML += '<tr><td id="y' + j + '" onmouseover="this.style.backgroundColor=\'#909090\'" onmouseout="this.style.backgroundColor=\'\'" style="cursor:pointer" onclick="selectYear('+j+');event.cancelBubble=true"><font color="#000066">&nbsp;' + sName + '&nbsp;</font></td></tr>';
				j++;
			}

			sHTML += '<tr><td align="center" onmouseover="this.style.backgroundColor=\'#909090\'" onmouseout="clearInterval(intervalID2);this.style.backgroundColor=\'\'" style="cursor:pointer" onmousedown="clearInterval(intervalID2);intervalID2=setInterval(\'incYear()\',30)" onmouseup="clearInterval(intervalID2)"><font color="#000066">+</font></td></tr>';

			document.getElementById('selectYear').innerHTML = '<table width="44" cellspacing="0" bgcolor="#f0f0f0" style="font-family:Arial;font-size:11px;border:1px solid #a0a0a0;" onmouseover="clearTimeout(timeoutID2)" onmouseout="clearTimeout(timeoutID2);timeoutID2=setTimeout(\'popDownYear()\',100)">' + sHTML + '</table>';

			yearConstructed = true;
		}
	}

	function popDownYear() {
		clearInterval(intervalID1);
		clearTimeout(timeoutID1);
		clearInterval(intervalID2);
		clearTimeout(timeoutID2);
		crossYearObj.display= 'none';
		visYear = 0;
	}

	function popUpYear() {
		var leftOffset
		if (visYear==1) {
			popDownYear();
			visYear--;
		} else {
			constructYear();
			crossYearObj.display	= '';
			leftOffset = parseInt(crossobj.left) + document.getElementById('spanYear').offsetLeft;
			if (ie) leftOffset += 6;
			if (getCalendarDirection() == 'rtl')
				leftOffset += 32;
			crossYearObj.left = cal_pixels(leftOffset);
			crossYearObj.top = cal_pixels(parseInt(crossobj.top) + 26);
			visYear++;
		}
	}

	/*** calendar ***/
	function WeekNbr(n) {
		// Algorithm used:
		// From Klaus Tondering's Calendar document (The Authority/Guru)
		// http://www.tondering.dk/claus/calendar.html
		// a = (14-month) / 12
		// y = year + 4800 - a
		// m = month + 12a - 3
		// J = day + (153m + 2) / 5 + 365y + y / 4 - y / 100 + y / 400 - 32045
		// d4 = (J + 31741 - (J mod 7)) mod 146097 mod 36524 mod 1461
		// L = d4 / 1460
		// d1 = ((d4 - L) mod 365) + L
		// WeekNumber = d1 / 7 + 1

		year = n.getFullYear();
		month = n.getMonth() + 1;
		if (startAt == 0) {
			day = n.getDate() + 1;
		} else {
			day = n.getDate();
		}

		a = Math.floor((14-month) / 12);
		y = year + 4800 - a;
		m = month + 12 * a - 3;
		b = Math.floor(y/4) - Math.floor(y/100) + Math.floor(y/400);
		J = day + Math.floor((153 * m + 2) / 5) + 365 * y + b - 32045;
		d4 = (((J + 31741 - (J % 7)) % 146097) % 36524) % 1461;
		L = Math.floor(d4 / 1460);
		d1 = ((d4 - L) % 365) + L;
		week = Math.floor(d1/7) + 1;

		return week;
	}
	function WeekYear(n) {
		// Algorithm used:
		// From Klaus Tondering's Calendar document (The Authority/Guru)
		// http://www.tondering.dk/claus/calendar.html
		// a = (14-month) / 12
		// y = year + 4800 - a
		// m = month + 12a - 3
		// J = day + (153m + 2) / 5 + 365y + y / 4 - y / 100 + y / 400 - 32045
		// d4 = (J + 31741 - (J mod 7)) mod 146097 mod 36524 mod 1461
		// L = d4 / 1460
		// d1 = ((d4 - L) mod 365) + L
		// WeekNumber = d1 / 7 + 1

		year = n.getFullYear();
		month = n.getMonth() + 1;
		if (startAt == 0) {
			day = n.getDate() + 1;
		} else {
			day = n.getDate();
		}

		a = Math.floor((14-month) / 12);
		y = year + 4800 - a;
		m = month + 12 * a - 3;
		b = Math.floor(y/4) - Math.floor(y/100) + Math.floor(y/400);
		J = day + Math.floor((153 * m + 2) / 5) + 365 * y + b - 32045;
		d4 = (((J + 31741 - (J % 7)) % 146097) % 36524) % 1461;
		L = Math.floor(d4 / 1460);
		d1 = ((d4 - L) % 365) + L;
		week = Math.floor(d1/7) + 1;
		if (week == 1 && month == 12)
			return year+1;
		if (week >= 52 && month == 1)
			return year-1;
		return year;
	}
	function constructCalendar () {
		var aNumDays = Array (31,0,31,30,31,30,31,31,30,31,30,31);
		var dateMessage;
		var startDate = new Date (yearSelected,monthSelected,1);
		var endDate;

		if (monthSelected==1) {
			endDate = new Date (yearSelected,monthSelected+1,1);
			endDate = new Date (endDate - (24*60*60*1000));
			numDaysInMonth = endDate.getDate();
		} else {
			numDaysInMonth = aNumDays[monthSelected];
		}

		datePointer = 0;
		dayPointer = startDate.getDay() - startAt;
		
		if (dayPointer<0) dayPointer = 6;

		sHTML = '<table border="0" style="font-family:verdana;font-size:10px;"><tr>';

		if (showWeekNumber == 1) {
			sHTML += '<td width="27"><b>' + weekString[language] + '</b></td><td width="1" rowspan="7" bgcolor="#d0d0d0" style="padding:0px"><img src="'+imgDir+'divider.gif" width="1"></td>';
		}

		for (i = 0; i<7; i++) {
			sHTML += '<td width="27" align="right"><b><font color="#000066">' + dayName[language][i] + '</font></b></td>';
		}

		sHTML += '</tr><tr>';
		
		if (showWeekNumber == 1) {
			sHTML += '<td align="right">' + WeekNbr(startDate) + '&nbsp;</td>';
		}

		for	( var i=1; i<=dayPointer;i++ ) {
			sHTML += '<td>&nbsp;</td>';
		}
	
		for	( datePointer=1; datePointer <= numDaysInMonth; datePointer++ ) {
			dayPointer++;
			sHTML += '<td align="right">';
			sStyle=styleAnchor;
			if ((datePointer == odateSelected) && (monthSelected == omonthSelected) && (yearSelected == oyearSelected))
			{ sStyle+=styleLightBorder }

			sHint = '';
			for (k = 0;k < HolidaysCounter; k++) {
				if ((parseInt(Holidays[k].d) == datePointer)&&(parseInt(Holidays[k].m) == (monthSelected+1))) {
					if ((parseInt(Holidays[k].y)==0)||((parseInt(Holidays[k].y)==yearSelected)&&(parseInt(Holidays[k].y)!=0))) {
						sStyle+= 'background-color:#fdd;';
						sHint += sHint=="" ? Holidays[k].desc : "\n"+Holidays[k].desc;
					}
				}
			}

			sHint = sHint.replace('/\"/g', '&quot;');

			dateMessage = 'onmousemove="window.status=\''+selectDateMessage[language].replace('[date]',constructDate(datePointer,monthSelected,yearSelected))+'\'" onmouseout="window.status=\'\'" ';


			//////////////////////////////////////////////
			//////////  Modifications PinoToy  //////////
			//////////////////////////////////////////////
			if (enablePast == 0 && ((yearSelected < yearNow) || (monthSelected < monthNow) && (yearSelected == yearNow) || (datePointer < dateNow) && (monthSelected == monthNow) && (yearSelected == yearNow))) {
				selDayAction = '';
				isPast = 1;
			} else {
				selDayAction = 'href="javascript:dateSelected=' + datePointer + ';closeCalendar();"';
				isPast = 0;
			}

			if ((datePointer == dateNow) && (monthSelected == monthNow) && (yearSelected == yearNow)) {	///// today
				sHTML += "<b><a "+dateMessage+" title=\"" + sHint + "\" style=\"white-space:nowrap;"+sStyle+"\" "+selDayAction+"><font color=#ff0000>&nbsp;" + datePointer + "</font>&nbsp;</a></b>";
			} else if (dayPointer % 7 == (startAt * -1)+1) {									///// SI ES DOMINGO
				if (isPast==1)
					sHTML += "<a "+dateMessage+" title=\"" + sHint + "\" style='"+sStyle+"' "+selDayAction+">&nbsp;<font color=#909090>" + datePointer + "</font>&nbsp;</a>";
				else
					sHTML += "<a "+dateMessage+" title=\"" + sHint + "\" style='"+sStyle+"' "+selDayAction+">&nbsp;<font color=#54A6E2>" + datePointer + "</font>&nbsp;</a>";
			} else if ((dayPointer % 7 == (startAt * -1)+7 && startAt==1) || (dayPointer % 7 == startAt && startAt==0)) {	///// SI ES SABADO
				if (isPast==1)
					sHTML += "<a "+dateMessage+" title=\"" + sHint + "\" style='"+sStyle+"' "+selDayAction+">&nbsp;<font color=#909090>" + datePointer + "</font>&nbsp;</a>";
				else
					sHTML += "<a "+dateMessage+" title=\"" + sHint + "\" style='"+sStyle+"' "+selDayAction+">&nbsp;<font color=#54A6E2>" + datePointer + "</font>&nbsp;</a>";
			} else {																			///// CUALQUIER OTRO DIA
				if (isPast==1)
					sHTML += "<a "+dateMessage+" title=\"" + sHint + "\" style='"+sStyle+"' "+selDayAction+">&nbsp;<font color=#909090>" + datePointer + "</font>&nbsp;</a>";
				else
					sHTML += "<a "+dateMessage+" title=\"" + sHint + "\" style='"+sStyle+"' "+selDayAction+">&nbsp;<font color=#000066>" + datePointer + "</font>&nbsp;</a>";
			}

			sHTML += '';
			if ((dayPointer+startAt) % 7 == startAt) {
				sHTML += '</tr><tr>';
				if ((showWeekNumber == 1) && (datePointer < numDaysInMonth)) {
					sHTML += '<td align="right">' + (WeekNbr(new Date(yearSelected,monthSelected,datePointer+1))) + '&nbsp;</td>';
				}
			}
		}

		document.getElementById('content').innerHTML   = sHTML
		document.getElementById('spanMonth').innerHTML = '&nbsp;' +	monthName[language][monthSelected] + '&nbsp;<img id="changeMonth" src="'+imgDir+'drop1.gif" width="12" height="10" border="0">'
		document.getElementById('spanYear').innerHTML  = '&nbsp;' + yearSelected	+ '&nbsp;<img id="changeYear" src="'+imgDir+'drop1.gif" width="12" height="10" border="0">';
	}

	function showCalendar(ctl, ctl2, format, lang, past, fx, fy) {
		if (lang != null && lang != '') language = lang;
		if (past != null) enablePast = past;
		else enablePast = 0;
		if (fx != null) fixedX = fx;
		else fixedX = -1;
		if (fy != null) fixedY = fy;
		else fixedY = -1;

		if (showToday == 1) {
			document.getElementById('lblToday').innerHTML = '<font color="#000066">' + todayString[language] + ' <a onmousemove="window.status=\''+gotoString[language]+'\'" onmouseout="window.status=\'\'" title="'+gotoString[language]+'" style="'+styleAnchor+'" href="javascript:monthSelected=monthNow;yearSelected=yearNow;constructCalendar();">'+dayName[language][(today.getDay()-startAt==-1)?6:(today.getDay()-startAt)]+', ' + dateNow + ' ' + monthName[language][monthNow].substring(0,3) + ' ' + yearNow + '</a></font>';
		}
		popUpCalendar(ctl, ctl2, null, format);
	}

	/*
		Pops up the calendar.
		ctl: the document node used to position the popup calendar
		target: the document node in which the formatted value should be placed (and from which the initial value should be taken)
		handler: a function to be called when a date has been selected. 
		         the function accepts 3 arguments (y, m, d) where y is the full year (e.g. 2004)
		         m is the month (1..12) and d is the day (1..31)
	*/
	
	function parseDate(text, format)
	{
		var formatChar = ' ';
		var aFormat = format.split(formatChar);
		if (aFormat.length < 3) {
			formatChar = '/';
			aFormat = format.split(formatChar);
			if (aFormat.length < 3) {
				formatChar = '.';
				aFormat = format.split(formatChar);
				if (aFormat.length < 3) {
					formatChar = '-';
					aFormat = format.split(formatChar);
					if (aFormat.length < 3) {
						return null;					// invalid date format

					}
				}
			}
		}

		var tokensChanged = 0;
		var aData =	text.split(formatChar);			// use user's date

		var y, m, d;
		for (var i=0; i<3; i++) {
			if ((aFormat[i] == "d") || (aFormat[i] == "dd")) {
				d = parseInt(aData[i], 10);
				tokensChanged++;
			} else if ((aFormat[i] == "m") || (aFormat[i] == "mm")) {
				m = parseInt(aData[i], 10) - 1;
				tokensChanged++;
			} else if (aFormat[i] == "yyyy") {
				y = parseInt(aData[i], 10);
				tokensChanged++;
			} else if (aFormat[i] == "mmm") {
				for (j=0; j<12; j++) {
					if (aData[i] == monthName[language][j]) {
						m=j;
						tokensChanged++;
					}
				}
			} else if (aFormat[i] == "mmmm") {
				for (j=0; j<12; j++) {
					if (aData[i] == monthName2[language][j]) {
						m = j;
						tokensChanged++;
					}
				}
			}
		}

		if ((tokensChanged != 3)|| isNaN(d) || isNaN(m) || isNaN(y)) 
			return null; // Invalid format or invalid date string

		return {d:d, m:m, y:y};
	}
	function popUpCalendar(ctl, target, handler, format) {
		initToday();
		var leftpos = 0;
		var toppos  = 0;
		if (bPageLoaded) {
			if (crossobj.display == 'none') {
				valueHandler = handler;
				targetNode = target;
				dateFormat = format;
	
				var date = parseDate(target.value, format);
				if (date == null)
				{
					date = {d:dateNow, m:monthNow, y:yearNow};
				}
				dateSelected = date.d;
				monthSelected = date.m;
				yearSelected = date.y;

				odateSelected  = dateSelected;
				omonthSelected = monthSelected;
				oyearSelected  = yearSelected;

				var p = screen_position(ctl);
				var s = visible_size(ctl);
				var window_size = page_size(window);
				
				
				crossobj.left = cal_pixels(0);
				crossobj.top = cal_pixels(0);
				constructCalendar (1, monthSelected, yearSelected);
				crossobj.display = '';
				var cal=document.getElementById('calendar');
				var calSize = visible_size(cal);
				var scroll = getScroll(cal);
				var left = (fixedX == -1) ? p.left  : fixedX;
				var left0=left;
				var top = (fixedY == -1) ? p.top + s.height + 2 : fixedY;
				if (left + calSize.width >window_size.width-10)
					left = window_size.width-calSize.width-10;
				if (left<0)
					left=0;
				if (top + calSize.height > window_size.height-10)
					top = window_size.height-10-calSize.height-10;
				if (top<0)
					top=0;
				crossobj.left = cal_pixels(left-scroll.left);
				crossobj.top = cal_pixels(top-scroll.top);

				hideElement('SELECT', document.getElementById('calendar'));
				hideElement('APPLET', document.getElementById('calendar'));			

				bShow = true;
			} else {
				hideCalendar();
				if (ctlNow!=ctl) popUpCalendar(ctl, target, handler, format);
			}
			ctlNow = ctl;
		}
	}

	document.onkeypress = function hidecal1 (e) {
		if (window.event && window.event.keyCode == 27) // IE
			hideCalendar();
		else if (e && e.keyCode && e.keyCode == 27) // Mozilla
			hideCalendar();
		
		
	}
	document.onclick = function hidecal2 () {
		if (!bShow) hideCalendar();
		bShow = false;
	}
	function visible_size(node)
	{
		var width = node.offsetWidth;
		var height = node.offsetHeight;
		if (node == node.ownerDocument.body && node.clientHeight)
		{
			height = node.clientHeight;
		};
		//TODO handle width
		return {'height':height,'width':width};
	};
	if (window.getScreenPosition)
		screen_position=getScreenPosition;
	else
	screen_position = function(node)
	{
		var x = 0;
		var y = 0;
		var currentNode = node;
		do
		{
			x+=currentNode.offsetLeft;
			y+=currentNode.offsetTop;
			currentNode = currentNode.offsetParent;
		}
		while (currentNode);
		currentNode = node;
		do
		{	
			if (currentNode.scrollLeft != undefined)
			{	
		  		x -= currentNode.scrollLeft;
  				y -= currentNode.scrollTop;
	  			currentNode = currentNode.offsetParent;
	  		}
	  		else
		  		currentNode = null;
		}
		while (currentNode);
		return {'left':x, 'top':y};
	};
function page_size(w)
{
	var size = {};
	if (w.innerHeight) 
	{
		size.width = w.innerWidth;
		size.height = w.innerHeight;
	}
	else if (w.document.documentElement && w.document.documentElement.clientHeight)
	{
		size.width = w.document.documentElement.clientWidth;
		size.height = w.document.documentElement.clientHeight;
	}
	else if (w.document.body) 
	{
		size.width = w.document.body.clientWidth;
		size.height = w.document.body.clientHeight;
	}
	return size;
};

function page_scroll(w)
{
	var scroll = {};
	if (w.pageYOffset) 
	{
		scroll.x = w.pageXOffset;
		scroll.y = w.pageYOffset;
	}
	else if (w.document.documentElement && w.document.documentElement.scrollTop)
	{
		scroll.x = w.document.documentElement.scrollLeft;
		scroll.y = w.document.documentElement.scrollTop;
	}
	else if (w.document.body) // all other Explorers
	{
		scroll.x = w.document.body.scrollLeft;
		scroll.y = w.document.body.scrollTop;
	}
	return scroll;
};

if (!window.getScroll)
{
	if (opener.getScroll)
		getScroll=opener.getScroll;
}	
// file: 062-reload-styles.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.reloadStyles = function()
{
	var linkElement = document.getElementById('main_stylesheet');
	if (linkElement)
	{
		if(window.tersusOriginalStyleURL == null)
			window.tersusOriginalStyleURL = linkElement.href;
		
		linkElement.href=window.tersusOriginalStyleURL+ '&_x=' +(new Date()).getTime();
	}	
}
// file: 073-send-message.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 function sendMessage(adapter, message, traceFileName)
{
	var url=tersus.rootURL+'File/'+encodeURIComponent(adapter);
	var req = createXMLHTTPRequest();
	var serviceNode = this;
	req.open('POST', url, true);
	var b = []
	Serializer.prototype._quote(message, b);
	req.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
	var requestBody='Message='+window.encodeURIComponent(b.join(''));
	if (traceFileName)
		requestBody+='&_traceFileName='+window.encodeURIComponent(traceFileName);
	req.send(requestBody);
	
}// file: 090-standalone.js
tersus.isNative=true;
tersus.preload = function (packages)
{
	tersus.repository = new ConstructorRepository();
    tersus.repository.packageTexts = packages;
};
tersus.init = function()
{
	if (! tersus.isIPhoneSafari)
		tersus.async.exec(tersus.initServiceResponseFrames);
	if (window.browser == 'IE')
		window.document.body.onunload = tersus.onUnload;
	else
		window.onunload = tersus.onUnload;
	tersus.async.timer = setInterval(tersus.async.worker, tersus.async.interval);
	tersus.keepAliveTimer = setInterval(tersus.keepAlive, tersus.KEEP_ALIVE_INTERVAL);
			
	tersus.createClassNameList();
	window.onresize = tersus.onResize;
	if (tersus.settings.dev_mode)
		tersus.updateCheckInterval = tersus.DEV_UPDATE_CHECK_INTERVAL;
	else
		tersus.updateCheckInterval = tersus.PROD_UPDATE_CHECK_INTERVAL;
};

tersus.checkUpdate = function()
{
};
// file: 100-GenericField.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.GenericField = function tersus_GenericField() {
};
tersus.GenericField.prototype = new DisplayNode();
tersus.GenericField.prototype.constructor = tersus.GenericField;
tersus.GenericField.prototype.currentReadOnly = null;
tersus.GenericField.prototype.value = null;
tersus.GenericField.prototype.isEditable = true;
tersus.GenericField.prototype.inputFieldTag = 'input';
tersus.GenericField.prototype.defaultTag = 'span';
tersus.GenericField.prototype.textTranslation = false;
tersus.GenericField.prototype.emptyOptionText = null;
tersus.GenericField.EMPTY_OPTION_TEXT = '<Empty Option Text>';
tersus.GenericField.prototype.init = function ()
{
	if (this.sharedProperties && tersus.isNumber(this.sharedProperties.maxLength))
    	this.maxLength = this.sharedProperties.maxLength;
};
tersus.GenericField.prototype.addGetter(tersus.GenericField.EMPTY_OPTION_TEXT, function()
{
	return this.emptyOptionText;
});
tersus.GenericField.prototype.updateOptions = function()
{
 	if (this.inputField && this.isChooser)
	{
		this.inputField.innerHTML = '';
		for (var i=0; i<this.options.length; i++)
		{
			var option = this.options[i];
			this.addOptionElement(option, i);
		}		
	}
 }
tersus.GenericField.prototype.addRemover(tersus.GenericField.EMPTY_OPTION_TEXT, function()
{
	if (this.emptyOptionText != null)
	{
		if (this.options)
			this.options.splice(0,1);//remove null from beginning of list
		this.emptyOptionText = null;
		this.updateOptions();
	}
});
tersus.GenericField.prototype.addSetter(tersus.GenericField.EMPTY_OPTION_TEXT,function(text)
{
	if (this.emptyOptionText == null)
	{
		if (this.options)
			this.options.splice(0,0,null); // add null to beginning of list
		else
			this.options = [null];
	}
	this.emptyOptionText = text;
	this.updateOptions();
});
tersus.GenericField.prototype.addGetter('<Max Length>', function()
{
	return this.maxLength;
});
tersus.GenericField.prototype.addSetter('<Max Length>', function(length)
{
	this.maxLength = length;
	if (this.inputField)
		this.inputField.maxLength = this.maxLength;
	// Reset value in order to truncate if needed
	this.setFieldValue(this.getFieldValue());
	
});
tersus.GenericField.prototype.reformat = function()
{
	this.setValueString();
};
tersus.GenericField.prototype.formatProperties = {
	currencySymbol:'<Currency Symbol>',
	currencyPosition:'<Currency Position>',
	useThousandsSeparator:'<Use Thousands Separator>',
	decimalPlaces:'<Decimal Places>'
};
tersus.GenericField.prototype.createFormatProps = function()
{
	this.formatProps = {};
	for (var p in this.formatProperties)
	{
		this.formatProps[p]=this.sprop(p);
	}	
};

tersus.GenericField.addFormatAccessor  = function (pt, prop, elementName)
{
	pt.addSetter(elementName, function(v)
	{
		if (! this.formatProps)
			this.createFormatProps();
		this.formatProps[prop] = v;	
		this.reformat();
	});
	pt.addGetter(elementName, function()
	{
		if (this.formatProps)
			return this.formatProps[prop];
		else
			return this.sprop(prop);
	});
};
for (var p in tersus.GenericField.prototype.formatProperties)
{
	tersus.GenericField.addFormatAccessor(tersus.GenericField.prototype, p,tersus.GenericField.prototype.formatProperties[p] );
};

tersus.GenericField.prototype.onClick  = function (domNode, callback)
{
	if (domNode == this.browseButton)
		this.queueEvent(Events.ON_BROWSE, null, null, callback);
	else if (this.currentReadOnly) 
		DisplayNode.prototype.onClick.call(this,domNode, callback);
	if (this.hyperlink)
		tersus.preventDefaultEventHandling();
};
tersus.GenericField.prototype['remove<Value>'] = function removeValue()
{
	this.value = null;
	if (this.viewNode)
	{
		if (this.inputField)
		{
			if (this.isChooser)
			{
				this.setSelectedIndex(0);
				if (this.options.length>0)
					this.value = this.options[0];
			}
			else
				this.inputField.value = '';
		}
		else if (this.hyperlink)
			this.hyperlink.innerHTML='';
		else
			this.viewNode.innerHTML = '';
		
	}
};

tersus.GenericField.prototype.updateSelfReadOnly = function tersus_GenericField_updateSelfReadOnly(readOnly)
{
	if (this.currentReadOnly == readOnly)
		return; // no change
	this.currentReadOnly = readOnly;
	if (this.inputField)
	{
		/* When we switch to read only mode, we get the current value from the input field, with the exception
		  of a chooser whose value doesn't match the options (selectedIndex<0) */
		var update = true;
		if (this.isChooser)
		{
			if (this.tmpSelectedIndex != null)
				update = this.tmpSelectedIndex != -1; // IE workaround
			else
				update = this.inputField.selectedIndex != -1;
		} 
   		if (update)
       		this.value = this.getFieldValue();
    }
	if (this.viewNode)
	{
		this.viewNode.innerHTML = '';
		if (readOnly)
		{
			if (this.hasEventHandler(Events.ON_CLICK))
				this.initLink()
			else
				this.initDisplay()
		}
		else
			this.initWritable();
	}
	this.setValueString();
};

tersus.GenericField.prototype.createViewNode = function tersus_GenericField_createViewNode()
{
	if (this.setDefaultProperties)
		this.setDefaultProperties();
	
    if (this.sharedProperties.inputFieldTag && this.sharedProperties.inputFieldTag != ' ')
    	this.inputFieldTag = this.sharedProperties.inputFieldTag;
    var optionsElement = this.getElement('<Options>');
    if (optionsElement)
    {
    	this.optionsElement = optionsElement;
    	this.isChooser = true;
    	this.options = [];
    	this.inputFieldTag = 'select';
    }
    DisplayNode.prototype.createViewNode.call(this);
};
tersus.GenericField.prototype.initViewNode = function tersus_GenericField_initViewNode()
{
    DisplayNode.prototype.initViewNode.call(this);
    this.updateSelfReadOnly(this.computedReadOnly());
   	if (this.hasEventHandler(Events.ON_CLICK))
   		this.registerEventHandler(Events.ON_CLICK,this.viewNode);
    
};
tersus.GenericField.prototype.attachHTMLNode = function (e)
{
	this.viewNode = e;
	if (e.firstChild && e.firstChild.tagName == 'INPUT')
		this.inputField = e.firstChild;
   	this.registerEventHandler(Events.ON_CHANGE,this.inputField);
	this.registerEventHandler(Events.ON_FOCUS,this.inputField);
   	if (this.hasEventHandler(Events.ON_BLUR))
	   	this.registerEventHandler(Events.ON_BLUR,this.inputField);
   	if (this.hasEventHandler(Events.ON_KEYPRESS))
	   	this.registerEventHandler(Events.ON_KEYPRESS,this.inputField);
   	if (this.hasEventHandler(Events.ON_KEYDOWN))
	   	this.registerEventHandler(Events.ON_KEYDOWN,this.inputField);
   	if (this.hasEventHandler(Events.ON_KEYUP))
	   	this.registerEventHandler(Events.ON_KEYUP,this.inputField);
   	if (this.hasEventHandler(Events.ON_CLICK))
	   	this.registerEventHandler(Events.ON_CLICK,this.hyperlink);
		
}

tersus.GenericField.prototype.setViewNode = function tersus_GenericField_setViewNode(viewNode)
{
	if (viewNode.tagName == 'INPUT')
	{	
		this.viewNode = this.createHTMLElement(this.getTag());
		viewNode.parentNode.replaceChild(this.viewNode, viewNode);
		this.viewNode.appendChild(viewNode);
		this.inputField = viewNode;
	}
	else
		this.viewNode = viewNode;
};
tersus.GenericField.nodesToAdjust = [];
tersus.GenericField.prototype.disabled = false;
tersus.GenericField.prototype.initWritable = function tersus_GenericField_initWritable()
{
	tersus.removeEventHandlers(this.viewNode);
    this.inputField = this.createHTMLElement(this.inputFieldTag);
    if (!this.isEditable)
    {
    	this.inputField.readOnly = true;
    }
    this.inputField.disabled = this.disabled;
    this.hyperlink = null;
    if (! this.isChooser && tersus.isNumber(this.sharedProperties.size))
    	this.inputField.size = this.sharedProperties.size;
    if (this.maxLength != null)
    	this.inputField.maxLength = this.maxLength;
    if (this.sharedProperties.width != undefined && this.sharedProperties.width != ' ')
    	this.inputField.style.width = this.sharedProperties.width;
    if (this.sharedProperties.styleClass && this.sharedProperties.styleClass != ' ')
	    this.inputField.className = this.sharedProperties.styleClass;
    if (this.sharedProperties.height && this.sharedProperties.height != ' ')
	    this.inputField.style.height = this.sharedProperties.height;
    var rows = this.sharedProperties.numberOfRows;
	if (rows && tersus.isNumber(rows))
		this.inputField.rows = parseInt(rows);
	var cols = this.sharedProperties.numberOfColumns;
	if (cols && tersus.isNumber(cols))
		this.inputField.cols = cols;
	if (this.isChooser)
	{	
		this.inputField.setAttribute('data-role','none');
		for (var i=0; i<this.options.length; i++)
		{
			var option = this.options[i];
			this.addOptionElement(option, i);
		}
		if (this.sharedProperties.size && this.sharedProperties.size != ' ')
			this.inputField.style.width = (2.5+0.57*this.sharedProperties.size) + "em"; // Trying to emulate the 'size' property of text input
		else if (this.sharedProperties.width && this.sharedProperties.width != ' ')
			this.inputField.style.width = this.sharedProperties.width;
			
	};
    	
    this.viewNode.innerHTML = '';
    this.viewNode.appendChild(this.inputField);
    if (this.hasEventHandler(Events.ON_BROWSE))
    {
	    this.viewNode.style.whiteSpace='nowrap';
		this.viewNode.style.position='relative';
    	this.inputField.className = 'pf_txt';

    	this.browseButton = this.createHTMLElement('button','pf_browse');
    	this.browseButton.innerHTML = '<img src="images/spacer.gif"/>';
	    //this.browseButton.type='text';
    	this.browseButton.id = this.getNodeId();
    	this.registerEventHandler(Events.ON_CLICK, this.browseButton);
	    this.viewNode.appendChild(this.browseButton);
	    var adjustList = tersus.GenericField.nodesToAdjust;
	    if (adjustList.length == 0)
		    setTimeout(tersus.GenericField.adjustPositions,10);
		adjustList.push(this);
	    //Adjust the position of the button based on the input field
    }
    this.inputField.setAttribute('node_id',this.getNodeId());
   	this.registerEventHandler(Events.ON_CHANGE,this.inputField);
	this.registerEventHandler(Events.ON_FOCUS,this.inputField);
   	if (this.hasEventHandler(Events.ON_BLUR))
	   	this.registerEventHandler(Events.ON_BLUR,this.inputField);
   	if (this.hasEventHandler(Events.ON_KEYPRESS))
	   	this.registerEventHandler(Events.ON_KEYPRESS,this.inputField);
   	if (this.hasEventHandler(Events.ON_KEYDOWN))
	   	this.registerEventHandler(Events.ON_KEYDOWN,this.inputField);
   	if (this.hasEventHandler(Events.ON_KEYUP))
	   	this.registerEventHandler(Events.ON_KEYUP,this.inputField);
};

tersus.GenericField.prototype.onChange = function(domNode, callback)
{
	this.validateFieldValue();
	DisplayNode.prototype.onChange.call(this, domNode, callback);
};
tersus.GenericField.prototype.onResize = function()
{
	if (this.browseButton)
	{
	    var adjustList = tersus.GenericField.nodesToAdjust;
	    this.browseButton.style.top='';
	    this.browseButton.style.left='';
	    if (adjustList.length == 0)
		    setTimeout(tersus.GenericField.adjustPositions,10);
		adjustList.push(this);
	}
};
tersus.GenericField.adjustPositions = function()
{
	var list = tersus.GenericField.nodesToAdjust;
	tersus.GenericField.nodesToAdjust = [];
	for (var i=0; i<list.length; i++)
	{
		var node = list[i];
		node.adjustPosition();
	}
};
/**
 * adjustPosition: align the browse button with the input field
 */
tersus.GenericField.prototype.adjustPosition = function()
{
	if (!this.inputField)
		return; // The field may have become read-only 
	var direction = getActualStyle(this.inputField,'direction');
	this.browseButton.style.height=getVisibleSize(this.inputField).height;
	var inputOffset = getOffsets(this.inputField);
	var inputTop=inputOffset.top;
	var buttonOffset = getOffsets(this.browseButton);
	var buttonTop = buttonOffset.top;
	var deltaTop = inputTop - buttonTop;
	this.browseButton.style.top = deltaTop+'px';
	if (direction == 'rtl')
	{
		var inputLeft = inputOffset.left;
		var buttonRight = buttonOffset.left + this.browseButton.offsetWidth-2; 	
		var deltaLeft = inputLeft - buttonRight;
		this.browseButton.style.borderRightWidth='0';
		this.browseButton.style.borderLeftWidth='2px';
		
	}
	else
	{
		var inputRight=inputOffset.left+this.inputField.offsetWidth -2; //Don't know why, but we need to substract 2
		var buttonLeft = buttonOffset.left;
		var deltaLeft = inputRight - buttonLeft;
		this.browseButton.style.borderRightWidth='2px';
		this.browseButton.style.borderLeftWidth='0px';
	}
	this.browseButton.style.left = deltaLeft+'px';
};
tersus.GenericField.prototype.getDefaultContent = function()
{
	return '';
};
tersus.GenericField.prototype.initLink = function tersus_GenericField_initLink()
{
	if (this.inputField)
		this.value = this.getFieldValue();
    this.inputField = null
    this.browseButton = null;
    this.hyperlink = this.createHTMLElement('a');
    var id = this.getNodeId();
    this.hyperlink.href='';
   	this.hyperlink.innerHTML = this.getDefaultContent();
    this.viewNode.appendChild(this.hyperlink);
};
tersus.GenericField.prototype.initDisplay = function tersus_GenericField_initDisplay()
{
	this.hyperlink = null;
	this.inputField = null;
	this.browseButton = null;
	DisplayNode.prototype.setEventHandlers.call(this);
};
tersus.GenericField.prototype.getFieldValue = tersus.GenericField.prototype['get<Value>'] = function tersus_GenericField_getFieldValue()
{
	if (this.inputField)
	{	
		if (this.isChooser)
			this.value = this.getSelectedOption();
		else
		{
			var valueElement = this.getElement('<Value>');
			var valuePrototype = valueElement.getChildConstructor().prototype;
			this.value = valuePrototype.parseInput(this.inputField.value, this.formatProps ? this.formatProps : this.sharedProperties);
		}
	}
	return this.value;
};
tersus.GenericField.prototype.setFieldValue = tersus.GenericField.prototype['set<Value>'] = function tersus_GenericField_setFieldValue(value)
{
	this.value = value;
	// Truncate to maxLength if needed (only if this is a text field)
	if (this.maxLength && value && value.isText && value.leafValue && value.leafValue.length > this.maxLength)
	{
		this.value = new value.constructor();
		this.value.leafValue =  value.leafValue.substring(0,this.maxLength); 
	}

	if (this.isChooser && ! this.currentReadOnly)
		this.setSelectedIndex(this.findIndex(this.value));
	else
		this.setValueString();
};
tersus.GenericField.prototype.setValueString = function()
{
	this.valueStr = '';
	if (this.value != null)
	{
		if (this.isChooser)
			this.valueStr = this.getOptionText(this.value);
		else if (this.inputField)
			this.valueStr = this.value.formatInput(this.formatProps ? this.formatProps : this.sharedProperties);
		else
			this.valueStr = this.value.formatString(this.formatProps ? this.formatProps : this.sharedProperties);
	}
	if (this.inputField)
	{
		this.inputField.value = this.valueStr;
	}
	else if (this.hyperlink)
		this.hyperlink.innerHTML = this.escapeHTML(this.translate(this.valueStr));
	else if (this.viewNode)
		this.viewNode.innerHTML =  this.escapeHTML(this.translate(this.valueStr));
};
tersus.GenericField.prototype.setValueStringOpt = function()
{
	this.valueStr = '';
	if (this.value != null)
	{
		if (this.isChooser)
			this.valueStr = this.getOptionText(this.value);
		else if (this.optimizeFormatString)
			this.valueStr = this.value.leafValue;
		else if (this.inputField && this.valueFormatInput)
			this.valueStr = this.value.formatInput(this.sharedProperties);
		else if (this.value.formatString)
			this.valueStr = this.value.formatString(this.sharedProperties);
		else
			this.valueStr = this.value.toString();
	}
	if (this.inputField)
	{
		this.inputField.value = this.valueStr;
	}
	else if (this.hyperlink)
		this.hyperlink.innerHTML = this.escapeHTML(this.translate(this.valueStr));
	else if (this.viewNode)
		this.viewNode.innerHTML =  this.escapeHTML(this.translate(this.valueStr));
};
tersus.GenericField.prototype.setFieldValue.requiresNodes = true;
tersus.GenericField.prototype.validateFieldValue = function()
{
	var valueElement = this.getElement('<Value>');
	var valuePrototype = valueElement.getChildConstructor().prototype;
	if (valuePrototype.validateInput)
	{
		var errorMessage =  valuePrototype.validateInput(this.inputField.value, this.formatProps ? this.formatProps : this.sharedProperties);
		this.setValidationMessage(errorMessage);
	}
};
tersus.GenericField.prototype.setEventHandlers = function ()
{
};
tersus.GenericField.prototype.showValidationMessage = function (message)
{
	var targetNode = this.viewNode;
	if (this.inputField)
		targetNode = this.inputField;
	this.currentWindow.alert(message);
	if (targetNode.focus && !targetNode.disabled)
		targetNode.focus();
	if (targetNode.select)
		targetNode.select();
};

tersus.GenericField.prototype.tmpSelectedIndex = null;
tersus.GenericField.prototype.getSelectedOption  = function tersus_GenericField_getSelectedOption()
{
	var selectedIndex;
	if (this.tmpSelectedIndex != null) // In IE, the selected index is updated asynchronously, so we use tmpSelectedIndex if it is set
		selectedIndex = this.tmpSelectedIndex;
	else
		selectedIndex = this.inputField.selectedIndex;
	if (selectedIndex < 0)
		return null;
	return this.options[selectedIndex];
};

tersus.GenericField.prototype.setSelectedIndex = function tersus_GenericField_setSelectedIndex(selectedIndex)
{
	if (!this.inputField)
		return;
	if (window.browser == 'IE') // IE needs a delay in order to recognize any newly added option/s
	{
		var inputField = this.inputField;
		var node = this;
		this.tmpSelectedIndex = selectedIndex;
		setTimeout(function(){	
		inputField.selectedIndex = selectedIndex;
		node.tmpSelectedIndex = null;}
		, 10);
	}
	else
		this.inputField.selectedIndex = selectedIndex;
};

tersus.GenericField.prototype.getOptions = tersus.GenericField.prototype['get<Options>'] = function tersus_GenericField_getOptions()
{
	if ( !this.options)
		return [];
	if (this.emptyOptionText != null)
	{
		return this.options.slice(1); // Ignoring the first item in the array, which is the empty option (null)
	}
	return this.options;
};
tersus.GenericField.prototype.reset = function()
{
	/* For choosers, the basic behavior sometimes leaves the value set as follows:
	 * When <Empty Option Text> is removed, the empty option is removed as the first on the list of options
	 * Then, clearing <Value> sets it as the first option
	 */ 
	DisplayNode.prototype.reset.call(this);
	this.value = null;
};
tersus.GenericField.prototype.addOption = tersus.GenericField.prototype['add<Options>'] = function tersus_GenericField_addOption(value)
{
	if (! this.options)
		this.options = [];
	this.options.push(value);
	if (this.inputField)
	{
		this.addOptionElement(value, this.options.length -1 );
	}
};
tersus.GenericField.prototype.addOptionElement = function tersus_GenericField_addOptionElement(value, index)
{
	var optionNode = this.createHTMLElement('option');
	optionNode.innerHTML = this.translate(tersus.escapeHTML(this.getOptionText(value)));
	this.inputField.appendChild(optionNode);
	/* Check if the new option matches the value (which may have been set previously)
	 In case of a match, select the new option */
	if (this.value != null &&  compareValues(this.value, value))
		this.setSelectedIndex(index);
};
tersus.GenericField.prototype.deepCopy = function deepCopy(value)
{
	var selectedIndex = null;
	this.emptyOptionText = value.getLeafChild(tersus.GenericField.EMPTY_OPTION_TEXT);
	this.options = this.emptyOptionText == null ? [] : [null];

	for (var elementRole in value.elements)
	{
		if (elementRole == tersus.GenericField.EMPTY_OPTION_TEXT)
			continue;
		var element = this.getElement(elementRole);
		if (element)
		{
			if (element.isRepetitive)
			{
				var childValues = value.getChildren(elementRole);
				if (elementRole == '<Options>') // Special optimization - setting the options of a large chooser as one chunk of HTML
				{
					var html = [];
					if (this.emptyOptionText != null)
					{
						html.push('<OPTION>');
						html.push(tersus.escapeHTML(this.emptyOptionText));
						html.push('</OPTION>');
					}
					if (childValues.length > 0)
					{
						var optimize_get_text = false;
						var optionText;
						var v0 = childValues[0];
						this.getOptionText(v0);
						var textElement = v0.__textElement;
						if (textElement && textElement.modelId == BuiltinModels.TEXT_ID && ! this.textTranslation)
							optimize_get_text = true;
						for (var i=0; i<childValues.length;i++)
						{
							var v = childValues[i];
							this.options.push(v);
							if (this.inputField)
							{
								var optionText = '';
								html.push('<OPTION>');
								if (optimize_get_text)
								{
									var c = v.children;
									if (c)
									{
										var tr = v.__textElementRole;
										if (tr)
										{
											var tn = c[tr];
											if (tn)
												optionText =this.escapeHTML(tn.leafValue); // Can be further optimized by checking in advance if escapeHTML is required
										}
										else
											optionText = this.escapeHTML(this.getOptionText(v)); // No text element found - reverting to un-optimized
									}
									html.push(optionText);
								}
								else
									html.push(this.translate(tersus.escapeHTML(this.getOptionText(v))));
								html.push('</OPTION>');
								if (this.value != null &&  compareValues(this.value, v))
								{
									selectedIndex = this.options.length-1; // Takes care of emptyOptionText as well
								}
							}
						}
						if (this.inputField)
						{
							var f=this.inputField;
							if (window.browser == 'IE')
							{
								// Don't remember why, but we need to re-recreate the input field in IE
								var oh = f.outerHTML;
								var p = f.parentNode;
								var ps = f.previousSibling;
								var endTag = '</SELECT>';
								f.outerHTML = oh.substring(0,oh.length-endTag.length) + html.join('') + endTag;
								if (ps)
									this.inputField = ps.nextSibling;
								else
									this.inputField = p.firstChild;
								this.attachHTMLNode(this.viewNode); // re-registers event handlers, which are lost when the input field is re-recreated.					

							}
							else
								f.innerHTML = html.join('');
						}
					}
					if (this.inputField && selectedIndex != null)
						this.setSelectedIndex(selectedIndex);
					
				}
				else
					element.setChildren(this, childValues, Operation.REPLACE);
			}
			else
			{
				var childValue = value.getChild(elementRole);
				if (childValue != null)
				{
					element.replaceChild(this, childValue);
				}
			}
		}
	}
	
};

tersus.GenericField.prototype.addOption.requiresNodes = true;
tersus.GenericField.prototype.removeOptions = tersus.GenericField.prototype['remove<Options>'] = function tersus_GenericField_removeOptions()
{
	if (this.emptyOptionText != null)
		this.options=[null];
	else
		this.options = [];
	this.updateOptions();
};

tersus.GenericField.prototype.findIndex = function tersus_GenericField_findIndex(value)
{
	if (this.options)
		for (var i=0; i<this.options.length; i++)
		{
			var option = this.options[i];
			if (compareValues(option, value))
				return i;
		}
	return -1;
};
tersus.GenericField.prototype.getOptionText = function tersus_GenericField_getOptionText(value)
{
	var outputValue = null;
	if (value && value.elementList)
	{
		var textElementRole = value.__textElementRole;
		if (textElementRole == null)
		{
			for (var i=0; i< value.elementList.length; i++)
			{
				var element = value.elementList[i];
				if (i==0)
				{
					textElementRole = element.role; // If no text element is found, the first element is used
					value.constructor.prototype.__textElement = element;
				}
				if (element.modelId == BuiltinModels.TEXT_ID)
				{
					textElementRole = element.role;
					value.constructor.prototype.__textElement = element;
					break;
				}
			}
		value.constructor.prototype.__textElementRole = textElementRole;
		}
		if (textElementRole && value.children)
			outputValue = value.children[textElementRole];
	}
	else
		outputValue = value;
	var text = '';
	if (outputValue != null)
		text =  outputValue.formatString(this.sharedProperties);
	else if (this.emptyOptionText != null)
		text = this.emptyOptionText;
	return text;
};
tersus.GenericField.prototype['get<Disabled>'] = function ()
{
	return this.disabled;
};

tersus.GenericField.prototype['set<Disabled>'] = function (disabled)
{
	this.disabled = disabled?true:false;
	if (this.inputField)
	{
		this.inputField.disabled=this.disabled;
	}
};
tersus.GenericField.prototype.getClickTarget = function()
{
	if (this.inputField)
		return this.inputField;
	else if (this.hyperlink)
		return this.hyperlink;
	else
		return this.viewNode;
};
tersus.GenericField.prototype.focus = function()
{
	if (this.inputField)
		this.inputField.focus();
	else if (this.viewNode.focus)
		this.viewNode.focus();
};
TextInputField = NumberInputField = tersus.GenericField; //backward compatability
// file: 101-DisplayField.js
/*********************************************************************************************************
 * Copyright (c) 2003-2009 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/
 tersus.DisplayField = function tersus_DisplayField() {};
tersus.DisplayField.prototype = new tersus.GenericField();
tersus.DisplayField.prototype.constructor = tersus.DisplayField;
tersus.DisplayField.prototype.explicitReadOnly = true;
TextDisplay = NumberDisplay = DateDisplay = tersus.DisplayField; // backward compatability
// file: 102-HTMLTag.js
/*********************************************************************************************************
 * Copyright (c) 2003-2010 Tersus Software Ltd. , All rights reserved.
 *
 * This program is made available under the terms of the GNU Lesser General Public License, version 2.1,
 * which is part of this distribution and is available at hhttp://www.gnu.org/licenses/lgpl-2.1.txt
 **********************************************************************************************************/

var SPECIAL_HTML_ATTRIBUTES='marquee.direction,marquee.scrollamount,marquee.loop';
var HTML_ATTRIBUTES = 'abbr,accept,accept-charset,accesskey,action,align,alink,alt,archive,autoplay,axis,background,bgcolor,border,cellpadding,cellspacing,char,charoff,charset,checked,cite,class,classid,clear,code,codebase,codetype,color,cols,colspan,compact,content,controls,coords,data,datetime,declare,defer,dir,disabled,enctype,face,for,frame,frameborder,headers,height,href,hreflang,hspace,http-equiv,id,ismap,label,lang,language,link,longdesc,marginheight,marginwidth,maxlength,media,method,multiple,name,nohref,noresize,noshade,nowrap,object,onblur,onchange,onclick,ondblclick,onfocus,onkeydown,onkeypress,onkeyup,onload,onmousedown,onmousemove,onmouseout,onmouseover,onmouseup,onreset,onselect,onsubmit,onunload,profile,prompt,readonly,rel,rev,rows,rowspan,rules,scheme,scope,scrolling,selected,shape,size,span,src,standby,start,style,summary,tabindex,target,text,title,type,usemap,valign,value,valuetype,version,vlink,vspace,width';
var STYLE_ATTRIBUTES = 'azimuth,background,background-attachment,background-color,background-image,background-position,background-repeat,border,border-collapse,border-color,border-spacing,border-style,border-top,border-right,border-bottom,border-left,border-top-color,border-right-color,border-bottom-color,border-left-color,border-top-style,border-right-style,border-bottom-style,border-left-style,border-top-width,border-right-width,border-bottom-width,border-left-width,border-width,bottom,caption-side,clear,clip,color,content,counter-increment,counter-reset,cue,cue-after,cue-before,cursor,direction,display,elevation,empty-cells,float,font,font-family,font-size,font-size-adjust,font-stretch,font-style,font-variant,font-weight,height,left,letter-spacing,line-height,list-style,list-style-image,list-style-position,list-style-type,margin,margin-top,margin-right,margin-bottom,margin-left,marker-offset,marks,max-height,max-width,min-height,min-width,orphans,outline,outline-color,outline-style,outline-width,overflow,padding,padding-top,padding-right,padding-bottom,padding-left,page,page-break-after,page-break-before,page-break-inside,pause,pause-after,pause-before,pitch,pitch-range,play-during,position,quotes,richness,right,size,speak,speak-header,speak-numeral,speak-punctuation,speech-rate,stress,table-layout,text-align,text-decoration,text-indent,text-shadow,text-transform,top,unicode-bidi,vertical-align,visibility,voice-family,volume,white-space,widows,width,word-spacing,z-index';
tersus.HTML_ATTRIBUTES = new Set(HTML_ATTRIBUTES.split(','));
tersus.SPECIAL_HTML_ATTRIBUTES = new Set(SPECIAL_HTML_ATTRIBUTES.split(','));
var BOOLEAN_HTML_ATTRIBUTES = 'readonly,disabled,autoplay,controls'
tersus.BOOLEAN_HTML_ATTRIBUTES = new Set(BOOLEAN_HTML_ATTRIBUTES.split(','));
tersus.STYLE_ATTRIBUTES = new Set(STYLE_ATTRIBUTES.split(','));
tersus.HTMLTag = DisplayNode.__subclass('tersus.HTMLTag', function(){});
tersus.HTMLTag.prototype.isHTMLTag = true;
tersus.propertyConversionMap = { 
	styleclass:'class',
	'style class':'class',
	wrappertag:'wrapperTags',
	wrappertags:'wrapperTags',
	wrapper_tags:'wrapperTags',
	wrapperstyleclass:'wrapperStyleClasses',
	wrapperstyleclasses:'wrapperStyleClasses',
	wrapper_classes:'wrapperStyleClasses',
	path:'src',
	icon:'data-icon',
	theme:'data-theme',
	inset:'data-inset'
};


tersus.HTMLTag.prototype.isHTMLTag = true;
tersus.convertProperty = function(p)
{
	var l = p.toLowerCase(); 
	var c = tersus.propertyConversionMap[l];
	return c ? c: l;
};
DisplayElement.prototype.isHTMLPrepared = false;
DisplayElement.prototype.prepareForHTML = function(parent)
{
	if (this.isHTMLPrepared)
		return;
	this.isHTMLPrepared = true;
	var childPrototype = this.getChildConstructor().prototype;
	if (!childPrototype || ! (childPrototype.isHTMLTag || childPrototype.isGroup ))
		return;
	this.prepHTML = childPrototype.prepareForHTML(this,parent);
	childPrototype.prepareChildrenForHTML();
};
DisplayNode.prototype.prepareChildrenForHTML = function()
{
	if (this.displayElements)
	{
		for (var i=0;i<this.displayElements.length;i++)
		{
			var e = this.displayElements[i];
			if (e.prepareForHTML)
				e.prepareForHTML(this);
		}
	}
};
tersus.HTMLTag.prototype.updateVisible = function()
{
	this.visible = !(this.getLeafChild('<Visible>') == false);
};
tersus.template = function(template, data)
{
	if (!tersus.templates)
		tersus.templates = {};
	if (!tersus.templates[template])
		tersus.templates[template]=$.template(template);
	return tersus.templates[template]($,{'data':data}).join('');
};
DisplayNode.prototype.prepareForHTML = function(e,parent)
{
	var props = {};
	var prep = {};
	if (!this.isHTMLTag)
		return prep;
	prep.updateVisible = parent && parent.isGroup || this.getElement('<Visible>'); //No need to update if parent is not a group and no <Visible>
	props.wrapperTags = this.defaultWrapperTags;
	props.wrapperStyleClasses = this.defaultWrapperStyleClasses;
	$.extend(props,this.baseProperties);
	props['class'] = this.defaultClass;
	props.tag = this.defaultTag;
	if (this.contentTag)
	{
		prep.contentPrefix = tersus.template('<${tag}{{if cls}} class="${cls}"{{/if}}>',{tag:this.contentTag,cls:this.contentClass});
		prep.contentSuffix = '</'+this.contentTag+'>';
	}
	for (var p in this.sharedProperties)
	{
		var v = this.sharedProperties[p];
		if (v != null && v != ' ')
			props[tersus.convertProperty(p)]=v;
	} 
	if (e)
	{
		for (var p in e.properties)
		{
			var v = e.properties[p];
			if (v != null && v != ' ')
				props[tersus.convertProperty(p)]=v;
		}
	}
	if (props.wrapperTags == '-')
		props.wrapperTags = null;
	if (props.wrapperStyleClasses == '-')
		props.wrapperStyleClasses = null; 
	prep.staticStyle = '';
	var html = '';
	for (var p in props)
	{
		var v = props[p];
		var dynamicElement =  this.dynamicAttributeElements[p];
		if (dynamicElement)
		{
			//Prepare default value
			if (prep.defaultValues == null)
				prep.defaultValues = {};
			prep.defaultValues[dynamicElement.role]=v;
		}
		else
		{
			//Parepare static value
			if (tersus.SPECIAL_HTML_ATTRIBUTES.contains(props.tag+'.'+p))
				html += ' ' + p + '="' +v+'"';
			else if (tersus.STYLE_ATTRIBUTES.contains(p))
				prep.staticStyle += p + ':' + v+';';
			else if (tersus.BOOLEAN_HTML_ATTRIBUTES.contains(p))
			{
				if (v)
					html+= " " + p;	
			}
			else if (p.match(/^data-/) || tersus.HTML_ATTRIBUTES.contains(p) && v != undefined)
				html += " " + p +'="'+tersus.escapeHTML(v)+'"'; 	
		}
	}
	if (e && e.displayOrder)
	{
		html+= ' display_order="'+e.displayOrder+'"';
	}
	prep.tag = props.tag;
	prep.isInput = prep.tag == 'INPUT' || prep.tag == 'TEXTAREA';
	prep.HTMLPrefix = '';
	prep.HTMLSuffix = '';
	prep.wrapperTags = null;
	if (props.wrapperTags)
	{
		var tags = prep.wrapperTags = props.wrapperTags.split(',');
		if (tags.length > 0)
			prep.wrapper_tag = tags[0];
		var classes = [];
		if (props.wrapperStyleClasses)
		{
			classes = props.wrapperStyleClasses.split(',');
		}
		
		for (var i=0;i<tags.length;i++)
		{
			var t = tags[i];
			var c = null;
			if (i< classes.length)
				c = classes[i];
			if (c)
				prep.HTMLPrefix += '<'+t+' class="'+c+'">';
			else
				prep.HTMLPrefix += '<'+t+'>';
			prep.HTMLSuffix = '</'+t+'>'+prep.HTMLSuffix;
		}
	}
	prep.HTMLPrefix += '<'+prep.tag + html;
	if (prep.staticStyle.length>0 && this.styleAttributeElements.length ==0)
		prep.HTMLPrefix += ' style="'+prep.staticStyle+'"';
	prep.HTMLPrefix += ' lid="';
	prep.HTMLSuffix = '</'+prep.tag+'>'+prep.HTMLSuffix;
	prep.simpleHTML = this.HTMLAttributeElements.length == 0 && this.styleAttributeElements.length ==0 && !prep.updateVisible;
	return prep;
};
 
tersus.HTMLTag.prototype.reset = function()
{
	this.removeAll();
	this.create();
};
tersus.HTMLTag.prototype.create = tersus.HTMLTag.prototype.copyValue = function(value)
{

	var oldViewNode = this.viewNode; // Shouldn't we care about "old wrapper node"
	this.viewNode = null; // Clear view node so that attribute values are set logically during the copy phase (and not placed on DOM nodes)
	if (this.element && !this.element.isHTMLPrepared)
		this.element.prepareForHTML(this.parent);
	var nodeMap = {};
	var htmlV = [];
	var legacyNodes = []; // nodes that are not instances of HTMLTag and need to be copied the old way
	this.copyValueOpt(value, nodeMap, htmlV, legacyNodes);
	var html = htmlV.join('');
	var domNode = $(html)[0];
	var prev = this.wrapperNode ? this.wrapperNode: oldViewNode; // TODO (x) use placeHolderNode;
	this.viewNode = domNode;
	var wt = this.prep.wrapperTags;
	this.wrapperNode = this.viewNode;
	if (wt)
	{
		for (var i=0;i<wt.length;i++)
			this.viewNode = this.viewNode.firstChild;
	}
	this.textContainer = this.viewNode;
	if (prev)
		prev.parentNode.replaceChild(domNode, prev);
	else
		this.attachViewNode();
	this.setEventHandlers();
	this.postAttach();
	this.attachHierarchy(nodeMap);
	for (var i=0;i<legacyNodes.length;i++)
	{
		var n = legacyNodes[i];
		n.create(n.valueToCopy);
		n.valueToCopy = null;
	}
	var t = this;
	tersus.async.exec(function(){t.onResize();},false);
};
tersus.HTMLTag.prototype.initModel = function()
{
	this.dynamicAttributeElements = {};
	this.HTMLAttributeElements = [];
	this.styleAttributeElements = [];
}
tersus.HTMLTag.prototype.onload = function()
{
	DisplayNode.prototype.onload.call(this);
	this.createEventHandlerList();
};
DisplayNode.prototype.copyValueOpt = function(value, nodeMap, htmlV, legacyNodes)
{
	var prep;
	if (this.element && this.element.prepHTML)
	{
		prep = this.prep = this.element.prepHTML;
	}
	else
	{
		if (!this.prepHTML)
		{
			{
				var prototype = this.getPrototype();
				prototype.prepHTML = prototype.prepareForHTML(null);
				prototype.prepareChildrenForHTML();
			}
		}
		prep = this.prep = this.prepHTML;
	}
	// Copy default values into instance
	var def = prep.defaultValues;
	if (def != null)
	{
		for (var role in def)
		{
			this.setLeafChild(role, def[role], Operation.REPLACE);
		}
	}
	if (value != null)
		this.leafValue = value.leafValue;
	else 
		this.leafValue = null;
	if (!this.isGroup)
	{
		htmlV.push(null); // placeHolder for tag opening; we can't create the tag before we copy values
		var openIndex = htmlV.length-1;
	}
	if (prep.contentPrefix)
		htmlV.push(prep.contentPrefix);
	var displayChildren=[];
	if (this.elements)
	{
		for (var elementRole in this.elements)
		{
			var element = this.getElement(elementRole);
			if (element)
			{
				if (element.isRepetitive)
				{
					if (value)
					{
						var childValues = value.getChildren(elementRole);
						for (var i=0;i<childValues.length;i++)
						{
							var oldChild = childValues[i];
							if (element.isDisplayElement)
							{
								var newChild =  element.createChildWithoutVisuals(this, oldChild.modelId);
								if (newChild.isHTMLTag || newChild.isDisplayGroup)
									newChild.copyValueOpt(oldChild, nodeMap,htmlV, legacyNodes);
								else
								{
									newChild.valueToCopy = oldChild;
									legacyNodes.push(newChild);
								}
							}
							else
							{
								var newChild = oldChild; //No need to clone non-display elements
								element.addAChild(this, newChild);
							}
							displayChildren.push(newChild);
						}
					}
				}
				else
				{
					var oldChild = value ? value.getChild(elementRole) : null;
					if (element.isDisplayElement && (element.alwaysCreate && ! this.initElement  || oldChild != null))
					{
						var newChild =  element.createChildWithoutVisuals(this, oldChild? oldChild.modelId:null);
						if (newChild.isHTMLTag || newChild.isDisplayGroup)
							newChild.copyValueOpt(oldChild, nodeMap,htmlV,legacyNodes);
						else
						{
							newChild.valueToCopy = oldChild;
							legacyNodes.push(newChild);
						}
						displayChildren.push(newChild);
						Element.prototype.replaceChild.call(element,this, newChild);
					}
					else if (oldChild != null)
						element.replaceChild(this, oldChild);
					
				}
			}
		}	
	}
	if (prep.contentSuffix)
		htmlV.push(prep.contentSuffix);
	
	if (! this.isGroup)
	{
		var nodeIdS = '' + this.getNodeId();
		nodeMap[nodeIdS] = this;
		var e = this.element;
		if (prep.simpleHTML)
		{
			htmlV[openIndex] = prep.HTMLPrefix + nodeIdS+'">';
		}
		else
		{
			var hv = [];
			hv.push(prep.HTMLPrefix);
			hv.push(nodeIdS);
			hv.push('" ');
			for (var i=0;i<this.HTMLAttributeElements.length; i++)
			{
				var e = this.HTMLAttributeElements[i];
				var v = e.getChild(this);
				if ( v!=null)
				{
					if (tersus.BOOLEAN_HTML_ATTRIBUTES.contains(e.role))
					{
						if (v)
						{
							hv.push(e.attributeName);
							hv.push(' ');
						}
						
					}
					else
					{
						hv.push(e.attributeName);
						hv.push('="');
						hv.push(tersus.escapeHTML(v.leafValue));
						hv.push('" ');
					}
				}
			}
			var invisible = false;
			if (prep.updateVisible)
			{
				this.updateVisible();
				invisible = this.isInvisible();
			}
			if (this.styleAttributeElements.length > 0 || e.staticStyle || invisible)
			{
				hv.push(' style="');
				hv.push(prep.staticStyle);
				for (var i=0;i<this.styleAttributeElements.length; i++)
				{
					var e = this.styleAttributeElements[i];
					var v = e.getChild(this);
					if ( v!=null && v.leafValue != null)
					{
						hv.push(e.attributeName);
						hv.push(':');
						hv.push(v.leafValue);
						hv.push(';');
					}
				};
				if (invisible)
					hv.push('display:none;');
				hv.push('"');
			}
			hv.push('>');
			htmlV[openIndex] = hv.join('');
		}
		this.addHTMLTagContent(htmlV, displayChildren, prep);
		htmlV.push(prep.HTMLSuffix);
	}
};
tersus.HTMLTag.prototype.addHTMLTagContent = function(htmlV, displayChildren, prep)
{
	if (!this.displayElements && !prep.isInput)
		htmlV.push(this.escapeHTML(this.valueStr));
};

tersus.HTMLTag.prototype.attachHTMLNode = function(e)
{
	this.viewNode = this.textContainer=e;
	var wt = this.prep.wrapperTags;
	this.wrapperNode = this.viewNode;
	if (wt)
	{
		for (var i=0; i< wt.length;i++)
			this.wrapperNode = this.wrapperNode.parentNode;
	}
	if (this.hasEventHandlers)
		this.setEventHandlers();
	this.postAttach();
}
tersus.HTMLTag.prototype.postAttach = function()
{
	var e = this.viewNode;
	if (e.tagName=='INPUT' || e.tagName=='TEXTAREA')
		this.inputField = e;
	
	var p = e.getAttribute('data-role');
	if (p=='button') p='buttonMarkup';
	var node = this;
	if (p && $.fn[p])
		$.fn[p].apply($(e));
	if (e.tagName=='A')
		$(e).find('span.ui-btn-text').each(function(i,e1){node.textContainer=e1;}); // Should make <Value> work, but not <Disabled>
	
};
tersus.HTMLTag.prototype.attachHierarchy = function (nodeMap) 
{
	var elements = this.viewNode.getElementsByTagName('*');
	for (var i=0;i<elements.length; i++)
	{
		var e = elements[i];
		var lid = e.getAttribute('lid');
		if (lid)
		{
			var n = nodeMap[lid];
			if (n)
				n.attachHTMLNode(e);
		}
	}
};
//Todo (?) - merge HTMLTag and DisplayNode so that optimization is the default
tersus.HTMLTag.prototype.createEventHandlerList = function createEventHandlerList()
{
	this.eventHandlerList = [];
	for (var key in Events)
	{
		var eventType = Events[key];
		if (this.hasEventHandler(eventType) && eventType.jsName)
			this.eventHandlerList.push(eventType);
	}
	this.isLocationChangeListener = this.hasEventHandler(Events.ON_LOCATION_CHANGE);
	
	this.hasEventHandlers = this.eventHandlerList.length > 0 || this.isLocationChangeListener;
};
tersus.HTMLTag.prototype.setEventHandlers = function DisplayNode_setEventHandlers()
{
	for (var i=0; i<this.eventHandlerList.length; i++)
	{
		var eventType = this.eventHandlerList[i];
		this.registerEventHandler(eventType);
	}
	if (this.isLocationChangeListener)
		tersus.locationChangeListeners.push(this.getNodeId());
};

tersus.HTMLTag.prototype.addIntermediateDataElement = function addIntermediateDataElement(role, modelId)
{
	//Interpret "distingushed" elements as CSS/HTML attributes (and throw error if there is no such HTML/CSS attribute)
	var element = null;
	if (this.getGetter(role) && role != '<Style Class>' && role != '<Style>')
	{
		element = new MethodElement(this, role, modelId);
	}
	else if (role == '<Valid Values>')
	{
		//<Valid Values> is used by the Table Wizard mechanism and is actaully a regular data element
		element = new FlowDataElement(role, modelId);
	}
	else if (role.search(/^<.*>$/)>=0) 
	{
		var p = tersus.convertProperty(role.substring(1,role.length-1));
		if (tersus.STYLE_ATTRIBUTES.contains(p))
		{
			element = new tersus.StyleAttributeElement(this,role,modelId);
			this.styleAttributeElements.push(element);
			this.dynamicAttributeElements[p]=element;
		}
		else if (p.match(/^data-/) || tersus.HTML_ATTRIBUTES.contains(p))
		{
			element = new tersus.HTMLAttributeElement(this,role,modelId);
			this.HTMLAttributeElements.push(element);
			this.dynamicAttributeElements[p]=element;
		}
		else
		 	modelError("Unknown CSS/HTML property \""+p+"\" for data element \""+role+"\"",this);
	}
	else
	{
		element = new FlowDataElement(role, modelId);
	}
	return this.addElement(element);
};

tersus.HTMLAttributeElement = DataElement.__subclass('HTMLAttributeElement', function(owner, role, modelId)
{
	this.owner = owner;
	this.role = role;
	this.modelId = modelId;
	this.attributeName = tersus.convertProperty(role.substring(1,role.length-1));
	this.changeHandler=owner['jq'+role];
});
tersus.HTMLAttributeElement.prototype.replaceChild = function replaceChild(parent, value)
{
	DataElement.prototype.replaceChild.call(this, parent,value);
	var v = value != null ? value.leafValue : null;
	if (parent.viewNode)
	{
		if (this.attributeName == 'class')
		{
			if (v == null) v = '';
			parent.viewNode.className = v;
		}
		else if (tersus.BOOLEAN_HTML_ATTRIBUTES.contains(this.attributeName))
		{
			parent.viewNode[this.attributeName] = v;
		}
		else
			parent.viewNode.setAttribute(this.attributeName, v);
			
		if (this.changeHandler)
			this.changeHandler.call(parent,value);
	}
};
tersus.HTMLAttributeElement.prototype.removeChild = function removeChild(parent)
{
	DataElement.prototype.removeChild.call(this, parent);
	if (parent.viewNode)
	{
		if (this.attributeName == 'class')
			parent.viewNode.className = '';
		else
			parent.viewNode.setAttribute(this.attributeName, null);
	}
};

tersus.HTMLAttributeElement.prototype.getChild = function getChild(parent)
{
	if (parent.viewNode)
	{
		var leafValue;
		if (this.attributeName == 'class')
		{
			leafValue = parent.viewNode.className;
			if (leafValue == '') 
				leafValue = null;
		}
		else if (tersus.BOOLEAN_HTML_ATTRIBUTES.contains(this.attributeName))
			leafValue = parent.viewNode[this.attributeName];
		else
			leafValue = parent.viewNode.getAttribute(this.attributeName);
		var dataValue = null;
		if (leafValue != null)
		{
			dataValue = this.createChildInstance();
			dataValue.setLeaf(leafValue);
		}
		return dataValue;

	}
	else
		return DataElement.prototype.getChild.call(this, parent);
};
tersus.StyleAttributeElement = DataElement.__subclass('StyleAttributeElement', function(owner, role, modelId)
{
	this.owner = owner;
	this.role = role;
	this.modelId = modelId;
	this.attributeName = tersus.convertProperty(role.substring(1,role.length-1));
	this.jsAttributeName = tersus.getJsAttributeName(this.attributeName);
});
tersus.StyleAttributeElement.prototype.replaceChild = function replaceChild(parent, value)
{
	DataElement.prototype.replaceChild.call(this, parent,value);
	if (parent.viewNode)
	{
		var v = value != null ? value.leafValue : null;
		if (v != null)
			parent.viewNode.style[this.jsAttributeName]= v;
	}
};
tersus.StyleAttributeElement.prototype.removeChild = function removeChild(parent)
{
	DataElement.prototype.removeChild.call(this, parent);
	if (parent.viewNode) // parent is a real display node
		parent.viewNode.style[this.jsAttributeName] = null;
};

tersus.StyleAttributeElement.prototype.getChild = function getChild(parent)
{
	if (parent.viewNode)
	{
		var leafValue = parent.viewNode.style[this.attributeName];
		var dataValue = this.createChildInstance();
		dataValue.setLeaf(leafValue);
		return dataValue;
	}
	else
		return DataElement.prototype.getChild.call(this, parent);
};
tersus.HTMLTag.prototype.formatProperties = tersus.GenericField.prototype.formatProperties;
tersus.HTMLTag.prototype.createFormatProps = tersus.GenericField.prototype.createFormatProps;
tersus.HTMLTag.prototype.reformat = tersus.GenericField.prototype.reformat;
for (var p in tersus.GenericField.prototype.formatProperties)
{
        tersus.GenericField.addFormatAccessor(tersus.HTMLTag.prototype, p,tersus.GenericField.prototype.formatProperties[p] );
};
tersus.HTMLTag.prototype.setValueString = function()
{
	this.valueStr = '';
	if (this.value != null)
	{
		if (this.optimizeFormatString)
			this.valueStr = this.value.leafValue;
		else if (this.value.formatString)
			this.valueStr = this.value.formatString(this.formatProps ? this.formatProps : this.sharedProperties);
		else
			this.valueStr = this.value.toString();
	}
	if (this.viewNode)
	{
		if (this.inputField)
			this.inputField.value=this.valueStr;
		else
			this.textContainer.innerHTML = this.escapeHTML(this.translate(this.valueStr));
	}
};
tersus.HTMLTag.getFieldValue = tersus.HTMLTag.prototype['get<Value>'] = function getFieldValue()
{
	if (this.inputField)
    {       
    	var valueElement = this.getElement('<Value>');
        var valuePrototype = valueElement.getChildConstructor().prototype;
        this.value = valuePrototype.parseInput(this.inputField.value, this.formatProps ? this.formatProps : this.sharedProperties);
    }
    return this.value;
};
tersus.HTMLTag.prototype.setFieldValue = tersus.HTMLTag.prototype['set<Value>'] = tersus.HTMLTag.prototype['remove<Value>'] = function setFieldValue(value) 
{
        this.value = value;
        if (this.inputField && this.inputField.tagName =='INPUT')
        {
	        // Truncate to maxLength if needed (only if this is a text field)
	        if (this.maxLength && value && value.isText && value.leafValue && value.leafValue.length > this.maxLength)
	        {
	                this.value = new value.constructor();
	                this.value.leafValue =  value.leafValue.substring(0,this.maxLength); 
	        }
	    }
        this.setValueString();
};
tersus.HTMLTag.prototype.refresh = function()
{
		DisplayNode.prototype.refresh.call(this);
		var $this  = $(this.viewNode);
		if ($this.parents().hasClass('ui-page'))
			$this.pageUpdate();
};
tersus.HTMLTag.prototype.onSubmit = function(domNode, callback)
{
	if (window.trace)
	{
		window.trace.event(this, Events.ON_SUBMIT);
	}
	this.queueEvent(Events.ON_SUBMIT, null, null, callback);
	tersus.currentEvent.preventDefault();
};


tersus.HTMLTag.prototype.setFieldValue.requiresNodes = true;// file: 103-FullScreen.js
tersus.FullScreen = tersus.HTMLTag.__subclass('FullScreen');
tersus.FullScreen.prototype.onResize = function onResize()
{
	var d = this.currentWindow.document;
	var s = getViewportSize(d);
/*	if (this.lastSize && this.lastSize.height == s.height && this.lastSize.width == s.width)
		return; // no change in size - nothing to do
	this.lastSize = s;*/

	var top = this.viewNode;
	var children = top.childNodes;
	var body;
	for (var i=0;i<children.length;i++)
	{
		var c=children[i];
		var cn = c.className;
		if (cn.indexOf('body')>=0)
			body = c;	
	}
	if (body)
	{
		var delta = top.offsetHeight - body.clientHeight;
		var height = s.height;	 //client height of body

		//Because clientHeight includes padding, but style.height doesn't include padding, we need to add padding to delta
		delta += getMeasure(body,'padding-bottom');
		delta += getMeasure(body,'padding-top');
		
		
		//Similarly, we need to remove body padding from the available height
		height -= parseInt(getActualStyle(d.body,'margin-bottom'));
		height -= parseInt(getActualStyle(d.body,'margin-top'));
		var newHeight = height-delta;
		if (newHeight>0 && newHeight < 3000)
		{
			body.style.height = newHeight+'px';
		}
	}
	this.onResizeChildren();
};
// file: 104-TagPopin.js
tersus.TagPopin = tersus.FullScreen.__subclass('TagPopin');
tersus.TagPopin.prototype.getCurrentRootViewNode = function()
{
	var root;
	var doc = this.currentWindow.document;
	root = doc.getElementById('root');
	if (! root)
		root = doc.getElementById('tersus.content');
	return root;
};
tersus.TagPopin.prototype.attachViewNode = function()
{
	this.originalRoot = this.getCurrentRootViewNode();
	this.rootNode = this.wrapperNode? this.wrapperNode:this.viewNode;
	this.originalRoot.parentNode.appendChild(this.rootNode, this.originalRoot);
};
tersus.TagPopin.prototype.isPopup = true;
tersus.TagPopin.prototype.isPopin = true;
tersus.TagPopin.prototype.start = function()
{
	if (this.element && ! this.element.prepareForHTML)
		this.element.prepareForHTML = DisplayElement.prototype.prepareForHTML;
	var w = this.currentWindow;
	var doc = w.document;
	this.originalRootDisplayNode = w.currentRootDisplayNode;
	tersus.popupNodes[this.getNodeId()]=this;
	clearSelection(w);
	this.create();
	w.currentRootDisplayNode = this;		
	FlowNode.prototype.start.call(this);
	this.isOpen = true;
	if (tersus.isIPhone)
		window.scrollTo(0,1);
	var w =this.rootNode;
	var cn = w.className;
	setTimeout(function() {w.className = cn +' open';},10);
};
tersus.TagPopin.prototype.destroy = function()
{
	if (! this.isDestroyed)
	{
		this.isDestroyed = true;
		DisplayNode.prototype.destroy.call(this);
		this.doClose();
	}
};
tersus.TagPopin.prototype.closeWindow = function() 
{
	this.destroyHierarchy();
};
tersus.TagPopin.prototype.doClose = function()
{
	delete tersus.popupNodes[this.getNodeId()];
	
	if (this.isOpen)
	{
		var doc = this.currentWindow.document;
//		currentRoot.parentNode.replaceChild(this.originalRoot, currentRoot);
		var r = this.rootNode;
		r.className = r.className.replace(/open/,'closed');
		setTimeout(function(){r.parentNode.removeChild(r);},500);
		this.currentWindow.currentRootDisplayNode = this.originalRootDisplayNode;
		this.viewNode = null;
		this.rootNode = null;
	}
	this.isOpen = false;
};

tersus.TagPopin.prototype.findById = function tersus_popIn_FindById(id, alreadyScanned)
{
	//Traverse only if its window is open
	if (this.isOpen)
		return tersus.Node.prototype.findById.call(this,id, alreadyScanned);
	else
		return null;
};// file: 105-Widgets.js

(function( $, undefined ) {
var setter = tersus.setter = function(f, requiresNodes)
{
	if (requiresNodes)
		f.requiresNodes = true;
	return f;
	
};

$.mobile.ajaxLinksEnabled = false;
var register = function(eventType, domNode)
		{
			if (eventType===Events.ON_CLICK)
			{
				tersus.HTMLTag.prototype.registerEventHandler.call(this,eventType, domNode);
				return;
			}
			if (!eventType.jsName)
			{
				throw {message:('No event dispatcher defined for '+eventType.elementName)};
				return;
			}
			var dispatcher = this.currentWindow.tersus_dispatchers[eventType.jsName];
			if (!dispatcher)
				throw {message:('No event dispatcher defined for '+eventType.jsName)};

			if (!domNode)
			{
				if (this.eventTargets)
					domNode = this.eventTargets[eventType.jsName];
				if (!domNode)
					domNode = this.viewNode;
			 }
			domNode.setAttribute('lid',this.getNodeId());
			$(domNode).bind(eventType.name,dispatcher);
		};
		
tersus.Widget = function(name, base, options)
{
	if (tersus[name])
		throw 'Duplicate definition of \'tersus.\''+name;
		
	var c = tersus[base].__subclass('tersus.'+name);
	tersus[name]=c;
	var p = c.prototype;
	$.extend(p,{
		
		postAttach:function()
		{
			var e = this.viewNode;
			var p = e.getAttribute('data-role');
			if (p=='button') p='buttonMarkup';
			var node = this;
			var pl = $.fn[this.jQuery];
			if (pl)
				pl.apply($(e));
			if (e.tagName=='A')
				$(e).find('span.ui-btn-text').each(function(i,e1){node.textContainer=e1;}); // Should make <Value> work, but not <Disabled>
			if (this.attach)
				this.attach();
			
		}
		,registerEventHandler:register
		
		,eachDisplay: function(callback)
		{
			if (this.elements)
			{
				var i=0;
				for (var elementRole in this.elements)
				{
					var element = this.getElement(elementRole);
					if (element.isDisplayElement)
					{
						if (element.isRepetitive)
						{
							var cc = element.getChildren(this);
							for (var j=0;j<cc.length;j++)
							{
								var c=cc[j];
								if (callback.call( c, i, c ) === false)
									return;
								++i;
							}
						}
						else
						{
							var c = element.getChild(this);
							if (c)
							{
								if (callback.call( c, i, c ) === false)
									return;
								++i;
							}
						}
					}							
				}
			}
		}		
	});
	
	$.extend(p,options);
	if (typeof options.template === 'string')
		p.template=$.template(null,options.template);
	if (p.template)
	{
		p.addHTMLTagContent = function(htmlV, displayChildren, prep)
		{
			var caption = null;
			if (this.getElement('<Caption>'))
				caption = this.getCaption();
			var data = {'caption':caption, value:this.valueStr, id:this.getNodeId(), readOnly:this.currentReadOnly};
			if (typeof this.templateData === 'function')
				$.extend(data, this.templateData.call(this, displayChildren)); // Add widget-specific data
			else if (typeof this.templateData === 'object')
				$.extend(data, this.templateData);
			var t = this.template($,{'data':data});
			$.merge(htmlV,t);
		};
	};
};

tersus.Widget('Toolbar','HTMLTag',{
	defaultTag:'div'
});
tersus.Widget('Label2','HTMLTag', {
	defaultTag:'h2'
	,template:'${caption}'
	,captionChanged:function(){$(this.viewNode).html(this.getCaption());}
});
tersus.Widget('Button2','HTMLTag', {
	defaultTag:'a'
	//,jQuery:'buttonMarkup'
	//,baseProperties:{'data-role':'button'}
	,template:'${caption}'
	,attach: function(){var node=this;$(this.wrapperNode).not('.ui-btn').buttonMarkup().prev().find('span.ui-btn-text').each(function(i,e1){node.textContainer=e1;;});}
	,onClick:function(domNode, callback){
		if (this.getLeafChild('<Disabled>'))
			return;
		DisplayNode.prototype.onClick.call(this, domNode, callback);
	}
	,captionChanged:function(){$(this.viewNode).find('.ui-btn-text').text(this.getCaption());}
	,'jq<Icon>': function(icon) {$(this.viewNode).data('icon',icon).buttonUpdate();}
});

/*

	// A version of Button2 with more explicit HTML Structure.  Doesn't support icons

tersus.Widget('Button2','HTMLTag', {
	defaultTag:'a'
	,defaultClass:'ui-btn ui-btn-corner-all ui-shadow ui-btn-up-c'
	//,jQuery:'buttonMarkup'
	//,baseProperties:{'data-role':'button'}
	,template:'<span class="ui-btn-inner"><span class="ui-btn-text">${caption}</span></span>'
	,attach: function(){
		this.textContainer=$(this.viewNode).find('.ui-btn-text')[0];
	}
	,onClick:function(domNode, callback){
		if (this.getLeafChild('<Disabled>'))
			return;
		DisplayNode.prototype.onClick.call(this, domNode, callback);
	}
	//,'jq<Icon>': function(icon) {$(this.viewNode).data('icon',icon).buttonUpdate();}
});
 */

tersus.Widget('FieldWidget','HTMLTag',{
	template:'{{if readOnly}}{{if caption}}<label for="${id}" class="ui-input-text" >${caption}</label>{{/if}}<span id="${id}" class="ter-field-readonly">${value}</span>'
			+'{{else}}{{if caption}}<label for="${id}" class="ui-input-text">${caption}</label>{{/if}}<input type="${fieldType}" id="${id}" value="${value}"/>{{/if}}'
	,defaultTag:'div'
	,jQuery:'fieldcontain'
	,setTextNode: function()
	{
		var $this = $(this.viewNode);
		this.textContainer = $this.find('.ter-field-readonly')[0];
		this.inputField = $this.find('input,textarea')[0];
	}
	,attach:function(){this.setTextNode();this.eventTargets={'onfocus':this.inputField};}
	,updateSelfReadOnly: function(ro) {
		this.currentReadOnly=ro;
		if (!this.viewNode)
			return;
		if (ro)
		{
			var $f = $('<span class="ter-field-readonly"/>');
			$f.text(this.valueStr);
			$f.attr('id',this.getNodeId());
			$(this.viewNode).find('input').replaceWith($f);
		}
		else
		{
			var $f = $('<input type="text" class="ui-input-text ui-body-null ui-corner-all ui-shadow-inset ui-body-c"/>');
			$f.val(this.valueStr);
			$f.attr('id',this.getNodeId());
			$(this.viewNode).find('span.ter-field-readonly').replaceWith($f);
		}
		this.setTextNode();
	}
});
tersus.Widget('TextArea2','FieldWidget',{
	template:'{{if caption}}<label for="${id}">${caption}</label>{{/if}}<textarea id="${id}" value="${value}"/>'
});

tersus.Widget('TextField','FieldWidget',{
	templateData:{fieldType:'text'}
});
tersus.Widget('PasswordField2','FieldWidget',{
	templateData:{fieldType:'password'}
});
tersus.Widget('NumberField','FieldWidget',{
	templateData:{fieldType:'number'}

});

tersus.Widget('DateField2','FieldWidget',{
	templateData:{fieldType:'text'}
//	,attach:function(){tersus.FieldWidget.prototype.attach.call(this);var $f=$(this.inputField); setTimeout(function(){$f.datebox();},0);}

});
tersus.Widget('List','HTMLTag',{
	defaultTag:'ul'
	,attach:function(){if(this.pluginVersion<1) $(this.viewNode).listview();}
});
tersus.Widget('Form','HTMLTag',{
	defaultTag:'form'
});
tersus.Widget('ListItem','HTMLTag',{
	defaultTag:'li'
	,attach:function()
	{
		if (this.pluginVersion<1)
		{
			if (this.sprop('data-role')=='none')
				return;
		 	var lv = $(this.viewNode).parent().data('listview'); if (lv) lv.refresh();
		 }
	}
	,prepareForHTML:function(e,parent)
	{
		if (this.pluginVersion<1 && this.hasEventHandler(Events.ON_CLICK))
		{
			$(this.displayElements)
				.filter(function(i){
					return this.getChildConstructor().prototype.plugin=='Tersus/Widgets/Text Display';
				})
				.first()
				.each(function(){
					this.properties = $.extend({wrapperTags:'a'},this.properties);
				});
		}
		return tersus.HTMLTag.prototype.prepareForHTML.call(this);
	}
});
tersus.Widget('ListDivider','HTMLTag',{
	defaultTag:'li'
	,baseProperties:{'data-role':'list-divider'}
	,template:'${caption}'
});

tersus.Widget('Image2','HTMLTag',{
	defaultTag:'img'
});
tersus.Widget('TextDisplay','HTMLTag',{
	defaultTag:'span'
});
tersus.Widget('NumberDisplay','HTMLTag',{
	defaultTag:'span'
});
tersus.Widget('DateDisplay','HTMLTag',{
	defaultTag:'span'
});

tersus.Widget('Slider', 'HTMLTag', {
	templateData: function(){return {min:13, max:77};}
	,template:'{{if caption}}<label for="${id}">${caption}</label>{{/if}}<input type="number" id="${id}" value="${value}" min=${min} max=${max}/>'
	,defaultTag:'div'
	,jQuery:'fieldcontain'
	,attach:function(){this.inputField=this.viewNode.lastChild; $(this.inputField).slider();}
});

tersus.Widget('RadioButton2','Button2',
{
	onClick:function(domNode, callback){
		var $p = $(this.parent.viewNode); $self = $(this.viewNode);
		$p.find('.ui-btn').removeClass('ui-btn-active');
		$self.addClass('ui-btn-active');
		DisplayNode.prototype.onClick.call(this, domNode, callback);
		this.onChange(domNode, callback);
		this.parent.onChange(domNode, callback);
	}
	,getButtonValue:function()
	{
		if (this.value != undefined)
			return this.value;
		else
		{
			var c = tersus.repository.get(BuiltinModels.TEXT_ID);
			var node = new c();
	        node.prototype = c.prototype;
	        node.setLeaf(this.getCaption());
			return node;
		}
	}
});

tersus.Widget('ButtonGroup','HTMLTag',{
	defaultTag:'fieldset',
	attach:function()
	{
		var $self = $(this.viewNode);
		$self.addClass('ui-corner-all ui-controlgroup ui-controlgroup-horizontal');
		setTimeout(function()
		{
			var rtl= $self.css('direction') == 'rtl';
			var $buttons = $self.find('.ui-btn');
			$self.find('.ui-btn-corner-all').removeClass('ui-btn-corner-all ui-shadow');
			$buttons.first().addClass(rtl ? 'ui-corner-right' : 'ui-corner-left');
			$buttons.last().addClass(rtl ? 'ui-corner-left' : 'ui-corner-right');
			
			if ($buttons.filter('.ui-btn-active').length == 0)
				$buttons.first().addClass('ui-btn-active');
		},0);
	}
	,'get<Value>':function()
	{
		var $active = $(this.viewNode).find('.ui-btn-active').tersus();
		if ($active.length>0)
			return $active[0].getButtonValue();
		 
	}	
	
	,'set<Value>':setter(function(value)
	{
		var $self = $(this.viewNode);
		$self.find('.ui-btn-active').removeClass('ui-btn-active');
		$self.find('.ui-btn').each(function(index){
			var node = $(this).tersus()[0];
			if (node && node.getButtonValue && compareValues(value, node.getButtonValue()))
				$(this).addClass('ui-btn-active');
		});
		
	},true)
});
$.fn.pageUpdate = function(options)
{
	var $elem=$(this);
	// If we're refreshing a direct component of a page - update the whole page
	var $dataEls = $elem.find( "[data-role]" ).andSelf().each(function() {
          				var $this = $( this ),
                        role = $this.data( "role" ),
                        theme = $this.data( "theme" );

                        //apply theming and markup modifications to page,header,content,footer
                        if ( role === "header" || role === "footer" ) {
                        	$this.addClass( "ui-bar-" + (theme || $this.parent('[data-role=page]').data( "theme" ) || "a") );

                            // add ARIA role
                            $this.attr( "role", role === "header" ? "banner" : "contentinfo" );

                            //right,left buttons
                            var $headeranchors = $this.children( "a" ),
                            leftbtn = $headeranchors.hasClass( "ui-btn-left" ),
                            rightbtn = $headeranchors.hasClass( "ui-btn-right" );

                            if ( !leftbtn ) {
                                leftbtn = $headeranchors.eq( 0 ).not( ".ui-btn-right" ).addClass( "ui-btn-left" ).length;
                            }

                            if ( !rightbtn ) {
                                rightbtn = $headeranchors.eq( 1 ).addClass( "ui-btn-right" ).length;
                            }
                            //page title
                            $this.children( "h1, h2, h3, h4, h5, h6" )
                                    .addClass( "ui-title" )
                                        //regardless of h element number in src, it becomes h1 for the enhanced page
                                        .attr({ "tabindex": "0", "role": "heading", "aria-level": "1" });
                        } else if ( role === "content" ) {
                            if ( theme ) {
                                    $this.addClass( "ui-body-" + theme );
                            }

                            // add ARIA role
                            $this.attr( "role", "main" );

                        } else if ( role === "page" ) {
                                $this.addClass( "ui-body-" + (theme || "c") );
                    }

					switch(role) {
                        case "header":
                        case "footer":
                        case "page":
                        case "content":
    	                    $this.addClass( "ui-" + role );
	                        break;
                        case "collapsible":
                        case "fieldcontain":
                        case "navbar":
                        case "listview":
                        case "dialog":
                            $this[ role ]();
                            break;
                    }
                });

                //links in bars, or those with data-role become buttons
                $elem.find( "[data-role='button'], .ui-bar > a, .ui-header > a, .ui-footer > a" )
                        .not( ".ui-btn" )
                        .buttonMarkup();

                $elem
                        .find("[data-role='controlgroup']")
                        .controlgroup();
                        
                                 var o = this.options;

             
                // enchance form controls
               $elem
                        .find( "[type='radio'], [type='checkbox']" )
                        .checkboxradio();

			   $elem
                        .find( "button, [type='button'], [type='submit'], [type='reset'], [type='image']" )
                        .button();

                $elem
                        .find( "input, textarea" )
                        .not( "[type='radio'], [type='checkbox'], button, [type='button'], [type='submit'], [type='reset'], [type='image']" )
                        .textinput();

                $elem
                        .find( "input, select" )
                        .filter( "[data-role='slider'], [data-type='range']" )
                        .slider();

           /*     $elem
                        .find( "select:not([data-role='slider'])" )
                        .selectmenu();*/
                        
		

};
$.fn.buttonUpdate = function( options ){
	return this.each( function() {
		var el = $( this ),
		    o = $.extend( {}, $.fn.buttonMarkup.defaults, el.data(), options),

			// Classes Defined
			buttonClass,
			innerClass = "ui-btn-inner",
			iconClass;


		// if not, try to find closest theme container
		if ( !o.theme ) {
			var themedParent = el.closest("[class*='ui-bar-'],[class*='ui-body-']"); 
			o.theme = themedParent.length ?
				/ui-(bar|body)-([a-z])/.exec( themedParent.attr("class") )[2] :
				"c";
		}

		buttonClass = "ui-btn ui-btn-up-" + o.theme;
		if ( o.inline ) {
			buttonClass += " ui-btn-inline";
		}
		
		if ( o.icon ) {
			o.icon = "ui-icon-" + o.icon;
			o.iconpos = o.iconpos || "left";

			iconClass = "ui-icon " + o.icon;

			if ( o.shadow ) {
				iconClass += " ui-icon-shadow"
			}
		}

		if ( o.iconpos ) {
			buttonClass += " ui-btn-icon-" + o.iconpos;
			
			if ( o.iconpos == "notext" && !el.attr("title") ) {
				el.attr( "title", el.text() );
			}
		}
		
		if ( o.corners ) { 
			buttonClass += " ui-btn-corner-all";
			innerClass += " ui-btn-corner-all";
		}
		
		if ( o.shadow ) {
			buttonClass += " ui-shadow";
		}
		
		el
			.attr( "data-theme", o.theme )
			.addClass( buttonClass );

		el.attr('class',buttonClass);
		el.find('.ui-btn-inner').attr('class',innerClass);
		if (o.icon)
			el.find('.ui-icon').attr('class',iconClass);
	});		
};

})( jQuery );// file: 110-MobileView.js
(function( $, undefined ) {

	tersus.isIphoneBrowser = navigator.userAgent.match(/Mobile.*Safari/) && !navigator.userAgent.match(/Android/) &&  !navigator.standalone;
	tersus.availableHeight = function(w)
	{
		if (!w) w = window;
		var h =  w.innerHeight ? w.innerHeight : $(w).height();
		return tersus.isIphoneBrowser & !tersus.safari.scrolled ? h+60 : h;	
		
	};
	var setter=tersus.setter;
	$.mobile.ajaxEnabled = false;
tersus.safari={scrolled:false}; //Keep track of whether we've already scrolled away the browser chrome (iPhone) as it changes the behavior of innerHeight;	
tersus.Widget('MobileView','HTMLTag',{
	defaultTag:'div'
	,hideNavbar:function()
	{
		if (this.navbar)
		{
			$(this.navbar).css('display','none');
		}
	}
	,showNavbar:function()
	{
		if (this.navbar)
		{
			$(this.navbar).css('display','block');
			$.fixedToolbars.show();
		}
	}
	,'set<Selected Page>':setter(function(p)
		{
			if (p.parent != this)
				throw {message:'Invalid element specified as selected page'};
			this.select(p.getNodeId());
		},true)
	,'get<Selected Page>':function(p)
	{
		return this.selectedPage;
	}
	,onload: function()
	{
		tersus.HTMLTag.prototype.onload.call(this);
	}
	,checkBrowser:function()
	{
	    if ( !$.mobile.gradeA() || !$.support.cssTransitions) {
	    	alert('Your browser does not have all features required to run this application. For best results, please use Safari or Chrome');
	    }
	}
	,create:function(value){
		this.checkBrowser();
		tersus.HTMLTag.prototype.create.call(this);
		if (!$.mobile.gradeA())
			return;
        //set up active page
        var p = $.mobile.startPage = $.mobile.activePage = $('[data-role=page]').first();
        //set page container
        $.mobile.pageContainer = p.parent().addClass('ui-mobile-viewport');
        window.tersusMobileInit();
        //initialize page as active page
        p.page({addBackBtn:false}).addClass( $.mobile.activePageClass);
		$(this.navbar).find('a').each(function(i)
			{
				$(this).find('.ui-icon').css('background-image',$(this).data('iconurl'));
			});        
		//remove rendering class
        $('html').removeClass('ui-mobile-rendering');
        $('html').removeClass('ui-loading');
        setTimeout(function(){window.scrollTo(0,1);tersus.safari.scrolled=true;},100);
        
	}
	,template:'${value}' 

	,navSetup:function()
	{
		//Detect mobile safari to know when we can use real fixed positioning
		this.template = $.template(null,'<div data-role="footer" data-position="bottom"><div data-role="navbar"><ul>{{each(i,p) panes}}<li {{if p.tabClass}}class="${p.tabClass}"{{/if}}><a href="javascript:tersus.findNode(${id}).select(${p.paneId})" data-paneid=${p.paneId} {{if icon}}data-icon="${icon}"{{/if}}{{if iconURL}}data-iconurl=url(${iconURL}){{/if}}>{{if p.caption}}${p.caption}{{/if}}</a></li>{{/each}}</ul></div></div>');
			
		
		this.contentTag = 'div';
		this.contentClass = 'ter-tp-content';
		this.sharedProperties = $.extend({'data-role':'page'},this.sharedProperties);
		this.attach = function()
		{
			this.navbar=this.viewNode.lastChild;
			this.content=this.viewNode.firstChild;
//			$(this.selectedPage.wrapperNode).addClass('ter-tp-active');
			$(this.content.firstChild).addClass('ter-tp-active');
			$(this.navbar).find('a[data-paneid='+this.selectedPage.getNodeId()+']').addClass('ui-btn-active');
			if (this.selectedPage.includeInNavbar)
				this.showNavbar();
			else
				this.hideNavbar();
	
		};
		this.onResize = function()
		{
			var height = tersus.availableHeight();
			$('.ui-mobile-viewport').height(height);
			var nh = $(this.navbar).outerHeight();
			var d = this.selectedPage.includeInNavbar ? nh:0;
			$(this.content).css('min-height',height-d);
			$(this.content).css('height',height-d);
			if (tlog) tlog('On Resize: height='+height+' nh='+nh+' d='+d);
			this.eachDisplay(function(i,c){
				var d = c.includeInNavbar ? nh:0;
				 $(c.viewNode).css('min-height',height-d).parent().css('height',height-d);
				 });
			this.onResizeChildren();
			if (this.selectedPage.includeInNavbar)
				this.showNavbar();
			else
				this.hideNavbar();
			
		};
		this.select = function(id)
		{
			$(this.navbar).find('a').removeClass('ui-btn-active').filter('[data-paneid='+id+']').addClass('ui-btn-active');
			var p =tersus.findNode(id);
			if (!p ||  p == this.selectedPage)
				return;
			window.scrollTo(0,0);
			if (this.selectedPage.includeInNavbar)
				this.showNavbar();
			var reverse = p.element.displayOrder>this.selectedPage.element.displayOrder ? '' : ' reverse';
			
			var from=$(this.selectedPage.wrapperNode);
			this.selectedPage = p;
			var to=$(this.selectedPage.wrapperNode);
			to.prependTo(to.parent());
			from.addClass('slide out'+reverse);
			to.addClass('ter-tp-active slide in'+reverse);
			this.onResize();
			var v = this;
			to.animationComplete(function() {
				from.removeClass('ter-tp-active');
				from.add( to ).removeClass("slide out in reverse");
				if (p.includeInNavbar)
					v.showNavbar();
				else
					v.hideNavbar()
				
			});
			tersus.invokeAction('<On Select>',null,null,p);
		};
		this.templateData =  function(displayChildren)
		{
			var panes = [];
			var view = this;
			$.each(displayChildren, function(i,child)
			{
				if (i==0)
					view.selectedPage=child;
				var iconP=child.sprop('icon'), iconJQ=iconP, iconURL=null;
				child.includeInNavbar = child.sprop('includeInNavbar') != false; // Todo: support local property
				if (!child.includeInNavbar)
					return;
				if (iconP && iconP.match(/[.]/))
				{
					iconJQ='custom';
					iconURL=iconP;
				};
				panes.push({caption:child.getElement('<Caption>')?child.getCaption():null,icon:iconJQ,iconURL:iconURL,paneId:child.getNodeId(), tabClass:child.sprop('tabStyleClass')});
			});
			return {'panes':panes};
		}
	}
	,nonavSetup: function()
	{
		var pe = this.displayElements[0];
		pe.properties = $.extend({'data-role':'page'},pe.properties);
  		this.onResize = function()
		{
			var height = tersus.availableHeight();
			$('.ui-mobile-viewport').height(height);
			$(this.viewNode).css('min-height',height);
			this.eachDisplay(function(i,c){
				 $(c.wrapperNode).css('min-height',height);
				 $(c.viewNode).css('min-height',null);
				 });
			this.onResizeChildren();
		};
  	}
	,prepareForHTML:function(e,parent)
	{
		if (this.displayElements && this.displayElements.length >1 )
		{
			this.navSetup();
		}
		else if (this.displayElements)
		{
			this.nonavSetup();
		}

		return tersus.HTMLTag.prototype.prepareForHTML.call(this,e,parent);
		
	}
	,attachViewNode:function()
	{
		$(this.wrapperNode).appendTo( this.parent ? this.parent.viewNode : '#tersus\\.content');
	}
});
})(jQuery);